
// Title:       A Synchronous bean for implementing a Z3950 Origin
// @version:    $Id: Z3950OriginBean.java,v 1.6 2006/01/12 13:52:24 jorpiell Exp $
// Copyright:   Copyright (C) 1999,2000 Knowledge Integration Ltd.
// @author:     Ian Ibbotson (ibbo@k-int.com)
// Company:     KI
// Description:
//
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2.1 of
// the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite
// 330, Boston, MA  02111-1307, USA.
//
package es.gva.cit.catalogClient.protocols;
import com.k_int.IR.InvalidQueryException;
import com.k_int.IR.QueryModel;
import com.k_int.IR.SearchException;
import com.k_int.IR.TimeoutExceededException;
import com.k_int.codec.util.OIDRegister;
import com.k_int.gen.AsnUseful.EXTERNAL_type;
import com.k_int.gen.RecordSyntax_explain.CategoryInfo_type;
import com.k_int.gen.RecordSyntax_explain.CategoryList_type;
import com.k_int.gen.RecordSyntax_explain.DatabaseInfo_type;
import com.k_int.gen.RecordSyntax_explain.Explain_Record_type;
import com.k_int.gen.Z39_50_APDU_1995.DefaultDiagFormat_type;
import com.k_int.gen.Z39_50_APDU_1995.DiagRec_type;
import com.k_int.gen.Z39_50_APDU_1995.InitializeResponse_type;
import com.k_int.gen.Z39_50_APDU_1995.NamePlusRecord_type;
import com.k_int.gen.Z39_50_APDU_1995.PDU_type;
import com.k_int.gen.Z39_50_APDU_1995.PresentResponse_type;
import com.k_int.gen.Z39_50_APDU_1995.Records_type;
import com.k_int.gen.Z39_50_APDU_1995.ScanResponse_type;
import com.k_int.gen.Z39_50_APDU_1995.SearchRequest_type;
import com.k_int.gen.Z39_50_APDU_1995.SearchResponse_type;
import com.k_int.gen.Z39_50_APDU_1995.record_inline13_type;
import com.k_int.util.LoggingFacade.LogContextFactory;
import com.k_int.util.LoggingFacade.LoggingContext;
import com.k_int.util.RPNQueryRep.RootNode;
import com.k_int.z3950.util.APDUEvent;
import com.k_int.z3950.util.APDUListener;
import com.k_int.z3950.util.GenericEventToOriginListenerAdapter;
import com.k_int.z3950.util.PDUTypeAvailableSemaphore;
import com.k_int.z3950.util.ReferencedPDUAvaialableSemaphore;
import com.k_int.z3950.util.ZEndpoint;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;
// for OID Register
// Used to represent query structures

/**
 * SynchronousOriginBean : A Z3950 session that trys to behave as a synchronous local resource. IE
 * Z39.50 Requests wait for their corresponding response before returning (Or time out)
 * 
 * 
 * @version: $Id: Z3950OriginBean.java,v 1.6 2006/01/12 13:52:24 jorpiell Exp $
 * @author: Ian Ibbotson (ian.ibbotson@k-int.com)
 */
public class Z3950OriginBean implements APDUListener {

/**
 * 
 * 
 */
    public static final int NO_CONNECTION = 1;

/**
 * 
 * 
 */
    public static final int CONNECTING = 2;

/**
 * 
 * 
 */
    public static final int CONNECTED = 3;

/**
 * 
 * 
 */
    public static final String NO_RS_NAME = "default";
/**
 * 
 * 
 */
    protected static LoggingContext cat = LogContextFactory.getContext(
            "SynchronousOriginBean");
/**
 * 
 * 
 */
    ZEndpoint assoc = null;
/**
 * 
 * 
 */
    OIDRegister reg = null;

/**
 * 
 * 
 */
    Vector db_names = new Vector();

/**
 * 
 * 
 */
    String record_syntax = "sutrs";
// Default is to ask for brief sutrs records

/**
 * 
 * 
 */
    int session_status = NO_CONNECTION;

/**
 * 
 * 
 */
    private HashMap responses = new HashMap();

/**
 * 
 * 
 */
    private Hashtable dbinfo = new Hashtable();

/**
 * 
 * 
 */
    private boolean supports_named_result_sets = false;

/**
 * 
 * 
 */
    public  Z3950OriginBean() {        
        db_names.add("Default");
        // The OIDRegister static initialiser now looks for an oidreg.default file via the classloader
        // instead of having lots of messy static data in this file. You can add new OID's to that file
        // (See the OIDRegister class in the A2J Project) Or by calling the OIDRegConfigurator load 
        // method with the name of the resource to be loaded.
        reg = OIDRegister.getRegister();
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public int getSessionStatus() {        
        return session_status;
    } 

/**
 * 
 * 
 * 
 * @param i 
 */
    public void setSessionStatus(int i) {        
        session_status = i;
    } 

/**
 * 
 * 
 * 
 * @param record_syntax 
 */
    public void setRecordSyntax(String record_syntax) {        
        this.record_syntax = record_syntax;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public String getRecordSyntax() {        
        return record_syntax;
    } 

/**
 * 
 * 
 */
    public void clearAllDatabases() {        
        db_names.clear();
    } 

/**
 * 
 * 
 * 
 * @param dbname 
 */
    public void addDatatabse(String dbname) {        
        db_names.add(dbname);
    } 

/**
 * 
 * 
 */
    public void disconnect() {        
        if (null != assoc) {
            System.err.println("Closing existing listener");
            try {
                assoc.shutdown();
            } catch (java.io.IOException ioe) {
                ioe.printStackTrace();
            }
        }
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param hostname 
 * @param portnum 
 */
    public InitializeResponse_type connect(String hostname, String portnum) {        
        // Close down any outstanding assoc
        disconnect();
        return connect(hostname, portnum, 0, null, null, null);
    } 
// 0 = none, 1=Anonymous, 2=open, 3=idpass
// for open, the access string, for idpass, the id
// group
// password

/**
 * 
 * 
 * 
 * @return 
 * @param hostname 
 * @param portnum 
 * @param auth_type 
 * @param principal 
 * @param group 
 * @param credentials 
 */
    public InitializeResponse_type connect(String hostname, String portnum, int auth_type, String principal, String group, String credentials) {        
        System.err.println("Connect");
        InitializeResponse_type retval = null;
        try {
            disconnect();
            Properties p = new Properties();
            p.setProperty("ServiceHost", hostname);
            p.setProperty("ServicePort", portnum);
            p.setProperty("service_auth_type", "" + auth_type);
            if (principal != null) {
                p.setProperty("service_user_principal", principal);
            }
            if (group != null) {
                p.setProperty("service_user_group", group);
            }
            if (credentials != null) {
                p.setProperty("service_user_credentials", credentials);
            }
            cat.debug(p.toString());
            System.out.println("Create listener & encoder");
            assoc = new ZEndpoint(p);
            // Convert incoming observer/observable notifications into PDU notifications.
            assoc.getPDUAnnouncer().addObserver(new GenericEventToOriginListenerAdapter(
                    this));
            // Look out for the init response
            PDUTypeAvailableSemaphore s = new PDUTypeAvailableSemaphore(PDU_type.initresponse_CID,
                    assoc.getPDUAnnouncer());
            assoc.start();
            try {
                s.waitForCondition(20000); // Wait up to 20 seconds for an init response
                retval = (InitializeResponse_type) s.the_pdu.o;
            } catch (Exception e) {
            } finally {
                s.destroy();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return retval;
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param query 
 * @param refid 
 * @param setname 
 */
    public SearchResponse_type sendSearch(QueryModel query, String refid, String setname) throws SearchException, InvalidQueryException {        
        return sendSearch(query, refid, setname, "F");
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param query 
 * @param refid 
 * @param setname 
 * @param elements 
 */
    public SearchResponse_type sendSearch(QueryModel query, String refid, String setname, String elements) throws SearchException, InvalidQueryException {        
        System.err.println("Sending search request....");
        SearchResponse_type retval = null;
        if (refid == null) {
            refid = assoc.genRefid("Search");
        }
        // ReferencedPDUAvaialableSemaphore s = new ReferencedPDUAvaialableSemaphore(refid, assoc.getPDUAnnouncer() );
        PDUTypeAvailableSemaphore s = new PDUTypeAvailableSemaphore(PDU_type.searchresponse_CID,
                assoc.getPDUAnnouncer());
        try {
            if (cat.isDebugEnabled()) {
                cat.debug("Sending Search request, setname=" + setname +
                    " syntax=" + record_syntax);
            }
            assoc.sendSearchRequest(db_names, query, refid, 3, 5, 10, true, // ssub, lslb, mspn
                ((supports_named_result_sets == true) ? setname : NO_RS_NAME), // Result set name
                elements, elements, reg.oidByName(record_syntax));
            s.waitForCondition(20000); // Wait up to 20 seconds for a message of type SearchResponse
            retval = (SearchResponse_type) s.the_pdu.o;
            if (retval.records != null) {
                if (retval.records.which == Records_type.nonsurrogatediagnostic_CID) {
                    // Record contains defaultDiagFormat object
                    DefaultDiagFormat_type d = (DefaultDiagFormat_type) retval.records.o;
                    if (d.addinfo != null) {
                        throw new SearchException("Diagnostic [" + d.condition +
                            "] Additional Info : " + d.addinfo.o, retval);
                    } else {
                        throw new SearchException("Diagnostic [" + d.condition +
                            "] no additional info", retval);
                    }
                } else if (retval.records.which == Records_type.multiplenonsurdiagnostics_CID) {
                    // Record contains DiagRec ( May have externally defined info )
                    DiagRec_type d = (DiagRec_type) retval.records.o;
                    if (d.which == DiagRec_type.defaultformat_CID) {
                        DefaultDiagFormat_type dd = (DefaultDiagFormat_type) d.o;
                        throw new SearchException("Diagnostic [" +
                            dd.condition + "] Additional Info : " +
                            dd.addinfo.o, retval);
                    } else {
                        throw new SearchException("Externally defined diagnostic record",
                            retval);
                    }
                } else {
                    System.err.println("Search has valid piggyback records");
                }
            }
        } catch (java.io.IOException ioe) {
            // Problem with comms sending PDU...
            ioe.printStackTrace();
        } catch (TimeoutExceededException tee) {
            tee.printStackTrace();
        } finally {
            s.destroy();
        }
        return retval;
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param query 
 */
    public SearchResponse_type sendSearch(RootNode query) throws SearchException, InvalidQueryException {        
        SearchResponse_type retval = null;
        String refid = assoc.genRefid("Search");
        ReferencedPDUAvaialableSemaphore s = new ReferencedPDUAvaialableSemaphore(refid,
                assoc.getPDUAnnouncer());
        try {
            assoc.sendSearchRequest(db_names, query, refid, 10, 20, 10, true,
                "simple", "f", "f", reg.oidByName(record_syntax));
            s.waitForCondition(20000); // Wait up to 20 seconds for a response
            retval = (SearchResponse_type) s.the_pdu.o;
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
        } catch (TimeoutExceededException tee) {
            tee.printStackTrace();
        } finally {
            s.destroy();
        }
        if (retval.records != null) {
            if (retval.records.which == Records_type.nonsurrogatediagnostic_CID) {
                // Record contains defaultDiagFormat object
                DefaultDiagFormat_type d = (DefaultDiagFormat_type) retval.records.o;
                throw new SearchException("Diagnostic [" + d.condition +
                    "] Additional Info : " + d.addinfo.o, retval);
            } else if (retval.records.which == Records_type.multiplenonsurdiagnostics_CID) {
                // Record contains DiagRec ( May have externally defined info )
                DiagRec_type d = (DiagRec_type) retval.records.o;
                if (d.which == DiagRec_type.defaultformat_CID) {
                    DefaultDiagFormat_type dd = (DefaultDiagFormat_type) d.o;
                    throw new SearchException("Diagnostic [" + dd.condition +
                        "] Additional Info : " + dd.addinfo.o, retval);
                } else {
                    throw new SearchException("Externally defined diagnostic record",
                        retval);
                }
            } else {
                System.err.println("Search seems has valid piggyback records");
            }
        }
        return retval;
    } 

/**
 * Alternate sendSearch that simply passes along a search request PDU. Added for proxy server.
 * 
 * 
 * @return 
 * @param req 
 */
    public SearchResponse_type sendSearch(PDU_type req) throws SearchException, InvalidQueryException {        
        SearchResponse_type retval = null;
        SearchRequest_type search_req = (SearchRequest_type) req.o;
        ReferencedPDUAvaialableSemaphore s = new ReferencedPDUAvaialableSemaphore(new String(
                    search_req.referenceId), assoc.getPDUAnnouncer());
        try {
            assoc.encodeAndSend(req);
            s.waitForCondition(20000); // Wait up to 20 seconds for a response
            retval = (SearchResponse_type) s.the_pdu.o;
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
        } catch (TimeoutExceededException tee) {
            tee.printStackTrace();
        } finally {
            s.destroy();
        }
        return retval;
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param qm 
 */
    public ScanResponse_type sendScan(QueryModel qm) {        
        ScanResponse_type retval = null;
        PDUTypeAvailableSemaphore s = new PDUTypeAvailableSemaphore(PDU_type.scanresponse_CID,
                assoc.getPDUAnnouncer());
        try {
            assoc.sendScanRequest("scan", db_names, qm, 0, 10, 5);
            s.waitForCondition(20000);
            retval = (ScanResponse_type) s.the_pdu.o;
        } catch (TimeoutExceededException tee) {
            tee.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            s.destroy();
        }
        System.err.println("Done scan request");
        return retval;
    } 
// Notification Handlers

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingAPDU(APDUEvent e) {        
        // We don't care about this
        // System.err.println("Incoming APDU notification");
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param start 
 * @param count 
 * @param element_set_name 
 * @param setname 
 */
    public PresentResponse_type sendPresent(long start, long count, String element_set_name, String setname) {        
        PresentResponse_type retval = null;
        String refid = assoc.genRefid("Present");
        PDUTypeAvailableSemaphore s = new PDUTypeAvailableSemaphore(PDU_type.presentresponse_CID,
                assoc.getPDUAnnouncer());
        // ReferencedPDUAvaialableSemaphore s = new ReferencedPDUAvaialableSemaphore(refid, assoc.getPDUAnnouncer() );
        try {
            if (cat.isDebugEnabled()) {
                cat.debug("Sending present request for " + count +
                    " records starting at " + start + ", setname=" + setname +
                    " syntax=" + record_syntax);
            }
            assoc.sendPresentRequest(refid,
                ((supports_named_result_sets == true) ? setname : NO_RS_NAME),
                start, count, record_syntax, element_set_name);
            s.waitForCondition(20000); // Wait up to 20 seconds for a response
            retval = (PresentResponse_type) s.the_pdu.o;
        } catch (TimeoutExceededException tee) {
            tee.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            s.destroy();
        }
        System.err.println("Done present request");
        return retval;
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingInitRequest(APDUEvent e) {        
        cat.debug("Incoming InitRequest");
        // Preparation for synchronous Retrieval API
        notifyAll();
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingInitResponse(APDUEvent e) {        
        InitializeResponse_type init_response = (InitializeResponse_type) (e.getPDU().o);
        responses.put(init_response.referenceId, init_response);
        session_status = CONNECTED;
        cat.debug("Incoming init response " + init_response.referenceId);
        if (init_response.result.booleanValue() == true) {
            if (init_response.options.isSet(14)) {
                cat.debug("Target supports named result sets");
            } else {
                cat.debug("Target does not support named result sets");
                supports_named_result_sets = false;
            }
        }
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingSearchRequest(APDUEvent e) {        
        cat.debug("\nIncoming SearchRequest");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingSearchResponse(APDUEvent e) {        
        SearchResponse_type search_response = (SearchResponse_type) e.getPDU().o;
        responses.put(search_response.referenceId, search_response);
        cat.debug("Incoming search response " + search_response.referenceId);
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingPresentRequest(APDUEvent e) {        
        cat.debug("Incoming PresentResponse");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingPresentResponse(APDUEvent e) {        
        PresentResponse_type present_response = (PresentResponse_type) e.getPDU().o;
        responses.put(present_response.referenceId, present_response);
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingDeleteResultSetRequest(APDUEvent e) {        
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingDeleteResultSetResponse(APDUEvent e) {        
        cat.debug("Incoming DeleteResultSetResponse");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingAccessControlRequest(APDUEvent e) {        
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingAccessControlResponse(APDUEvent e) {        
        System.err.println("Incoming AccessControlResponse");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingResourceControlRequest(APDUEvent e) {        
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingResourceControlResponse(APDUEvent e) {        
        System.err.println("Incoming ResourceControlResponse");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingTriggerResourceControlRequest(APDUEvent e) {        
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingResourceReportRequest(APDUEvent e) {        
        System.err.println("Incoming ResourceReportResponse");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingResourceReportResponse(APDUEvent e) {        
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingScanRequest(APDUEvent e) {        
        System.err.println("Incoming ScanResponse");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingScanResponse(APDUEvent e) {        
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingSortRequest(APDUEvent e) {        
        System.err.println("Incoming SortResponse");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingSortResponse(APDUEvent e) {        
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingSegmentRequest(APDUEvent e) {        
        System.err.println("Incoming SegmentResponse");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingExtendedServicesRequest(APDUEvent e) {        
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingExtendedServicesResponse(APDUEvent e) {        
        System.err.println("Incoming ExtendedServicesResponse");
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void incomingClose(APDUEvent e) {        
        // Preparation for synchronous Retrieval API
        synchronized (this) {
            notifyAll();
        }
    } 

/**
 * 
 * 
 */
    public void doExplain() throws SearchException, InvalidQueryException {        
        clearAllDatabases();
        addDatatabse("IR-Explain-1");
        setRecordSyntax("explain");
        SearchResponse_type r = sendSearch(new com.k_int.IR.QueryModels.PrefixString(
                    "@attrset exp-1 @attr 1=1 categoryList"), "exp", "expset");
        if (r.searchStatus.booleanValue()) {
            // OK, Our search for a category list in the explain database seems to have worked...
            // Figure out what useful info we can from the available categories
            if (r.resultCount.intValue() == 1) {
                // That's good, only one category list
                CategoryList_type cl = null;
                if (r.numberOfRecordsReturned.intValue() == 1) {
                    // Even better, it looks like the category list has been piggybacked
                    if (r.records.which == Records_type.responserecords_CID) {
                        Vector response_records = (Vector) r.records.o;
                        NamePlusRecord_type npr = (NamePlusRecord_type) response_records.get(0);
                        System.err.println("Got record from " + npr.name);
                        switch (npr.record.which) {
                        case record_inline13_type.retrievalrecord_CID: {
                            EXTERNAL_type et = (EXTERNAL_type) npr.record.o;
                            Object o = et.encoding.o;
                            System.err.println("Got retrieval record");
                            if (o instanceof Explain_Record_type) {
                                cl = (CategoryList_type) (((Explain_Record_type) o).o);
                                for (int i = 0; i < cl.categories.size();
                                        i++) {
                                    CategoryInfo_type ci = (CategoryInfo_type) cl.categories.get(i);
                                    System.err.println("Aviailable category : " +
                                        ci.category);
                                    if (null != ci.category) {
                                        if (ci.category.equals("DatabaseInfo")) {
                                            explainDatabaseInfo();
                                        }
                                    }
                                }
                            }
                            break;
                        }
                        default:
                            System.err.println(
                                "Unhandled name plus record option : " +
                                npr.record.which);
                        }
                    } else {
                        System.err.println("Might have a diagnostic here...");
                    }
                    // If not, it's probably a diagnostic....
                }
                // Let's hope we have managed to set cl in some of the above
            }
        }
    } 

/**
 * 
 * 
 */
    private void explainDatabaseInfo() throws SearchException, InvalidQueryException {        
        clearAllDatabases();
        addDatatabse("IR-Explain-1");
        dbinfo.clear();
        setRecordSyntax("explain");
        SearchResponse_type r = sendSearch(new com.k_int.IR.QueryModels.PrefixString(
                    "@attrset exp-1 @attr 1=1 databaseInfo"), "expdbinfo",
                "dbinfoset");
        if (r.searchStatus.booleanValue()) {
            // OK, Our search for database info in the explain database seems to have worked...
            // Figure out what useful info we can from the available categories
            if (r.resultCount.intValue() > 0) {
                if (r.numberOfRecordsReturned.intValue() > 0) {
                    // Even better, it looks like we have some piggyback records
                    if (r.records.which == Records_type.responserecords_CID) {
                        processDatabaseInfoRecords((Vector) (r.records.o));
                    }
                    // If not, it's probably a diagnostic....
                }
                // Use present to get the records (For now we will rely on number of databases
                // being less that ssub ).
            }
        }
    } 

/**
 * 
 * 
 * 
 * @param records 
 */
    private void processDatabaseInfoRecords(Vector records) {        
        for (int i = 0; i < records.size(); i++) {
            NamePlusRecord_type npr = (NamePlusRecord_type) records.get(i);
            if (npr.record.which == record_inline13_type.retrievalrecord_CID) {
                EXTERNAL_type et = (EXTERNAL_type) npr.record.o;
                Object o = et.encoding.o;
                if (o instanceof Explain_Record_type) {
                    DatabaseInfo_type di = (DatabaseInfo_type) (((Explain_Record_type) o).o);
                    dbinfo.put(di.name, di);
                    System.err.println("Aviailable database: " + di.name);
                }
            }
        }
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public Enumeration getDatabases() {        
        return dbinfo.elements();
    } 
 }
