
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.catalogClient.schemas.discoverer;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.schemas.discoverer.Record;
import es.gva.cit.catalogClient.schemas.parsers.DeegreeISO19115Parser;
import es.gva.cit.catalogClient.schemas.parsers.DublinCoreParser;
import es.gva.cit.catalogClient.schemas.parsers.GeonetworkISO19115Parser;
import es.gva.cit.catalogClient.schemas.parsers.IdecISO19115Parser;
import es.gva.cit.catalogClient.schemas.parsers.IdeeISO19115Parser;
import com.iver.utiles.swing.jcomboServer.ServerData;
import java.net.URL;

/**
 * This class is used to discover the format of the retrieved records.
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class DiscovererFormat {

/**
 * 
 * 
 */
    private String protocol;

/**
 * 
 * 
 */
    private URL url;

/**
 * Constructor
 * 
 * 
 * @param protocol Catalog protocol
 * @param url Server url
 */
    public  DiscovererFormat(String protocol, URL url) {        
        this.protocol = protocol;
        this.url = url;
    } 

/**
 * This method discovers the record format and parses it.
 * 
 * 
 * @return 
 * @param node XMLNode
 */
    public Record DiscoverAndParse(XMLNode node) {        
       if (node == null){
    	   return new DublinCoreParser(node);
       }
    	if ((node.getName().equals("dc:metadata")) ||
                (node.getName().equals("simpledc"))){
            return new DublinCoreParser(node);
        }
        
        if (node.getName().equals("iso19115:MD_Metadata")){
            return new DeegreeISO19115Parser(node);
        }        
        
        if (protocol.equals(ServerData.SERVER_SUBTYPE_CATALOG_Z3950)) {
             return new GeonetworkISO19115Parser(node,url);
        } else if (protocol.equals(ServerData.SERVER_SUBTYPE_CATALOG_SRW)) {
            return new IdeeISO19115Parser(node);
        } else {
            if (node.getName().equals("dc:metadata")){
                return new DublinCoreParser(node);
            }else{
                return new IdecISO19115Parser(node);
            }
        }
        
    } 
 }
