
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.catalogClient.ui.serverConnect;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import es.gva.cit.catalogClient.CatalogClient;
import es.gva.cit.catalogClient.traductor.ITranslator;
import es.gva.cit.catalogClient.traductor.Translator;
import es.gva.cit.catalogClient.ui.search.SearchDialog;
import com.iver.utiles.swing.jcomboServer.ServerData;

/**
 * 
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class ServerConnectDialogPanel extends JPanel implements ActionListener {

/**
 * 
 * 
 */
    private static TreeMap serverList = new TreeMap();
//Panels
/**
 * 
 * 
 */
    private JPanel ppalPanel = null;
/**
 * 
 * 
 */
    protected ServerConnectPanel controlsPanel = null;
/**
 * 
 * 
 */
    private JPanel buttonsPanel = null;
//Buttons
/**
 * 
 * 
 */
    private JButton connect = null;
/**
 * 
 * 
 */
    private JButton search = null;
/**
 * 
 * 
 */
    private JButton close = null;
//Others
/**
 * 
 * 
 */
    private CatalogClient client = null;

/**
 * 
 * 
 */
    private String serversFile = "servers/CatalogServers.txt";

/**
 * 
 * 
 */
    private String currentServer = "";
/**
 * 
 * 
 */
    private ITranslator translator = null;
/**
 * 
 * 
 */
    private JFrame parent = null;

/**
 * 
 * 
 * 
 * @param parent 
 * @param translator 
 */
    public  ServerConnectDialogPanel(JFrame parent, ITranslator translator) {        
        this.parent = parent;        
        this.translator = translator;
        
        ppalPanel = new JPanel();
        ppalPanel.setLayout(new BoxLayout(ppalPanel, BoxLayout.Y_AXIS));
        ppalPanel.setPreferredSize(new Dimension(585,240));
        ppalPanel.add(getControlsPanel(), null);
        ppalPanel.add(getButtonPanel(), null);
        
        add(ppalPanel);
        setDefaultButtonListeners();
        loadServerList(serversFile);
        
        
    } 

/**
 * Gets the panel where are all the controls
 * 
 * 
 * @return JPanel
 */
    public ServerConnectPanel getControlsPanel() {        
        if (controlsPanel == null) {
            controlsPanel = new ServerConnectPanel(translator);
            controlsPanel.getZ3950Button().addActionListener(this);
            controlsPanel.getSrwButton().addActionListener(this);
            controlsPanel.getCswButton().addActionListener(this);
            controlsPanel.getServidoresCombo().addActionListener(this);
        }
        return controlsPanel;
    } 

/**
 * Gets the panel where are all the buttons
 * 
 * 
 * @return JPanel
 */
    public JPanel getButtonPanel() {        
        if (buttonsPanel == null) {
        	FlowLayout flowLayout = new FlowLayout();
			flowLayout.setAlignment(FlowLayout.RIGHT);
            buttonsPanel = new JPanel(flowLayout);
            buttonsPanel.add(getConnectButton());
            buttonsPanel.add(getSearchButton());
            buttonsPanel.add(getCloseButton());
        }
        return buttonsPanel;
    } 

/**
 * Gets the Connect Button
 * 
 * 
 * @return JButton
 */
    public JButton getConnectButton() {        
        if (connect == null) {
            connect = new JButton(Translator.getText(translator,"connectButton"));
            connect.setPreferredSize(new Dimension(80, 23));
            connect.setActionCommand("Connect");
        }
        return connect;
    } 

/**
 * Gets the Serach Button
 * 
 * 
 * @return JButton
 */
    public JButton getSearchButton() {        
        if (search == null) {
            search = new JButton(Translator.getText(translator,"searchButton"));
            search.setPreferredSize(new Dimension(80, 23));
            search.setActionCommand("Search");
            search.setEnabled(false);
        }
        return search;
    } 

/**
 * Gets the Close Button
 * 
 * 
 * @return JButtonServerConnectDialogPanel
 */
    public JButton getCloseButton() {        
        if (close == null) {
            close = new JButton(Translator.getText(translator,"close"));
              close.setPreferredSize(new Dimension(80, 23));
            close.setActionCommand("close");
              
            }
            return close;
    } 

/**
 * It adds a server in the TreeMap Object
 * 
 * 
 * @param server 
 */
    public static void addTreeMapServer(ServerData server) {        
        if (ServerConnectDialogPanel.serverList == null) {
            ServerConnectDialogPanel.serverList = new TreeMap();
        }
        serverList.put(server.getServerAddress(), server);
      
    } 

/**
 * Sets the listeners
 * 
 */
    public void setDefaultButtonListeners() {        
        getConnectButton().addActionListener(this);
        getSearchButton().addActionListener(this);
        getCloseButton().addActionListener(this);
    } 

/**
 * This method loads a server list in the combo
 * 
 * 
 * @param sfile 
 */
    public void loadServerList(String sfile) {        
        loadServersFromFile(sfile);
        
        Iterator iter = serverList.keySet().iterator();
        while (iter.hasNext()) {
            ServerData server = (ServerData) serverList.get((String) iter.next());
            controlsPanel.getServerCombo().addServer(server);
        }            
    } 

/**
 * It loads a server list from a text file
 * 
 * 
 * @param sfile 
 */
    public void loadServersFromFile(String sfile) {        
        File file = null;
        try {
            file = new File(sfile);
            if (file.exists()) {
                BufferedReader fr = new BufferedReader(new FileReader(file));
                String s;
                while ((s = fr.readLine()) != null) {
                    addTreeMapServer(new ServerData(s,"",""));
                }
            } else {
                System.out.println("No se encuentra el fichero '" +
                    file.getPath() + "'");
            }
        } catch (FileNotFoundException e) {
            System.out.println("No se encuentra el fichero '" + file.getPath() +
                "'");
            //e.printStackTrace();
        } catch (IOException e) {
            System.out.println("Error de entrada salida en la lectura del fichero");
            //e.printStackTrace();
        }
    } 
/* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */

/**
 * 
 * 
 * 
 * @param e 
 */
    public void actionPerformed(ActionEvent e) {        
        //Connect
        if (e.getActionCommand() == "Connect") {
            connectButtonActionPerformed();
        }
        //BUSCAR
        if (e.getActionCommand() == "Search") {
            searchButtonActionPerformed(); 
        }
        if ((e.getActionCommand() == "Z39.50") ||
                (e.getActionCommand() == "SRU/SRW") ||
                (e.getActionCommand() == "CS-W") ||
                (e.getActionCommand() == "comboBoxChanged")) {
            search.setEnabled(false);
        }
        if (e.getActionCommand() == "Z39.50") {
            controlsPanel.getDbText().setEnabled(true);
        }
        if ((e.getActionCommand() == "SRU/SRW") ||
                (e.getActionCommand() == "CS-W")) {
            controlsPanel.getDbText().setEnabled(false);
        }
        
        if (e.getActionCommand() == "servidoresCombo"){
            try {
                controlsPanel.setProtocol(controlsPanel.getServer().getServiceSubType());
                controlsPanel.setDatabase();
            }catch(NullPointerException ex){
                //The server is not loaded 
            }
            
        }
        if (e.getActionCommand() == "close") {
             closeButtonActionPerformed();
        }
        
    } 

/**
 * Search Button action
 * 
 */
    public void searchButtonActionPerformed() {        
	    setEnabled(false);
      new SearchDialog(client,true,"",parent);
    } 

/**
 * Search Button action
 * 
 */
    public void closeButtonActionPerformed() {        
        parent.setVisible(false);
    } 

/**
 * This method try to connect with the server
 * 
 */
    private void connectButtonActionPerformed() {        
        search.setEnabled(false);
        //Create a new atalogClient
        client = new CatalogClient(controlsPanel.getServerAddress(),
                controlsPanel.getProtocol(), controlsPanel.getDatabase());
        
        if (client.getCapabilities()){
            search.setEnabled(true);
            currentServer = controlsPanel.getServerAddress();
            searchButtonActionPerformed();
        }
        
        controlsPanel.setRespuesta(translator.getText(client.getServerStatus()));
        
    } 

/**
 * 
 * 
 * 
 * @return Returns the serversFile.
 */
    public String getServersFile() {        
        return serversFile;
    } 

/**
 * 
 * 
 * 
 * @param serversFile The serversFile to set.
 */
    public void setServersFile(String serversFile) {        
        this.serversFile = serversFile;
    } 

/**
 * 
 * 
 * 
 * @return Returns the currentServer.
 */
    public String getCurrentServer() {        
        return currentServer;
    } 

/**
 * 
 * 
 * 
 * @return Returns the cliente.
 */
    public CatalogClient getClient() {        
        return client;
    } 
 }
