/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import java.awt.Component;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import javax.swing.JOptionPane;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.cresques.cts.gt2.CoordSys;
import org.cresques.cts.gt2.CoordTrans;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;
import com.iver.cit.gvsig.fmap.drivers.WithDefaultLegend;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.rendering.LegendFactory;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.cit.gvsig.gui.FOpenDialog;
import com.iver.cit.gvsig.gui.FileOpenDialog;
import com.iver.cit.gvsig.gui.View;
import com.iver.cit.gvsig.gui.WizardPanel;


/**
 * Extensin que abre un dilogo para seleccionar la capa o capas que se
 * quieren aadir a la vista.
 *
 * @author Fernando Gonzlez Corts
 */
public class AddLayer implements Extension {
	public FOpenDialog fopen = null;
	
	private static ArrayList wizardStack = null;
	
	static {
		AddLayer.wizardStack = new ArrayList();
	}
	
	public static void addWizard(Class wpClass) {
		AddLayer.wizardStack.add(wpClass);
	}
	
	public static WizardPanel getInstance(int i) throws IllegalArgumentException, SecurityException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		Class wpClass = (Class) AddLayer.wizardStack.get(i);
		Class [] args = {};
		Object [] params = {};
		WizardPanel wp = (WizardPanel) wpClass.getConstructor(args).newInstance(params);

		wp.initWizard();
		
		return wp;
	}


	/**
	 * @see com.iver.mdiApp.plugins.Extension#isVisible()
	 */
	public boolean isVisible() {
		com.iver.andami.ui.mdiManager.View f = PluginServices.getMDIManager()
															 .getActiveView();

		if (f == null) {
			return false;
		}

		return (f.getClass() == View.class);
	}

	/**
	 * @see com.iver.andami.plugins.Extension#inicializar()
	 */
	public void inicializar() {
	}

    private void checkProjection(FLayer lyr, ViewPort viewPort)
    {
        if (lyr instanceof FLyrVect)
        {
            FLyrVect lyrVect = (FLyrVect) lyr;
            IProjection proj = lyr.getProjection();
            // Comprobar que la projeccin es la misma que la vista
            if (proj == null)
            {
                // SUPONEMOS que la capa est en la proyeccin que 
                // estamos pidiendo (que ya es mucho suponer, ya).
                lyrVect.setProjection(viewPort.getProjection());
                return;
            }
            if (proj != viewPort.getProjection()) {
                int option = JOptionPane.showConfirmDialog(null,
                        PluginServices.getText(this, "reproyectar_aviso"),
                        PluginServices.getText(this, "reproyectar_pregunta"),
                        JOptionPane.YES_NO_OPTION);

                if (option == JOptionPane.NO_OPTION) {
                    return;
                } else {
                    ICoordTrans ct = new CoordTrans((CoordSys) proj,
                            (CoordSys) viewPort.getProjection());
                    lyrVect.setCoordTrans(ct);
                    System.err.println("coordTrans = " +
                        proj.getAbrev() + " " +
                        viewPort.getProjection().getAbrev());
                }
            }
        }                    

    }
	/**
	 * @see com.iver.andami.plugins.Extension#execute(java.lang.String)
	 */
	public void execute(String actionCommand) {
		//Project project = ((ProjectExtension) PluginServices.getExtension(ProjectExtension.class)).getProject();
		View theView = (View) PluginServices.getMDIManager().getActiveView();
		fopen = new FOpenDialog();

		FileOpenDialog fileDlg = new FileOpenDialog(new Class[] {
					VectorialFileDriver.class, RasterDriver.class
				});
		fopen.addTab(PluginServices.getText(this, "Fichero"), fileDlg);
		for (int i=0; i<wizardStack.size(); i++) {
			WizardPanel wp;
			try {
				wp = AddLayer.getInstance(i);
				fopen.addWizardTab(wp.getTabName(), wp);			
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}
		
		PluginServices.getMDIManager().addView(fopen);

		if (fopen.isAccepted()) {
			FLayer lyr = null;

			if (fopen.getSelectedTab() == fileDlg) {
				if (fileDlg.getFiles() == null) {
					return;
				}

				IProjection proj = FOpenDialog.getLastProjection();
				File[] files = fileDlg.getFiles();
				String[] driverNames = fileDlg.getDriverNames();
				Driver[] drivers = new Driver[driverNames.length];
				for (int i = 0; i < drivers.length; i++) {
					try {
						drivers[i] = LayerFactory.getDM().getDriver(driverNames[i]);
					} catch (DriverLoadException e) {
						NotificationManager.addError("No se pudo cargar el driver", e);
					}
				}
				Rectangle2D[] rects=new Rectangle2D[files.length];
				boolean first=false;
				theView.getMapControl().getMapContext()
				   .beginAtomicEvent();

				for (int iFile = 0; iFile < files.length; iFile++) {
					File fich = files[iFile];
					String layerName = fich.getName();
					//String layerPath = fich.getAbsolutePath();

					try {
						// FJP: Comento esto (if (fileDlg.accept(fich))) para resolver el bug 75. Esto estaba antes
                        // o lo apuesto alguien por algo en concreto?.
						// if (fileDlg.accept(fich))
    						if (drivers[iFile] instanceof VectorialFileDriver){
    							lyr = LayerFactory.createLayer(layerName,
    									(VectorialFileDriver) drivers[iFile], fich, proj);
    						}else if (drivers[iFile] instanceof RasterDriver){
    							lyr = LayerFactory.createLayer(layerName,
    									(RasterDriver) drivers[iFile], fich, proj);
    						}

						if (lyr != null) {
							lyr.setVisible(true);
							if (theView.getMapControl().getMapContext().getViewPort().getExtent()==null){
								first=true;
							}


                            
                            try {
                                
                                // Le asignamos tambin una legenda por defecto acorde con
                                // el tipo de shape que tenga. Tampoco s si es aqu el
                                // sitio adecuado, pero en fin....
                                checkProjection(lyr, theView.getMapControl().getViewPort());
                                theView.getMapControl().getMapContext().getLayers()
                                   .addLayer(lyr);
                                
                                if (lyr instanceof FLyrVect)
                                {
                                    FLyrVect lyrVect = (FLyrVect) lyr;
                                    if (drivers[iFile] instanceof WithDefaultLegend) {
                                        WithDefaultLegend aux = (WithDefaultLegend) drivers[iFile];
                                        lyrVect.setLegend((VectorialLegend) aux.getDefaultLegend());                                        
                                    } else {
                                        lyrVect.setLegend(LegendFactory.createSingleSymbolLegend(
                                                lyrVect.getShapeType()));
                                    }
                                }
                            } catch (FieldNotFoundException e) {
                                //Esta no puede saltar
                            }                            
							rects[iFile]=lyr.getFullExtent();
							
							// TODO: Poner una variable y dibujar solo cuando
							// todas las capas hayan sido cargadas.
							// TODO Se deber de redibujar mediante la captura de los eventos, por
							//eso se comenta la parte anterior
							//							theView.getMapControl().drawMap();
							//							theView.getTOC().refresh();
						}

					} catch (DriverException e) {
						NotificationManager.addError("Error al crear la capa", e);
					}
				}

				//Esto permite que cuando se cargan varias capas de golpe y la vista est vacia,se ponga como extent la suma de todos sus extents.
				if (rects.length > 1) {
					Rectangle2D rect = new Rectangle2D.Double();
					rect.setRect(rects[0]);

					if (first) {
						for (int i = 0; i < rects.length; i++) {
							rect.add(rects[i]);
						}

						theView.getMapControl().getMapContext().getViewPort()
							   .setExtent(rect);
					}
				}
				theView.getMapControl().getMapContext()
				   .endAtomicEvent();

			} else if (fopen.getSelectedTab() instanceof WizardPanel) {
				WizardPanel wp = (WizardPanel) fopen.getSelectedTab();
				wp.setMapCtrl(theView.getMapControl());
				wp.execute();
				lyr = wp.getLayer();
				
				if (lyr != null) {
					lyr.setVisible(true);
					theView.getMapControl().getMapContext().beginAtomicEvent();
                    checkProjection(lyr, theView.getMapControl().getViewPort());
					theView.getMapControl().getMapContext().getLayers()
						   .addLayer(lyr);
					theView.getMapControl().getMapContext().endAtomicEvent();
                    
				}
			/* } else if (fopen.getSelectedTab() == wms) {
				String layerName = wms.getLayerName();

				lyr = LayerFactory.createLayer(layerName,
						wms.getLayersRectangle(), wms.getHost(),
						wms.getFormat(), wms.getLayersQuery(),
						wms.getQueryableLayerQuery(), wms.getSRS());

				if (lyr != null) {
					lyr.setVisible(true);
					theView.getMapControl().getMapContext().beginAtomicEvent();
					theView.getMapControl().getMapContext().getLayers()
						   .addLayer(lyr);
					theView.getMapControl().getMapContext().endAtomicEvent();
				}
			} else if (fopen.getSelectedTab() == wiz) {
                try {
					String dbURL = wiz.getConnectionString();
					String user = wiz.getUser();
					String pwd = wiz.getPassword();
					String layerName = wiz.getLayerName();
					String fields = StringUtilities.getComaSeparated(wiz.getFields());
					String tableName = wiz.getTable();
					String whereClause = "";
					int fidField = wiz.getFID();;
					
                    
                    // TODO: Cmo se hace esto para que sea genrico?.
                    // Supongo que el wizard te tiene que decir qu driver
                    // quieres usar....
				    // Class.forName("org.postgresql.Driver");
				    // Class.forName("com.mysql.jdbc.Driver");
                    Connection conn = DriverManager.getConnection(dbURL, user, pwd);
                    conn.setAutoCommit(false);

                    VectorialDatabaseDriver driver = wiz.getDriver();
                    if (driver instanceof DefaultDBDriver)
                    {
                        DefaultDBDriver dbDriver = (DefaultDBDriver) driver;
                        dbDriver.setData(conn, tableName, fields, whereClause, fidField);
                    }
                    
                    lyr = LayerFactory.createDBLayer(driver, layerName, null);
                    
    				if (lyr != null) {
    					lyr.setVisible(true);
    					theView.getMapControl().getMapContext().beginAtomicEvent();
    					theView.getMapControl().getMapContext().getLayers()
    						   .addLayer(lyr);
    					theView.getMapControl().getMapContext().endAtomicEvent();
    				}
                    
                } catch (SQLException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                // } catch (ClassNotFoundException e) {
                    // TODO Auto-generated catch block
                    //e.printStackTrace(); 
                } catch (DriverLoadException e) {
					e.printStackTrace();
				}


			} else if (fopen.getSelectedTab() == wfs) 
            {
				String[] layers=wfs.getLayers();
				if (layers.length>0)
                {
					String layerName = wfs.getLayerName();
					String user = wfs.getUser();
					String pwd = wfs.getPassword();
					String url = wfs.getUrlConnect();
				
					boolean protocol=wfs.getProtocol();
					int numfeatures=wfs.getNumFeatures();
					int timeout=wfs.getTimeout();
				//String fields = wfs.getSqlFields();
				//String whereClause = dbWiz.getSqlWhere();
					WFSDriver driver = new WFSDriver();
					driver.setData(url,user,pwd,layers,protocol,numfeatures,timeout,2);
					lyr = LayerFactory.createWFSLayer(layerName,driver,wfs.getHost(),
						wfs.getProjection());

					if (lyr != null) {
						lyr.setVisible(true);
						theView.getMapControl().getMapContext().beginAtomicEvent();
						theView.getMapControl().getMapContext().getLayers()
						   .addLayer(lyr);
						theView.getMapControl().getMapContext().endAtomicEvent();
					}
                }
            } else if (fopen.getSelectedTab() == arcsde_wizard) {
                    String dbHost = arcsde_wizard.getHost();
                    String port = arcsde_wizard.getPort();
                    String user = arcsde_wizard.getUser();
                    String pwd = arcsde_wizard.getPassword();
                    // String layerName = arcsde_wizard.getLayerName();
                    // String fields = StringUtilities.getComaSeparated(wiz.getFields());
                    String tableName = arcsde_wizard.getLayerName();
                    String schema = arcsde_wizard.getSchema();
                    String whereClause = "";
                    String [] fields = null;
                    
                    
                    ArcSdeDriver driver = new ArcSdeDriver();
                    driver.setData(dbHost, Integer.parseInt(port), schema, user, pwd, tableName, fields, whereClause);
                    
                    lyr = LayerFactory.createDBLayer(driver, tableName, null);
                    
                    if (lyr != null) {
                        lyr.setVisible(true);
                        theView.getMapControl().getMapContext().beginAtomicEvent();
                        theView.getMapControl().getMapContext().getLayers()
                               .addLayer(lyr);
                        theView.getMapControl().getMapContext().endAtomicEvent();
                    }                   
                    
			} // for*/
			} else {
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),"Ninguna capa seleccionada");
			}
		}
	}

	/**
	 * @see com.iver.andami.plugins.Extension#isEnabled()
	 */
	public boolean isEnabled() {
		return true;
	}
}
