/*
 * Created on 17-feb-2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.JProgressBar;

import org.cresques.io.Rasterizer;
import org.cresques.io.raster.RasterBuf;
import org.cresques.io.raster.RasterStats;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.fmap.operations.Cancellable;

/**
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 * Sirve datos solicitados por los drivers que salvan a raster. 
 * Hereda de Rasterizer y reescribe el mtodo readData que es el que 
 * ser llamado desde el driver cada vez que vacie el buffer y 
 * necesite ms datos.
 */
public class RasterizerLayer extends Rasterizer{
	
	private ViewPort		viewPort = null;
	private ViewPort		viewPortBlock = null;
	private FLayers			flayers = null;
	private Color			backgroundColor = null;
	private JProgressBar	progressBar = null;
	private double			incrProgressBar = 0;
	private boolean			firstRead = true;
    private int 			nBlocks = 0;
    private double 			percentMax = 100.0D;
    private double			incrTemp = 0;
 	
	/**
	 * Calculo del viewPort
	 * @param vp
	 */
	private void calcViewPort(ViewPort vp){
		Rectangle2D ext = null;

		if(viewPortBlock == null){
			ext = new Rectangle2D.Double(	vp.getExtent().getMinX(),
											vp.getExtent().getMaxY()-wcIntervalo,
											vp.getExtent().getWidth(),
											wcIntervalo
										);
		}else{
			ext = new Rectangle2D.Double(	vp.getExtent().getMinX(),
											vp.getExtent().getMinY()-wcIntervalo,
											vp.getExtent().getWidth(),
											wcIntervalo
										);
		}

		viewPortBlock = new ViewPort(vp.getProjection());
		viewPortBlock.setExtent(ext);
		viewPortBlock.setImageSize(dimension);	
		viewPortBlock.setScale();
		//System.out.println("--->bloque="+contBlocks+" ImageHeight="+viewPortBlock.getImageHeight()+" ImageWidth="+viewPortBlock.getImageWidth());
		//System.out.println("--->minX="+viewPortBlock.getExtent().getMinX()+" minY="+viewPortBlock.getExtent().getMinY()+" maxX="+viewPortBlock.getExtent().getMaxX()+" maxY="+viewPortBlock.getExtent().getMaxY()+" width="+viewPortBlock.getExtent().getWidth()+" height="+viewPortBlock.getExtent().getHeight());
	}
	
	/**
	 * Constructor
	 * @param flyrs capas
	 * @param vp viewport
	 * @param sizeBlock altura del bloque que se lee de una vez en la imagen de entrada 
	 * @param mapCtrl Mapcontrol
	 */
	public RasterizerLayer(FLayers flyrs, ViewPort vp, int sizeBlock, MapControl mapCtrl){
		super(sizeBlock);
		//System.out.println("===>VP="+vp.getExtent().getMinX()+" "+vp.getExtent().getMinY()+" "+vp.getExtent().getMaxX()+" "+vp.getExtent().getMaxY());
		backgroundColor = vp.getBackColor();
		viewPort = new ViewPort(vp.getProjection());
		viewPort.setImageSize(vp.getImageSize());
		viewPort.setExtent(vp.getExtent());
						
		//Calculo del viewPort del primer bloque
		
		wcAlto = viewPort.getExtent().getMaxY() - viewPort.getExtent().getMinY();
		wcIntervalo = (sizeBlock*wcAlto)/viewPort.getImageHeight();		
		dimension = new Dimension(viewPort.getImageWidth(), sizeBlock);
		
		nBlocks = (int)(vp.getImageHeight()/sizeBlock);
				
		incrProgressBar = percentMax/((double)nBlocks);
		//System.out.println("===>NBLOCKS="+nBlocks+" INCR="+incrProgressBar);
		
		//Tamao de ltimo bloque en pixeles
		lastBlock = vp.getImageHeight()-(nBlocks*sizeBlock);
				
		//System.out.println("1--->SizeBlock="+sizeBlock+" ImageHeight="+viewPort.getImageHeight()+" ImageWidth="+viewPort.getImageWidth());
		//System.out.println("2--->wcAlto="+wcAlto+" wcIntervalo="+wcIntervalo);
		//System.out.println("3--->nBlocks="+nBlocks+" lastBlock="+lastBlock);
		//System.out.println("4--->minX="+viewPort.getExtent().getMinX()+" minY="+viewPort.getExtent().getMinY()+" maxX="+viewPort.getExtent().getMaxX()+" maxY="+viewPort.getExtent().getMaxY()+" width="+viewPort.getExtent().getWidth()+" height="+viewPort.getExtent().getHeight());
		
		calcViewPort(viewPort);
		
		this.flayers = flyrs;
		this.sizeBlock = sizeBlock;		
	}
	
	
	/* (non-Javadoc)
	 * @see org.cresques.io.IDataWriter#readData(int, int, int)
	 */
	public int[] readData(int sX, int sY, int nBand){
		if(nBand == 0){ //Con nBand==0 se devuelven las 3 bandas
			image = new BufferedImage(sX, sY, BufferedImage.TYPE_INT_RGB);
			g = (Graphics2D) image.getGraphics();
			BufferedImage img = new BufferedImage(	viewPortBlock.getImageWidth(), 
													viewPortBlock.getImageHeight(),
													BufferedImage.TYPE_INT_ARGB
													);
			g.setColor(backgroundColor);
			g.fillRect(0, 0, viewPortBlock.getImageWidth(), viewPortBlock.getImageHeight());
			try{
				//Si es la primera lectura salvamos los valores de mximo y mnimo para la aplicacin
				//de realce si la imagen es de 16 bits.
				if(firstRead){
					for(int i=0;i<flayers.getLayersCount();i++){
						if(flayers.getLayer(i) instanceof FLyrRaster){
							FLyrRaster raster = (FLyrRaster)flayers.getLayer(i);
							if(	raster.getSource().getDataType() == RasterBuf.TYPE_SHORT || 
								raster.getSource().getDataType() == RasterBuf.TYPE_USHORT){
								RasterStats stats = raster.getSource().getFilterStack().getStats();
								stats.history.add(stats.new History(raster.getName(), stats.minBandValue, stats.maxBandValue, stats.secondMinBandValue, stats.secondMaxBandValue));	
							}	
						}
					}
					firstRead = false;
				}
				
				flayers.draw(img, g, viewPortBlock,new Cancellable() {
					public boolean isCanceled() {
						return false;
					}
				},flayers.getFMap().getScaleView());
									
				//Si es el ltimo bloque vaciamos el historial de mximos y mnimos
				if((contBlocks+1)==nBlocks){
					for(int i=0;i<flayers.getLayersCount();i++){
						if(flayers.getLayer(i) instanceof FLyrRaster){
							FLyrRaster raster = (FLyrRaster)flayers.getLayer(i);
							if(	raster.getSource().getDataType() == RasterBuf.TYPE_SHORT || 
								raster.getSource().getDataType() == RasterBuf.TYPE_USHORT){
								raster.getSource().getFilterStack().getStats().history.clear();
								RasterStats stats = raster.getSource().getFilterStack().getStats();
							}
						}
					}
				}
					
				
			}catch (DriverException e) {
				e.printStackTrace();
			}
			rasterData = image.getRGB(0, 0, sX, sY, rasterData, 0, sX);				
			
			//Calculamos el viewPort del sgte bloque
			
			if(((contBlocks+1)*sizeBlock)<=viewPort.getImageHeight()){
				dimension = new Dimension(sX, sY);
			}
			else{ //Calculo de la altura del ltimo bloque
				dimension = new Dimension(sX, (int)(viewPort.getImageHeight()-(contBlocks*sizeBlock)));
				wcIntervalo = (lastBlock*wcAlto)/viewPort.getImageHeight();
			}
			
			calcViewPort(viewPortBlock);			
			
			if(progressBar != null){
				this.incrTemp = (incrTemp - ((int)incrTemp)) + progressBar.getValue() + incrProgressBar;
				if(((int)incrTemp) !=  progressBar.getValue())
					progressBar.setValue((int)incrTemp);	
				//System.out.println("===>progressBar.getValue()="+incrProgressBar+" "+incrTemp+" "+Math.floor(incrTemp)+" "+progressBar.getValue());
			}
			
			contBlocks++;
			return rasterData;
		}
	
		return null;
	}
	
	/**
	 * Asigna la barra de progreso
	 * @param bar Barra de progreso
	 */
	public void setProgressBar(JProgressBar bar, double percentMax){
		this.progressBar = bar;
		if(percentMax != 0){
			this.percentMax = percentMax;
			incrProgressBar = percentMax/((double)nBlocks);
		}
		//progressBar.setValue((int)Math.round(incrProgressBar));
	}
}