package com.iver.cit.gvsig;

import java.io.IOException;
import java.sql.Types;
import java.util.BitSet;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.instruction.EvaluationException;
import com.hardcode.gdbms.engine.instruction.SemanticException;
import com.hardcode.gdbms.engine.values.NumericValue;
import com.hardcode.gdbms.parser.ParseException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.View;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.gui.Table;
import com.iver.cit.gvsig.gui.tables.Statistics;

/**
 * @author Fernando Gonzlez Corts
 */
public class TableFieldOperations implements Extension{

    /**
     * @see com.iver.andami.plugins.Extension#inicializar()
     */
    public void inicializar() {
    }

    /**
     * @see com.iver.andami.plugins.Extension#execute(java.lang.String)
     */
    public void execute(String actionCommand) {
		View v = PluginServices.getMDIManager().getActiveView();

		if (v != null) {
			if (v.getClass() == Table.class) {
			    
			    Table t = (Table) v;
			    
			    int fieldIndex = t.getSelectedFieldIndices().nextSetBit(0);
			    
		        DataSource sds = t.getModel().getModelo();
			    String dsName = sds.getName();
			    try {
                    String sql = "select * from '" + dsName + "' order by " + sds.getFieldName(fieldIndex);
                    if ("ORDERASC".equals(actionCommand)){
                        sql += " asc;";
				    }else{
	                    sql += " desc;";
				    }
				    DataSource ds = sds.getDataSourceFactory().executeSQL(sql, DataSourceFactory.AUTOMATIC_OPENING);
				    t.setOrder(ds.getWhereFilter());
				} catch (DriverException e) {
                    NotificationManager.addError("No se pudo ordenar", e);
                } catch (ParseException e) {
                    throw new RuntimeException(e);
                } catch (DriverLoadException e) {
                    NotificationManager.addError("Error con la carga de drivers", e);
                } catch (SemanticException e) {
                    throw new RuntimeException(e);
                } catch (IOException e) {
                    throw new RuntimeException(e);
                } catch (EvaluationException e) {
                    throw new RuntimeException(e);
                }
            }
		}
    }

    /**
     * @see com.iver.andami.plugins.Extension#isEnabled()
     */
    public boolean isEnabled() {
		View v = PluginServices.getMDIManager().getActiveView();

		if (v == null) {
			return false;
		}

		if (v.getClass() == Table.class) {
		    Table t = (Table) v;
		    BitSet indices = t.getSelectedFieldIndices();
		    if (indices.cardinality() == 1){
		        return true;
		    }
		}
		return false;
    }

    /**
     * @see com.iver.andami.plugins.Extension#isVisible()
     */
    public boolean isVisible() {
		View v = PluginServices.getMDIManager().getActiveView();

		if (v == null) {
			return false;
		}

		if (v.getClass() == Table.class) {
		    return true;
		}
		return false;
    }

}
