package com.iver.cit.gvsig;

import java.math.BigDecimal;
import java.sql.Types;
import java.util.BitSet;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.NumericValue;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.View;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.gui.Table;
import com.iver.cit.gvsig.gui.tables.Statistics;

/**
 * @author Fernando Gonzlez Corts
 */
public class TableNumericFieldOperations implements Extension{

    /**
     * @see com.iver.andami.plugins.Extension#inicializar()
     */
    public void inicializar() {
    }

    /**
     * @see com.iver.andami.plugins.Extension#execute(java.lang.String)
     */
    public void execute(String actionCommand) {
		View v = PluginServices.getMDIManager().getActiveView();

		if (v != null) {
			if (v.getClass() == Table.class) {
			    
			    Table t = (Table) v;
			    
			    int fieldIndex = t.getSelectedFieldIndices().nextSetBit(0);
			    
			    SelectableDataSource ds = t.getModel().getModelo();
			    BitSet selectedRows = ds.getSelection();
                try {
                    int rc = (int) ds.getRowCount();
                    //Si no hay seleccin se usan todos los registros
    			    if (selectedRows.cardinality() == 0){
    			        selectedRows.set(0, rc);
    			    }else{
    			        rc = selectedRows.cardinality();
    			    }
                    BigDecimal suma = new BigDecimal(0);
                    BigDecimal min = new BigDecimal(Double.MAX_VALUE);
                    BigDecimal max = new BigDecimal(Double.MIN_VALUE);
                    for(int i=selectedRows.nextSetBit(0); i>=0; i=selectedRows.nextSetBit(i+1)) {
	                    double d = ((NumericValue) ds.getFieldValue(i, fieldIndex)).doubleValue();
	                    suma = suma.add(new BigDecimal(d));
	                    if (d < min.doubleValue()) min = new BigDecimal(d);
	                    if (d > max.doubleValue()) max = new BigDecimal(d);
	                }
	                BigDecimal media = suma.divide(new BigDecimal(rc), BigDecimal.ROUND_HALF_DOWN);
	                BigDecimal varianza = new BigDecimal(0);
                    for(int i=selectedRows.nextSetBit(0); i>=0; i=selectedRows.nextSetBit(i+1)) {
	                    double d = ((NumericValue) ds.getFieldValue(i, fieldIndex)).doubleValue();
	                    BigDecimal dif = new BigDecimal(d).subtract(media);
	                    varianza = dif.multiply(dif).add(varianza);
	                }
	                varianza = varianza.divide(new BigDecimal(rc), BigDecimal.ROUND_HALF_DOWN);
	                double desviacion = Math.sqrt(varianza.doubleValue());

	                Statistics st = new Statistics();
				    st.setStatistics(media.doubleValue(), max.doubleValue(), min.doubleValue(), varianza.doubleValue(), desviacion, rc, new BigDecimal(max.doubleValue()).subtract(min).doubleValue(), suma.doubleValue());
				    PluginServices.getMDIManager().addView(st);
				    
                } catch (DriverException e) {
                    NotificationManager.addError("No se pudo acceder a los datos", e);
                }
			}
		}
    }

    /**
     * @see com.iver.andami.plugins.Extension#isEnabled()
     */
    public boolean isEnabled() {
		View v = PluginServices.getMDIManager().getActiveView();

		if (v == null) {
			return false;
		}

		if (v.getClass() == Table.class) {
		    Table t = (Table) v;
		    BitSet indices = t.getSelectedFieldIndices();
            System.out.println("TableNumericFieldOperations.isEnabled: Tabla: " + t.getModel().getModelo().getName() );
		    if (indices.cardinality() == 1){
		        try {
                    int type = t.getModel().getModelo().getFieldType(indices.nextSetBit(0));
                    if ((type == Types.BIGINT) ||
                            (type == Types.DECIMAL) ||
                            (type == Types.DOUBLE) ||
                            (type == Types.FLOAT) ||
                            (type == Types.INTEGER) ||
                            (type == Types.SMALLINT) ||
                            (type == Types.TINYINT) ||
                            (type == Types.REAL) ||
                            (type == Types.NUMERIC)){
                        return true;
                    }
                } catch (DriverException e) {
                    return false;
                }
		    }
		}
		return false;
    }

    /**
     * @see com.iver.andami.plugins.Extension#isVisible()
     */
    public boolean isVisible() {
		View v = PluginServices.getMDIManager().getActiveView();

		if (v == null) {
			return false;
		}

		if (v.getClass() == Table.class) {
		    return true;
		}
		return false;
    }

}
