/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.BitSet;

import javax.swing.DefaultListSelectionModel;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;

import org.apache.log4j.Logger;

import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.values.LongValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueCollection;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiFrame.MainFrame;
import com.iver.andami.ui.mdiManager.SingletonView;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.cit.gvsig.fmap.FMap;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.SelectionEvent;
import com.iver.cit.gvsig.fmap.layers.SelectionListener;
import com.iver.cit.gvsig.gui.layout.Popupmenu;
import com.iver.cit.gvsig.gui.tables.PopupMenu;
import com.iver.cit.gvsig.project.ProjectTable;
import com.iver.utiles.swing.jtable.FieldSelectionEvent;
import com.iver.utiles.swing.jtable.FieldSelectionListener;
import com.iver.utiles.swing.jtable.JTable;
import com.iver.utiles.swing.jtable.SelectionHeaderSupport;
import com.iver.utiles.swing.jtable.TextFieldCellEditor;


/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class Table extends JPanel implements SingletonView {
    private static Logger logger = Logger.getLogger(Table.class.getName());
    private javax.swing.JScrollPane jScrollPane = null;
    private JTable table = null;
    private ProjectTable model = null;
    private DateFormat df = DateFormat.getDateInstance();
    private NumberFormat nf = NumberFormat.getInstance();
    private JLabel jLabelStatus = null;
    private ViewInfo m_viewInfo = null;
    private FMap fmap;
    
    private boolean updating = false;
    
    private TableSelectionListener selectionListener = new TableSelectionListener();
    private long numReg=0;
    private SelectionHeaderSupport headerSelectionSupport = new SelectionHeaderSupport();
    
    private long[] orderIndexes = null;
    private long[] orderIndexesInverted = null;
    
    private DataWare dw;
    private ArrayList rowsCopied=new ArrayList();
    
    /**
     * This is the default constructor
     */
    public Table() {
        super();
        initialize();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public ProjectTable getModel() {
        return model;
    }
    
    public BitSet getSelectedFieldIndices(){
        return headerSelectionSupport.getSelectedColumns();
    }
    public int[] getSelectedRowIndices(){
    	return getTable().getSelectedRows();
    }
    private void refreshControls(){
    	try{
            MainFrame mF = PluginServices.getMainFrame();
			if (mF != null) 
                PluginServices.getMDIManager().getViewInfo(Table.this).setTitle(PluginServices.getText(this, "Tabla")+": " + model.getName());
	        	        
	        if (model.getAssociatedTable() != null){
	        	this.fmap = ((FLayer)model.getAssociatedTable()).getFMap();
	        }else{
	        	this.fmap = null;
	        }
	        
	        SelectableDataSource dataSource = model.getModelo();
	        logger.debug("dataSource.start()");
	        dataSource.start();
	
	        AbstractTableModel dataModel = new DataSourceDataModel(model);
	
			getTable().setModel(dataModel);

			headerSelectionSupport.setTableHeader(getTable().getTableHeader());
			headerSelectionSupport.addFieldSelectionListener(new FieldSelectionListener() {
                public void fieldSelected(FieldSelectionEvent e) {
                    if (PluginServices.getMainFrame() != null)
                        PluginServices.getMainFrame().enableControls();
                }
            });
	
	        model.getModelo().addSelectionListener(selectionListener);
	        
	        updateSelection();
    	}catch (DriverException e){
    		NotificationManager.addError("No se pudo leer la informacin", e);
    	}
    	
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param table DOCUMENT ME!
     */
    public void setModel(ProjectTable table) {
        SelectableDataSource dataSource;
        model = table;

        //Gestin del nombre de la ventana
        model.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (evt.getPropertyName().equals("name")){
					PluginServices.getMDIManager().getViewInfo(Table.this).setTitle(PluginServices.getText(this, "Tabla")+": " + (String)evt.getNewValue());
				}else if (evt.getPropertyName().equals("model")){
					refreshControls();
				}
			}
		});
        
        refreshControls();
    }

    /**
     *
     */
    private void updateSelection() {
    	updating = true;
    	
        DefaultListSelectionModel sm = (DefaultListSelectionModel) getTable()
        	.getSelectionModel();
        sm.clearSelection();
        BitSet bs = model.getModelo().getSelection();
        sm.setValueIsAdjusting(true);

        if (orderIndexes != null){
            for (int i = 0; i < orderIndexes.length; i++) {
                if (bs.get(i)){
                    sm.addSelectionInterval((int) orderIndexesInverted[i], (int) orderIndexesInverted[i]);
                }
            }
        }else{
            for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1)) {
                sm.addSelectionInterval(i, i);
            }
        }

        sm.setValueIsAdjusting(false);
        
		try {
			numReg=model.getModelo().getRowCount();
		} catch (DriverException e) {
			NotificationManager.addError("", e);
		}
        jLabelStatus.setText(" " + model.getModelo().getSelection().cardinality() + " / " + numReg +" "+
            PluginServices.getText(this, "registros_seleccionados_total") + ".");
        
        updating = false;
    }

    /**
     * This method initializes this
     */
    private void initialize() {
        jLabelStatus = new JLabel();
        setLayout(new java.awt.BorderLayout());
        jLabelStatus.setText("");
        jLabelStatus.setName("");
        jLabelStatus.setPreferredSize(new java.awt.Dimension(100, 18));
        add(getJScrollPane(), java.awt.BorderLayout.CENTER);
        this.add(jLabelStatus, java.awt.BorderLayout.SOUTH);
    }

    /**
     * This method initializes table
     *
     * @return javax.swing.JTable
     */
    private javax.swing.JTable getTable() {
        if (table == null) {
            table = new JTable();
            table.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
            table.setSelectionModel(new DefaultListSelectionModel());
            table.getTableHeader().addMouseListener(new MouseHandler());
            table.addMouseListener(new MouseRow());
            table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                    public void valueChanged(ListSelectionEvent e) {
                    	if (updating) return;
                    	
                    	SelectableDataSource dataModel = Table.this.model.getModelo();
                    	
                    	DefaultListSelectionModel model = (DefaultListSelectionModel) table.getSelectionModel();
                        BitSet selection = dataModel.getSelection();

                        for (int i = e.getFirstIndex();
                                i <= e.getLastIndex(); i++) {
                            if (orderIndexes != null){
                                selection.set((int) orderIndexes[i], model.isSelectedIndex(i));
                            }else{
                                selection.set(i, model.isSelectedIndex(i));
                            }
                        }

                        if (e.getValueIsAdjusting() == false) {
                        	if (fmap != null){
                        		fmap.endAtomicEvent();
                        	}
                            dataModel.fireSelectionEvents();
                        }else{
                        	if (fmap != null)
                        	fmap.beginAtomicEvent();
                        }
						jLabelStatus.setText(" " + selection.cardinality() + " / " + numReg + " " +
					            PluginServices.getText(this, "registros_seleccionados_total") + ".");
                    }
                });
            TextFieldCellEditor ce = new TextFieldCellEditor();
            for (int i = 0; i < table.getColumnModel().getColumnCount(); i++) {
                table.getColumnModel().getColumn(i).setCellEditor(ce);
            }
        }

        return table;
    }

    /**
     * This method initializes jScrollPane
     *
     * @return javax.swing.JScrollPane
     */
    private javax.swing.JScrollPane getJScrollPane() {
        if (jScrollPane == null) {
            jScrollPane = new javax.swing.JScrollPane();
            jScrollPane.setViewportView(getTable());
        }

        return jScrollPane;
    }

    /**
     * @see com.iver.mdiApp.ui.MDIManager.SingletonView#getViewModel()
     */
    public Object getViewModel() {
        return model;
    }

    /**
     * @see com.iver.mdiApp.ui.MDIManager.View#getViewInfo()
     */
    public ViewInfo getViewInfo() {
    	ViewInfo m_viewInfo = new ViewInfo(ViewInfo.ICONIFIABLE | ViewInfo.MAXIMIZABLE |
                ViewInfo.RESIZABLE);
            m_viewInfo.setTitle(PluginServices.getText(this, "Tabla"));
            m_viewInfo.setWidth(300);
            m_viewInfo.setHeight(200);

        return m_viewInfo;
    }

    /**
     * DOCUMENT ME!
     *
     * @author Fernando Gonzlez Corts
     */
    public class DataSourceDataModel extends AbstractTableModel {
        private SelectableDataSource dataSource;
        private ProjectTable pt;
        private String[] aliases;
        private int[] mapping;

        /**
         * Crea un nuevo DataSourceDataModel.
         *
         * @param ds DOCUMENT ME!
         */
        public DataSourceDataModel(ProjectTable pt) {
            this.pt = pt;
            dataSource = pt.getModelo();
            aliases = pt.getAliases();
            mapping = pt.getMapping();
        }

        /**
         * Returns the name of the field.
         *
         * @param col index of field
         *
         * @return Name of field
         */
        public String getColumnName(int col) {
                return aliases[mapping[col]];
        }

        /**
         * Returns the number of fields.
         *
         * @return number of fields
         */
        public int getColumnCount() {
            return mapping.length;
        }

        /**
         * Returns number of rows.
         *
         * @return number of rows.
         */
        public int getRowCount() {
            try {
                return (int) dataSource.getRowCount();
            } catch (DriverException e) {
                return 0;
            }
        }

        /**
         * DOCUMENT ME!
         *
         * @param row DOCUMENT ME!
         * @param col DOCUMENT ME!
         *
         * @return DOCUMENT ME!
         */
        public Object getValueAt(int row, int col) {
            try {
                if (dw != null){
                    return dw.getFieldValue(row, mapping[col]);
                }else{
                    if (orderIndexes != null){
                        return dataSource.getFieldValue(orderIndexes[row], mapping[col]);
                    }else{
                        return dataSource.getFieldValue(row, mapping[col]);
                    }
                }
            } catch (DriverException e) {
                return ValueFactory.createValue("").toString();
            }
        }
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return (dw != null);
        }
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            Value v;
            try {
                v = ValueFactory.createValueByType(aValue.toString(), dataSource.getFieldType(columnIndex));
                dw.setFieldValue(rowIndex, mapping[columnIndex], v);
            } catch (DriverException e1) {
                throw new RuntimeException(e1);
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
        }
    }

    public class TableSelectionListener implements SelectionListener{

		/**
		 * @see com.iver.cit.gvsig.fmap.layers.LegendListener#selectionChanged(com.iver.cit.gvsig.fmap.layers.LayerEvent)
		 */
		public void selectionChanged(SelectionEvent e) {
			updateSelection();
			Table.this.repaint();
		}
    	
    }

    /**
     * @param ds
     * @throws IOException
     */
    public void setOrder(long[] indexes) throws IOException {
        orderIndexes = indexes;
        orderIndexesInverted = new long[orderIndexes.length];
        for (int i = 0; i < indexes.length; i++) {
            orderIndexesInverted[(int) indexes[i]] = i;
        }
        
        updating=true;
        ((DataSourceDataModel)getTable().getModel()).fireTableDataChanged();
        updating=false;
        updateSelection();
        
    }
   
    public void startEditing() throws DriverException{
        dw = model.getModelo().getDataWare(DataSourceFactory.DATA_WARE_COHERENT_ROW_ORDER);
        dw.beginTrans();
    }

    /**
     * @throws DriverException
     * 
     */
    public void cancelEditing() throws DriverException {
        dw.rollBackTrans();
        dw = null;
    }

    /**
     * @throws DriverException
     * 
     */
    public void stopEditing() throws DriverException {
        dw.commitTrans();
        dw = null;
        model.getModelo().stop();
        model.getModelo().start();
    }

    /**
     * @return
     */
    public boolean isEditing() {
        return dw != null;
    }
    
    /**
     * Quita los campos seleccionados
     */
    public void clearSelectedFields()
    {
        headerSelectionSupport.clearSelectedColumns();
    }
    private class MouseHandler extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
           /* JTableHeader h = (JTableHeader) e.getSource();
            TableColumnModel columnModel = h.getColumnModel();
            int viewColumn = columnModel.getColumnIndexAtX(e.getX());
            int column = columnModel.getColumn(viewColumn).getModelIndex();
            if (column != -1) {
            
            }*/
        }
    }
    private class MouseRow extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
        	PluginServices.getMainFrame().enableControls();
        	if (e.getButton()==MouseEvent.BUTTON3){
        		new PopupMenu(e.getPoint());
        	}
        }
        
    }
    public Value[] getValueRow(int index){
    	DataSourceDataModel dsdm=(DataSourceDataModel)getTable().getModel();
    	Value [] values=new Value[dsdm.getColumnCount()];
    	for (int i=0;i<dsdm.getColumnCount();i++){
    		values[i]=(Value)dsdm.getValueAt(index,i);
    	}
    	return values;
    }
    private void refresh() throws DriverException{
    	dw.commitTrans();
	    model.getModelo().stop();
	    dw.beginTrans();
		DataSourceDataModel dsdm=(DataSourceDataModel)getTable().getModel();
		dsdm.fireTableDataChanged();
    }
    public void addEmptyRow() throws DriverException{
    	ValueCollection valuePK=new ValueCollection();
		valuePK.setValues(new Value[]{ValueFactory.createValue(dw.getRowCount())});
		dw.insertEmptyRow(valuePK);
		refresh();
	}

	public void copySelectedRows() throws DriverException{
		int[] sel=getSelectedRowIndices();
		for(int i=0;i<sel.length;i++){
			rowsCopied.add(getValueRow(sel[i]));
		}
	}

	public void addSelectionToEnd() throws DriverException {
		for (int i=0;i<rowsCopied.size();i++){
			dw.insertFilledRow((Value[])rowsCopied.get(i));
		}
		refresh();
	}
	public void delSelectionRow() throws DriverException{
		int[] sel=getSelectedRowIndices();
		for(int i=sel.length-1;i>=0;i--){
			dw.deleteRow(sel[i]);
		}
		refresh();
		
	}
	public boolean isCopy(){
		return !rowsCopied.isEmpty();
	}
	/*
	public void addSelectionToRow() throws DriverException {
		int[] sel=getSelectedRowIndices();
			dw.insertFilledRow((Value[])rowsCopied.get(i),sel[0]);
		refresh();
	}
	*/
	
}
