/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;


import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import org.cresques.cts.IProjection;

import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.SingletonView;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.andami.ui.mdiManager.ViewListener;
import com.iver.cit.gvsig.fmap.ColorEvent;
import com.iver.cit.gvsig.fmap.ErrorEvent;
import com.iver.cit.gvsig.fmap.ErrorListener;
import com.iver.cit.gvsig.fmap.ExtentEvent;
import com.iver.cit.gvsig.fmap.FMap;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPortListener;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.tools.ZoomOutRightButtonListener;
import com.iver.cit.gvsig.fmap.tools.Behavior.Behavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.MouseMovementBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.MoveBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.PointBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.PolygonBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.PolylineBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.RectangleBehavior;
import com.iver.cit.gvsig.gui.thememanager.FThemeManagerWindow;
import com.iver.cit.gvsig.gui.toc.TOC;
import com.iver.cit.gvsig.gui.toolListeners.AreaListener;
import com.iver.cit.gvsig.gui.toolListeners.InfoListener;
import com.iver.cit.gvsig.gui.toolListeners.LinkListener;
import com.iver.cit.gvsig.gui.toolListeners.MeasureListener;
import com.iver.cit.gvsig.gui.toolListeners.PanListener;
import com.iver.cit.gvsig.gui.toolListeners.PointSelectListener;
import com.iver.cit.gvsig.gui.toolListeners.RectangleSelectListener;
import com.iver.cit.gvsig.gui.toolListeners.SaveRasterListener;
import com.iver.cit.gvsig.gui.toolListeners.SelectImageListener;
import com.iver.cit.gvsig.gui.toolListeners.StatusBarListener;
import com.iver.cit.gvsig.gui.toolListeners.ZoomInListener;
import com.iver.cit.gvsig.gui.toolListeners.ZoomOutListener;
import com.iver.cit.gvsig.gui.toolListeners.ZoomPixelCursorListener;
import com.iver.cit.gvsig.project.ProjectView;
import com.iver.utiles.exceptionHandling.ExceptionListener;


/**
 * DOCUMENT ME!
 *
 * @author vcn
 */
public class View extends JPanel implements com.iver.andami.ui.mdiManager.View,
    SingletonView,ViewListener, IView{

    /** DOCUMENT ME! */
    private MapControl m_MapControl;
    private MapOverview m_MapLoc;

    /** DOCUMENT ME! */
    private FThemePropertiesDialog m_Themeproperties;
    private FThemeManagerWindow fThemeManagerWindow;
    private JScrollPane jscrollTOC;
    private TOC m_TOC;
    private ProjectView modelo;
    private Table table = null;
	private ViewExceptionListener mapControlExceptionListener = new ViewExceptionListener();
    	
    private class MyError implements ErrorListener
    {

		public void errorThrown(ErrorEvent e) {
			
			System.err.println("View#ErrorListener: " + e.getMessage());
			
			JOptionPane.showMessageDialog(
				(Component)PluginServices.getMainFrame(),
				PluginServices.getText(this,"fallo_capas")+" : \n"+ e.getMessage()
			);
			
		}
    	
    }

	/**
    /**
     * Creates a new View object.
     */
    public View() {
        //this.setName(Utilities.getMessage(this, "Vista")+ " : "+modelo.getName());
        // this.setPreferredSize(new Dimension(500, 300));
        initComponents();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public ProjectView getModel() {
        return modelo;
    }

    /**
     * DOCUMENT ME!
     *
     * @param model DOCUMENT ME!
     */
    public void setModel(ProjectView model) {
        this.modelo = model;
        //Se registra como listener de cambios en FMap
        FMap fmap = modelo.getMapContext();

        fmap.addErrorListener(new MyError());

        //Se configura el mapControl
        m_MapControl.setMapContext(fmap);
        m_TOC.setMapContext(fmap);
        m_MapControl.setBackground(new Color(255, 255, 255));
        if (modelo.getMapOverViewContext()!=null){
        	m_MapLoc.setModel(modelo.getMapOverViewContext());
        }
        model.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (evt.getPropertyName().equals("name")){
					PluginServices.getMDIManager().getViewInfo(View.this).setTitle("Vista: " + (String)evt.getNewValue());
				}
			}
		});
        if (m_MapControl.getViewPort() != null){
	        m_MapControl.getViewPort().addViewPortListener(new ViewPortListener() {
				public void extentChanged(ExtentEvent e) {
					if (PluginServices.getMainFrame() != null){			
						PluginServices.getMainFrame().getStatusBar().setMessage("6", "1:"+m_MapControl.getMapContext().getScaleView());
						PluginServices.getMainFrame().getStatusBar().setMessage("7", getMapControl().getViewPort().getProjection().getAbrev());
					}
				}

				public void backColorChanged(ColorEvent e) {
				}
			}); 
	      }
    }

    /**
     * DOCUMENT ME!
     */
    private void initComponents() { //GEN-BEGIN:initComponents
        m_MapControl = new MapControl(); //Default is paintEnabled = false. Remember to activate it 
        m_MapControl.addExceptionListener(mapControlExceptionListener);
        // modelo.setMapContext(m_MapControl.getMapContext());

        // m_MapControl.setAcceptEvents(true); Lo comento porque ya se hace en el setModel
        m_TOC = new TOC();
       
        JSplitPane tempMainSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        tempMainSplit.setPreferredSize(new Dimension(500, 300));

        // Ponemos el localizador
        m_MapLoc = new MapOverview(m_MapControl);
        m_MapLoc.setPreferredSize(new Dimension(150, 200));
        JSplitPane tempSplitToc = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

        tempSplitToc.setTopComponent((TOC) m_TOC);
        tempSplitToc.setBottomComponent(m_MapLoc);
        tempSplitToc.setResizeWeight(0.7);

        // tempSplitToc.resetToPreferredSizes(); 
        tempMainSplit.setLeftComponent(tempSplitToc);

        m_TOC.setVisible(true);

        // m_TOC.setPreferredSize(new Dimension(150,50));
        tempMainSplit.setRightComponent(m_MapControl);
        this.setLayout(new BorderLayout());
        this.add(tempMainSplit, BorderLayout.CENTER);

        //Listener de eventos de movimiento que pone las coordenadas del ratn en la barra de estado
        StatusBarListener sbl = new StatusBarListener(m_MapControl);
        
        // Zoom out (pinchas y el mapa se centra y te muestra ms).
        // No es dibujando un rectngulo, es solo pinchando.

        ZoomOutListener zol = new ZoomOutListener(m_MapControl);
        m_MapControl.addMapTool("zoomOut", new Behavior[]{new PointBehavior(zol), new MouseMovementBehavior(sbl)});
        
        // pan

        PanListener pl = new PanListener(m_MapControl);
        m_MapControl.addMapTool("pan", new Behavior[]{new MoveBehavior(pl), new MouseMovementBehavior(sbl)});
        
        // Medir

        MeasureListener mli = new MeasureListener(m_MapControl);
        m_MapControl.addMapTool("medicion", new Behavior[]{new PolylineBehavior(mli), new MouseMovementBehavior(sbl)});

        // Area

        AreaListener ali = new AreaListener(m_MapControl);
        m_MapControl.addMapTool("area", new Behavior[]{new PolygonBehavior(ali), new MouseMovementBehavior(sbl)});
        
        //Seleccion por punto
        PointSelectListener psl = new PointSelectListener(m_MapControl);
        m_MapControl.addMapTool("pointSelection", new Behavior[]{new PointBehavior(psl), new MouseMovementBehavior(sbl)});
        
        //Info por punto
        InfoListener il = new InfoListener(m_MapControl);
        m_MapControl.addMapTool("info", new Behavior[]{new PointBehavior(il), new MouseMovementBehavior(sbl)});
        
//      Link por punto
        LinkListener ll = new LinkListener(m_MapControl);
        m_MapControl.addMapTool("link", new Behavior[]{new PointBehavior(ll), new MouseMovementBehavior(sbl)});
        
        //Seleccin por rectngulo
        RectangleSelectListener rsl = new RectangleSelectListener(m_MapControl);
        m_MapControl.addMapTool("rectSelection", new Behavior[]{new RectangleBehavior(rsl), new MouseMovementBehavior(sbl)});

        // Zoom por rectngulo
        ZoomOutRightButtonListener zoil = new ZoomOutRightButtonListener(m_MapControl);
        ZoomInListener zil = new ZoomInListener(m_MapControl);
        m_MapControl.addMapTool("zoomIn", new Behavior[]{new RectangleBehavior(zil),
        				new PointBehavior(zoil), new MouseMovementBehavior(sbl)});
        
        //Salvar a Raster
        SaveRasterListener srl = new SaveRasterListener(m_MapControl);
        m_MapControl.addMapTool("saveRaster", new Behavior[]{
        				new RectangleBehavior(srl), new MouseMovementBehavior(sbl)});
              
        SelectImageListener sil = new SelectImageListener(m_MapControl);
        m_MapControl.addMapTool("selectImage", new Behavior[]{
				new PointBehavior(sil), new MouseMovementBehavior(sbl)});
        
        ZoomPixelCursorListener zp = new ZoomPixelCursorListener(m_MapControl);
        m_MapControl.addMapTool("zoom_pixel_cursor", new Behavior[]{new PointBehavior(zp), new MouseMovementBehavior(sbl)});
        
        m_MapControl.setTool("zoomIn"); // Por defecto
        // m_MapControl.setPaintEnabled(true);
    }

    /**
     * DOCUMENT ME!
     */
   /*public void openPropertiesWindow() {
    }
*/
    /**
     * DOCUMENT ME!
     */
  /*  public void openQueryWindow() {
    }
*/
    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public MapOverview getMapOverview() {
        return m_MapLoc;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public MapControl getMapControl() {
        return m_MapControl;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public TOC getTOC() {
        return m_TOC;
    }

    /**
     * DOCUMENT ME!
     */
    public void repaintMap() {
        m_MapControl.drawMap(false);
    }

    /* (non-Javadoc)
     * @see com.iver.mdiApp.ui.MDIManager.SingletonView#getModel()
     */
    public Object getViewModel() {
        return modelo;
    }

    /**
     * @see com.iver.mdiApp.ui.MDIManager.View#getViewInfo()
     */
    public ViewInfo getViewInfo() {
    	ViewInfo m_viewInfo = new ViewInfo(ViewInfo.ICONIFIABLE | ViewInfo.RESIZABLE |
	                ViewInfo.MAXIMIZABLE);

			m_viewInfo.setWidth(500);
			m_viewInfo.setHeight(300);
		m_viewInfo.setTitle(PluginServices.getText(this, "Vista") + " : " +
                modelo.getName());
		
		return m_viewInfo;
    	
    }

    /**
     * @see com.iver.mdiApp.ui.MDIManager.View#viewActivated()
     */
    public void viewActivated() {
    	PluginServices.getMainFrame().getStatusBar().setMessage("1",
    			FConstant.NAMES[getMapControl().getMapContext().getViewPort().getDistanceUnits()]);
    	PluginServices.getMainFrame().getStatusBar().setMessage("6", "1:"+m_MapControl.getMapContext().getScaleView());
		PluginServices.getMainFrame().getStatusBar().setMessage("7", getMapControl().getViewPort().getProjection().getAbrev());
    }

	/**
	 * @return
	 */
	public BufferedImage getImage() {
		return m_MapControl.getImage();
	}
	
	public void setProjection(IProjection proj)
	{
		getMapControl().setProjection(proj);
	}
	public IProjection getProjection()
	{
		return getMapControl().getProjection();
	}
	
	public class ViewExceptionListener implements ExceptionListener {

		/**
		 * @see com.iver.cit.gvsig.fmap.ExceptionListener#exceptionThrown(java.lang.Throwable)
		 */
		public void exceptionThrown(Throwable t) {
			NotificationManager.addError(t.getMessage(), t);
		}
		
	}

	/**
	 * @see com.iver.andami.ui.mdiManager.ViewListener#viewClosed()
	 */
	public void viewClosed() {
		///PluginServices.getMainFrame().getStatusBar().setMessage("1","");
	}
	
}
