/*
 * Created on 27-sep-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.JOptionPane;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.Print;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameView;
import com.iver.cit.gvsig.gui.layout.fframes.IFFrame;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfWriter;


/**
 * Clase que implementa los mtodos del Layout que dibujan sobre el Graphics.
 *
 * @author Vicente Caballero Navarro
 */
public class FLayoutDraw {
	private Layout layout;
	/**
	 * Crea un nuevo FLayoutDraw.
	 *
	 * @param l Referencia al Layout.
	 */
	public FLayoutDraw(Layout l) {
		layout = l;
	}

    /**
     * Mtodo para dibujar el Layout y modificar la matriz de transformacin  a
     * partir del tamao en pixels que tenga rect y el formato de papel
     * seleccionado.
     *
     * @param g2
     * @param imgBase Si es null, ests imprimiendo. Si no, la usas para el
     *        cdigo de  optimizacin.
     *
     * @throws DriverException
     */
    public void drawLayout(Graphics2D g2, BufferedImage imgBase)
        throws DriverException {
        layout.setCancelDrawing(false);
        double scale = 0;
        scale = layout.getRect().height / layout.getAtributes().m_sizePaper.getAlto() * 1;
        AffineTransform escalado = new AffineTransform();
        AffineTransform translacion = new AffineTransform();
        translacion.setToTranslation(layout.getRect().getMinX(),
            layout.getRect().getMinY());
        escalado.setToScale(scale, scale);
        layout.getAT().setToIdentity();
        layout.getAT().concatenate(translacion);
        layout.getAT().concatenate(escalado);
        layout.getAtributes().setDistanceUnitX(layout.getRect());
        layout.getAtributes().setDistanceUnitY(layout.getRect());
        for (int i = 0; i < layout.getFFrames().size(); i++) {
        	IFFrame f = (IFFrame) layout.getFFrames().get(i);
        	f.draw(g2, layout.getAT(), layout.getVisibleRect(),
                    imgBase);
            //          Dibuja el smbolo de que contiene un tag.
            if ((f.getTag() != null) && (layout.isModeDebug())) {
                f.drawSymbolTag(g2);
            }
        }

        if (!(layout.getFFrames().size()==0)) {
            layout.setStatus(Layout.ACTUALIZADO);
        } else {
            layout.setStatus(Layout.DESACTUALIZADO);
        }
    }

	/**
	 * Dibuja sobre un Graphics2D el rectngulo que representa al folio.
	 *
	 * @param g Graphics2D
	 */
	public void drawRectangle(Graphics2D g) {
		double unidadesX = layout.getAtributes().getUnitInPixelsX();
		double unidadesY = layout.getAtributes().getUnitInPixelsY();

		if ((unidadesX == 0) && (unidadesY == 0)) {
			return;
		}

		g.setColor(Color.red);

		Rectangle2D.Double rectBig = new Rectangle2D.Double(layout.getRect().x,
				layout.getRect().y, layout.getRect().width,
				layout.getRect().height);

		g.fill3DRect((int) rectBig.x + 10, (int) rectBig.y + 10,
			(int) rectBig.width, (int) rectBig.height, true);

		Rectangle2D.Double r = new Rectangle2D.Double();

		if (layout.getAtributes().isMargin()) {
			r = new Rectangle2D.Double((layout.getRect().x +
					FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[2], layout.getAT())),
					(layout.getRect().y +
					FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[0], layout.getAT())),
					(layout.getRect().width -
					FLayoutUtilities.fromSheetDistance(layout.getAtributes().m_area[2] +
						layout.getAtributes().m_area[3], layout.getAT())),
					(layout.getRect().height -
					FLayoutUtilities.fromSheetDistance(layout.getAtributes().m_area[0] +
						layout.getAtributes().m_area[1], layout.getAT())));
		} else {
			r.setRect(layout.getRect());
		}

		g.setColor(Color.white);
		g.fill(layout.getRect());

		g.setColor(Color.blue);
		g.drawRect((int) layout.getRect().getMinX(),
			(int) layout.getRect().getMinY(),
			(int) layout.getRect().getWidth(),
			(int) layout.getRect().getHeight());

		if (layout.getAtributes().isMargin()) {
			g.setColor(Color.black);

			g.drawRect((int) r.x, (int) r.y, (int) r.width, (int) r.height);
		}
	}

	public void drawGrid(Graphics2D g)
	{
		int unidadesMin = 6;
		double unidadesX = layout.getAtributes().getUnitInPixelsX();
		double unidadesY = layout.getAtributes().getUnitInPixelsY();

		if ((unidadesX == 0) && (unidadesY == 0)) {
			return;
		}
		Rectangle2D.Double r = new Rectangle2D.Double();

		if (layout.getAtributes().isMargin()) {
			r = new Rectangle2D.Double((layout.getRect().x +
					FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[2], layout.getAT())),
					(layout.getRect().y +
					FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[0], layout.getAT())),
					(layout.getRect().width -
					FLayoutUtilities.fromSheetDistance(layout.getAtributes().m_area[2] +
						layout.getAtributes().m_area[3], layout.getAT())),
					(layout.getRect().height -
					FLayoutUtilities.fromSheetDistance(layout.getAtributes().m_area[0] +
						layout.getAtributes().m_area[1], layout.getAT())));
		} else {
			r.setRect(layout.getRect());
		}

		g.setColor(Color.BLUE);
		if (((unidadesX > unidadesMin) || (unidadesY > unidadesMin)) &&
				layout.getIsDrawGrid()) {
			double ax = r.getMinX();
			double ay;

			while (ax < (r.getMaxX())) {
				ay = (r.getMinY());

				while (ay < (r.getMaxY())) {
					g.drawLine((int) ax, (int) ay, (int) ax, (int) ay);
					ay = ay + unidadesY;
				}

				ax = ax + unidadesX;
			}
		}

	}
	/**
	 * Dibuja sobre el graphics2d las reglas.
	 *
	 * @param g graphics2d sobre el que se dibuja.
	 * @param color Color de la regla.
	 */
	public void drawRuler(Graphics2D g, Color color) {
		if (layout.getRuler()) {
			int ini = 10;
			int w = 30;
			int wi = 16;

			g.setColor(new Color(250, 255, 250, 180));
			g.fillRect(ini, w, wi, layout.getHeight() - w);
			g.fillRect(w, ini, layout.getWidth() - w, wi);

			g.setColor(new Color(100, 155, 150, 180));
			g.fillRect(w, ini, (int) layout.getRect().x - w, wi);
			g.fillRect((int) layout.getRect().getMaxX(), ini,
				(int) layout.getWidth() - (int) layout.getRect().getMaxX(), wi);

			g.fillRect(ini, w, wi, (int) layout.getRect().y - w);
			g.fillRect(ini, (int) layout.getRect().getMaxY(), wi,
				(int) layout.getHeight() - (int) layout.getRect().getMaxY());

			if (layout.getAtributes().isMargin()) {
				g.setColor(new Color(50, 55, 50, 180));
				g.fillRect((int) layout.getRect().x, ini,
					(int) FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[2], layout.getAT()), wi);
				g.fillRect((int) layout.getRect().getMaxX() -
					(int) FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[3], layout.getAT()), ini,
					(int) FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[3], layout.getAT()), wi);

				g.fillRect(ini, (int) layout.getRect().y, wi,
					(int) FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[0], layout.getAT()));
				g.fillRect(ini,
					(int) layout.getRect().getMaxY() -
					(int) FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[1], layout.getAT()), wi,
					(int) FLayoutUtilities.fromSheetDistance(
						layout.getAtributes().m_area[1], layout.getAT()));
			}

			g.setColor(color);
			g.drawLine(w, wi + ini, layout.getWidth(), wi + ini);
			g.drawLine(w, ini, layout.getWidth(), ini);
			g.drawLine(ini, w, ini, layout.getHeight());
			g.drawLine(wi + ini, w, wi + ini, layout.getHeight());

			drawLineY(g, 5, 12, 22, layout.getRect().y, 0);
			drawLineX(g, 5, 12, 22, layout.getRect().x, 0);

			if (FLayoutUtilities.fromSheetDistance(1, layout.getAT()) > 15) {
				drawLineY(g, 1, 12, 22, layout.getRect().y, 0);
				drawLineX(g, 1, 12, 22, layout.getRect().x, 0);

				if (FLayoutUtilities.fromSheetDistance(1, layout.getAT()) > 25) {
					drawLineY(g, 1, 18, 22, layout.getRect().y, 0.5);
					drawLineY(g, 0.1, 21, 22, layout.getRect().y, 0);
					drawLineX(g, 1, 18, 22, layout.getRect().x, 0.5);
					drawLineX(g, 0.1, 21, 22, layout.getRect().x, 0);
				}
			}
		}
	}

	/**
	 * Dibuja sobre el graphics2d las lneas verticales que representan a las
	 * unidades de medida.
	 *
	 * @param g Graphics2d sobre el que se dibuja.
	 * @param dist distancia en centmetros de una lnea a otra.
	 * @param init inicio de la lnea.
	 * @param end fin de la lnea.
	 * @param y rectngulo, dentro del cual se dibujan las lneas.
	 * @param desp Desplazamiento.
	 */
	private void drawLineY(Graphics2D g, double dist, int init, int end,
		double y, double desp) {
		double distY = FLayoutUtilities.fromSheetDistance(dist, layout.getAT());
		double rota = Math.toRadians(90);

		if (distY > 4) {
			double despY = FLayoutUtilities.fromSheetDistance(desp,
					layout.getAT());

			double posUnitY = y + despY;
			double posUnitYNeg = posUnitY;
			int num = 0;
			double iniY = 40;
			Point2D.Double pfin = FLayoutUtilities.fromSheetPoint(new Point2D.Double(
						layout.getWidth(), layout.getHeight()), layout.getAT());

			while (posUnitY < (pfin.y - 5)) {
				posUnitYNeg -= distY;

				if (distY > 16) {
					if (init == 12) {
						if (posUnitY > iniY) {
							g.rotate(-rota, 20, posUnitY - 12);
							g.drawString(String.valueOf(num), (int) 10,
								(int) posUnitY - 12);
							g.rotate(rota, 20, posUnitY - 12);
						}

						if (dist == 5) {
							num = num + 5;
						} else {
							num++;
						}

						if (posUnitYNeg > iniY) {
							g.rotate(-rota, 20, posUnitYNeg - 12);
							g.drawString(String.valueOf(-num), (int) 10,
								(int) posUnitYNeg - 12);
							g.rotate(rota, 20, posUnitYNeg - 12);
						}
					}
				}

				if (posUnitY > iniY) {
					g.drawLine((int) 2 + init, (int) posUnitY, (int) 2 + end,
						(int) posUnitY);
				}

				if (posUnitYNeg > iniY) {
					g.drawLine((int) 2 + init, (int) posUnitYNeg,
						(int) 2 + end, (int) posUnitYNeg);
				}

				posUnitY += distY;
			}
		}
	}

	/**
	 * Dibuja sobre el graphics2d las lneas horizontales que representan a las
	 * unidades de medida.
	 *
	 * @param g Graphics2d sobre el que se dibuja.
	 * @param dist distancia en centmetros de una lnea a otra.
	 * @param init inicio de la lnea.
	 * @param end fin de la lnea.
	 * @param x rectngulo, dentro del cual se dibujan las lneas.
	 * @param desp Desplazamiento.
	 */
	private void drawLineX(Graphics2D g, double dist, int init, int end,
		double x, double desp) {
		double distX = FLayoutUtilities.fromSheetDistance(dist, layout.getAT());

		if (distX > 4) {
			double despX = FLayoutUtilities.fromSheetDistance(desp,
					layout.getAT());
			double posUnitX = x + despX;
			double posUnitXNeg = posUnitX;
			int num = 0;
			double iniX = 40;
			Point2D.Double pfin = FLayoutUtilities.fromSheetPoint(new Point2D.Double(
						layout.getWidth(), layout.getHeight()), layout.getAT());

			while (posUnitX < (pfin.x - 5)) {
				posUnitXNeg -= distX;

				if (init == 12) {
					if (distX > 16) {
						if (posUnitX > iniX) {
							g.drawString(String.valueOf(num),
								(int) posUnitX + 3, (int) 20);
						}

						if (dist == 5) {
							num = num + 5;
						} else {
							num++;
						}

						if (posUnitXNeg > iniX) {
							g.drawString(String.valueOf(-num),
								(int) posUnitXNeg + 3, (int) 20);
						}
					}
				}

				if (posUnitX > iniX) {
					g.drawLine((int) posUnitX, (int) 2 + init, (int) posUnitX,
						(int) 2 + end);
				}

				if (posUnitXNeg > iniX) {
					g.drawLine((int) posUnitXNeg, (int) 2 + init,
						(int) posUnitXNeg, (int) 2 + end);
				}

				posUnitX += distX;
			}
		}
	}

	/**
	 * Dibuja los handlers sobre los fframes que esten seleccionados.
	 *
	 * @param g Graphics sobre el que se dibuja.
	 * @param color Color de los Handlers.
	 */
	public void drawHandlers(Graphics2D g, Color color) {
		g.setColor(color);

		for (int i = 0; i < layout.getFFrames().size(); i++) {
			IFFrame fframe = (IFFrame) layout.getFFrames().get(i);

			if (fframe.getSelected() != IFFrame.NOSELECT) {
				fframe.drawHandlers((Graphics2D) g);
			}
		}
	}
	/**
	 * A partir de un fichero que se pasa como parmetro se crea un pdf con el
	 * contenido del Layout.
	 *
	 * @param pdf
	 */
	public void toPDF(File pdf) {
		double w = 0;
		double h = 0;
		Document document = new Document();

		if (layout.getAtributes().isLandSpace()) {
			w = ((layout.getAtributes().m_sizePaper.getAlto() * Attributes.DPISCREEN) / Attributes.PULGADA);
			h = ((layout.getAtributes().m_sizePaper.getAncho() * Attributes.DPISCREEN) / Attributes.PULGADA);
		} else {
			w = ((layout.getAtributes().m_sizePaper.getAncho() * Attributes.DPISCREEN) / Attributes.PULGADA);
			h = ((layout.getAtributes().m_sizePaper.getAlto() * Attributes.DPISCREEN) / Attributes.PULGADA);
		}

		document.setPageSize(new com.lowagie.text.Rectangle((float)w, (float)h));

		try {
			PdfWriter writer = PdfWriter.getInstance(document,
					new FileOutputStream(pdf));
			document.open();

			Print print = new Print();
			print.setLayout(layout);

			PdfContentByte cb = writer.getDirectContent();
			Graphics2D g2 = cb.createGraphicsShapes((float)w, (float)h);

			try {
				if (layout.getAtributes().isLandSpace()) {
					g2.rotate(Math.toRadians(-90),
						0 + (w / (h /w)), 0 + (h / 2));
					print.print(g2, new PageFormat(), 0);
					g2.rotate(Math.toRadians(90),
						0 + (w / (h / w)), 0 + (h / 2));
				} else {
					print.print(g2, new PageFormat(), 0);
				}
			} catch (PrinterException e) {
				e.printStackTrace();
			}

			g2.dispose();
		} catch (DocumentException de) {
			System.err.println(de.getMessage());
		} catch (IOException ioe) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),ioe.getMessage());
		}

		document.close();
		layout.fullRect();
	}
}
