/*
 * Created on 22-jun-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout.fframes;

import com.iver.andami.PluginServices;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.FPoint2D;
import com.iver.cit.gvsig.fmap.core.FPolygon2D;
import com.iver.cit.gvsig.fmap.core.FPolyline2D;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FGraphicUtilities;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.gui.layout.FLayoutUtilities;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.gui.project.SaveException;
////import com.iver.cit.gvsig.gui.layout.Line;

import com.iver.utiles.XMLEntity;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;


/**
 * FFrame para contener un grfico.
 *
 * @author Vicente Caballero Navarro
 */
public class FFrameGraphics extends FFrame {
    private int m_type = FConstant.SHAPE_TYPE_POINT;
    private FSymbol m_symbol = null;
    private Color m_color = null;
    private AffineTransform mT = null;
    private int size = 0;
    ////private Line polyline;
    //private AffineTransform aT = null;

    /**
     * Crea un nuevo FFrameGraphics.
     */
    public FFrameGraphics() {
        mT = new AffineTransform();
        mT.setToIdentity();
    }
  /*  public FFrameGraphics(Line line){
    	  mT = new AffineTransform();
          mT.setToIdentity();
          polyline=line;
    }
*/
    /**
     * Rellena el color que se utlizar para dibujar el smbolo.
     *
     * @param color
     */
    public void setColor(Color color) {
        m_color = color;
    }

    /**
     * Actualiza el Fsymbol a partir del tipo de Grfico que se pase como
     * parmetro.
     *
     * @param type tipo de grfico.
     * @param at Transformada.
     */
    public void update(int type, AffineTransform at) {
        m_type = type;

        //aT = at;
        if (m_color == null) {
            m_color = Color.red;
        }

        switch (m_type) {
            case (Layout.POINT):
                m_symbol = new FSymbol(FConstant.SHAPE_TYPE_POINT, m_color);

                break;

            case (Layout.RECTANGLESIMPLE):
                m_symbol = new FSymbol(FConstant.SYMBOL_TYPE_FILL, m_color);
                m_symbol.setColor(null);

                break;

            case (Layout.LINE):
                m_symbol = new FSymbol(FConstant.SYMBOL_TYPE_LINE, m_color);

                break;

            case (Layout.POLYLINE):
                m_symbol = new FSymbol(FConstant.SYMBOL_TYPE_POINT, m_color);

                break;

            case (Layout.POLYGON):
                m_symbol = new FSymbol(FConstant.SYMBOL_TYPE_POINT, m_color);

                break;

            case (Layout.CIRCLE):
                m_symbol = new FSymbol(FConstant.SYMBOL_TYPE_FILL, m_color);
                m_symbol.setStyle(FConstant.SYMBOL_STYLE_MARKER_CIRCLE);
                m_symbol.setOutlined(true);
                m_symbol.setOutlineColor(Color.red);
                m_symbol.setColor(null);

                break;
        }
    }

    /**
     * Devuelve el FSymbol que se representa.
     *
     * @return DOCUMENT ME!
     */
    public FSymbol getFSymbol() {
        return m_symbol;
    }

    /**
     * Rellena el FSymbol que se representara al dibujar.
     *
     * @param symbol
     */
    public void setFSymbol(FSymbol symbol) {
        m_symbol = symbol;
    }

    /**
     * Mtodo que dibuja sobre el graphics que se le pasa como parmetro, segn
     * la transformada afin que se debe de aplicar y el rectngulo que se debe
     * de dibujar.
     *
     * @param g Graphics
     * @param at Transformada afin.
     * @param rv rectngulo sobre el que hacer un clip.
     * @param imgBase DOCUMENT ME!
     */
    public void draw(Graphics2D g, AffineTransform at, Rectangle2D rv,
        BufferedImage imgBase) {
        Rectangle2D.Double rect = getBoundingBox(at);
        g.rotate(Math.toRadians(getRotation()), rect.x + (rect.width / 2),
            rect.y + (rect.height / 2));

        if (intersects(rv, rect)) {
            g.setColor(Color.black);

            FShape m_shape = null;

            switch (m_type) {
                case (Layout.POINT):

                    if (size != 0) {
                        rect.width = size + 10;
                        rect.height = size + 10;
                        setBoundBox(FLayoutUtilities.toSheetRect(rect, at));
                        size = 0;
                    } else {
                        if (rect.width < rect.height) {
                            m_symbol.setSize((int) rect.width - 10);
                        } else {
                            m_symbol.setSize((int) rect.height - 10);
                        }
                    }

                    Point2D.Double p = new Point2D.Double((int) rect.getCenterX(),
                            (int) rect.getCenterY());
                    m_shape = new FPoint2D(p.x, p.y);

                    ///m_symbol = new FSymbol(FConstant.SHAPE_TYPE_POINT, Color.red);
                    FGraphicUtilities.DrawShape(g, mT, m_shape, m_symbol);

                    break;

                case (Layout.RECTANGLESIMPLE):

                    GeneralPathX rectAux = new GeneralPathX(rect);
                    rectAux.transform(mT);
                    m_shape = new FPolygon2D(rectAux); //FConstant.SHAPE_TYPE_POLYLINE, rectAux);

                    ///m_symbol = new FSymbol(FConstant.SYMBOL_TYPE_LINE, Color.red);
                    FGraphicUtilities.DrawShape(g, mT, m_shape, m_symbol);

                    break;

                case (Layout.LINE):

                    Line2D line = new Line2D.Double();
                    line.setLine(new Point2D.Double(rect.x, rect.y),
                        new Point2D.Double(rect.getMaxX(), rect.getMaxY()));

                    GeneralPathX rectA = new GeneralPathX(line);
                    rectA.transform(mT);
                    m_shape = new FPolyline2D(rectA);

                    ///m_symbol = new FSymbol(FConstant.SYMBOL_TYPE_LINE, Color.red);
                    FGraphicUtilities.DrawShape(g, mT, m_shape, m_symbol);

                    //g.drawString("POLYLINE",(float)rect.getCenterX(),(float)rect.getCenterY());
                    break;

                case (Layout.POLYLINE):
                	////GeneralPathX polyLine=new GeneralPathX(polyline.getShape());
                	////polyLine.transform(at);
                	////m_shape = new FPolyline2D(polyLine);
                	////FGraphicUtilities.DrawShape(g, at, m_shape, m_symbol);
                	g.drawString("POLYLINE", (float) rect.getCenterX(),
                        (float) rect.getCenterY());
                	
                    break;

                case (Layout.POLYGON):
                    g.drawString("POLYGON", (float) rect.getCenterX(),
                        (float) rect.getCenterY());

                    break;

                case (Layout.CIRCLE):

                    Point2D.Double pc = new Point2D.Double((int) rect.getCenterX(),
                            (int) rect.getCenterY());
                    m_shape = new FPoint2D(pc.x, pc.y);

                    double x = rect.getX();
                    double y = rect.getY();
                    double w = rect.getWidth();
                    double h = rect.getHeight();

                    if (w < h) {
                        y = rect.getCenterY() - (w / 2);
                        h = w;
                        m_symbol.setSize((int) rect.width - 10);
                    } else {
                        x = rect.getCenterX() - (h / 2);
                        w = h;
                        m_symbol.setSize((int) rect.height - 10);
                    }

                    Shape circle = new Ellipse2D.Double(x, y, w, h);
                    m_shape = new FPolygon2D(new GeneralPathX(circle));

                    ///m_symbol = new FSymbol(FConstant.SHAPE_TYPE_POINT, Color.red);
                    FGraphicUtilities.DrawShape(g, mT, m_shape, m_symbol);

                    break;
            }
        }

        g.rotate(Math.toRadians(-getRotation()), rect.x + (rect.width / 2),
            rect.y + (rect.height / 2));
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#getXMLEntity()
     */
    public XMLEntity getXMLEntity() throws SaveException{
    	XMLEntity xml = new XMLEntity();
    	try{
    	xml.putProperty("className", this.getClass().getName());
        xml.putProperty("m_name", m_name);
        xml.putProperty("x", getBoundBox().x);
        xml.putProperty("y", getBoundBox().y);
        xml.putProperty("w", getBoundBox().width);
        xml.putProperty("h", getBoundBox().height);
        xml.putProperty("m_Selected", m_Selected);
        xml.putProperty("type", Layout.GRAPHICS);
        xml.putProperty("m_type", m_type);
        xml.putProperty("tag", getTag());
        xml.putProperty("m_rotation", getRotation());
        xml.addChild(m_symbol.getXMLEntity());
    	}catch (Exception e) {
			throw new SaveException(e,this.getClass().getName());
		}
        return xml;
    }

    /**
     * Crea un Objeto de esta clase a partir de la informacin del XMLEntity.
     *
     * @param xml XMLEntity
     *
     * @return Objeto de esta clase.
     */
    public static FFrameGraphics createFFrameGraphics(XMLEntity xml) {
        FFrameGraphics fframe = new FFrameGraphics();

        if (xml.getIntProperty("m_Selected") != 0) {
            fframe.setSelected(true);
        } else {
            fframe.setSelected(false);
        }

        /// fframe.m_type = xml.getIntProperty("m_type");
        /// fframe.m_symbol = FSymbol.createFSymbol(xml.getChild(0));
        return fframe;
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#updateNum()
     */
    public void updateNum() {
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#getNum()
     */
    public int getNum() {
        return 0;
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#setXMLEntity(com.iver.utiles.XMLEntity,
     *      com.iver.cit.gvsig.project.Project)
     */
    public void setXMLEntity(XMLEntity xml) {
        m_Selected = xml.getIntProperty("m_Selected");
        m_type = xml.getIntProperty("m_type");
        setRotation(xml.getDoubleProperty("m_rotation"));
        m_symbol = FSymbol.createFromXML(xml.getChild(0));
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#setXMLEntity(com.iver.utiles.XMLEntity,
     *      com.iver.cit.gvsig.project.Project)
     */
    public void setXMLEntity03(XMLEntity xml, Layout p) {
        m_Selected = xml.getIntProperty("m_Selected");
        m_type = xml.getIntProperty("m_type");
        m_symbol = FSymbol.createFromXML03(xml.getChild(0));
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#getNameFFrame()
     */
    public String getNameFFrame() {
        return PluginServices.getText(this, "Grficos") + num;
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#print(java.awt.Graphics2D,
     *      java.awt.geom.AffineTransform)
     */
    public void print(Graphics2D g, AffineTransform at)
        throws DriverException {
        draw(g, at, null, null);
    }

    /**
     * Inserta el tamao del punto.
     *
     * @param size entero que representa el tamao del punto.
     */
    public void setSize(int size) {
        this.size = size;
    }

    /**
     * Devuelve el tipo de grfico que contiene el fframe.
     *
     * @return tipo de
     */
    public int getType() {
        return m_type;
    }
	public void initialize() {
		// TODO Auto-generated method stub
		
	}
}
