/*
 * Created on 15-jul-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout.fframes;

import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.gui.layout.FLayoutUtilities;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.gui.project.OpenException;
import com.iver.cit.gvsig.gui.project.SaveException;
import com.iver.cit.gvsig.project.Project;

import com.iver.utiles.XMLEntity;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import java.util.ArrayList;


/**
 * FFrame que contiene a su vez un ArrayList de FFrames de cualquier tipo
 * incluso de si mismo.
 *
 * @author Vicente Caballero Navarro
 */
public class FFrameGroup extends FFrame implements IFFrameUseProject {
    private ArrayList m_fframes = new ArrayList();
    private Rectangle2D.Double rg = null;
    private AffineTransform m_at;
    private Project project;

    /**
     * Crea un nuevo FFrameGroup.
     */
    public FFrameGroup() {
    }

    /**
     * Aade al Arraylist un nuevo FFrame para formar parte del grupo.
     *
     * @param fframe FFrame a aadir.
     */
    public void addFFrame(IFFrame fframe) {
        m_fframes.add(fframe);
    }

    /**
     * Devuelve una ArrayList que contiene todos los FFrames que forman parte
     * del grupo.
     *
     * @return Arraylist con los fframes.
     */
    public ArrayList getFFrames() {
        return m_fframes;
    }

    /**
     * Devuelve el rectngulo que contiene a todos los fframes seleccionados.
     *
     * @param at Matriz de transformacin
     *
     * @return Rectngulo.
     */
    public Rectangle2D.Double getRectangle(AffineTransform at) {
        boolean first = true;
        Rectangle2D.Double rec = new Rectangle2D.Double();

        for (int i = 0; i < m_fframes.size(); i++) {
            Rectangle2D.Double rs = ((IFFrame) m_fframes.get(i)).getBoundingBox(at);

            if (first) {
                rec.setRect(rs);
                first = false;
            }

            rec.add(rs);
        }

        rg = new Rectangle2D.Double();
        rg.setRect(FLayoutUtilities.toSheetRect(rec, m_at));

        return rec;
    }

    /**
     * Mtodo que dibuja sobre el graphics que se le pasa como parmetro, segn
     * la transformada afin que se debe de aplicar y el rectngulo que se debe
     * de dibujar.
     *
     * @param g Graphics
     * @param at Transformada afin.
     * @param rv rectngulo sobre el que hacer un clip.
     * @param imgBase Imagen utilizada para acelerar el dibujado.
     *
     * @throws DriverException
     */
    public void draw(Graphics2D g, AffineTransform at, Rectangle2D rv,
        BufferedImage imgBase) throws DriverException {
        Rectangle2D.Double r = getBoundingBox(at);
        g.rotate(Math.toRadians(getRotation()), r.x + (r.width / 2),
            r.y + (r.height / 2));
        m_at = at;

        for (int i = 0; i < m_fframes.size(); i++) {
            ((IFFrame) m_fframes.get(i)).draw(g, at, rv, imgBase);
        }

        g.rotate(Math.toRadians(-getRotation()), r.x + (r.width / 2),
            r.y + (r.height / 2));
    }

    /**
     * Rellena la transformada que se esta utilizando en el Layout.
     *
     * @param at Matriz de transformacin.
     */
    public void setAt(AffineTransform at) {
        m_at = at;
    }

    /**
     * Reimplementacin del mtodo papa poder modificar los BoundBox  de cada
     * uno de los FFrames que contiene dentro este FFrameGroup.
     *
     * @param r Rectngulo.
     */
    public void setBoundBox(Rectangle2D r) {
        getBoundBox().setRect(r.getX(), r.getY(), r.getWidth(), r.getHeight());

        double dx = 1;
        double dy = 1;
        double dw = 1;
        double dh = 1;

        if (rg != null) {
            Rectangle2D.Double raux1 = new Rectangle2D.Double(rg.x, rg.y,
                    rg.width, rg.height);
            dx = r.getX() - raux1.x;
            dy = r.getY() - raux1.y;
            dw = r.getWidth() / raux1.width;
            dh = r.getHeight() / raux1.height;

            for (int i = 0; i < getFFrames().size(); i++) {
                IFFrame fframe = (IFFrame) getFFrames().get(i);
                Rectangle2D.Double raux = new Rectangle2D.Double();
                raux.setRect(fframe.getBoundBox());

                AffineTransform escalado = new AffineTransform();

                escalado.setToScale(dw, dh);
                escalado.translate(dx - r.getX(), dy - r.getY());

                Point2D.Double pd = new Point2D.Double();
                escalado.transform(new Point2D.Double(raux.x, raux.y), pd);

                raux.x = pd.x + r.getX();
                raux.y = pd.y + r.getY();
                raux.width = raux.width * dw;
                raux.height = raux.height * dh;

                fframe.setBoundBox(raux);
            }
        } else {
            rg = new Rectangle2D.Double();
            rg.setRect(r);
        }

        rg.setRect(r);
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws XMLException
     *
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#getXMLEntity()
     */
    public XMLEntity getXMLEntity() throws SaveException {
        XMLEntity xml = new XMLEntity();
        xml.putProperty("className", this.getClass().getName());
        xml.putProperty("m_name", m_name);
        xml.putProperty("x", getBoundBox().x);
        xml.putProperty("y", getBoundBox().y);
        xml.putProperty("w", getBoundBox().width);
        xml.putProperty("h", getBoundBox().height);
        xml.putProperty("m_Selected", m_Selected);
        xml.putProperty("type", Layout.RECTANGLEGROUP);
        xml.putProperty("tag", getTag());
        xml.putProperty("m_rotation", getRotation());

        for (int i = 0; i < getFFrames().size(); i++) {
            xml.addChild(((IFFrame) getFFrames().get(i)).getXMLEntity());
        }

        return xml;
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#setXMLEntity(com.iver.utiles.XMLEntity)
     */
    public void setXMLEntity03(XMLEntity xml, Layout l) {
        if (xml.getIntProperty("m_Selected") != 0) {
            this.setSelected(true);
        } else {
            this.setSelected(false);
        }

        IFFrame fframechild = null;

        for (int i = 0; i < xml.getNumChild(); i++) {
            try {
                Class clase = Class.forName(xml.getChild(i).getStringProperty("className"));
                fframechild = (IFFrame) clase.newInstance();
            } catch (Exception e) {
                NotificationManager.addError("Clase de Frame sobre el Layout no reconocida",
                    e);
            }

            fframechild.setName(xml.getStringProperty("m_name"));

            fframechild.setBoundBox(new Rectangle2D.Double(
                    xml.getChild(i).getDoubleProperty("x"),
                    xml.getChild(i).getDoubleProperty("y"),
                    xml.getChild(i).getDoubleProperty("w"),
                    xml.getChild(i).getDoubleProperty("h")));
            fframechild.setTag(xml.getChild(i).getStringProperty("tag"));
            fframechild.setXMLEntity03(xml.getChild(i), l);
            this.addFFrame(fframechild);
        }
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#setXMLEntity(com.iver.utiles.XMLEntity)
     */
    public void setXMLEntity(XMLEntity xml) {
        if (xml.getIntProperty("m_Selected") != 0) {
            this.setSelected(true);
        } else {
            this.setSelected(false);
        }

        setRotation(xml.getDoubleProperty("m_rotation"));

        for (int i = 0; i < xml.getNumChild(); i++) {
            try {
				this.addFFrame(FFrame.createFFrame(xml.getChild(i), project));
			} catch (OpenException e) {
				e.showError();
			}
        }
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#getNameFFrame()
     */
    public String getNameFFrame() {
        return PluginServices.getText(this, "grupo");
    }

    /**
     * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#print(java.awt.Graphics2D,
     *      java.awt.geom.AffineTransform)
     */
    public void print(Graphics2D g, AffineTransform at)
        throws DriverException {
        Rectangle2D.Double r = getBoundingBox(at);
        g.rotate(Math.toRadians(getRotation()), r.x + (r.width / 2),
            r.y + (r.height / 2));

        for (int i = 0; i < m_fframes.size(); i++) {
            ((IFFrame) m_fframes.get(i)).print(g, at);
        }

        g.rotate(Math.toRadians(-getRotation()), r.x + (r.width / 2),
            r.y + (r.height / 2));
    }

    /**
     * Inserta una referencia al proyecto nesecario.
     *
     * @param project DOCUMENT ME!
     */
    public void setProject(Project project) {
        this.project = project;
    }

	public void initialize() {
		// TODO Auto-generated method stub
		
	}
}
