/*
 * Created on 20-feb-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout.fframes;

import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;

import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.ColorEvent;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ExtentEvent;
import com.iver.cit.gvsig.fmap.FMap;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.ViewPortListener;
import com.iver.cit.gvsig.fmap.layers.LegendListener;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.fmap.rendering.LegendChangedEvent;
import com.iver.cit.gvsig.gui.layout.FLayoutUtilities;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.gui.project.SaveException;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.ProjectView;

import com.iver.utiles.XMLEntity;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import java.util.ArrayList;


/**
 * FFrame para introducir una vista en el Layout.
 *
 * @author Vicente Caballero Navarro
 */
public class FFrameView extends FFrame implements ViewPortListener,LegendListener,IFFrameUseProject  {
	//private static Logger logger = Logger.getLogger(FFrameView.class.getName());
	public static final int AUTOMATICO = 0;
	public static final int CONSTANTE = 1;
	public static final int MANUAL = 2;
	public static final int PRESENTACION = 0;
	public static final int BORRADOR = 1;
	private int m_Mode;
	private int m_typeScale = AUTOMATICO;
	private int m_extension = 0;
	private int m_quality = PRESENTACION;
	private int m_viewing = 0;
	private boolean m_bLinked = true;
	private ProjectView view = null;
	private FMap m_fmap = null;
	private double m_Scale = 0;
	private int m_mapUnits = 1; //Metros.
	//private int num = 0;
	private BufferedImage m_image = null;
	private AffineTransform at = null;
	private Project project = null;
	private Layout layout;

	/**
	 * Creates a new FFrameView object.
	 */
	public FFrameView() {
		num++;
	}

	/**
	 * Devuelve una descripcin del FFrameView.
	 *
	 * @return Descripcin.
	 */
	public String toString() {
		if (getView() == null) {
			return "FFrameView " + num + ": " + "Vacio";
		}

		return "FFrameView " + num + ": " + getView().getName();
	}

	/**
	 * Rellena la escala de la vista que contiene el fframe.
	 *
	 * @param d escala de la vista.
	 */
	public void setScale(double d) {
		m_Scale = d;
	}

	/**
	 * Inserta el nuevo extent a la FFrameView.
	 *
	 * @param r Rectngulo a ocupar por el FFrameView.
	 */
	public void setNewExtent(Rectangle2D r) {
		getFMap().getViewPort().setExtent(r);
		m_Scale = FLayoutUtilities.getScaleView(getFMap().getViewPort(), getBoundBox().width,getBoundingBox(null).width/getBoundBox().width);
	}

	/**
	 * Devuelve el FMap de la vista o una clonacin de este si se utiliza una
	 * escala fija.
	 *
	 * @return FMap.
	 */
	public FMap getFMap() {
		return m_fmap;
	}

	/**
	 * Rellena la calidad que se quiere aplicar.
	 *
	 * @param q entero que representa la calidad a aplicar.
	 */
	public void setQuality(int q) {
		m_quality = q;
	}

	/**
	 * Devuelve un entero que representa la calidad que est seleccionada.
	 *
	 * @return tipo de calidad selccionada.
	 */
	public int getQuality() {
		return m_quality;
	}

	/**
	 * Devuelve un entero que representa la forma en que se actualiza la vista.
	 *
	 * @return forma que se actualiza la vista.
	 */
	public int getViewing() {
		return m_viewing;
	}

	/**
	 * Rellena la forma de actualizar la vista(cuando activo o siempre). De momento esta opcin esta deshabilitada.
	 *
	 * @param v entero que representa la forma de actualizar la vista.
	 */
	public void setViewing(int v) {
		m_viewing = v;
	}

	/**
	 * Inserta el ProjectView de donde obtener las propiedades de la vista a
	 * mostrar.
	 *
	 * @param v Modelo de la vista.
	 */
	public void setView(ProjectView v) {
		view = v;

		ViewPort vp = v.getMapContext().getViewPort().cloneViewPort();

		vp.setImageSize(new Dimension((int)getBoundingBox(null).width,(int)getBoundingBox(null).height));
		if (m_bLinked) {
			if (getTypeScale() == AUTOMATICO) {
				m_fmap = v.getMapContext().createNewFMap(v.getMapContext()
														  .getViewPort()
														  .cloneViewPort());
				m_fmap.getViewPort().setImageSize(new Dimension((int)getBoundingBox(null).width,(int)getBoundingBox(null).height));
				v.getMapContext().getViewPort().removeViewPortListener(this);
				v.getMapContext().getViewPort().addViewPortListener(this);
				v.getMapContext().removeLayerListener(this);
				v.getMapContext().addLayerListener(this);
			} else if (getTypeScale() == CONSTANTE) {
				vp.setExtent((Rectangle2D) v.getMapContext().getViewPort()
											.getExtent().clone());
				m_fmap = v.getMapContext().createNewFMap(vp);
				v.getMapContext().getViewPort().removeViewPortListener(this);
				v.getMapContext().removeLayerListener(this);
				v.getMapContext().addLayerListener(this);
			} else if (getTypeScale() == MANUAL) {
                m_fmap = v.getMapContext().createNewFMap(vp);
				vp.setExtent(getNewExtent(getScale()));
				v.getMapContext().getViewPort().removeViewPortListener(this);
				v.getMapContext().removeLayerListener(this);
				v.getMapContext().addLayerListener(this);
			}
		} else if (!m_bLinked) {
			try {
				if (getTypeScale() == AUTOMATICO) {
					m_fmap = v.getMapContext().cloneFMap(); //(v.getMapContext().getViewPort().cloneViewPort());
					m_fmap.setViewPort(v.getMapContext().getViewPort()
										.cloneViewPort());
					v.getMapContext().getViewPort().removeViewPortListener(this);
					v.getMapContext().getViewPort().addViewPortListener(this);
				} else if (getTypeScale() == CONSTANTE) {
					vp.setExtent((Rectangle2D) v.getMapContext().getViewPort()
												.getExtent().clone());
					m_fmap = v.getMapContext().cloneFMap();
					m_fmap.setViewPort(vp);
					v.getMapContext().getViewPort().removeViewPortListener(this);
				} else if (getTypeScale() == MANUAL) {
					m_fmap = v.getMapContext().cloneFMap();
					vp.setExtent(getNewExtent(getScale()));
					m_fmap.setViewPort(vp);
					v.getMapContext().getViewPort().removeViewPortListener(this);
				}
			} catch (XMLException e1) {
				NotificationManager.addError("Cuando se aade una vista al Layout",
					e1);
			}
		}

		/*
		   m_fmap = v.getMapContext();

		   v.getMapContext().getViewPort();
		   ViewPort vp = new ViewPort();
		   vp.setExtent(v.getMapContext().getViewPort().getExtent());
		   m_fmap = m_fmap.createNewFMap(vp);
		   view = v;
		 */
	}

	/**
	 * Devuelve el modelo de la vista.
	 *
	 * @return Modelo de la vista.
	 */
	public ProjectView getView() {
		return view;
	}

	/**
	 * Devuelve un Rectngulo que representa  el extent de la vista que se
	 * requiere a partir  de una escala.
	 *
	 * @param scale Escala a mostrar.
	 *
	 * @return Rectngulo.
	 */
	private Rectangle2D.Double getNewExtent(long scale) {
		double hview = getBoundBox().getHeight();
		double wview = getBoundBox().getWidth();
		double hextent = (scale * hview) / 100.0;
		double wextent = (scale * wview) / 100.0;
		// double dif = hextent / m_fmap.getViewPort().getExtent().getHeight();
		double newx = m_fmap.getViewPort().getExtent().getCenterX() -
			(wextent / 2.0);
		double newy = m_fmap.getViewPort().getExtent().getCenterY() -
			(hextent / 2.0);

		return new Rectangle2D.Double(newx, newy, wextent, hextent);
	}

	/**
	 * Mtodo que dibuja sobre el graphics que se le pasa como parmetro, segn
	 * la transformada afin que se debe de aplicar y el rectngulo que se debe
	 * de dibujar.
	 *
	 * @param g Graphics2D
	 * @param at Transformada afn.
	 * @param rv rectngulo sobre el que hacer un clip.
	 * @param imgBase Imagen para acelerar el dibujado.
	 *
	 * @throws com.iver.cit.gvsig.fmap.DriverException
	 */
	public void draw(Graphics2D g, AffineTransform at, Rectangle2D rv,
		BufferedImage imgBase) throws com.iver.cit.gvsig.fmap.DriverException {
		Rectangle2D.Double r = getBoundingBox(at);
		g.rotate(Math.toRadians(getRotation()), r.x + (r.width / 2),
				r.y + (r.height / 2));
		if (intersects(rv, r)) {
			if (getFMap() == null) {
				drawEmpty(g);
			} else {
				if (rv != null) {
					// Dibujamos en pantalla
					Rectangle rclip = g.getClipBounds();
					g.clipRect((int) r.getMinX(), (int) r.getMinY(),
						(int) r.getWidth(), (int) r.getHeight());

					if (getQuality() == PRESENTACION) {
						/*     this.getFMap().setTransparent(true);
						   this.getFMap().drawView((Graphics2D) g,
						       new Rectangle((int) (r.x), (int) (r.y),
						           (int) (r.width), (int) (r.height)),
						       (Rectangle) rv, false, imgBase);
						 */
						if (rv.intersects(r)) {
							this.getFMap().getViewPort().setOffset(new Point2D.Double(
									r.x, r.y));
							this.getFMap().getViewPort().setImageSize(new Dimension(
									(int) r.width, (int) r.height));

							try {
							    ViewPort viewPort = this.getFMap().getViewPort();
							    Color theBackColor = viewPort.getBackColor();
							    if (theBackColor != null)
							    {
							        g.setColor(theBackColor);
							        g.fillRect((int) r.x, (int) r.y, viewPort.getImageWidth(), viewPort.getImageHeight());
							    }

								this.getFMap().draw(imgBase, g,getScale());
							} catch (com.iver.cit.gvsig.fmap.DriverException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						}

						if (rclip != null) {
							g.setClip(rclip.x, rclip.y, rclip.width,
								rclip.height);
						}
					} else {
						drawDraft(g);
					}
				} else {
					printX(g, at);
				}
			}
		}
		g.rotate(Math.toRadians(-getRotation()),
				r.x + (r.width / 2), r.y + (r.height / 2));
		setAt((AffineTransform)getFMap().getViewPort().getAffineTransform().clone());
	}

	/**
	 * @throws DriverException
	 * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#print(java.awt.Graphics2D,
	 * 		java.awt.geom.AffineTransform)
	 */
	public void print(Graphics2D g, AffineTransform at) throws DriverException {
		draw(g, at, null, null);
	}
	private void printX(Graphics2D g, AffineTransform at){
		Rectangle2D.Double r = getBoundingBox(at);

		// Dibujamos en impresora
		Rectangle rclip = g.getClipBounds();
		g.clipRect((int) r.getMinX(), (int) r.getMinY(), (int) r.getWidth(),
			(int) r.getHeight());
		this.getFMap().getViewPort().setOffset(new Point2D.Double(r.x, r.y));
		this.getFMap().getViewPort().setImageSize(new Dimension((int) r.width,
				(int) r.height));

		try {
			this.getFMap().print(g,getScale());
		} catch (com.iver.cit.gvsig.fmap.DriverException e) {
			NotificationManager.addError(e.getMessage(), e);
		}

		g.setClip(rclip.x, rclip.y, rclip.width, rclip.height);
	}
	/**
	 * Rellena la unidad de medida en la que est la vista.
	 *
	 * @param i entero que representa la unidad de medida de la vista.
	 */
	public void setMapUnits(int i) {
		m_mapUnits = i;
	}

	/**
	 * Obtiene la unidad de medida en la que est la vista.
	 *
	 * @return Unidad de medida.
	 */
	public int getMapUnits() {
		return m_mapUnits;
	}

	/**
	 * Devuelve la escala segn el tipo de escala que se haya seleccionado al
	 * aadida la vista.
	 *
	 * @return escala.
	 */
	public long getScale() {
		/*if (m_bLinked){
		   return getScaleView1(METROS);
		       }
		 */

		if (getTypeScale() == AUTOMATICO) {
			return FLayoutUtilities.getScaleView(getFMap().getViewPort(), getBoundBox().width,getBoundingBox(null).width);
		} else if (getTypeScale() == CONSTANTE) {
			return (long) m_Scale;
		} else if (getTypeScale() == MANUAL) {
			return (long) m_Scale;
		}

		return (long) m_Scale;
	}

	/**
	 * Seleccionar si la vista esta relacionada o no con la original.
	 *
	 * @param b true si est ligada y false si no lo est.
	 */
	public void setLinked(boolean b) {
		m_bLinked = b;
	}

	/**
	 * Devuelve si est ligada o no el FFrameView con la vista.
	 *
	 * @return True si la vista est ligada.
	 */
	public boolean getLinked() {
		return m_bLinked;
	}

	/**
	 * Devuelve la opcin seleccionada:Rellenar marco de la vista o recorte a
	 * la vista.
	 *
	 * @return entero que representa la opcin elegida.
	 */
	public int getExtension() {
		return m_extension;
	}

	/**
	 * Devuelve el tipo de escala que est seleccionada AUTOMATICO,CONSTANTE o
	 * MANUAL.
	 *
	 * @return entero que representa el tipo seleccionado.
	 */
	public int getTypeScale() {
		return m_typeScale;
	}

	/**
	 * Rellenar si se quiere:Rellenar marco de la vista o recorte a la vista.
	 *
	 * @param i entero que representa la opcin elegida.
	 */
	public void setExtension(int i) {
		m_extension = i;
	}

	/**
	 * Rellenar el tipo de escala que se desea.
	 *
	 * @param i entero que representa el tipo de escala.
	 */
	public void setTypeScale(int i) {
		m_typeScale = i;
	}

	/**
	 * @throws XMLException
	 * @throws SaveException
	 * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#getXMLEntity()
	 */
	public XMLEntity getXMLEntity() throws SaveException {
		XMLEntity xml = new XMLEntity();
		try{
		xml.putProperty("className",this.getClass().getName());
		xml.putProperty("m_name", m_name);
		xml.putProperty("x", getBoundBox().x);
		xml.putProperty("y", getBoundBox().y);
		xml.putProperty("w", getBoundBox().width);
		xml.putProperty("h", getBoundBox().height);
		xml.putProperty("m_Selected", m_Selected);
		xml.putProperty("type", Layout.RECTANGLEVIEW);
		xml.putProperty("m_Mode", m_Mode);
		xml.putProperty("m_typeScale", m_typeScale);
		xml.putProperty("m_extension", m_extension);
		xml.putProperty("m_quality", m_quality);
		xml.putProperty("m_viewing", m_viewing);
		xml.putProperty("m_bLinked", m_bLinked);
		xml.putProperty("m_mapUnits", m_mapUnits);
		xml.putProperty("m_Scale", m_Scale);
		xml.putProperty("m_rotation",getRotation());

		xml.putProperty("tag", getTag());

		ProjectExtension pe = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);
		ArrayList views = pe.getProject().getViews();

		boolean hasIndex = false;

		for (int i = 0; i < views.size(); i++) {
			if (view !=null && view.getName().equals(((ProjectView) views.get(i)).getName())) {
				xml.putProperty("indice", i);
				hasIndex = true;
			}
		}

		if (!hasIndex) {
			xml.putProperty("indice", -1);
		}
		if (getFMap()!=null){
			xml.putProperty("extentX", getFMap().getViewPort().getExtent().getX());
			xml.putProperty("extentY", getFMap().getViewPort().getExtent().getY());
			xml.putProperty("extentW",
				getFMap().getViewPort().getExtent().getWidth());
			xml.putProperty("extentH",
				getFMap().getViewPort().getExtent().getHeight());

		xml.addChild(getFMap().getXMLEntity());
		}
		}catch (Exception e) {
			throw new SaveException(e,this.getClass().getName());
		}
		return xml;
	}

	/**
	 * Inserta la imagen para repintar el FFrameView.
	 *
	 * @param bi Imagen para repintar.
	 */
	public void setBufferedImage(BufferedImage bi) {
		m_image = bi;
	}

	/**
	 * Devuelve la imagen para repintar.
	 *
	 * @return Imagen para repintar.
	 */
	public BufferedImage getBufferedImage() {
		return m_image;
	}

	/**
	 * Devuelve la MAtriz de transformacin utilizada por la FFrameView.
	 *
	 * @return MAtriz de transformacin.
	 */
	public AffineTransform getAt() {
		return at;
	}

	/**
	 * Inserta la matriz de transformacin.
	 *
	 * @param transform Matriz de transformacin.
	 */
	public void setAt(AffineTransform transform) {
		at = transform;
	}

	/**
	 * Inserta el proyecto.
	 *
	 * @param p Proyecto.
	 */
	public void setProject(Project p) {
		project = p;
	}
	/**
	 * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#setXMLEntity(com.iver.utiles.XMLEntity,
	 * 		com.iver.cit.gvsig.project.Project)
	 */
	public void setXMLEntity03(XMLEntity xml, Layout l) {
		if (xml.getIntProperty("m_Selected") != 0) {
			this.setSelected(true);
		} else {
			this.setSelected(false);
		}

		this.setName(xml.getStringProperty("m_name"));
		this.setBoundBox(new Rectangle2D.Double(xml.getDoubleProperty("x"),
				xml.getDoubleProperty("y"), xml.getDoubleProperty("w"),
				xml.getDoubleProperty("h")));

		this.m_Mode = xml.getIntProperty("m_Mode");
		this.m_typeScale = xml.getIntProperty("m_typeScale");
		this.m_extension = xml.getIntProperty("m_extension");
		this.m_quality = xml.getIntProperty("m_quality");
		this.m_viewing = xml.getIntProperty("m_viewing");
		this.m_bLinked = xml.getBooleanProperty("m_bLinked");
		this.m_mapUnits = xml.getIntProperty("m_mapUnits");

		//ProjectExtension pe = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);
		this.m_Scale = xml.getDoubleProperty("m_Scale");

		int indice = xml.getIntProperty("indice");

		if (indice != -1) {
			ProjectView view = (ProjectView) project.getViews().get(indice);
			this.m_fmap = view.getMapContext();
			this.setView((ProjectView) view);

			try {
				//this.m_fmap = view.getMapContext();

				/*        this.getFMap().getViewPort().setExtent(new Rectangle2D.Double(
				   xml.getDoubleProperty("extentX"),
				   xml.getDoubleProperty("extentY"),
				   xml.getDoubleProperty("extentW"),
				   xml.getDoubleProperty("extentH")));
				 */
				if (m_bLinked) {
					this.getFMap().getViewPort().setExtent(new Rectangle2D.Double(
							xml.getDoubleProperty("extentX"),
							xml.getDoubleProperty("extentY"),
							xml.getDoubleProperty("extentW"),
							xml.getDoubleProperty("extentH")));
				} else if (!m_bLinked) {
					this.m_fmap = FMap.createFromXML03(xml.getChild(0));
				}
			} catch (XMLException e) {
				NotificationManager.addError("Pasando las propiedades del XMLEntity al objeto",
					e);
			}
		}
	}

	/**
	 * @see com.iver.cit.gvsig.gui.layout.fframes.IFFrame#getNameFFrame()
	 */
	public String getNameFFrame() {
		return PluginServices.getText(this, "Vista") + num;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.ExtentListener#extentChanged(com.iver.cit.gvsig.fmap.ExtentEvent)
	 */
	public void extentChanged(ExtentEvent e) {
		if (getTypeScale() == AUTOMATICO) {
			m_fmap.getViewPort().setExtent(e.getNewExtent());
			if (layout!=null){
				layout.setStatus(Layout.DESACTUALIZADO);
				//setBufferedImage(null);
			}
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.ViewPortListener#backColorChanged(com.iver.cit.gvsig.fmap.ColorEvent)
	 */
	public void backColorChanged(ColorEvent e) {
		if (getLinked()){
	    m_fmap.getViewPort().setBackColor(e.getNewColor());
		layout.setStatus(Layout.DESACTUALIZADO);
		//setBufferedImage(null);
		}
	}
	public void setLayout(Layout l){
		layout=l;
	}

	public void legendChanged(LegendChangedEvent e) {
		if (getLinked()){
			layout.setStatus(Layout.DESACTUALIZADO);
			//setBufferedImage(null);
		}
	}

	public void setXMLEntity(XMLEntity xml) {
		if (xml.getIntProperty("m_Selected") != 0) {
			this.setSelected(true);
		} else {
			this.setSelected(false);
		}

		this.setName(xml.getStringProperty("m_name"));
		this.setBoundBox(new Rectangle2D.Double(xml.getDoubleProperty("x"),
				xml.getDoubleProperty("y"), xml.getDoubleProperty("w"),
				xml.getDoubleProperty("h")));

		this.m_Mode = xml.getIntProperty("m_Mode");
		this.m_typeScale = xml.getIntProperty("m_typeScale");
		this.m_extension = xml.getIntProperty("m_extension");
		this.m_quality = xml.getIntProperty("m_quality");
		this.m_viewing = xml.getIntProperty("m_viewing");
		this.m_bLinked = xml.getBooleanProperty("m_bLinked");
		this.m_mapUnits = xml.getIntProperty("m_mapUnits");
		setRotation(xml.getDoubleProperty("m_rotation"));
		//ProjectExtension pe = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);
		this.m_Scale = xml.getDoubleProperty("m_Scale");

		int indice = xml.getIntProperty("indice");

		if (indice != -1) {
			try{
			ProjectView view = (ProjectView) project.getViews().get(indice);
			this.m_fmap = view.getMapContext();
			this.setView((ProjectView) view);

			try {
				//this.m_fmap = view.getMapContext();

				/*        this.getFMap().getViewPort().setExtent(new Rectangle2D.Double(
				   xml.getDoubleProperty("extentX"),
				   xml.getDoubleProperty("extentY"),
				   xml.getDoubleProperty("extentW"),
				   xml.getDoubleProperty("extentH")));
				 */
				if (xml.contains("extentX")){
				if (m_bLinked) {
					this.getFMap().getViewPort().setExtent(new Rectangle2D.Double(
							xml.getDoubleProperty("extentX"),
							xml.getDoubleProperty("extentY"),
							xml.getDoubleProperty("extentW"),
							xml.getDoubleProperty("extentH")));
				} else if (!m_bLinked) {
					this.m_fmap = FMap.createFromXML(xml.getChild(0));
				}
				}
			} catch (XMLException e) {
				NotificationManager.addError("Pasando las propiedades del XMLEntity al objeto",
					e);
			}
        }catch (IndexOutOfBoundsException e) {

		}
		}
        else if (!m_bLinked) {
            try {
                this.m_fmap = FMap.createFromXML(xml.getChild(0));
            } catch (XMLException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

	}

	public Layout getLayout() {
		return layout;
	}

	/*public boolean equals(Object arg0) {
		if (!this.getName().equals(((FFrameView)arg0).getName())){
			return false;
		} if (!this.getLayout().equals(((FFrameView)arg0).getLayout())){
			return false;
		}
		return super.equals(arg0);
}*/
}
