/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.selectionByTheme;

import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.operations.strategies.QueryByThemeVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;


/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class MySelectionByThemeListener implements SelectionByThemeListener {
    /**
     * @see com.iver.cit.gvsig.gui.selectionByTheme.SelectionByThemeListener#newSet(int, int,
     *      int)
     */
    public void newSet(FLayer[] toSelect, FLayer selectionLayer, int action) {
        long t1 = System.currentTimeMillis();
        for (int i = 0; i < toSelect.length; i++) {
        	Selectable capa = (Selectable) toSelect[i];
            QueryByThemeVisitor visitor = new QueryByThemeVisitor(toSelect[i], selectionLayer, action);
            capa.clearSelection();
            try {
				((VectorialData) selectionLayer).process(visitor, ((Selectable)selectionLayer).getSelection());
				((Selectable) toSelect[i]).setSelection(visitor.getBitset());
			} catch (DriverException e) {
				NotificationManager.addError("Error leyendo las capas", e);
			} catch (VisitException e) {
				NotificationManager.addError("Error leyendo las capas", e);
			}
        }
        long t2 = System.currentTimeMillis();
        System.out.println("Tiempo de consulta: " + (t2-t1) + " milisegundos");
//        doSelection(toSelect, selectionLayer, action, false);
    }

    /**
     * @see com.iver.cit.gvsig.gui.selectionByTheme.SelectionByThemeListener#addToSet(int,
     *      int, int)
     */
    public void addToSet(FLayer[] toSelect, FLayer selectionLayer, int action) {
        for (int i = 0; i < toSelect.length; i++) {
        	Selectable capa = (Selectable) toSelect[i];
            QueryByThemeVisitor visitor = new QueryByThemeVisitor(toSelect[i], selectionLayer, action);
            FBitSet selection = capa.getSelection();
            try {
				((VectorialData) selectionLayer).process(visitor, ((Selectable)selectionLayer).getSelection());
				selection.or(visitor.getBitset());
				((Selectable) toSelect[i]).setSelection(selection);
			} catch (DriverException e) {
				NotificationManager.addError("Error leyendo las capas", e);
			} catch (VisitException e) {
				NotificationManager.addError("Error leyendo las capas", e);
			}
        }
    }

    /**
     * @see com.iver.cit.gvsig.gui.selectionByTheme.SelectionByThemeListener#fromSet(int, int,
     *      int)
     */
    public void fromSet(FLayer[] toSelect, FLayer selectionLayer, int action) {
        for (int i = 0; i < toSelect.length; i++) {
        	Selectable capa = (Selectable) toSelect[i];
            QueryByThemeVisitor visitor = new QueryByThemeVisitor(toSelect[i], selectionLayer, action);
            FBitSet selection = capa.getSelection();
            try {
				((VectorialData) selectionLayer).process(visitor, ((Selectable)selectionLayer).getSelection());
				selection.and(visitor.getBitset());
				((Selectable) toSelect[i]).setSelection(selection);
			} catch (DriverException e) {
				NotificationManager.addError("Error leyendo las capas", e);
			} catch (VisitException e) {
				NotificationManager.addError("Error leyendo las capas", e);
			}
        }
    }
}
