/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.tables;

import com.hardcode.gdbms.engine.data.driver.DriverException;

import com.iver.andami.PluginServices;

import com.iver.cit.gvsig.AddLayer;
import com.iver.cit.gvsig.gui.Table;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;


/**
 * PopupMenu with options to operate on the table.
 *
 * @author Vicente Caballero Navarro
 */
public class PopupMenu extends JPopupMenu {
    private static final ImageIcon editcopy = new ImageIcon(AddLayer.class.getClassLoader()
                                                                          .getResource("images/editcopy.png"));
    private static final ImageIcon editcut = new ImageIcon(AddLayer.class.getClassLoader()
                                                                         .getResource("images/editcut.png"));
    private static final ImageIcon editpaste = new ImageIcon(AddLayer.class.getClassLoader()
                                                                           .getResource("images/editpaste.png"));
    private static final ImageIcon editdelete = new ImageIcon(AddLayer.class.getClassLoader()
                                                                            .getResource("images/editdelete.png"));
    private JMenuItem copy = null;
    private JMenuItem cut = null;
    private JMenuItem paste = null;
    private JMenuItem insertRow = null;
    private JMenuItem insertColumn = null;
    private JMenuItem removeRow = null;
    private JMenuItem removeColumn = null;
    private JMenuItem startEdition = null;
    private JMenuItem stopEdition = null;
    private Table table = null;
    private Point point = null;

    /**
     * Create a new PopupMenu.
     *
     * @param p Point to location.
     */
    public PopupMenu(Point p) {
        point = p;
        initialize();
    }

    /**
     * Initialize the components of Popupmenu.
     */
    private void initialize() {
        copy = new JMenuItem(PluginServices.getText(this, "copiar"), editcopy);
        cut = new JMenuItem(PluginServices.getText(this, "cortar"), editcut);
        paste = new JMenuItem(PluginServices.getText(this, "pegar"), editpaste);
        insertRow = new JMenuItem(PluginServices.getText(this, "insertar_fila"));
        insertColumn = new JMenuItem(PluginServices.getText(this,
                    "insertar_columna"));
        removeRow = new JMenuItem(PluginServices.getText(this, "eliminar_fila"),
                editdelete);
        removeColumn = new JMenuItem(PluginServices.getText(this,
                    "eliminar_columna"));
        startEdition = new JMenuItem(PluginServices.getText(this,
                    "comenzar_edicion"));
        stopEdition = new JMenuItem(PluginServices.getText(this,
                    "terminar_edicion"));

        add(startEdition);
        add(stopEdition);
        addSeparator();
        add(copy);
        add(cut);
        add(paste);
        addSeparator();
        add(insertRow);
        add(removeRow);
        add(insertColumn);
        add(removeColumn);

        table = (Table) PluginServices.getMDIManager().getActiveView();
        startEdition.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        table.startEditing();
                        PluginServices.getMainFrame().enableControls();
                    } catch (DriverException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                }
            });
        stopEdition.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        table.stopEditing();
                        PluginServices.getMainFrame().enableControls();
                    } catch (DriverException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                }
            });
        copy.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        table.copySelectedRows();
                        PluginServices.getMainFrame().enableControls();
                    } catch (DriverException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                }
            });
        cut.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        table.copySelectedRows();
                        table.delSelectionRow();
                        PluginServices.getMainFrame().enableControls();
                    } catch (DriverException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                }
            });
        paste.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    //TODO este mtodo aade las filas seleccionadas previamente al final de la tabla.
                    //De momento no se puede aadir filas en una posicin en concreto.
                    try {
                        table.addSelectionToEnd();
                        PluginServices.getMainFrame().enableControls();
                    } catch (DriverException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                }
            });
        insertRow.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    //TODO Este mtodo aade filas al final de la tabla y debera aadirlas justo antes de la fila que tengamos seleccionada.
                    try {
                        table.addEmptyRow();
                        PluginServices.getMainFrame().enableControls();
                    } catch (DriverException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                }
            });
        removeRow.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        table.delSelectionRow();
                        PluginServices.getMainFrame().enableControls();
                    } catch (DriverException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                }
            });
        insertColumn.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    //TODO Falta que implementar.
                	
                }
            });
        removeColumn.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    //TODO Falta que implementar.
                }
            });
        editingMenu(table.isEditing());
        setEnablePaste(table.isCopy());
        this.show(table, point.x, point.y);
    }

    /**
     * Set the menu enabled if its possible.
     *
     * @param b boolean
     */
    private void editingMenu(boolean b) {
        copy.setEnabled(false);
        cut.setEnabled(false);
        startEdition.setEnabled(!b);
        stopEdition.setEnabled(b);
        insertRow.setEnabled(b);
        insertColumn.setEnabled(b);
        removeRow.setEnabled(false);
        if (table.getSelectedRowIndices().length > 0){
        	copy.setEnabled(true);
        	if(b) {
        		cut.setEnabled(true);
        		removeRow.setEnabled(true);
        	}
        }
           
        if ((table.getSelectedFieldIndices().length() > 0) && b) {
            removeColumn.setEnabled(true);
        } else {
            removeColumn.setEnabled(false);
        }
    }

    /**
     * Set the menu of paste enabled if its possible
     *
     * @param b boolean
     */
    private void setEnablePaste(boolean b) {
        paste.setEnabled(b);
    }
}
