/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.toolListeners;

import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.BitSet;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import com.hardcode.gdbms.engine.data.DataSource;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.RasterOperations;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.InfoByPoint;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.operations.strategies.QueryByPointVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.PointEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.PointListener;
import com.iver.cit.gvsig.gui.Dialogs.FInfoDialogXML;
import com.iver.utiles.xmlViewer.TextXMLContent;
import com.iver.utiles.xmlViewer.XMLContent;

/**
 * DOCUMENT ME!
 * 
 * @author Vicente Caballero Navarro
 */
public class InfoListener implements PointListener {
	// private final Image img = new ImageIcon(NewMapControl.class.getResource(
	//"images/InfoCursor.png")).getImage();
	private static Logger logger = Logger.getLogger(InfoListener.class
			.getName());

	private final Image img = new ImageIcon(MapControl.class
			.getResource("images/InfoCursor.gif")).getImage();

	private Cursor cur = Toolkit.getDefaultToolkit().createCustomCursor(img,
			new Point(16, 16), "");

	private MapControl mapCtrl;

	/**
	 * Crea un nuevo AreaListenerImpl.
	 * 
	 * @param mc
	 *            DOCUMENT ME!
	 */
	public InfoListener(MapControl mc) {
		this.mapCtrl = mc;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param event
	 *            DOCUMENT ME!
	 * 
	 * @throws DriverIOException
	 * 
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.PointListener#point(com.iver.cit.gvsig.fmap.tools.Events.PointEvent)
	 */
	public void point(PointEvent event) throws BehaviorException {
		Point2D pReal = mapCtrl.getMapContext().getViewPort().toMapPoint(
				event.getPoint());
		Point imagePoint = new Point((int) event.getPoint().getX(), (int) event
				.getPoint().getY());

		// Dentro de queryByPoint tenemos que separar los registros que vienen
		// asociados a una
		// capa o a otra. Supongo que lo correcto sera que montaramos un XML y
		// usar el visor
		// de XML que ha empezado Fernando.
		// String strResul =
		// m_MapControl.queryByPoint(pScreen,m_MapControl.toMapDistance(3),"");
		//FInfoDialog dlg = new FInfoDialog();
		//     dlg.clearAll();
		//   dlg.setViewModel((ProjectView) proj);
		FInfoDialogXML dlgXML = new FInfoDialogXML();

		//MultipleXMLContent xmlModel = new MultipleXMLContent();

		FLayer[] sel = mapCtrl.getMapContext().getLayers().getActives();

		final XMLItem[] items = new XMLItem[sel.length];
		int numLayersInfoable = 0;

		for (int i = 0; i < sel.length; i++) {
			FLayer laCapa = (FLayer) sel[i];

			if ((laCapa instanceof VectorialData)
					&& (laCapa instanceof AlphanumericData)) {
				QueryByPointVisitor visitor = new QueryByPointVisitor();
				visitor.setLayer(laCapa);
				visitor.setQueriedPoint(pReal);
				visitor.setTolerance(mapCtrl.getMapContext().getViewPort()
						.toMapDistance(3));
				try {
					((VectorialData) laCapa).process(visitor);
				} catch (DriverException e) {
					throw new BehaviorException("No se pudo procesar la capa",
							e);
				} catch (VisitException e) {
					throw new RuntimeException(e);
				}

				items[i] = new VectorialXMLItem(visitor.getBitSet(), laCapa);
				numLayersInfoable++;
			}
			// TODO: PROVISIONAL PARA LA CAPA WMS
			if (laCapa instanceof RasterOperations) {
				RasterOperations layer = (RasterOperations) laCapa;
				//System.out.println(".........InfoByPoint");
				String text;
				try {

					ArrayList attr = ((RasterOperations) laCapa)
							.getAttributes();
					int anchoRaster = 0;
					int altoRaster = 0;

					for (int j = 0; j < attr.size(); j++) {
						Object[] a = (Object[]) attr.get(j);
						if (a[0].toString().equals("Width"))
							anchoRaster = Integer.parseInt(a[1].toString());
						if (a[0].toString().equals("Height"))
							altoRaster = Integer.parseInt(a[1].toString());
					}

					double xwc = ((RasterOperations) laCapa).getMaxX()
							- ((RasterOperations) laCapa).getMinX();//((FLyrDefault)laCapa).getFullExtent().getMaxX()-((FLyrRaster)laCapa).getFullExtent().getMinX();
					double ywc = ((RasterOperations) laCapa).getMaxY()
							- ((RasterOperations) laCapa).getMinY();//((FLyrDefault)laCapa).getFullExtent().getMaxY()-((FLyrRaster)laCapa).getFullExtent().getMinY();
					double ancho = ((RasterOperations) laCapa).getWidth();//((FLyrDefault)laCapa).getFullExtent().getWidth();
					double alto = ((RasterOperations) laCapa).getHeight();//((FLyrDefault)laCapa).getFullExtent().getHeight();

					//ptoX y ptoY son el pixel de la imagen donde se ha
					// pinchado a escala 1:1
					int ptoX = (int) (((pReal.getX() - ((RasterOperations) laCapa)
							.getMinX()) * anchoRaster) / xwc);//(int)(((pReal.getX()-((FLyrDefault)laCapa).getFullExtent().getMinX())*anchoRaster)/xwc);
					int ptoY = (int) (((pReal.getY() - ((RasterOperations) laCapa)
							.getMinY()) * altoRaster) / ywc);//(int)(((pReal.getY()-((FLyrDefault)laCapa).getFullExtent().getMinY())*altoRaster)/ywc);
					((RasterOperations) laCapa).setPos(ptoX, ptoY);
					ViewPort v = mapCtrl.getMapContext().getViewPort();

					int[] px = ((RasterOperations) laCapa).getPixel(pReal
							.getX(), pReal.getY());

					if (px != null)
						((RasterOperations) laCapa).setRGB(px[1], px[2], px[3]);
					((RasterOperations) laCapa).setPosWC(pReal.getX(), pReal
							.getY());

					text = layer.queryByPoint(imagePoint);

				} catch (DriverException e) {
					throw new BehaviorException("No se pudo procesar la capa",
							e);
				}
				items[i] = new StringXMLItem(text);
				numLayersInfoable++;
			} else if (laCapa instanceof InfoByPoint) {
				// TODO Hecho para el WMS. No deberia hacer falta 
				String text;
				try {
					InfoByPoint layer = (InfoByPoint) laCapa;
					text = layer.queryByPoint(imagePoint);
					System.out.println(text);
				} catch (DriverException e) {
					throw new BehaviorException("No se pudo procesar la capa",
							e);
				}
				items[i] = new StringXMLItem(text);
				numLayersInfoable++;

			}

		}

		if (numLayersInfoable > 0) {
			try {
				if (PluginServices.getMainFrame() == null) {
					JDialog dialog = new JDialog();
					dlgXML.setPreferredSize(dlgXML.getSize());
					dialog.getContentPane().add(dlgXML);
					dialog.setModal(false);
					dialog.pack();
					dialog.show();

				} else {
					dlgXML = (FInfoDialogXML) PluginServices.getMDIManager()
							.addView(dlgXML);
				}

				dlgXML.setModel(new XMLContent() {
					private ContentHandler handler;

					public void setContentHandler(ContentHandler arg0) {
						handler = arg0;
					}

					public void parse() throws SAXException {
						handler.startDocument();

						for (int i = 0; i < items.length; i++) {
							items[i].parse(handler);
						}

						handler.endDocument();
					}
				});
				dlgXML.getXmlTree().setRootVisible(false);
				DefaultTreeModel treeModel = (DefaultTreeModel) dlgXML
						.getXmlTree().getModel();
				DefaultMutableTreeNode n;
				DefaultMutableTreeNode root = (DefaultMutableTreeNode) dlgXML
						.getXmlTree().getModel().getRoot();
				n = root.getFirstLeaf();
				TreePath path = new TreePath(treeModel.getPathToRoot(n));
				dlgXML.getXmlTree().expandPath(path);

				dlgXML.getXmlTree().setSelectionPath(path);

			} catch (SAXException e) {
				NotificationManager.addError(
						"Error formateando los resultados", e);
			}
		}

	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#getCursor()
	 */
	public Cursor getCursor() {
		return cur;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#cancelDrawing()
	 */
	public boolean cancelDrawing() {
		return false;
	}

	private interface XMLItem {
		public void parse(ContentHandler handler) throws SAXException;
	}

	private class VectorialXMLItem implements XMLItem {
		private BitSet bitset;

		private FLayer layer;

		public VectorialXMLItem(BitSet bitset, FLayer layer) {
			this.bitset = bitset;
			this.layer = layer;
		}

		/**
		 * @see com.iver.cit.gvsig.gui.toolListeners.InfoListener.XMLItem#parse(org.xml.sax.ContentHandler)
		 */
		public void parse(ContentHandler handler) throws SAXException {
			AttributesImpl aii = new AttributesImpl();
			handler.startElement("", "", ((FLayer) layer).getName(), aii);
			try {
				// Project p = ((ProjectExtension)
				// PluginServices.getExtension(ProjectExtension.class)).getProject();
				// DataSource ds = p.getDataSourceByLayer(layer);
				DataSource ds = ((AlphanumericData) layer).getRecordset();
				ds.start();

				for (int j = bitset.nextSetBit(0); j >= 0; j = bitset
						.nextSetBit(j + 1)) {
					// for (int j = 0; j < bitset[i].length();
					//        j++) {
					//if (bitset[i].get(j)) {
					AttributesImpl ai = new AttributesImpl();

					for (int k = 0; k < ds.getFieldCount(); k++) {
						ai.addAttribute("", ds.getFieldName(k), "",
								"xs:string", ds.getFieldValue(j, k).toString());
					}

					handler.startElement("", "", String.valueOf(j), ai);

					handler.endElement("", "", String.valueOf(j));

				}

				logger.debug("ds.stop()");
				ds.stop();

			} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
				throw new SAXException(e);
			} catch (DriverException e) {
				throw new SAXException(e);
			}
			handler.endElement("", "", ((FLayer) layer).getName());
		}
	}

	private class StringXMLItem extends TextXMLContent implements XMLItem {

		/**
		 * @param text
		 */
		public StringXMLItem(String text) {
			super(text);

		}

		/**
		 * @see com.iver.cit.gvsig.gui.toolListeners.InfoListener.XMLItem#parse(org.xml.sax.ContentHandler)
		 */
		public void parse(ContentHandler handler) throws SAXException {
			setContentHandler(new FilterContentHandler(handler));
			parse();
		}

		private class FilterContentHandler implements ContentHandler {

			private ContentHandler handler;

			public FilterContentHandler(ContentHandler handler) {
				this.handler = handler;
			}

			/**
			 * @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator)
			 */
			public void setDocumentLocator(Locator arg0) {
			}

			/**
			 * @see org.xml.sax.ContentHandler#startDocument()
			 */
			public void startDocument() throws SAXException {
			}

			/**
			 * @see org.xml.sax.ContentHandler#endDocument()
			 */
			public void endDocument() throws SAXException {
			}

			/**
			 * @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String,
			 *      java.lang.String)
			 */
			public void startPrefixMapping(String arg0, String arg1)
					throws SAXException {
			}

			/**
			 * @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String)
			 */
			public void endPrefixMapping(String arg0) throws SAXException {
			}

			/**
			 * @see org.xml.sax.ContentHandler#startElement(java.lang.String,
			 *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
			 */
			public void startElement(String arg0, String arg1, String arg2,
					Attributes arg3) throws SAXException {
				handler.startElement(arg0, arg1, arg2, arg3);
			}

			/**
			 * @see org.xml.sax.ContentHandler#endElement(java.lang.String,
			 *      java.lang.String, java.lang.String)
			 */
			public void endElement(String arg0, String arg1, String arg2)
					throws SAXException {
				handler.endElement(arg0, arg1, arg2);
			}

			/**
			 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
			 */
			public void characters(char[] arg0, int arg1, int arg2)
					throws SAXException {
				handler.characters(arg0, arg1, arg2);
			}

			/**
			 * @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int,
			 *      int)
			 */
			public void ignorableWhitespace(char[] arg0, int arg1, int arg2)
					throws SAXException {
			}

			/**
			 * @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String,
			 *      java.lang.String)
			 */
			public void processingInstruction(String arg0, String arg1)
					throws SAXException {
			}

			/**
			 * @see org.xml.sax.ContentHandler#skippedEntity(java.lang.String)
			 */
			public void skippedEntity(String arg0) throws SAXException {
			}

		}
	}
}