/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project;

import java.io.IOException;
import java.text.DateFormat;
import java.util.Date;

import javax.swing.JOptionPane;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.NoSuchTableException;
import com.hardcode.gdbms.engine.data.driver.ObjectDriver;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.hardcode.gdbms.engine.instruction.SemanticException;
import com.hardcode.gdbms.parser.ParseException;
import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.cit.gvsig.gui.project.OpenException;
import com.iver.cit.gvsig.gui.project.SaveException;
import com.iver.utiles.XMLEntity;


/**
 * Tabla del proyecto
 *
 * @author Fernando Gonzlez Corts
 */
public class ProjectTable extends ProjectElement {
    private static int numTables = 0;
    private SelectableDataSource modelo;
    private SelectableDataSource original;
    private String linkTable;
    private String field1;
    private String field2;

    /* No es necesaria para operar, slo para guardar el proyecto */
    private AlphanumericData associatedTable;

	private int[] mapping;
	private String[] alias;
    
    /**
     * Establece a true el bit index-simo del bitmap de campos visibles. Los
     * campos cuyos bits estn a true se considerarn visibles y viceversa
     *
     * @param index indice del bit que se quiere establecer a true
     */
    public void set(int index) {
        //TODO implementar bien
        //        modelo.set(index);

        change.firePropertyChange("visibles", true, true);
    }

    /**
     * Obtiene el valor del bit index-simo del bitmap de campos visibles
     *
     * @param index indice del bit que se quiere obtener
     *
     * @return devuelve true si el campo index-simo es visible y false en caso
     *         contrario
     */
    public boolean get(int index) {
        //TODO implementar bien
        //        return modelo.get(index);
        return false;
    }

    /**
     * Obtiene el modelo de la Tabla. Es decir, una clase con capacidad para
     * leer la informacin de la tabla
     *
     * @return
     */
    public SelectableDataSource getModelo() {
        return modelo;
    }

    /**
     * Establece el valor del bit index-simo al valor  'value'
     *
     * @param bitIndex indice del bit que se quiere tocar
     * @param value valor que se quiere establecer en el bit indicado
     */
    public void set(int bitIndex, boolean value) {
        //TODO implementar bien
        //        modelo.set(bitIndex, value);
        change.firePropertyChange("visibles", value, value);
    }

    /**
     * DOCUMENT ME!
     *
     * @param name DOCUMENT ME!
     * @param sds DOCUMENT ME!
     *
     * @return
     * @throws com.hardcode.gdbms.engine.data.driver.DriverException
     */
    public static ProjectTable createTable(String name, SelectableDataSource sds) throws com.hardcode.gdbms.engine.data.driver.DriverException {
        ProjectTable t = new ProjectTable();
        t.modelo = sds;

        t.createAlias();
        
        t.setName(name);
        t.setCreationDate(DateFormat.getInstance().format(new Date()));
        numTables++;

        return t;
    }

    private void createAlias() throws com.hardcode.gdbms.engine.data.driver.DriverException{
        mapping = new int[modelo.getFieldCount()];
		alias = new String[modelo.getFieldCount()];
		for (int i = 0; i < modelo.getFieldCount(); i++) {
            mapping[i] = i;
            alias[i] = modelo.getFieldName(i);
        }
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param sds DOCUMENT ME!
     */
    public void setDataSource(SelectableDataSource sds) {
        modelo = sds;

        setName(sds.getName());
        setCreationDate(DateFormat.getInstance().format(new Date()));
        change.firePropertyChange("model", modelo, modelo);
    }

    /**
     * DOCUMENT ME!
     *
     * @param ds DOCUMENT ME!
     * @throws com.hardcode.gdbms.engine.data.driver.DriverException 
     */
    public void replaceDataSource(SelectableDataSource ds) throws com.hardcode.gdbms.engine.data.driver.DriverException {
        if (original == null){
            original = modelo;
        }
        modelo = ds;
        ds.setSelectionSupport(original.getSelectionSupport());
        createAlias();
        
        // FJP:
        // Si la tabla proviene de un layer, cambiamos su recordset
        if (associatedTable != null)
        {
            if (associatedTable instanceof FLyrVect)
            {                
                try {
                    ((FLyrVect) associatedTable).setRecordset(ds);
                } catch (FieldNotFoundException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (DriverException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
                ((FLyrVect) associatedTable).setIsJoined(true);
            }
        }
        
        change.firePropertyChange("model", original, modelo);
    }

    /**
     * DOCUMENT ME!
     * @throws com.hardcode.gdbms.engine.data.driver.DriverException 
     */
    public void restoreDataSource() throws com.hardcode.gdbms.engine.data.driver.DriverException {
        // FJP:
        // Si la tabla proviene de un layer, cambiamos su recordset
        if (associatedTable != null)
        {
            if (associatedTable instanceof FLyrVect)
            {      
                // Miramos si la leyenda que est usando es una
                // leyenda basada en un campo de los de la unin.
                // Si lo es, no dejamos pegarle el cambiazo al recordset
                FLyrVect lyr = ((FLyrVect) associatedTable);
                if (lyr.getLegend() instanceof ClassifiedLegend)
                {
                    ClassifiedLegend legend = (ClassifiedLegend) lyr.getLegend();
                    VectorialLegend aux = (VectorialLegend) legend;
                    String fieldName = legend.getFieldName();
                    int idField = original.getFieldIndexByName(fieldName);
                    int idLabelField = -2;
                    if (aux.getLabelField() != null)
                    {
                        idLabelField = original.getFieldIndexByName(aux.getLabelField());
                    }
                    if ((idField == -1) || (idLabelField == -1))
                    {
                        // No se ha encontrado ese campo, o se est etiquetando por ese campo
                        JOptionPane.showMessageDialog(null,
                                PluginServices.getText(this, "leyenda_campo_unido")
                                );

                        return;
                    }
                }
                
                try {
                    lyr.setRecordset(original);
                } catch (FieldNotFoundException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (DriverException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
                lyr.setIsJoined(false);
            }
        }

        
        modelo = original;
        original = null;
        createAlias();

        
        change.firePropertyChange("model", original, modelo);
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     * @throws SaveException 
     *
     * @throws XMLException
     */
    public XMLEntity getXMLEntity() throws SaveException {
        XMLEntity xml = super.getXMLEntity();
        try {
        //xml.putProperty("nameClass", this.getClass().getName());
        xml.putProperty("numTables", numTables);

        if (getLinkTable() != null) {
            xml.putProperty("linkTable", linkTable);
            xml.putProperty("field1", field1);
            xml.putProperty("field2", field2);
        }

        if (getOriginal() != null) {
            xml.addChild(getOriginal().getXMLEntity());
        }

        xml.addChild(modelo.getXMLEntity());

        Object di = LayerFactory.getDataSourceFactory().getDriverInfo(modelo.getName());

        if (associatedTable != null) {
            xml.putProperty("layerName", ((FLayer) associatedTable).getName());
            xml.putProperty("viewName",
                project.getView((FLayer) associatedTable));
        }
        
        xml.putProperty("mapping", mapping);
        xml.putProperty("aliases", alias);
        }catch (Exception e) {
			throw new SaveException(e,this.getClass().getName());
		}
        return xml;
    }

    /**
     * DOCUMENT ME!
     *
     * @param xml DOCUMENT ME!
     * @param p DOCUMENT ME!
     *
     * @throws XMLException
     * @throws DriverException
     *
     * @see com.iver.cit.gvsig.project.ProjectElement#setXMLEntity(com.iver.utiles.XMLEntity)
     */
    public void setXMLEntity03(XMLEntity xml, Project p)
        throws XMLException, DriverException {
        numTables = xml.getIntProperty("numTables");

        if (xml.getStringProperty("type").equals("otherDriverFile")) {
            LayerFactory.getDataSourceFactory().addFileDataSource(xml.getStringProperty(
                    "driverName"), xml.getStringProperty("gdbmsname"),
                xml.getStringProperty("file"));

            setSelectableDataSource03(xml);
        } else if (xml.getStringProperty("type").equals("sameDriverFile")) {
            String layerName = xml.getStringProperty("layerName");

            ProjectView vista = project.getViewByName(xml.getStringProperty(
                        "viewName"));
            FLayer layer = vista.getMapContext().getLayers().getLayer(layerName);

            modelo = ((AlphanumericData) layer).getRecordset();
            associatedTable = (AlphanumericData) layer;

            LayerFactory.getDataSourceFactory().addDataSource((ObjectDriver) ((SingleLayer) layer).getSource()
                                                                              .getDriver(),
                xml.getStringProperty("gdbmsname"));
        } else if (xml.getStringProperty("type").equals("db")) {
            LayerFactory.getDataSourceFactory().addDBDataSourceByTable(xml.getStringProperty(
                    "gdbmsname"), xml.getStringProperty("host"),
                xml.getIntProperty("port"), xml.getStringProperty("user"),
                xml.getStringProperty("password"),
                xml.getStringProperty("dbName"),
                xml.getStringProperty("tableName"),
                xml.getStringProperty("driverInfo"));

            setSelectableDataSource03(xml);
        }

        setName(xml.getStringProperty("name"));
    }

    /**
     * DOCUMENT ME!
     *
     * @param xml DOCUMENT ME!
     * @param p DOCUMENT ME!
     *
     * @throws XMLException
     * @throws DriverException
     * @throws OpenException 
     *
     * @see com.iver.cit.gvsig.project.ProjectElement#setXMLEntity(com.iver.utiles.XMLEntity)
     */
    public void setXMLEntity(XMLEntity xml, Project p)
        throws XMLException, DriverException, OpenException {
        try{
    	setName(xml.getStringProperty("name"));
        numTables = xml.getIntProperty("numTables");

        setSelectableDataSource(xml);

        /*
           if (xml.getStringProperty("type").equals("otherDriverFile")) {
           } else if (xml.getStringProperty("type").equals("sameDriverFile")) {
                   String layerName = xml.getStringProperty("layerName");
                   ProjectView vista = project.getViewByName(xml.getStringProperty(
                   "viewName"));
                   FLayer layer = vista.getMapContext().getLayers().getLayer(layerName);
        
                   modelo = ((AlphanumericData) layer).getRecordset();
                   associatedTable = (AlphanumericData) layer;
           } else if (xml.getStringProperty("type").equals("db")) {
                   setSelectableDataSource(xml);
           }
         */
        setName(xml.getStringProperty("name"));

        if (xml.contains("linkTable")) {
            setLinkTable(xml.getStringProperty("linkTable"),
                xml.getStringProperty("field1"), xml.getStringProperty("field2"));
        }

        if (xml.contains("mapping")) {
        	mapping = (int[]) xml.getIntArrayProperty("mapping");
        	alias = (String[]) xml.getStringArrayProperty("aliases");
        } else {
        	try {
        		createAlias();
        	} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
        		throw new XMLException(e);
        	}
        }
        }catch (Exception e) {
        	throw new OpenException(e,this.getClass().getName());
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param xml DOCUMENT ME!
     *
     * @throws XMLException DOCUMENT ME!
     * @throws DriverException DOCUMENT ME!
     */
    private void setSelectableDataSource03(XMLEntity xml)
        throws XMLException, DriverException {
        String layerName = null;

        if (xml.contains("layerName")) {
            layerName = xml.getStringProperty("layerName");
        }

        if (layerName == null) {
            DataSource dataSource;

            try {
                dataSource = LayerFactory.getDataSourceFactory()
                                         .createRandomDataSource(xml.getStringProperty(
                            "gdbmsname"),
                        DataSourceFactory.AUTOMATIC_OPENING);

                SelectableDataSource sds = new SelectableDataSource(dataSource);

                sds.setXMLEntity03(xml.getChild(0));

                setDataSource(sds);
            } catch (NoSuchTableException e) {
                throw new XMLException(e);
            } catch (DriverLoadException e) {
                throw new XMLException(e);
            } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
                throw new XMLException(e);
            }

        } else {
            ProjectView vista = project.getViewByName(xml.getStringProperty(
                        "viewName"));
            FLayer layer = vista.getMapContext().getLayers().getLayer(layerName);

            modelo = ((AlphanumericData) layer).getRecordset();
            associatedTable = (AlphanumericData) layer;
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param xml DOCUMENT ME!
     *
     * @throws XMLException DOCUMENT ME!
     * @throws DriverException DOCUMENT ME!
     */
    private void setSelectableDataSource(XMLEntity xml)
        throws XMLException, DriverException {
        String layerName = null;
        boolean bNeedToReplace = false;
        XMLEntity xmlAux = null;

        try {
            SelectableDataSource sds;

            if (xml.contains("layerName")) {
                layerName = xml.getStringProperty("layerName");

                ProjectView vista = project.getViewByName(xml.getStringProperty(
                            "viewName"));
               	FLayer layer=getLayer(vista.getMapContext().getLayers(),layerName);
                sds = ((AlphanumericData) layer).getRecordset();
            } else {
                sds = SelectableDataSource.createSelectableDataSource(xml.getChild(
                            0));
            }

            setDataSource(sds);

            if (xml.getNumChild() == 2) {
                bNeedToReplace = true;
                xmlAux = xml.getChild(1);
            }
            
            if (bNeedToReplace) {
				if (layerName != null) {
					ProjectView vista = project.getViewByName(xml
							.getStringProperty("viewName"));
					FLayer layer = getLayer(vista.getMapContext().getLayers(),
							layerName);

					// modelo = ((AlphanumericData) layer).getRecordset();
					associatedTable = (AlphanumericData) layer;
				}
				replaceDataSource(SelectableDataSource.createSelectableDataSource(xmlAux));
			}
        } catch (NoSuchTableException e) {
            throw new XMLException(e);
        } catch (ParseException e) {
            throw new XMLException(e);
        } catch (DriverLoadException e) {
            throw new XMLException(e);
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            throw new XMLException(e);
        } catch (SemanticException e) {
            throw new XMLException(e);
        } catch (IOException e) {
            throw new XMLException(e);
        }

        if (layerName != null) {
            ProjectView vista = project.getViewByName(xml.getStringProperty(
                        "viewName"));
        	FLayer layer=getLayer(vista.getMapContext().getLayers(),layerName);

            //			modelo = ((AlphanumericData) layer).getRecordset();
            associatedTable = (AlphanumericData) layer;
            
            if (bNeedToReplace)
            {
                try {
                    replaceDataSource(SelectableDataSource.createSelectableDataSource(xmlAux));
                } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (NoSuchTableException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (ParseException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (DriverLoadException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (SemanticException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (XMLException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
    }
    private FLayer getLayer(FLayers layers,String name){
    	for (int i=0;i<layers.getLayersCount();i++){
    		if (layers.getLayer(i) instanceof FLayers){
    			return getLayer((FLayers)layers.getLayer(i),name);
    		}else if (layers.getLayer(i).getName().equals(name)){
    			return layers.getLayer(i);
    		}
    	}
    return null;
    }
    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public AlphanumericData getAssociatedTable() {
        return associatedTable;
    }

    /**
     * DOCUMENT ME!
     *
     * @param associatedTable DOCUMENT ME!
     */
    public void setAssociatedTable(AlphanumericData associatedTable) {
        this.associatedTable = associatedTable;
    }

    /**
     * Obtiene la fuente de datos original de la tabla si se ha invocado
     * replaceDataSource. Si no se invoc este mtodo o se invoc
     * posteriormente restoreDataSource se devuelve null
     *
     * @return Returns the original.
     */
    public SelectableDataSource getOriginal() {
        return original;
    }

    /**
     * Devuelve el identificador de la tabla que contiene el link.
     *
     * @return identificador nico de la tabla.
     */
    public String getLinkTable() {
        return linkTable;
    }

    /**
     * Devuelve el nombre del campo de la tabla a enlazar.
     *
     * @return Nombre del campo de la tabla a enlazar.
     */
    public String getField1() {
        return field1;
    }

    /**
     * Devuelve el nombre del campo de la tabla enlazada.
     *
     * @return Nombre del campo de la tabla enlazada.
     */
    public String getField2() {
        return field2;
    }

    /**
     * Inserta el identificador de la tabla, el campo de la primera tabla y el
     * campo de la segunda tabla.
     *
     * @param lt identificado de la tabla.
     * @param f1 nombre del campo de la primera tabla.
     * @param f2 nombre del campo de la segunda tabla.
     */
    public void setLinkTable(String lt, String f1, String f2) {
        linkTable = lt;
        field1 = f1;
        field2 = f2;
    }

    /**
     * Borra el identificador de la tabla y elimina del array de listener los
     * listener que sean del tipo: LinkSelectionListenr
     */
    public void removeLinkTable() {
        linkTable = null;
        getModelo().removeLinksSelectionListener();
    }
    public String[] getAliases() {
        return alias;
    }
    public void setAliases(String[] alias) {
        this.alias = alias;
    }
    public int[] getMapping() {
        return mapping;
    }
    public void setMapping(int[] mapping) {
        this.mapping = mapping;
    }
}
