/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project;

import java.awt.Color;
import java.awt.Component;
import java.text.DateFormat;
import java.util.Date;

import javax.swing.JOptionPane;

import org.cresques.cts.IProjection;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ErrorEvent;
import com.iver.cit.gvsig.fmap.ErrorListener;
import com.iver.cit.gvsig.fmap.FMap;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.gui.project.OpenException;
import com.iver.cit.gvsig.gui.project.SaveException;
import com.iver.utiles.XMLEntity;


/**
 * Clase que representa una vista del proyecto
 *
 * @author Fernando Gonzlez Corts
 */
public class ProjectView extends ProjectElement implements ErrorListener {
	public static int numViews = 0;

	//public static int METROS = 0;
	//public static int KILOMETROS = 1;
	//public static int[] unidades = new int[] { METROS, KILOMETROS };
	///private Color backgroundColor = new Color(255, 255, 255);
	private FMap mapOverViewContext;
	private FMap mapContext;
	private int m_typeLink = 0;
	private String m_extLink;
	private String m_selectedField = null;

	/**
	 * DOCUMENT ME!
	 *
	 * @return
	 */
	public FMap getMapContext() {
		return mapContext;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return
	 */
	public FMap getMapOverViewContext() {
		return mapOverViewContext;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param baseName
	 *
	 * @return
	 */
	public static ProjectView createView(String baseName) {
		ProjectView v = new ProjectView();
		v.mapContext = new FMap(new ViewPort(Project.getProjection()));
		v.mapOverViewContext = new FMap(null);

		//v.mapOverViewContext = null;
		v.setName(baseName + " - " + numViews);
		v.setCreationDate(DateFormat.getInstance().format(new Date()));
		numViews++;

		return v;
	}

	/**
	 * @see com.iver.cit.gvsig.project.ProjectView#setMapContext(com.iver.cit.gvsig.fmap.FMap)
	 */
	public void setMapContext(FMap fmap) {
		mapContext = fmap;
		fmap.addErrorListener(this);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param fmap DOCUMENT ME!
	 */
	public void setMapOverViewContext(FMap fmap) {
		mapOverViewContext = fmap;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 * @throws XMLException
	 * @throws SaveException 
	 */
	public XMLEntity getXMLEntity() throws SaveException {
		XMLEntity xml = super.getXMLEntity();
		//xml.putProperty("nameClass", this.getClass().getName());
		try{
		xml.putProperty("numViews", numViews);
		xml.putProperty("m_selectedField", m_selectedField);
		xml.putProperty("m_typeLink", m_typeLink);
		xml.putProperty("m_extLink", m_extLink);
		xml.addChild(mapContext.getXMLEntity());

		if (mapOverViewContext != null) {
			if (mapOverViewContext.getViewPort() != null) {
				xml.putProperty("mapOverView", true);
				xml.addChild(mapOverViewContext.getXMLEntity());
			} else {
				xml.putProperty("mapOverView", false);
			}
		} else {
			xml.putProperty("mapOverView", false);
		}
		}catch (Exception e) {
			throw new SaveException(e,this.getClass().getName());
		}
		return xml;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 * @param p DOCUMENT ME!
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 *
	 * @see com.iver.cit.gvsig.project.ProjectElement#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity03(XMLEntity xml, Project p)
		throws XMLException, DriverException, DriverIOException {
		numViews = xml.getIntProperty("numViews");
		m_selectedField = xml.getStringProperty("m_selectedField");
		m_typeLink = xml.getIntProperty("m_typeLink");
		m_extLink = xml.getStringProperty("m_extLink");
		setMapContext(FMap.createFromXML03(xml.getChild(0)));
		
		if (xml.getBooleanProperty("mapOverView")) {
			setMapOverViewContext(FMap.createFromXML03(xml.getChild(1)));
		}
		
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 * @param p DOCUMENT ME!
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 * @throws OpenException 
	 *
	 * @see com.iver.cit.gvsig.project.ProjectElement#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity(XMLEntity xml, Project p)
		throws XMLException, DriverException, DriverIOException, OpenException {
		try{
			numViews = xml.getIntProperty("numViews");
			m_selectedField = xml.getStringProperty("m_selectedField");
			m_typeLink = xml.getIntProperty("m_typeLink");
			m_extLink = xml.getStringProperty("m_extLink");
			setMapContext(FMap.createFromXML(xml.getChild(0)));
	
			if (xml.getBooleanProperty("mapOverView")) {
				setMapOverViewContext(FMap.createFromXML(xml.getChild(1)));
			}
			showErrors();
		}catch (Exception e) {
			throw new OpenException(e,this.getClass().getName());
		}
	}

	public void showErrors(){
		if (mapContext.getLayersError().size()>0){
			String layersError="";
			for (int i=0;i<mapContext.getLayersError().size();i++){
				layersError=layersError+"\n"+(String)mapContext.getLayersError().get(i);
			}
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this,"fallo_capas")+" : \n"+
					layersError);
		}
	}
	/**
	 * DOCUMENT ME!
	 *
	 * @param p DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 * @throws OpenException 
	 */
	/*public ProjectView cloneProjectView(Project p)
		throws XMLException, DriverException, DriverIOException, OpenException {
		return (ProjectView) createFromXML(getXMLEntity(), p);
	}
*/
	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public IProjection getProjection() {
		return mapContext.getProjection();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public IProjection getOverViewProjection() {
		return mapOverViewContext.getProjection();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getExtLink() {
		return m_extLink;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public int getTypeLink() {
		return m_typeLink;
	}

	/**
     * Se selecciona la extensin para realizar cuando se quiera el link.
     *
     * @param s nombre del campo.
     */
	public void setExtLink(String s) {
		m_extLink = s;
	}

	/**
     * Se selecciona el tipo de fichero para realizar cuando se quiera el link.
     *
     * @param i tipo de fichero.
     */
	public void setTypeLink(int i) {
		m_typeLink = i;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param c DOCUMENT ME!
	 */
	public void setBackColor(Color c) {
//		getMapContext().getViewPort().addViewPortListener(getMapContext()
	//													   .getEventBuffer());
		getMapContext().getViewPort().setBackColor(c);
		//getMapContext().getViewPort().removeViewPortListener(getMapContext()
			//												  .getEventBuffer());
	}

	/**
	 * Se selecciona el nombre del campo para realizar cuando se quiera el
	 * link.
	 *
	 * @param s nombre del campo.
	 */
	public void setSelectedField(String s) {
		m_selectedField = s;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getSelectedField() {
		return m_selectedField;
	}

	public void errorThrown(ErrorEvent e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this,"fallo_capas")+" : \n"+
					e.getMessage());
	
	}
}
