/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package es.gva.cit.gvsig.catalogClient.gui;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import javax.swing.JOptionPane;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;

import es.gva.cit.catalogClient.schemas.discoverer.Resource;
import es.gva.cit.catalogClient.traductor.Translator;
import es.gva.cit.catalogClient.ui.chooseResource.ChooseResourceDialogPanel;
import es.gva.cit.gvsig.catalogClient.loaders.LayerLoader;
import es.gva.cit.gvsig.catalogClient.loaders.LayerLoaderException;
import es.gva.cit.gvsig.catalogClient.translation.AndamiTranslation;

/**
 * This class implements the resources list view.
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class ChooseResourceDialog extends ChooseResourceDialogPanel
implements View {
	
	/**
	 * @param resources
	 * Found resources array
	 */
	public ChooseResourceDialog(Collection resources) {
		super(resources,new AndamiTranslation());
	}   
	
	public ViewInfo getViewInfo() {
		ViewInfo m_viewinfo = new ViewInfo(ViewInfo.MODALDIALOG);
		m_viewinfo.setTitle(getName());
		return m_viewinfo;
	}
	
	public void closeButtonActionPerformed() {
		closeJDialog();
	}
	
	public void closeJDialog() {
		setVisible(true);
		PluginServices.getMDIManager().closeView(ChooseResourceDialog.this);
	}
	
	/**
	 * This method is invocated when a resource button is clicked
	 */
	public void resourceButtonActionPerformed(Resource resource){
		LayerLoader loader = null;
		try {
			loader = LayerLoader.getLoader(resource,translator);
			if (loader != null) {
				loader.loadLayer();
			} else {
				JOptionPane.showMessageDialog(this,
						Translator.getText(translator,"pluginNotFound") ,
						Translator.getText(translator,"pluginNotFoundTitle"),			
						JOptionPane.INFORMATION_MESSAGE);	
				return;
			}
			
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LayerLoaderException e) {
			JOptionPane.showMessageDialog(this,
					e.getMessage(),
					e.getWindowMessage(),			
					JOptionPane.ERROR_MESSAGE);	
		}		
		closeJDialog();
	}   
}





