/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gvsig.catalogClient.loaders;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.StringTokenizer;
import java.util.TreeMap;

import com.hardcode.driverManager.DriverLoadException;
import com.iver.cit.gvsig.fmap.drivers.DBLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.jdbc.postgis.PostGisDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;

import es.gva.cit.catalogClient.schemas.discoverer.Resource;
import es.gva.cit.catalogClient.traductor.ITranslator;
import es.gva.cit.catalogClient.traductor.Translator;



/**
 * This class is used to load a POSTGIS resource in gvSIG
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */

public class PostgisLayerLoader extends LayerLoader{
	
	
	
	public PostgisLayerLoader(Resource resource, ITranslator translator) {
		super(resource, translator);		
	}

	/**
     * This function adds a Postgis Layer to gvSIG 
     * @param jdbcUrl
   	 * JDBC url connection
   	 * @param table
   	 * Table to load
	 * @throws LayerLoaderException 
   	 */	
   	public void loadLayer() throws LayerLoaderException {
       	FLayer flayer;
       	String jdbcUrl = getResource().getLinkage();
       	String table = getResource().getName();
        try {
            flayer = createPostgisLayer(jdbcUrl, table);
           	addLayerToView(flayer);
        } catch (SQLException e) {
        	throw new LayerLoaderException(e.getMessage(),getWindowMessage());
        } catch (DriverLoadException e) {
        	throw new LayerLoaderException(e.getMessage(),getWindowMessage());
        } catch (ClassNotFoundException e) {
        	throw new LayerLoaderException(e.getMessage(),getWindowMessage());
        }     
        
   	}
    	
    /**
   	 * It returns a Postgis Layer
   	 * @param jdbcUrl
   	 * JDBC url connection
   	 * @param table
   	 * Table to load
   	 * @return
     * @throws SQLException
     * @throws DriverLoadException
     * @throws ClassNotFoundException
   	 */
    private FLayer createPostgisLayer(String jdbcUrl, String table) throws SQLException, DriverLoadException, ClassNotFoundException  {
        //jdbc:postgresql://sercartlin/carto_300k?user=gis&password=gis     
        //NAME=Comunicaciones&TABLENAME=comunic_lin_300k&ID=6&FIELDS=entity,layer,codigo,tipo,gid&GEOMFIELD=the_geom
      StringTokenizer sti = new StringTokenizer(jdbcUrl,"?");
        String dbURL = sti.nextToken();
        
        String p = sti.nextToken();
               
        TreeMap credentials = separateParams(p);
		String user = (String) credentials.get((String) "USER");
		String pwd = (String) credentials.get((String) "PASSWORD");
		
		TreeMap params = separateParams(table);
		String layerName = (String) params.get((String) "NAME");
		String fidField = (String) params.get((String) "ID");
		String sFields = (String) params.get((String) "FIELDS");
		String[] fields = sFields.split(",");
		String geomField = (String) params.get((String) "GEOMFIELD");
		String tableName = (String) params.get((String) "TABLENAME");
		String whereClause = "";
			
				
		 Connection conn = DriverManager.getConnection(dbURL, user, pwd);
         conn.setAutoCommit(false);
           
         DBLayerDefinition lyrDef = new DBLayerDefinition();                
         lyrDef.setLayerName(layerName);
         lyrDef.setTableName(tableName);
         lyrDef.setWhereClause(whereClause);
         lyrDef.setFieldNames(fields);
         lyrDef.setFieldGeometry(geomField);
         lyrDef.setFieldID(fidField);

         PostGisDriver pgd = new PostGisDriver();
         pgd.setData(conn, lyrDef);
                 
         return LayerFactory.createDBLayer(pgd, layerName, null);
   }
    
    private TreeMap separateParams(String pairValues){
        TreeMap map = new TreeMap(); 
		String[] params = pairValues.split("&");
		for (int i = 0; i < params.length; i++) {
			String[] nameValue = params[i].split("=");
			map.put(nameValue[0].toUpperCase(), nameValue[1]);
		}
		return map;
    }

	private String getErrorMessage() {
		return Translator.getText(getTranslator(),"postgisError") + ".\n" +
				Translator.getText(getTranslator(),"server") + ": " + 
				getResource().getLinkage() + "\n" +
				Translator.getText(getTranslator(),"parameters") + ": " +
				getResource().getName();
	}

	private String getWindowMessage() {
		return Translator.getText(getTranslator(),"postgisLoad");
	}

	
}
