/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gvsig.catalogClient.loaders;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.util.Vector;

import org.exolab.castor.xml.ValidationException;

import com.iver.cit.gvsig.fmap.drivers.UnsupportedVersionException;
import com.iver.cit.gvsig.fmap.drivers.WMSException;
import com.iver.cit.gvsig.fmap.drivers.wms.FMapWMSDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWMS;
import com.iver.cit.gvsig.fmap.layers.WMSLayerNode;

import es.gva.cit.catalogClient.schemas.discoverer.Resource;
import es.gva.cit.catalogClient.traductor.ITranslator;
import es.gva.cit.catalogClient.traductor.Translator;


/**
 * This class is used to load a WMS layer in gvSIG
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class WMSLayerLoader extends LayerLoader{
	
		
    public WMSLayerLoader(Resource resource, ITranslator translator) {
		super(resource, translator);
		//load(getResource().getLinkage(),getResource().getName());
	}

	/**
	 * This function loads a WMS resource 
	 * @param host
	 * URL where the server is located
	 * @param sLayer
	 * Layer name
	 * @param crs
	 * Coordenates System
	 * @throws LayerLoaderException 
	 */
	
	public void loadLayer() throws LayerLoaderException {
		String host = getResource().getLinkage();
		String sLayer = getResource().getName();
		
		    FLayer flayer;
		    
			try {
				flayer = createWMSLayer(host, sLayer);
				addLayerToView(flayer);
			} catch (ProtocolException e) {
				throw new LayerLoaderException(e.getMessage(),getWindowMessage());
			} catch (MalformedURLException e) {
				throw new LayerLoaderException(e.getMessage(),getWindowMessage());
			} catch (IllegalStateException e) {
				throw new LayerLoaderException(e.getMessage(),getWindowMessage());
			} catch (ValidationException e) {
				throw new LayerLoaderException(e.getMessage(),getWindowMessage());
			} catch (WMSException e) {
				throw new LayerLoaderException(e.getMessage(),getWindowMessage());
			} catch (IOException e) {
				throw new LayerLoaderException(e.getMessage(),getWindowMessage());
			} catch (UnsupportedVersionException e) {
				throw new LayerLoaderException(e.getMessage(),getWindowMessage());
			} 		   		    

	}
	
	/**
	 * It retrieves a getCapabilities file and loads the layer in gvSIG
	 * @param host
	 * WMS URL
	 * @param sLayer
	 * Layer Name
	 * @return
	 * FLayer
	 * @throws ServerOutOfOrderException
	 * @throws ProtocolException
	 * @throws MalformedURLException
	 * @throws WMSException
	 * @throws IOException
	 * @throws UnsupportedVersionException 
	 * @throws ValidationException 
	 * @throws IllegalStateException 
	 * @throws LayerLoaderException 
	 */
	private FLayer createWMSLayer(String host, String sLayer) throws ProtocolException, MalformedURLException, WMSException, IOException, IllegalStateException, ValidationException, UnsupportedVersionException, LayerLoaderException {
		FMapWMSDriver drv = new FMapWMSDriver();
		drv.createClient(new URL(host));
		
		if (!drv.connect())
			throw new LayerLoaderException(getErrorMessage(),getWindowMessage()); 
		
		WMSLayerNode wmsNode = drv.getLayer(sLayer);
						
		if (wmsNode == null){
			throw new LayerLoaderException(getErrorMessage(),getWindowMessage());
		}		
		
		String[] sLayers = sLayer.split(",");
				
		FLyrWMS flyr = new FLyrWMS();
		flyr.setHost(new URL(host));
		flyr.setFullExtent(drv.getLayersExtent(sLayers,(String)wmsNode.getAllSrs().get(0)));
		flyr.setFormat(getGreatFormat(drv.getFormats()));
		flyr.setLayerQuery(sLayer);
		flyr.setInfoLayerQuery("");
		flyr.setSRS((String)wmsNode.getAllSrs().get(0));
		flyr.setName(sLayer);
		
		return flyr;
	   
	}
	
	/**
	 * It choose the best format to load different maps if the server 
	 * supports it. This format could be png, because it supports 
	 * transparency.
	 * @param formats
	 * Array with all the formats supported by the server
	 * @return
	 */	
	private String getGreatFormat(Vector formats){
	    for (int i=0 ; i<formats.size() ; i++){
	        String format = (String) formats.get(i);
	    	if (format.equals("image/jpg")){
	            return format;
	    	}
	    	if (format.equals("image/jpeg")){
	            return format;
	    	}
	    }
		    
	    return (String)formats.get(0);
	}

	private String getErrorMessage() {
		return Translator.getText(getTranslator(),"wmsError") + ".\n" +
				Translator.getText(getTranslator(),"server") + ": " + 
				getResource().getLinkage() + "\n" +
				Translator.getText(getTranslator(),"layer") + ": " +
				getResource().getName();		
	}

	private String getWindowMessage() {
		return Translator.getText(getTranslator(),"loadWMS");
	}
	
	
	
}
