/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gvsig.gazetteer.gui;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Rectangle2D;

import javax.swing.JDialog;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.cit.gvsig.fmap.ColorEvent;
import com.iver.cit.gvsig.fmap.ExtentEvent;
import com.iver.cit.gvsig.fmap.ViewPortListener;

import es.gva.cit.catalogClient.querys.Coordinates;
import es.gva.cit.catalogClient.utils.Frames;
import es.gva.cit.gazetteer.GazetteerClient;
import es.gva.cit.gazetteer.querys.Feature;
import es.gva.cit.gazetteer.ui.search.SearchDialogPanel;
import es.gva.cit.gvsig.gazetteer.gui.ConnectDialog;
import es.gva.cit.gvsig.catalogClient.translation.AndamiTranslation;


/**
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class SearchDialog extends SearchDialogPanel
implements View,ViewPortListener{
    private JDialog frame = null;
    
    public SearchDialog(GazetteerClient client,JDialog frame, Object serverConnectFrame){
        super(client,serverConnectFrame,new AndamiTranslation());
        this.frame = frame;  
        setViewChangeListener();	
    	loadViewPortCoordinates();
    }
    
    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.View#getViewInfo()
     */
    public ViewInfo getViewInfo() {
        ViewInfo m_viewinfo=new ViewInfo(ViewInfo.MODALDIALOG);
		m_viewinfo.setTitle(PluginServices.getText(this, "gazetteer_search"));
		return m_viewinfo;
    }
    
    public void ShowResultsActionPerformed(Feature[] features) {
        JDialog panel = new JDialog((Frame) PluginServices.getMainFrame(), false);
        Frames.centerFrame(panel,420,258);
      panel.setTitle(PluginServices.getText(this, "gazetteer_search")); 
        panel.setResizable(false);
        
        ShowResultsDialog dialog = new ShowResultsDialog(panel,
                client,
                features,
                lowerPanel.getNPaginas(),
                doQuery());
		
        panel.getContentPane().add(dialog);
        panel.show(); 	 
       
        //PluginServices.getMDIManager().addView(panel);
    }
    
    public void closeButtonActionPerformed() {
	     frame.setVisible(false);
	 }
    
	 /**
	  * Size button action performed
	  */	 
	 public void sizeButtonActionPerformed(){
	        if (isMinimized){
	        	frame.setSize(525,520);
				ppalPanel.setPreferredSize(new Dimension(514,490));
				getLowerPanel().setPreferredSize(new Dimension(514,440));
				getLowerPanel().setVisible(true);
				frame.setVisible(true);
				getUpperPanel().setUpIcon();
	         }else{
	        	frame.setSize(525,117);
	 			ppalPanel.setPreferredSize(new Dimension(514,91));
	 			getLowerPanel().setPreferredSize(new Dimension(514,0));
	 			getLowerPanel().setVisible(false);
	 			frame.setVisible(true);
	 			getUpperPanel().setDownIcon();
	        }
	        isMinimized = !isMinimized;
	 }
	 /**
	  * Return button action
	  */
	public void returnButtonActionPerformed() {  
		closeJDialog();
		ConnectDialog serverConnect = (ConnectDialog)serverConnectFrame;
		serverConnect.setVisible(true);
		serverConnect.getSearchButton().setEnabled(false);
		PluginServices.getMDIManager().addView(serverConnect);
	} 
    
    public void closeJDialog() {
    	frame.setVisible(false);
	}
    
	 /**
	  * This method loads the view coordinates to the catalog search dialog
	  *
	  */
	 private void loadViewPortCoordinates(){
		 com.iver.cit.gvsig.gui.View activeView = 
				(com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveView();
	       
	        Rectangle2D r2d= activeView.getMapControl().getViewPort().getExtent();
	        
	        try{
	        getLowerPanel().setCoordinates(new Coordinates(r2d.getMinX(),
	                r2d.getMaxY(),
	                r2d.getMaxX(),
	                r2d.getMinY()));
	        }catch(NullPointerException E){
	            //We cant retrieve the coordinates if it doesn't 
	            //exist a loaded layer
	        }
	 }
	 /*
     * This methos joins the viewPort event to the listener
     */
    private void setViewChangeListener(){
        com.iver.cit.gvsig.gui.View activeView = 
			(com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveView();
               
        activeView.getMapControl().getViewPort().addViewPortListener(this);
       
    }

	public void extentChanged(ExtentEvent e) {
		loadViewPortCoordinates();   
		
	}

	public void backColorChanged(ColorEvent e) {
		// TODO Auto-generated method stub
		
	}


    
 
    
   
}
