/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.TreeMap;

import javax.swing.JOptionPane;

import org.cresques.io.GeoRasterFile;
import org.cresques.io.raster.RasterFilterStack;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.wcs.FMapWCSDriver;
import com.iver.cit.gvsig.fmap.operations.Cancellable;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.XMLEntity;

import es.uji.lsi.wcs.client.ServerErrorResponseException;

/**
 * Class for the WCS layer.
 * 
 * Capa para el WCS.
 * 
 * @author jaume - jaume.dominguez@iver.es
 */
public class FLyrWCS extends FLyrDefault implements RasterOperations{
	private FMapWCSAdapter wcs = null;

	private String 						label;
	private StatusRasterInterface		status = null;
	private int 						posX = 0, posY = 0;
	private double 						posXWC = 0, posYWC = 0;
	private int 						r = 0, g = 0, b = 0;
		

	/**
	 * Returns the XMLEntity containing the necessary info for reproduce
	 * the layer.
	 * 
	 * Devuelve el XMLEntity con la informacin necesaria para reproducir
	 * la capa.
	 * 
	 * @return XMLEntity.
	 * @throws XMLException
	 */
	public XMLEntity getXMLEntity() throws XMLException {
		XMLEntity xml = super.getXMLEntity();

		xml.putProperty("wcs.host", getHost());
		try {
            xml.putProperty("wcs.fullExtent",
            	StringUtilities.rect2String(wcs.getFullExtent()));
        } catch (IOException e) {
        } catch (DriverException e) {
        }
		xml.putProperty("wcs.layerQuery", wcs.getCoverageQuery());
		xml.putProperty("wcs.format", getFormat());
		xml.putProperty("wcs.srs", getSRS());
		xml.putProperty("wcs.time", getTime());
		xml.putProperty("wcs.parameter", getParameter());
		xml.putProperty("wcs.label", label);
		xml.putProperty("wcs.coverageName", getCoverageName());
		xml.putProperty("wcs.maxResX", getMaxResolution().getX());
		xml.putProperty("wcs.maxResY", getMaxResolution().getY());
		if (status!=null)
			status.getXMLEntity(xml, true, this);
		else{
			status = new StatusLayerRaster();
			status.getXMLEntity(xml, true, this);
		}
		return xml;
	}

	/**
	 * Reproduces the layer from an XMLEntity.
	 * 
	 * A partir del XMLEntity reproduce la capa.
	 * 
 	 * @param xml XMLEntity
	 *
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 */
	public void setXMLEntity(XMLEntity xml) throws XMLException {
		super.setXMLEntity(xml);
		//System.out.println(" Cargo la capa WCS. ... o no ;-)");
		setHost(xml.getStringProperty("wcs.host"));
		setFullExtent(StringUtilities.string2Rect(
			xml.getStringProperty("wcs.fullExtent")));
		wcs.setCoverageQuery(xml.getStringProperty("wcs.layerQuery"));
		setFormat(xml.getStringProperty("wcs.format"));
		setSRS(xml.getStringProperty("wcs.srs"));
		setTime(xml.getStringProperty("wcs.time"));
		setParameter(xml.getStringProperty("wcs.parameter"));
		label = xml.getStringProperty("wcs.label");
		setCoverageName(xml.getStringProperty("wcs.coverageName"));
		if (xml.contains("wcs.maxRes"))
			setMaxResolution(new Point2D.Double(xml.getDoubleProperty("wcs.maxRes"), xml.getDoubleProperty("wcs.maxRes")));
		else if (xml.contains("wcs.maxResX") && xml.contains("wcs.maxResY")) 
			setMaxResolution(new Point2D.Double(xml.getDoubleProperty("wcs.maxResX"), xml.getDoubleProperty("wcs.maxResY")));

		
		String claseStr = null;
		if (xml.contains("raster.class")) {
			claseStr = xml.getStringProperty("raster.class");
		}
		if(status!=null)
			status.setXMLEntity(xml, this);
		else{
			
			//Cuando cargamos un proyecto 
			
			if(claseStr!=null && !claseStr.equals("")){
				try{
					Class clase = Class.forName(claseStr);
					Constructor constr = clase.getConstructor(null);
					status = (StatusRasterInterface)constr.newInstance(null);
					if(status!=null)
						status.setXMLEntity(xml, this);
				}catch(ClassNotFoundException exc){
					exc.printStackTrace();
				}catch(InstantiationException exc){
					exc.printStackTrace();
				}catch(IllegalAccessException exc){
					exc.printStackTrace();
				}catch(NoSuchMethodException exc){
					exc.printStackTrace();
				}catch(InvocationTargetException exc){
					exc.printStackTrace();
				}					
			}
		}
		wcs.firstLoad = true;
	}
	
	/**
	 * The full extent of the layer.
	 * 
	 * La extensin completa de la layer.
	 * 
	 * @return Rectangle2D
	 */
	public Rectangle2D getFullExtent(){// throws DriverException {
		try {
            return getWCSAdaptor().getFullExtent();
        } catch (IOException e) {
            JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
            		"error_comunicacion_servidor", "Error", JOptionPane.ERROR_MESSAGE);
        } catch (DriverException e) {
            JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
            		"servidor_wcs_no_responde", "Error", JOptionPane.ERROR_MESSAGE);
        }
        return null;
	}

	/**
	 * Draws the layer on the ViewPort using PxRaster.
	 * 
	 * Dibuja la capa en el ViewPort con PxRaster.
	 */
	public void draw(BufferedImage image, Graphics2D g,
			ViewPort viewPort, Cancellable cancel, double scale) throws DriverException {
		/* This try loop I've moved here is for show an alert window when it is not
		 * possible to download the coverage. 
		 */
			try {
				if (!isWithinScale(scale))
					return;
				
				if(status!=null && wcs.firstLoad){
					wcs.draw(image, g, viewPort, cancel);
					status.applyStatus(this);
					wcs.firstLoad = false;
				}
				
				wcs.draw(image, g, viewPort, cancel);
			} catch (ServerErrorResponseException e) {
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(), e.getMessage(), "Error",
						JOptionPane.ERROR_MESSAGE);
			} catch (IOException e){
				this.setActive(false);
				this.setVisible(false);
				throw new DriverException("Error en la conexin", e);
			}
	}
	/**
	 * Prints the layer.
	 * 
	 * Imprime la cobertura.
	 */	
	public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel, double scale)
			throws DriverException {
		draw(null, g, viewPort, cancel, scale);
	}

	/**
	 * Sets the layer's label. That what is going to be used at the TOC.
	 * 
	 * Establece la etiqueta de la cobertura. La que se usar en el TOC.
	 * 
	 * @param lbl
	 */
	public void setLabel(String lbl) {
		label = lbl;
	}

	/**
	 * Sets the layer's full extent.
	 * 
	 * Establece la extensin mxima de la capa.
	 * 
	 * @param rect
	 */
	public void setFullExtent(Rectangle2D rect) {
		wcs.setFullExtent(rect);
	}

	/**
	 * Recovers the name of the layer shown on the TOC. The text used for the name
	 * is not the actual coverage's name but that what appears in the wizard's text
	 * field which initially correponds with the "label" specified by the server for
	 * the coverage and may be edited by the user for choose whatever she/he wants. 
	 * 
	 * Recupera el nom de la capa mostrada al TOC. El text per al nom no s el
	 * nom de la cobertura sin el text que apareix al camp de text del wizard,
	 * que es correspon amb la "label" que el servidor especifica per a la
	 * covertura, i pot ser modificat per l'usuari al wizard per a gastar el que
	 * dessitge.
	 * 
	 * @return String
	 */
	public String getLabel() {
		return this.label;
	}

	/**
	 * Remote source layers have a bunch of properties that are required for get them from
	 * the servers. This method supplies a hash table containing any needed field. This hash
	 * table may be used to let the client to connect to a server and restore a previously saved
	 * layer. So, the layer itself may not be saved to the disk since the actual saved
	 * info is just its properties.
	 * 
	 * @return Returns a hash table containing all the required information for
	 * set up a wms layer
	 */
	public Hashtable getProperties(){
		Hashtable info = new Hashtable();
		info.put(   "name", getCoverageName());
		info.put(   "host", getHost());
		info.put(    "crs", wcs.getSRS());
		info.put( "format", wcs.getFormat());
		String str = wcs.getTime();
		if (str==null) 
			str = "";
		info.put(   "time", str);
		str = wcs.getParameter();
		if (str==null)
			str = "";
		info.put("parameter", str);	
		
		return info;
	}

	
	/**
	 * Returns the coverage name.
	 * 
	 * Obtiene el nombre de la cobertura.
	 * @return String
	 */
	public String getCoverageName() {
		return wcs.getCoverageName();
	}
	
	/**
	 * Returns the coverage format.
	 * 
	 * Obtiene el formato en que la cobertura est codificado.
	 * @return String
	 */
	public String getFormat() {
		return wcs.getFormat();
	}
	
	/**
	 * Returns the WCS server URL.
	 * 
	 * Obtiene la direccin URL del servidor WCS
	 * 
	 * @return String
	 */
	public String getHost() {
		return wcs.getHost();
	}
	
	/**
	 * Returns the parameter value.
	 * 
	 * Obtiene el valor del parmetro.
	 * @return String
	 */
	public String getParameter() {
		return wcs.getParameter();
	}
	
	/**
	 * Returns the current SRS.
	 * 
	 * Obtiene el SRS actual de la cobertura.
	 * @return String
	 */
	public String getSRS() {
		return wcs.getSRS();
	}
	
	/**
	 * Returns the TIME value in the WCS query.
	 * 
	 * Obtiene el valor para TIME en la consulta WCS.
	 * @return String
	 */
	public String getTime() {
		return wcs.getTime();
	}
	
	/**
	 * Sets the FORMAT parameter that we are requesting the WCS coverage.
	 * 
	 * Establece el parmetro FORMAT en que se pide la cobertura WCS
	 * @param format
	 */
	public void setFormat(String format) {
		wcs.setFormat(format);
	}
	
	/**
	 * Sets the server's URL.
	 * 
	 * Establece la direccin URL del servidor WCS
	 * @param host
	 */
	public void setHost(String host) {
		if (wcs == null) {
			wcs = new FMapWCSAdapter();
			wcs.setDriver(new FMapWCSDriver()); 
		}
		wcs.setHost(host);
	}
	
	/**
	 * Sets the parameter. It is defined by a name-value pair and the value is
	 * a comma-separated list of values.
	 * 
	 * Establece el nombre y el valor (lista de valores separados por comas)
	 * del parmetro de la consulta WCS
	 * @param parameter
	 */
	public void setParameter(String parameter) {
		wcs.setParameter(parameter);
	}
	
	/**
	 * Sets the SRS parameter within the WCS query.
	 * 
	 * Establece el SRS de la consulta WCS.
	 * @param srs
	 */
	public void setSRS(String srs) {
		wcs.setSRS(srs);
	}
	
	/**
	 * Sets the TIME parameter within the WCS query.
	 * 
	 * Establece el parmetro TIME de la consulta WCS.
	 * @param time
	 */
	public void setTime(String time) {
		 wcs.setTime(time);
	}
	
	/**
	 * Sets the COVERAGE parameter within the WCS query.
	 * 
	 * Establece el parmetro COVERAGE de la consulta WCS.
	 * @param coverageName
	 */
	public void setCoverageName(String coverageName) {
		wcs.setCoverageName(coverageName);
	}
	
	/**
	 * Gets the max resolution allowed by the coverage. Requesting a higher resolution
	 * than this value does not cause any error, but the info responsed is just an
	 * interpolation.
	 * 
	 * In exchange for obtaining a greater file and without additional information,
	 * we can easily connect it at the View.
	 * 
	 * 
	 * Obtiene la resolucin mxima soportada por la cobertura. La peticin
	 * de una resolucin superior a la soportada no provoca ningn error, aunque
	 * la informacin obtenida slo es una mera interpolacin de informacin. 
	 * 
	 * A cambio de obtener un archivo mayor y sin informacin adicional, podemos
	 * fcilmente acoplarlo a la vista.
	 * 
	 * @return double
	 */
	public Point2D getMaxResolution() {
		return wcs.getMaxResolution();
	}
	
	/**
	 * Sets the MAX resolution supported by the coverage.
	 * 
	 * The max resolution is given by the server within the DescribeCoverage document
	 * in the SpationDomain section.
	 * 
	 * Use this method only to load this value from there (the DescribeCoverage document)
	 * of from an already saved project because an incorrect value will bring
	 * computing errors when doing "zoom to raster resolution".
	 * 
	 * 
	 * 
	 * Establece la resolucin MXIMA que soporta la cobertura.
	 * 
	 * La resolucin mxima viene fijada por el servidor dentro documento
	 * DescribeCoverage en la seccin SpatialDomain.
	 * 
	 * Use este mtodo nicamente para cargar el valor para la resolucin mxima
	 * desde all o desde un proyecto guardado en anterioridad, puesto que un
	 * valor incorrecto reportar (conllevar) errores de clculo cuando se haga
	 * un "zoom a la resolucin del rster".
	 * 
	 * @param res
	 */
	//public void setMaxResolution(double res){
	public void setMaxResolution(Point2D res){
		wcs.setMaxResolution(res);
	}
	
	/**
	 * Returns the adaptor.
	 * 
	 * Obtiene el adaptor.
	 */
	public FMapWCSAdapter getWCSAdaptor(){
	    return wcs;
	}
	
	/**
	 * Sets the adaptor.
	 * 
	 * Establece el adaptador.
	 * 
	 * @deprecated
	 * @param wcs
	 */
	public void setWCSAdaptor(FMapWCSAdapter wcs){
		this.wcs = wcs;
	}
	
	/**
	 * Returns the bounding box that is being used.
	 * 
	 * Obtiene la Bounding Box sobre la que se est operando
	 * @return Rectangle2D
	 */
	private Rectangle2D getBbox() {
		return wcs.getBbox();
	}
	
	/**
	 * Returns the current resolution for the coverage at the view.
	 * 
	 * Obtiene la resolucin actual para la cobertura de la vista.
	 * @return
	 */
	private Point2D getRes() {
		return wcs.getRes();
	}
	
	/**
	 * Sets the current resolution for the coverage at the view.
	 * This value is automatically calculated by gvSIG. Thus, this method should
	 * be used only to reload this value from a previously saved project.
	 * 
	 * Establece la resolucin actual para la cobertura de la vista.
	 * Este valor es calculado automticamente por gvSIG. ste mtodo debera
	 * ser usado nicamente para recuperar este valor de un proyecto
	 * anteriormente guardado. 
	 * @param res
	 */
	//private void setRes(double res) {
	private void setRes(Point2D res) {
		wcs.setRes(res);
	}
	
	/**
	 * Sets the area of the coverage to request.
	 * This value is automatically calculated by gvSIG. Thus, this method should
	 * be used only to reload this value from a previuosly saved project.
	 * 
	 * Establece el rea de la cobertura a pedir.
	 * Este valor es calculado automticamente por gvSIG. ste mtodo debera
	 * ser usado nicamente para recuperar el valor de un proyecto anteriormente
	 * guardado.
	 * 
	 * @param rect
	 */
	private void setBBox(Rectangle2D rect) {
		wcs.setBBox(rect);
	}
	
	/**
	 * Builds a coverage starting from a full GetCoverage URL.
	 * (Using this is not a regular function)
	 * 
	 * Inicializa una cobertura a partir de una URL GetCoverage completa.
	 * (To use this method supposes to skip all the regular operation)
	 * 
	 */
	public void initFromQueryString(String queryString){
		System.out.println("Query = "+queryString);
		TreeMap map = new TreeMap(); 
		String[] params = queryString.split("&");
		for (int i = 0; i < params.length; i++) {
			String[] nameValue = params[i].split("=");
			map.put(nameValue[0].toUpperCase(), nameValue[1]);
		}
		try {
			String coverageName = (String) map.get((String) "COVERAGE");
			String crs = (String) map.get((String) "CRS");
			String format = (String) map.get((String) "FORMAT");
			String time = (String) map.get((String) "TIME");
			String bands = (String) map.get((String) "BANDS");

			setSRS(crs);
			setCoverageName(coverageName);
			if (time != null)
				setTime("TIME="+time);
			FMapWCSDriver service = wcs.getDriver();
			service.connect();
			
			setLabel( service.getLabel(coverageName) );
			setName( getLabel() );
			setFullExtent ( service.getFullExtent(coverageName, crs) );
			setFormat( format );
			setParameter( "BANDS="+bands );
			setMaxResolution ( service.getMaxResolution(coverageName) );
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (DriverException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Asignar el estado del raster
	 * @param status
	 */
	public void setStatus(StatusRasterInterface status){
		this.status = status;
	}
	
	/**
	 * Obtiene el estado del raster
	 * @return
	 */
	public StatusRasterInterface getStatus(){
		return this.status;
	}
	
	/**
	 * Builds a coverage starting from a full GetCoverage URL.
	 * (Using this is not a regular function)
	 * 
 	 * Inicializa una cobertura a partir de una URL GetCoverage completa.
	 * (Usar este mtodo supone saltarse todo el funcionamiento normal)
	 * 
	 * TODO Por completar y probar
	 * @param queryString
	 */
	public void initFromQueryString2(String queryString){

		String host = queryString.substring(0, queryString.indexOf('?'));
		String[] params = queryString.substring(queryString.indexOf('?')+1, queryString.length()).split("=");
		setHost(host);
		for (int i = 0; i < params.length; i++) {
			String[] par_nombreValor = params[i].split("=");
			String nombre = par_nombreValor[0].toUpperCase();
			String valor = par_nombreValor[1];
			if (nombre.equals("COVERAGE"))			setCoverageName(valor);
			else if (nombre.equals("TIME"))			setTime(valor);
			else if (nombre.equals("FORMAT"))		setFormat(valor);
			else if (nombre.equals("CRS"))			setSRS(valor);
			else if (nombre.equals("EXCEPTIONS"))	continue;
			else if (nombre.equals("RESX"))			continue;
			else if (nombre.equals("RESY"))			continue;
			else if (nombre.equals("RESZ"))			continue;
			else if (nombre.equals("WIDTH"))		continue;
			else if (nombre.equals("HEIGHT"))		continue;
			else if (nombre.equals("DEPTH"))		continue;
			else if (nombre.equals("BBOX"))			continue;
			else if (nombre.equals("RESPONSE_CRS"))	continue;
			else									setParameter(nombre+"="+valor);
		}	
		FMapWCSDriver service = wcs.getDriver();
		try {
			service.connect();
			String coverageName = getCoverageName();
			setLabel( service.getLabel(coverageName) );
			setName( getLabel() );
			setFullExtent ( service.getFullExtent(coverageName, getSRS()) );
			setMaxResolution ( service.getMaxResolution(coverageName) );
		} catch (IOException e) {
			e.printStackTrace();
		} catch (DriverException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Devuelve la pila de filtros aplicada sobre  la capa raster.
	 *
	 * @return RasterFilterStack.
	 */
	public RasterFilterStack getFilterStack() {
		return wcs.getFilterStack();
	}
	
	/**
	 * Asigna la pila de filtros aplicada al raster
	 * @return
	 */
	public void setFilterStack(RasterFilterStack stack){
		wcs.setFilterStack(stack);		
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#setTransparency(int)
	 */
	public void setTransparency(int trans) {
		wcs.setTransparency(trans);
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setBand(int, int)
	 */
	public void setBand(int flag, int nBand) {
		switch(flag){
		case GeoRasterFile.RED_BAND:wcs.setBandR(nBand);break;
		case GeoRasterFile.GREEN_BAND:wcs.setBandG(nBand);break;
		case GeoRasterFile.BLUE_BAND:wcs.setBandB(nBand);break;
		}
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getAttributes()
	 */
	public ArrayList getAttributes() {
		return wcs.getAttributes();
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getHeight()
	 */
	public double getHeight() {
		if(wcs.getGeoRasterFile() != null)
			return wcs.getGeoRasterFile().getHeight();
		else
			return 0;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getMaxX()
	 */
	public double getMaxX() {
		if(wcs.getPxRaster() != null)
			return wcs.getPxRaster().getExtent().getMax().getX();
		else
			return 0;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getMaxY()
	 */
	public double getMaxY() {
		if(wcs.getPxRaster() != null)
			return wcs.getPxRaster().getExtent().getMax().getY();
		else
			return 0;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getMinX()
	 */
	public double getMinX() {
		if(wcs.getPxRaster() != null)
			return wcs.getPxRaster().getExtent().getMin().getX();
		else
			return 0;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getMinY()
	 */
	public double getMinY() {
		if(wcs.getPxRaster() != null)
			return wcs.getPxRaster().getExtent().getMin().getY();
		else
			return 0;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getPixel(double, double)
	 */
	public int[] getPixel(double wcx, double wcy) {
		if(wcs.getPxRaster() != null)
			return wcs.getPxRaster().getPixel(wcx, wcy);
		else
			return null;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getSource()
	 */
	public RasterAdapter getSource() {
		// TODO Auto-generated method stub
		return null;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#getWidth()
	 */
	public double getWidth() {
		if(wcs.getGeoRasterFile() != null)
			return wcs.getGeoRasterFile().getWidth();
		else
			return 0;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setPos(int, int)
	 */
	public void setPos(int x, int y) {
		this.posX = x;
		this.posY = y;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setPosWC(double, double)
	 */
	public void setPosWC(double x, double y) {
		this.posXWC = x;
		this.posYWC = y;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setRGB(int, int, int)
	 */
	public void setRGB(int r, int g, int b) {
		this.r = r;
		this.g = g;
		this.b = b;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setSource(com.iver.cit.gvsig.fmap.layers.RasterAdapter)
	 */
	public void setSource(RasterAdapter ra) {
		// TODO Auto-generated method stub

	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.InfoByPoint#queryByPoint(java.awt.Point)
	 */
	public String queryByPoint(Point p) throws DriverException {
		String data = "<file:"+getName().replaceAll("[^a-zA-Z0-9]","")+">\n";
		ArrayList attr = this.getAttributes();
		data += "  <raster\n";
		data += "    File=\""+getName()+"\"\n";
		for (int i=0; i<attr.size(); i++) {
			Object [] a = (Object []) attr.get(i);

			data += "    "+a[0].toString()+"=";
			if (a[1].toString() instanceof String)
				data += "\""+a[1].toString()+"\"\n";
			else
				data += a[1].toString()+"\n";
		}
		data += "    Point=\""+posX+" , "+posY+"\"\n";
		data += "    Point_WC=\""+posXWC+" , "+posYWC+"\"\n";
		data += "    RGB=\""+r+", "+g+", "+b+"\"\n";
		data += "  />\n";

		data += "</file:"+getName().replaceAll("[^a-zA-Z0-9]","")+">\n";
		System.out.println(data);
		return data;
	}
	

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.RasterOperations#setTempExtent(com.iver.cit.gvsig.fmap.ViewPort)
	 */
	public void setTempExtent(ViewPort vp){
		
	}
}