/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.fmap.layers;

import java.util.ArrayList;
import java.util.Iterator;

import es.uji.lsi.wcs.XmlWcsParsing.SingleValue;

/**
 * Class that holds the info about the parameters that a coverage may have.
 * It only support the single value type parameters.
 * 
 * TODO Write the support for interval value parameters
 * 
 * Clase que contiene la informacin de los parmetros que puede contener una
 * cobertura.

 * @author jaume - jaume.dominguez@iver.es
 *
 */
public class WCSParameterModel {
	private String name;
	private ArrayList lista_valores_simples;
	private double min;
	private double max;
	private double res;
	private String type;
	
	/**
	 * Returns an ArrayList containing a list of single value list.
	 * @return Returns the lista_valores_simples.
	 */
	public ArrayList getLista_valores_simples() {
		return this.lista_valores_simples;
	}
	/**
	 * Sets the SINGLE value list of the parameter.
	 * 
	 * Establece la lista de valores SIMPLES del parmetro en cuestin
	 * @param lista_valores_simples The lista_valores_simples to set.
	 */
	public void setLista_valores_simples(ArrayList lista_valores_simples) {
		ArrayList valores = new ArrayList();
		Iterator it = lista_valores_simples.iterator();
		while (it.hasNext()){
			SingleValue sv = (SingleValue) it.next();
			valores.add(sv.getValue());
		}
		this.lista_valores_simples = valores;
	}
	/**
	 * Returns the parameter's name.
	 * 
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * Sets the parameter's name
	 * 
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Returns the parameter's type (currently only "singleValue" is supported)
	 * @return Returns the type.
	 */
	public String getType() {
		return type;
	}
	/**
	 * Sets the parameter's type (curretly only "singleValue" is supported)
	 * @param type The type to set.
	 */
	public void setType(String type) {
		this.type = type;
	}
	/** 
	 * returns the max value allowed for this parameter
	 * 
	 * @return double
	 */
	public double getMax() {
		return max;
	}
	
	/**
	 * sets the max value allowed for this parameter
	 * @param min
	 */
	
	public void setMax(double max) {
		this.max = max;
	}
	
	/**
	 * returns the min value allowed for this parameter
	 * @return double
	 */
	public double getMin() {
		return min;
	}
	
	/**
	 * sets the min value allowed for this parameter
	 * @param min
	 */
	public void setMin(double min) {
		this.min = min;
	}
	
	/**
	 * returns the resolution value supported by this parameter
	 * @return
	 */
	public double getRes() {
		return res;
	}
	
	/**
	 * sets the resolution supported by this parameter
	 * @param res
	 */
	public void setRes(double res) {
		this.res = res;
	}
	
}
