/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;

import com.hardcode.driverManager.Driver;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.drivers.wcs.FMapWCSDriver;
import com.iver.cit.gvsig.gui.wcs.CoverageInfo;
import com.iver.cit.gvsig.gui.wcs.WCSWizardData;
import com.iver.cit.gvsig.gui.wcs.WCSWizardDataSource;
import com.iver.cit.gvsig.gui.wizards.WizardData;


/**
 * Adapts the FMap's WMS layer to the WCSWizard's data source.
 * It is a very good idea to have the OGC's WCS's documentation
 * to understand what is it cooking here. 
 * 
 * Adapta las capa wms de Fmap a los datasource del WCSWizard
 * Es muy recomendable disponer de la documentacin WCS de OGC
 * para entender el juego de objetos utilizado.
 *
 * @author Jaume Domnguez Faus - jaume.dominguez@iver.es
 */
public class WCSDataSourceAdapter implements WCSWizardDataSource {
	//OGCWCSService service = null;
    private FMapWCSDriver client;
    WCSWizardData data = null;
    
	/**
	 * Establishes and loads the initial data to keep a regular
	 * session with the WCS server.
	 * @throws DriverException 
	 */
	public WCSWizardData detalles(URL host) throws IOException, DriverException {

		//client = new FMapWCSDriver("WCS", host.toString(), 80);
		client = new FMapWCSDriver();
		client.setHost(host.toString());
		
		client.connect();
		data = new WCSWizardData();
		data.setCoverageNames(client.getCoverageNames());
		data.setTitle(client.getLabel());
		data.setDescription(client.getDescription());
		
		rellena();
		return data;
	}

	/**
	 * Reads the coverage specific data
	 * 
	 * Rellenar los datos especficos de las coberturas
	 */
	private void rellena() {
		String[] coberturas = client.getCoverageNames();
		ArrayList listaCoberturas = new ArrayList();
		for (int i=0; i<coberturas.length; i++){
			CoverageInfo nueva_cobertura = new CoverageInfo();
			nueva_cobertura.setName(coberturas[i]);
			nueva_cobertura.setLabel(client.getLabel(coberturas[i]));
			nueva_cobertura.setText(client.getCoverageDescription(coberturas[i]));
			nueva_cobertura.setSRSList(client.getSRSs(coberturas[i]));
			nueva_cobertura.setFormats(client.getFormats(coberturas[i]));
			nueva_cobertura.setMaxRes(client.getMaxResolution(coberturas[i]));
			nueva_cobertura.setTiempos(client.getTimes(coberturas[i]));
			nueva_cobertura.setParametros(client.getParameters(coberturas[i]));
			listaCoberturas.add(nueva_cobertura);
		}
		data.setCoveragesList(listaCoberturas);
	}


	/**
	 * Searches the max estent bounding box given for an specific SRS of an specified
	 * coverage.
	 * 
	 * Busca la boundingbox de extensin mxima dada para el SRS especificado.
	 * 
	 * @return Rectangle2D 
	 */
	public Rectangle2D getBoundingBox(String coverageName, String srs) {
		try {
            return client.getFullExtent(coverageName, srs);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (DriverException e) {
            e.printStackTrace();
        }
        return null;
	}

	/**
	 * Get the driver that controls the communication between gvSIG and WCSClient
	 * 
	 * @return Driver
	 */
	public Driver getDriver() {
		return client;
	}
}
