/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.gui.dialog;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWCS;
import com.iver.cit.gvsig.gui.WCSDataSourceAdapter;
import com.iver.cit.gvsig.gui.panels.WCSParamsPanel;
import com.iver.cit.gvsig.gui.wcs.WCSWizardData;
import com.iver.cit.gvsig.gui.wcs.WCSWizardDataSource;

/**
 * @author jaume - jaume.dominguez@iver.es
 *
 */
public class WCSPropsDialog extends JPanel implements View {
	JDialog dlg = null;  //  @jve:decl-index=0:visual-constraint="10,20"
	private JPanel buttonsPanel = null;
	//private FLyrDefault fLayer = null;
	private FLayer fLayer = null;
	boolean applied;
	private ViewInfo m_ViewInfo = null;
	
	private JButton btnApply = null;
	private JButton btnOk = null;
	private JButton btnCancel = null;
	private WCSParamsPanel wcsParamsTabbedPane;
    private ComandosListener m_actionListener;

    public WCSPropsDialog(FLayer layer) {
		super();
		initialize(layer);
	}
    
    
	private void initialize(FLayer layer) {
		setBounds(0,0,500,333);
        setLayout(null);

        setFLayer(layer);
        wcsParamsTabbedPane = getParamsPanel(((FLyrWCS) layer).getProperties());
        this.add(wcsParamsTabbedPane);
		wcsParamsTabbedPane.getJTabbedPane().addMouseListener(
				new java.awt.event.MouseAdapter() {
					public void mouseClicked(java.awt.event.MouseEvent e) {
						getBtnOk().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
					}
				});
		wcsParamsTabbedPane.getLstCoverages().addListSelectionListener(
				new javax.swing.event.ListSelectionListener() { 
					public void valueChanged(javax.swing.event.ListSelectionEvent e) {    
						wcsParamsTabbedPane.actualizaWizard();
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
					}
				});
		wcsParamsTabbedPane.getLstCRSs().addListSelectionListener(
				new javax.swing.event.ListSelectionListener() { 
					public void valueChanged(javax.swing.event.ListSelectionEvent e) {
						getBtnOk().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
					}
				});
		wcsParamsTabbedPane.getLstFormats().addListSelectionListener(
				new javax.swing.event.ListSelectionListener() { 
					public void valueChanged(javax.swing.event.ListSelectionEvent e) {
						getBtnOk().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
					}
				});
		wcsParamsTabbedPane.getLstSelectedTimes().addListSelectionListener(
				new javax.swing.event.ListSelectionListener() { 
					public void valueChanged(javax.swing.event.ListSelectionEvent e) {
						getBtnOk().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
					}
				});
		wcsParamsTabbedPane.getSingleParamValuesList().addListSelectionListener(
				new javax.swing.event.ListSelectionListener() { 
					public void valueChanged(javax.swing.event.ListSelectionEvent e) {
						getBtnOk().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
					}
				});
		wcsParamsTabbedPane.getCmbParam().addItemListener(
				new java.awt.event.ItemListener() {
					public void itemStateChanged(java.awt.event.ItemEvent e) {
						String s = (String) wcsParamsTabbedPane.getLstCoverages().getSelectedValue();
						wcsParamsTabbedPane.refreshParamValues(s);
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
					}
				});
		wcsParamsTabbedPane.getBtnAddTemps().addActionListener(
				new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
						wcsParamsTabbedPane.addTiempo();
					}
				});
		wcsParamsTabbedPane.getBtnDelTemps().addActionListener(
				new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
						wcsParamsTabbedPane.delTiempo();
					}
				});
		wcsParamsTabbedPane.getTxtInterval().addKeyListener(
				new java.awt.event.KeyAdapter() { 
					public void keyReleased(java.awt.event.KeyEvent e) {    
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
					}
				});
		wcsParamsTabbedPane.getTxtIntervalEnd().addKeyListener(
				new java.awt.event.KeyAdapter() { 
					public void keyReleased(java.awt.event.KeyEvent e) {    
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
					}
				});
		wcsParamsTabbedPane.getTxtIntervalBegin().addKeyListener(
				new java.awt.event.KeyAdapter() { 
					public void keyReleased(java.awt.event.KeyEvent e) {    
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
						wcsParamsTabbedPane.addTiempo();
					}
				});
		wcsParamsTabbedPane.getRadioSingleInterval().addChangeListener(
				new javax.swing.event.ChangeListener() { 
					public void stateChanged(javax.swing.event.ChangeEvent e) {    
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
						wcsParamsTabbedPane.addTiempo();
					}
				});
		wcsParamsTabbedPane.getRadioMultipleInterval().addChangeListener(
				new javax.swing.event.ChangeListener() { 
					public void stateChanged(javax.swing.event.ChangeEvent e) {    
						applied = false;
						getBtnApply().setEnabled(wcsParamsTabbedPane.isCorretlyConfigured());
						wcsParamsTabbedPane.addTiempo();
					}
				});
		
        this.add(getButtonsPanel(), null);        
	}
	

	public void setFLayer(FLayer f) {
		fLayer = f;
	}

	/**
	 * With getParamsPanel we have access to the coverage config TabbedPane.
	 * If this panel doesn't exist yet (which can occur when an existing project is
	 * recovered) it is been automatically constructed by connecting to the server,
	 * reloading the necessary data, filling up the content and setting the selected
	 * values that were selected when the projet was saved.
	 * 
	 * Since a connection to the server is needed when rebuiliding the panel, this
	 * causes a delay for the panel's showing up or a nullPointer error if there is
	 * no path to the server.
	 * 
	 * 
	 * Con getParamsPanel tenemos acceso a juego de pestaas de configuracin
	 * de la cobertura. Si este panel todava no existe (como puede ser cuando
	 * recuperamos un proyecto guardado) ste se crea automticamente conectando
	 * al servidor, recuperando los datos necesarios, rellenando el contenido y
	 * dejando seleccionados los valores que estaban seleccionados cuando se
	 * guard el proyecto.
	 * 
	 * Como para reconstruirse requiere una conexin con el servidor esto causa
	 * un retardo en la aparicin en el toc o un error de nullPointer si no
	 * hay conexin hasta el servidor.
	 * 
	 * @return WCSParamsPanel
	 */
	public WCSParamsPanel getParamsPanel(Hashtable info) {
		try {
			WCSWizardDataSource dataSource = new WCSDataSourceAdapter();
			WCSWizardData data = dataSource.detalles( new URL((String) info.get("host")) );
			
			WCSParamsPanel toc = new WCSParamsPanel();
			
			
			toc.setWizardData(data);
			toc.setDataSource(dataSource);
			
			toc.getLstCoverages().setListData(data.getCoverageNames());
			
			String coverageName = (String) info.get("name");
			int index = toc.getCoverageIndex( coverageName );
			if (index != -1)
				toc.getLstCoverages().setSelectedIndex(index);
			
			toc.actualizaWizard();
			
			index = toc.getSRSIndex((String) info.get("crs") );
			if (index != -1)
				toc.getLstCRSs().setSelectedIndex(index);
			index = toc.getFormatIndex((String) info.get("format"));
			if (index != -1)
				toc.getLstFormats().setSelectedIndex(index);
			String time = (String) info.get("time");
			if (!time.equals(""))
				toc.getLstSelectedTimes().setListData(time.split("=")[1].split(","));
			String parameter = (String) info.get("parameter");
			if (!parameter.equals("")){
				String[] s = parameter.split("=");
				String pName = s[0];
				
				String regexDouble = "-?[0-9]+(\\.[0-9]+)?";
				String regexInterval = regexDouble+"/"+regexDouble;
				String regexIntervalList = regexInterval+"(,"+regexInterval+")*";
				if (s[1].matches(regexInterval)){
					// Single Interval
					toc.getJScrollPane5().setVisible(false);
					toc.lblInterval.setVisible(true);
					toc.lblFrom.setVisible(true);
					toc.getTxtIntervalBegin().setVisible(true);
					toc.lblTo.setVisible(true);
					toc.getTxtIntervalEnd().setVisible(true);
					toc.getTxtInterval().setVisible(true);
					toc.lblSample.setVisible(true);
					toc.getRadioMultipleInterval().setVisible(true);
					toc.getRadioSingleInterval().setVisible(true);
					toc.getSingleParamValuesList().setEnabled(false);
					
					String[] pVals = s[1].split("/");
					String s1 = pVals[0];
					toc.setIntervalBegin(s1);
					s1 = pVals[1];
					
					toc.setIntervalEnd(s1);
					toc.getRadioSingleInterval().setSelected(true);
				
				} else if (s[1].matches(regexIntervalList)){
					// Multiple Interval
					toc.getJScrollPane5().setVisible(false);
					toc.lblInterval.setVisible(true);
					toc.lblFrom.setVisible(true);
					toc.getTxtIntervalBegin().setVisible(true);
					toc.lblTo.setVisible(true);
					toc.getTxtIntervalEnd().setVisible(true);
					toc.getTxtInterval().setVisible(true);
					toc.lblSample.setVisible(true);
					toc.getRadioMultipleInterval().setVisible(true);
					toc.getRadioSingleInterval().setVisible(true);
					toc.getSingleParamValuesList().setEnabled(false);
					
					toc.setMultipleInterval(s[1]);
					toc.getRadioMultipleInterval().setSelected(true);
					
				} else {
					// Single values
					toc.getJScrollPane5().setVisible(true);
					toc.lblInterval.setVisible(false);
					toc.lblFrom.setVisible(false);
					toc.getTxtIntervalBegin().setVisible(false);
					toc.lblTo.setVisible(false);
					toc.getTxtIntervalEnd().setVisible(false);
					toc.getTxtInterval().setVisible(false);
					toc.lblSample.setVisible(false);
					toc.getRadioMultipleInterval().setVisible(false);
					toc.getRadioSingleInterval().setVisible(false);
					toc.getSingleParamValuesList().setEnabled(true);
					
					String[] pVals = s[1].split(",");
					index = toc.getParamIndex(pName); 
					toc.getCmbParam().setSelectedIndex(index);
					toc.refreshParamValues( coverageName );
					int[] indexes = new int[pVals.length];
					for (int i = 0; i < pVals.length; i++) {
						indexes[i] = toc.getValueIndex(pVals[i]);
					}
					
					toc.getSingleParamValuesList().setSelectedIndices(indexes);
				}
			}	
			return toc;
		} catch (DriverException soe) {
			JOptionPane.showMessageDialog(null, "servidor_wcs_no_responde", "Error", JOptionPane.ERROR_MESSAGE);
		} catch (IOException ioe){
			JOptionPane.showMessageDialog(null, "error_comunicacion_servidor", "Error", JOptionPane.ERROR_MESSAGE);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	public JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			m_actionListener = new ComandosListener(this);
			buttonsPanel = new JPanel();
	        buttonsPanel.setBounds(10, 283, 472, 43);
			buttonsPanel.setLayout(null);     
			buttonsPanel.setName("buttonPanel");
			
	        buttonsPanel.add(getBtnOk(), null);
	        buttonsPanel.add(getBtnApply(), null);
	        buttonsPanel.add(getBtnCancel(), null);
		}
		return buttonsPanel;
	}
	
	public JButton getBtnOk() {
		if (btnOk == null) {
	        btnOk = new JButton("Aceptar");
	        btnOk.setText(PluginServices.getText(this,"Aceptar"));
	        btnOk.setActionCommand("OK");
	        btnOk.addActionListener(m_actionListener);
	        btnOk.setBounds(367, 9, 90, 25);
		}
		return btnOk;
	}
	
	public JButton getBtnApply() {
		if (btnApply == null) {
	        btnApply = new JButton("Aplicar");
	        btnApply.setText(PluginServices.getText(this,"Aplicar"));
	        btnApply.setEnabled(false);
	        btnApply.setActionCommand("APPLY");
	        btnApply.addActionListener(m_actionListener);
	        btnApply.setBounds(267, 9, 90, 25);
		}
		return btnApply;
	}
	
	public JButton getBtnCancel() {
		if (btnCancel == null) {
	        btnCancel = new JButton("Cancelar");
	        btnCancel.setText(PluginServices.getText(this,"Cancelar"));
	        btnCancel.setActionCommand("CANCEL");
	        btnCancel.addActionListener(m_actionListener);
	        btnCancel.setBounds(137, 9, 90, 25);
		}
		return btnCancel;
	}
	
    private class ComandosListener implements ActionListener {
        private WCSPropsDialog m_tp;

        /**
         * Creates a new ComandosListener object.
         *
         * @param lg DOCUMENT ME!
         */
        public ComandosListener(WCSPropsDialog tp) {
            m_tp = tp;
        }

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand() == "OK") {
            	//final WCSParamsPanel wcsParamsTabbedPane = getParamsPanel(((FLyrWCS) fLayer).getWCSProperties());
            	((FLyrWCS) fLayer).setCoverageName(wcsParamsTabbedPane.getNomCobertura());
            	((FLyrWCS) fLayer).setLabel(wcsParamsTabbedPane.getEtiquetaCobertura());
            	((FLyrWCS) fLayer).setSRS(wcsParamsTabbedPane.getSRS());
            	((FLyrWCS) fLayer).setFormat(wcsParamsTabbedPane.getFormat());
            	((FLyrWCS) fLayer).setFullExtent(wcsParamsTabbedPane.getLayersRectangle());
            	((FLyrWCS) fLayer).setTime(wcsParamsTabbedPane.getTiempos());
            	((FLyrWCS) fLayer).setParameter(wcsParamsTabbedPane.getParametro());
            	if (!applied) {
            		com.iver.cit.gvsig.gui.View vista = (com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveView();
            		MapControl mapCtrl = vista.getMapControl();
            		mapCtrl.getMapContext().invalidate();
            	}
            	closeWCSPropertiesDialog();
             } else if (e.getActionCommand() == "APPLY") {
             	//final WCSParamsPanel wcsParamsTabbedPane = getParamsPanel(((FLyrWCS) fLayer).getWCSProperties());
            	((FLyrWCS) fLayer).setCoverageName(wcsParamsTabbedPane.getNomCobertura());
            	((FLyrWCS) fLayer).setLabel(wcsParamsTabbedPane.getEtiquetaCobertura());
            	((FLyrWCS) fLayer).setSRS(wcsParamsTabbedPane.getSRS());
            	((FLyrWCS) fLayer).setFormat(wcsParamsTabbedPane.getFormat());
            	((FLyrWCS) fLayer).setFullExtent(wcsParamsTabbedPane.getLayersRectangle());
            	((FLyrWCS) fLayer).setTime(wcsParamsTabbedPane.getTiempos());
            	((FLyrWCS) fLayer).setParameter(wcsParamsTabbedPane.getParametro());
            	com.iver.cit.gvsig.gui.View vista = (com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveView();
        		MapControl mapCtrl = vista.getMapControl();
        		mapCtrl.getMapContext().invalidate();
        		applied = true;
        		getBtnApply().setEnabled(!applied);
             } else if (e.getActionCommand() == "CANCEL") {
            	closeWCSPropertiesDialog();
             }
		}
    }
    
	public ViewInfo getViewInfo() {
		if (m_ViewInfo==null){
			m_ViewInfo=new ViewInfo(ViewInfo.MODALDIALOG);
			m_ViewInfo.setTitle(PluginServices.getText(this,"Ajustar_cobertura_WCS"));
		}
		return m_ViewInfo;
	}
	
	public void viewActivated() {
	}

	public void openWCSPropertiesDialog() {
		if (PluginServices.getMainFrame() == null) {
			dlg = new JDialog();
			Rectangle bnds = getBounds();
			bnds.width += 10;
			bnds.height += 33;
			dlg.setBounds(bnds);
			dlg.setSize(getSize());
			dlg.getContentPane().add(this);
			dlg.setModal(true);			
			dlg.pack();
			dlg.show();
		} else
			PluginServices.getMDIManager().addView(this);
	}
	
	public void closeWCSPropertiesDialog() {
		if (PluginServices.getMainFrame() == null) {
			dlg.hide();
			dlg.dispose();
			dlg = null;
		} else
			PluginServices.getMDIManager().closeView(this);		
	}

}
