/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.gui.panels;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.WCSParameterModel;
import com.iver.cit.gvsig.gui.wcs.WCSWizardData;
import com.iver.cit.gvsig.gui.wcs.WCSWizardDataSource;

/**
 * This class implements the coverage option panel.
 * 
 * It includes a set of Listeners that implement some control rules which
 * refresh the component values that depends on those selected in the other
 * components to avoid to choose an invalid set of options. It also includes a
 * method (isCorrectlyConfigured()) that checks if the current set of values is enough to
 * correctly launch a GetCoverage request.
 * 
 * The information is obtained from a WCSWizardDataSource object.
 * 
 * 
 * Esta clase implementa el panel de opciones disponibles sobre la cobertura.
 * 
 * Incluye una serie de Listeners que implementan unas reglas de control que
 * refrescan los valores de componentes cuyos valores dependen de aqullos 
 * seleccionados en otros componentes para evitar que se escoja una combinacin
 * de opciones errnea as como una funcin (isCorrectlyConfigured()) que
 * comprueba si la combinacin escogida actualmente es suficiente para lanzar
 * una operacin GetCoverage correctamente.
 * 
 * La informacin obtiene a partir de un objeto WCSWizardDataSource.
 * 
 * @author jaume - jaume dominguez faus
 *
 */
public class WCSParamsPanel extends JPanel{
	private final int SINGLE_VALUE = 0;
	private final int INTERVAL = 1;
		
	private JTabbedPane jTabbedPane = null;  //  @jve:decl-index=0:visual-constraint="37,30"
	private JPanel panelFormato = null;
	private JPanel panelTemps = null;
	private JPanel panelParametres = null;
	private JList lstCRSs = null;
	private JList lstTemps = null;
	private JComboBox cmbParam = null;
	private JList lstParamValues = null;
	private JButton btnDelTemps = null;
	private JList lstSelectedTimes = null;
	private JList lstFormats = null;
	private JPanel jPanel = null;
	private JPanel jPanel1 = null;
	private JPanel jPanel2 = null;
	private JPanel jPanel3 = null;
	private JPanel panelCovertures = null;
	private JPanel jPanel4 = null;
	private JTextField txtNomCobertura = null;
	private JScrollPane jScrollPane = null;
	private JButton btnAddTemps = null;
	private JList lstCoverages = null;
	private JLabel jLabel = null;
	private JScrollPane jScrollPane2 = null;
	private JScrollPane jScrollPane3 = null;
	private JScrollPane jScrollPane4 = null;
	private JScrollPane jScrollPane5 = null;
	private JScrollPane jScrollPane1 = null;
	private WCSWizardData data;
	private WCSWizardDataSource dataSource;
	public JLabel lblInterval = null;
	public JLabel lblSample = null;
	private JTextField txtInterval = null;
	private JTextField txtIntervalBegin = null;
	private JTextField txtIntervalEnd = null;
	private JRadioButton radioSingleInterval = null;
	private JRadioButton radioMultipleInterval = null;
	private ButtonGroup group = new ButtonGroup();
	public JLabel lblFrom = null;
	public JLabel lblTo = null;
	private int parameterType;
    /**
	 * This method initializes jTabbedPane	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */    
	
	public WCSParamsPanel(){
		super();
		initialize();
	}
	/**
	 * Returns the tab amount that the WCSParamsPanel currently have
	 * 
	 * Devuelve el nmero de solapas que tiene actualmente el WCSParamsPanel
	 * 
	 * @return int
	 */
	public int getNumTabs(){
	 	return getJTabbedPane().getTabCount();
	}
	
	private void initialize(){
		this.setLayout(null);
		//this.setPreferredSize(new java.awt.Dimension(750, 320));
		this.setVisible(true);
		this.setSize(498, 274);
		this.add(getJTabbedPane(), null);
	}
	
	/**
	 * Sets the focus to the tab specified by the index.
	 * 
	 * Establece el foco en la solapa especificada en index
	 * 
	 * @param index
	 */
	private void setTabFocus(int index){
		this.getJTabbedPane().setSelectedIndex(index);
	}
	
	/**
	 * Sets the focus to the tab next to the current one.
	 * 
	 * Enfoca a la solapa siguiente a la actualmente enfocada del TabbedPane
	 *
	 */
	public void avanzaTab(){
	    int currentPage = currentPage();
	    int nPages = getNumTabs();
		if (nPages -1 > currentPage){
			getJTabbedPane().setSelectedIndex(nextPageEnabled());
		}
	}
	
	/**
	 * Sets the focus to the tab previous to the current one. 
	 * 
	 * Enfoca a la solapa anterior a la actualmente enfocada del TabbedPane
	 * 
	 */
	public void retrocedeTab(){
		this.getJTabbedPane().setSelectedIndex(previousEnabledPage());

	}
	
	/**
	 * Returns the current tab index.
	 * 
	 * Devuelve el nmero de solapa en que se encuentra
	 * 
	 * @return
	 */
	public int getIndiceSolapaActual(){
		return this.getJTabbedPane().getSelectedIndex();
	}
	
	public JTabbedPane getJTabbedPane() {
		if (jTabbedPane == null) {
			jTabbedPane = new JTabbedPane();
			jTabbedPane.setSize(492, 274);
			jTabbedPane.addTab(PluginServices.getText(this, "Coberturas"), null, getPanelCovertures(), null);
			jTabbedPane.addTab(PluginServices.getText(this, "Formato"), null, getPanelFormato(), null);
			jTabbedPane.addTab(PluginServices.getText(this, "Tiempo"), null, getPanelTemps(), null);
			jTabbedPane.addTab(PluginServices.getText(this, "Parametros"), null, getPanelParametres(), null);
		}
		return jTabbedPane;
	}
	/**
	 * This method initializes panelFormato	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPanelFormato() {
		if (panelFormato == null) {
			panelFormato = new JPanel();
			panelFormato.setLayout(null);
			panelFormato.add(getJPanel3(), null);
		}
		return panelFormato;
	}
	/**
	 * This method initializes panelTemps	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPanelTemps() {
		if (panelTemps == null) {
			panelTemps = new JPanel();
			panelTemps.setLayout(null);
			panelTemps.add(getJPanel(), null);
		}
		return panelTemps;
	}
	/**
	 * This method initializes panelParametres	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPanelParametres() {
		if (panelParametres == null) {
			panelParametres = new JPanel();
			panelParametres.setLayout(null);
			panelParametres.add(getJPanel1(), null);
		}
		return panelParametres;
	}
	/**
	 * This method initializes lstCRSs	
	 * 	
	 * @return javax.swing.JList	
	 */    
	public JList getLstCRSs() {
		if (lstCRSs == null) {
			lstCRSs = new JList();
			lstCRSs.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		}
		return lstCRSs;
	}
	/**
	 * This method initializes lstTemps	
	 * 	
	 * @return javax.swing.JList	
	 */    
	public JList getLstTemps() {
		if (lstTemps == null) {
			lstTemps = new JList();
		}
		return lstTemps;
	}
	/**
	 * This method initializes cmbParam	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	public JComboBox getCmbParam() {
		if (cmbParam == null) {
			cmbParam = new JComboBox();
			cmbParam.setBounds(23, 26, 188, 24);
		}
		return cmbParam;
	}
	/**
	 * This method initializes lstParamValues	
	 * 	
	 * @return javax.swing.JList	
	 */    
	public JList getSingleParamValuesList() {
		if (lstParamValues == null) {
			lstParamValues = new JList();
		}
		return lstParamValues;
	}
	


	/**
	 * This method initializes btnDelTemps	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBtnDelTemps() {
		if (btnDelTemps == null) {
			btnDelTemps = new JButton();
			btnDelTemps.setText("<");
			btnDelTemps.setBounds(190, 97, 41, 26);
		}
		return btnDelTemps;
	}
	/**
	 * This method initializes lstSelectedTimes	
	 * 	
	 * @return javax.swing.JList	
	 */    
	public JList getLstSelectedTimes() {
		if (lstSelectedTimes == null) {
			lstSelectedTimes = new JList();
		}
		return lstSelectedTimes;
	}
	/**
	 * This method initializes lstFormats	
	 * 	
	 * @return javax.swing.JList	
	 */    
	public JList getLstFormats() {
		if (lstFormats == null) {
			lstFormats = new JList();
			lstCRSs.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		}
		return lstFormats;
	}
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(null);
			jPanel.setBounds(5, 17, 477, 209);
			jPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "seleccionar_tiempo"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			jPanel.add(getBtnAddTemps(), null);
			jPanel.add(getBtnDelTemps(), null);
			jPanel.add(getJScrollPane3(), null);
			jPanel.add(getJScrollPane4(), null);
		}
		return jPanel;
	}
	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			lblTo = new JLabel();
			lblTo.setBounds(55, 123, 124, 23);
			lblTo.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
			lblTo.setText(PluginServices.getText(this, "hasta"));
			lblTo.setVisible(false);
			lblFrom = new JLabel();
			lblFrom.setBounds(54, 98, 125, 23);
			lblFrom.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
			lblFrom.setText(PluginServices.getText(this, "desde"));
			lblFrom.setVisible(false);
			lblSample = new JLabel();
			lblSample.setBounds(218, 172, 197, 23);
			lblSample.setText("("+PluginServices.getText(this, "muestra")+": 1.2/2.6,3/4.5)");
			lblSample.setVisible(false);
			lblInterval = new JLabel();
			lblInterval.setBounds(23, 52, 188, 23);
			lblInterval.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
			lblInterval.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
			lblInterval.setText(PluginServices.getText(this, "introduzca_intervalo_deseado"));
			lblInterval.setVisible(false);
			jPanel1 = new JPanel();
			jPanel1.setLayout(null);
			jPanel1.setBounds(5, 23, 447, 202);
			jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "seleccionar_parametros"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			jPanel1.add(getCmbParam(), null);
			jPanel1.add(getJScrollPane5(), null);
			jPanel1.add(lblInterval, null);
			jPanel1.add(lblSample, null);
			jPanel1.add(getTxtInterval(), null);
			jPanel1.add(getTxtIntervalBegin(), null);
			jPanel1.add(getTxtIntervalEnd(), null);
			jPanel1.add(getRadioSingleInterval(), null);
			jPanel1.add(getRadioMultipleInterval(), null);
			jPanel1.add(lblFrom, null);
			jPanel1.add(lblTo, null);
		}
		return jPanel1;
	}
	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new JPanel();
			jPanel2.setLayout(null);
			jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "seleccionar_CRS"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			jPanel2.setBounds(250, 33, 228, 204);
			jPanel2.add(getJScrollPane2(), null);
		}
		return jPanel2;
	}
	/**
	 * This method initializes jPanel3	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			jPanel3 = new JPanel();
			jPanel3.setLayout(null);
			jPanel3.setBounds(20, 22, 207, 203);
			jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "seleccionar_formato"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			jPanel3.add(getJScrollPane1(), null);
		}
		return jPanel3;
	}
	/**
	 * This method initializes panelCovertures	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPanelCovertures() {
		if (panelCovertures == null) {
			jLabel = new JLabel();
			
 			panelCovertures = new JPanel();
			panelCovertures.setLayout(null);
			jLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
			jLabel.setBounds(7, 5, 130, 22);
			jLabel.setText(PluginServices.getText(this, "nombre_cobertura"));
			panelCovertures.add(getJPanel4(), null);
			panelCovertures.add(getJPanel2(), null);
			panelCovertures.add(getTxtNomCobertura(), null);
			panelCovertures.add(jLabel, null);
		}
		return panelCovertures;
	}
	/**
	 * This method initializes jPanel4	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel4() {
		if (jPanel4 == null) {
			jPanel4 = new JPanel();
			jPanel4.setLayout(null);
			jPanel4.setBounds(7, 33, 228, 204);
			jPanel4.add(getJScrollPane(), null);
			jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "seleccionar_coberturas"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
		}
		return jPanel4;
	}
	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	public JTextField getTxtNomCobertura() {
		if (txtNomCobertura == null) {
 			txtNomCobertura = new JTextField();
			txtNomCobertura.setBounds(141, 6, 337, 21);
		}
		return txtNomCobertura;
	}
	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(6, 17, 213, 179);
			jScrollPane.setViewportView(getLstCoverages());
		}
		return jScrollPane;
	}
	/**
	 * This method initializes jButton2	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBtnAddTemps() {
		if (btnAddTemps == null) {
			btnAddTemps = new JButton();
			btnAddTemps.setBounds(239, 97, 41, 26);
			btnAddTemps.setText(">");
		}
		return btnAddTemps;
	}
	/**
	 * This method initializes jList1	
	 * 	
	 * @return javax.swing.JList	
	 */    
	public JList getLstCoverages() {
		if (lstCoverages == null) {
			lstCoverages = new JList();
			lstCoverages.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		}
		return lstCoverages;
	}
	/**
	 * This method initializes jScrollPane2	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane2() {
		if (jScrollPane2 == null) {
			jScrollPane2 = new JScrollPane();
			jScrollPane2.setBounds(6, 17, 213, 179);
			jScrollPane2.setViewportView(getLstCRSs());
		}
		return jScrollPane2;
	}

	public JLabel getCoverageLabel(){
		return jLabel;
	}
	/**
	 * This method initializes jScrollPane3	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane3() {
		if (jScrollPane3 == null) {
			jScrollPane3 = new JScrollPane();
			jScrollPane3.setBounds(6, 17, 170, 185);
			jScrollPane3.setViewportView(getLstTemps());
		}
		return jScrollPane3;
	}
	/**
	 * This method initializes jScrollPane4	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane4() {
		if (jScrollPane4 == null) {
			jScrollPane4 = new JScrollPane();
			jScrollPane4.setBounds(299, 17, 170, 185);
			jScrollPane4.setViewportView(getLstSelectedTimes());
		}
		return jScrollPane4;
	}
	/**
	 * This method initializes jScrollPane5	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	public JScrollPane getJScrollPane5() {
		if (jScrollPane5 == null) {
			jScrollPane5 = new JScrollPane();
			jScrollPane5.setBounds(226, 26, 193, 155);
			jScrollPane5.setViewportView(getSingleParamValuesList());
		}
		return jScrollPane5;
	}
	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setBounds(10, 32, 188, 160);
			jScrollPane1.setViewportView(getLstFormats());
		}
		return jScrollPane1;
	}
	

	/**
	 * Verifies that the selected parameters are enough to request the coverage
	 * to the server.
	 * 
	 * Comprueba que los parmetros seleccionados son suficientes para pedir la
	 * cobertura al servidor.
	 * 
	 * @return boolean
	 */
	public boolean isCorretlyConfigured() {
				
		if (getNomCobertura()==null) return false;
		/* 
		 * Segn el estndar WCS, si se especifica bounding box el parmetro TIME
		 * no es necesario. ste es el caso de gvSIG, que siempre especifica BBOX
		 * As que, en teora, no deberamos comprobar si se ha seleccionado algo
		 * o no. Pero en las pruebas, el servidor que usbamos responda con una
		 * cobertura en blanco (negro) as que se ha optado por evitar que se pueda
		 * consultar sin ninguna posicin para el tiempo (siempre que la cobertura
		 * especifique posiciones para el tiempo).
		 * 
		 * En cualquier caso con comentarizar la lnea basta.
		 */
		if (timeRequired() && getTiempos()==null) return false;
		if (getCmbParam().getSelectedItem()!=null){
			if (parameterType == SINGLE_VALUE && getParametro()==null) return false;
			if (parameterType == INTERVAL){
				if (radioSingleInterval.isSelected()){
					String beginText = getTxtIntervalBegin().getText();
					String endText = getTxtIntervalEnd().getText();
					try{
						Double.parseDouble(beginText);
						Double.parseDouble(endText);
					} catch (Exception e){
						return false;
					}
				} else {
					String regexDouble = "-?[0-9]+(\\.[0-9]+)?";
					String regexInterval = regexDouble+"/"+regexDouble;
					String regexIntervalList = regexInterval+"(,"+regexInterval+")*";
					
					if (!getTxtInterval().getText().matches(regexIntervalList))
						return false;
					
				}
			}
		}
		if (getSRS()==null) return false;
		if (getFormat()==null) return false;
		
		return true;
	}
	
	/**
	 * Returns the name and the selected values (if they are) of the parameter in
	 * a query string. Such string may directly be concatenated to the query string.
	 * 
	 * Devuelve el nombre y los valores seleccionados (si hay alguno) del parmetro
	 * en forma de querystring, de manera que puede ser concatenado a la query.
	 * 
	 * Si no se han seleccionado devuelve null.
	 * 
	 * @return String
	 */
	public String getParametro(){
		if (getCmbParam().getSelectedItem()!=null){
			String s = null;
			if (parameterType == SINGLE_VALUE){
				if (getSingleParamValuesList().getSelectedValues().length>0){
					Object[] o = getSingleParamValuesList().getSelectedValues();
					if (o.length > 0){
						s = (String) o[0];
						for (int i = 1; i < o.length; i++) {
							s += ","+(String) o[i];
						}			
					}
				}	
			}else if (parameterType == INTERVAL){
				if (radioSingleInterval.isSelected())
					s = getTxtIntervalBegin().getText()+"/"+txtIntervalEnd.getText();
				else{
					s = getTxtInterval().getText();
				}
			}
			if (s!=null)
				return (String) getCmbParam().getSelectedItem()+"="+s;		
		}
		return null;
	}
	
	/** 
	 * True if the TIME parameter is required, else false.
	 * 
	 * True si se requiere especificar una posicin para el tiempo.
	 * (jo crec que a no val per a res, per per si de cas la tinc)
	 * 
	 * @return
	 */
	private boolean timeRequired(){
		return getLstTemps().getModel().getSize() > 0;
	}
	
	/**
	 * Made for internal uses. Takes the selected values at the parameter page.
	 * 
	 * Usos internos. Recoge los valores seleccionados en los parmetros
	 * @param jList
	 * @return
	 */
	private Object[] recogeParametros(JList jList){
		ArrayList o = new ArrayList();
		for (int i = 0; i < jList.getModel().getSize(); i++) {
			o.add(jList.getModel().getElementAt(i));
		}
		
		return o.toArray();
	}
	
	/**
	 * Returns the time positions in a string that may directly be concatenated
	 * to the query string.
	 * 
	 * Devuelve las posiciones para el tiempo en forma de querystring, 
	 * de manera que puede ser concatenado a la query.
	 * 
	 * @return String
	 */
	public String getTiempos(){
		Object[] o = recogeParametros(getLstSelectedTimes());
		String s = null;
		if (o.length > 0){
			s = (String) o[0];
			for (int i = 1; i < o.length; i++) {
				s += ","+ (String) o[i];
			}
		}
		if (s!= null) return "TIME="+s;
		return s;
	}
	
	/**
	 * Returns the selected CRS.
	 * 
	 * Devuelve el CRS seleccionado.
	 * 
	 * @return String
	 */
	public String getSRS() {
		return (String) getLstCRSs().getSelectedValue();
	}
	
	/**
	 * Returns the selected format.
	 * 
	 * Devuelve el formato seleccionado.
	 * 
	 * @return String
	 */
	public String getFormat() {
		return (String) getLstFormats().getSelectedValue();
	}
	
	/**
	 * Returns the name of the selected coverage.
	 * 
	 * Devuelve el nombre de la cobertura seleccionada.
	 * 
	 * @return String
	 */
	public String getLayerName() {
		return getTxtNomCobertura().getText();
	}

	/**
	 * Cleans up the wizard's components but the server's layers list.
	 * 
	 * Limpia todos los componentes del wizard excepto la lista de capas del
	 * servidor.
	 */
	public void limpiaWizard() {
		Object[] nada = new Object[0];
		
		getLstCRSs().clearSelection();
		getLstCRSs().setListData(nada);

		getLstFormats().clearSelection();
		getLstFormats().setListData(nada);

		getLstTemps().clearSelection();
		getLstTemps().setListData(nada);

		getLstSelectedTimes().clearSelection();
		getLstSelectedTimes().setListData(nada);

		getCmbParam().removeAllItems();
		getSingleParamValuesList().setVisible(true);

		getSingleParamValuesList().clearSelection();
		getSingleParamValuesList().setListData(nada);
		getTxtInterval().setText("");
		getTxtIntervalBegin().setText("");
		getTxtIntervalEnd().setText("");
		getTxtInterval().setVisible(false);
		getTxtIntervalBegin().setVisible(false);
		getTxtIntervalEnd().setVisible(false);
	}

	/**
	 * Returns the selected coverage name.
	 * 
	 * Devuelve el nombre de la cobertura seleccionada
	 * @return String
	 */
	public String getNomCobertura(){
		return (String) getLstCoverages().getSelectedValue();	
	}

	/**
	 * Refreshes the wizard components data each time a coverage is selected.
	 * 
	 * Actualiza los datos de los componentes del wizard cada vez que se selecciona
	 * una cobertura diferente.
	 */
	public void actualizaWizard(){
		String cobertura = getNomCobertura();
		limpiaWizard();
		if (cobertura != null){
			getTxtNomCobertura().setText(data.getCoverageLabel(cobertura));
			
			getLstCRSs().clearSelection();
			getLstCRSs().setListData(data.getCoverageSRSs(cobertura).toArray());

			getLstFormats().clearSelection();
			getLstFormats().setListData(data.getCoverageFormatos(cobertura).toArray());

			getLstTemps().clearSelection();
			Object[] lista = data.getCoverageTimes(cobertura).toArray();
			
			// Activar/Desactivar pestaa tiempos
			if (lista.length!=0){
			    jTabbedPane.setEnabledAt(2, true);
				getLstTemps().setListData(lista);
				jTabbedPane.setEnabledAt(2, lista.length!=0);
				getBtnAddTemps().setEnabled(true);
				getBtnDelTemps().setEnabled(true);
			}
			else{
			    jTabbedPane.setEnabledAt(2, false);
				getLstTemps().setEnabled(false);
				getLstSelectedTimes().setEnabled(false);
				getBtnAddTemps().setEnabled(false);
				getBtnDelTemps().setEnabled(false);
			}
				
			// ComboBox Parmetros
			// la lista de valores se actualiza con un listener
			ArrayList listaParametros = data.getCoverageInfo(cobertura).getParametros();
			jTabbedPane.setEnabledAt(3, !listaParametros.isEmpty());
			if (listaParametros.isEmpty()){
				getCmbParam().setEnabled(false);
			}
			else{
				getCmbParam().setEnabled(true);
				Iterator it = listaParametros.iterator();
				while (it.hasNext()) {
					WCSParameterModel p = (WCSParameterModel) it.next();
					getCmbParam().addItem(p.getName());
				}
			}
		}		
	}

	/** 
	 * Refreshes the parameters name at the cmbParam.
	 * 
	 * Actualiza los nombres de parmetros en el cmbParam.
	 * 
	 * @param coverage
	 */
	public void actualizarParametros(String coverage){
		ArrayList listaParametros = data.getCoverageInfo((String) coverage).getParametros();
		Iterator it = listaParametros.iterator();
		while (it.hasNext()) {
			WCSParameterModel p = (WCSParameterModel) it.next();
			getCmbParam().addItem(p.getName());
		}
	}

	/**
	 * Sets the object that holds the wizard data.
	 * 
	 * Establece el objeto que contiene los datos del wizard.
	 * 
	 * @param data
	 */
	public void setWizardData(WCSWizardData data) {
		this.data = data;
		
	}
	
	/**
	 * Adds the selected items from the time list to the selected times list.
	 * 
	 * Aade los items seleccionados de la lista de tiempos a la 
	 * lista de tiempos seleccionados.
	 */
	public void addTiempo() {
		ArrayList tiempos = new ArrayList();
		for (int i = 0; i < getLstSelectedTimes()
				.getModel().getSize(); i++) {
			tiempos.add(getLstSelectedTimes().getModel()
					.getElementAt(i));
		}

		Object[] obj = getLstTemps().getSelectedValues();
		for (int i = 0; i < obj.length; i++) {
			if (!tiempos.contains(obj[i]))
				tiempos.add(obj[i]);
		}
		getLstSelectedTimes()
				.setListData(tiempos.toArray());
	}
	
	/**
	 * Refreshes the parameter values. This method is called by a listener when
	 * the state of the cmbParam changes. It is also called from FlyrWCS when the
	 * data is reloaded from a saved project.
	 * 
	 * Actualiza la lista de valores para el parmetro. ste mtodo es lanzado
	 * por un listener cuando el estado del cmbParam (combo de los nombres de parmetro)
	 * cambia. Tambin es lanzado desde FLyrWCS cuando recuperamos los datos en un
	 * proyecto guardado.
	 * 
	 * @param coverage
	 */
	public void refreshParamValues(String coverage){
		if (getCmbParam().getItemCount()==0){
			getSingleParamValuesList().setEnabled(false);
		}
		if (coverage!=null && ( getCmbParam().getSelectedItem() != null) ){
			String nombreParametro = (String) getCmbParam().getSelectedItem();
			WCSParameterModel p = data.getCoverageInfo(coverage).getParametro(nombreParametro);
			if (p.getType().equals("singleValue")){
				parameterType = SINGLE_VALUE;
				ArrayList valores = data.getCoverageInfo(coverage).getParametro(nombreParametro).getLista_valores_simples();
				getJScrollPane5().setVisible(true);
				lblInterval.setVisible(false);
				lblFrom.setVisible(false);
				getTxtIntervalBegin().setVisible(false);
				lblTo.setVisible(false);
				getTxtIntervalEnd().setVisible(false);
				getTxtInterval().setVisible(false);
				lblSample.setVisible(false);
				getRadioMultipleInterval().setVisible(false);
				getRadioSingleInterval().setVisible(false);
				getSingleParamValuesList().setEnabled(true);
				getSingleParamValuesList().setListData(valores.toArray());
			} else if (p.getType().equals("interval")){
				parameterType = INTERVAL;
				group.setSelected(getRadioSingleInterval().getModel(), true);
				double min = p.getMin();
				double max = p.getMax();
				String s = min+"";
				if (s.indexOf('.')>1 && (s.length()-s.indexOf('.')>3))
					s = s.substring(0, s.indexOf('.')+3);
				lblFrom.setText(PluginServices.getText(this, "desde")+" (min: "+s+"):");
				getTxtIntervalBegin().setText(min+"");
				s = max+"";
				if (s.indexOf('.')>1 && (s.length()-s.indexOf('.')>3))
					s = s.substring(0, s.indexOf('.')+3);
				lblTo.setText(PluginServices.getText(this, "hasta")+" (max: "+s+"):");
				getTxtIntervalEnd().setText(max+"");
				getJScrollPane5().setVisible(false);
				lblInterval.setVisible(true);
				lblFrom.setVisible(true);
				getTxtIntervalBegin().setVisible(true);
				lblTo.setVisible(true);
				getTxtIntervalEnd().setVisible(true);
				getTxtInterval().setVisible(true);
				lblSample.setVisible(true);
				getRadioMultipleInterval().setVisible(true);
				getRadioSingleInterval().setVisible(true);
				getSingleParamValuesList().setEnabled(false);
				
			}
		}
	}

	/**
	 * Removes the selected items from the selected times list.
	 * 
	 * Quita posiciones de tiempo de la lista de tiempos seleccionados.
	 */
	public void delTiempo() {
		ArrayList tiempos = new ArrayList();
		Object[] obj = getLstSelectedTimes()
				.getSelectedValues();
		for (int i = 0; i < getLstSelectedTimes()
				.getModel().getSize(); i++)
			tiempos.add(getLstSelectedTimes().getModel()
					.getElementAt(i));

		for (int j = 0; j < obj.length; j++)
			if (tiempos.contains(obj[j]))
				tiempos.remove(obj[j]);
		obj = new Object[tiempos.size()];
		for (int i = 0; i < obj.length; i++) {
			obj[i] = tiempos.get(i);
		}
		getLstSelectedTimes()
				.setListData(tiempos.toArray());
	}
	
	/**
	 * The coverage label.
	 * 
	 * La etiqueta de la cobertura
	 * @return String
	 */
	public String getEtiquetaCobertura() {
		return getTxtNomCobertura().getText();
	}
	
	/**
	 * Returns the extent of the currently selected coverage for the currently
	 * selected SRS.
	 * 
	 * Devuelve el Extent de la cobertura actualmente seleccionada para el SRS
	 * actualmente seleccionado
	 * 
	 * @return Rectangle2D
	 */
	public Rectangle2D getLayersRectangle() {
		return dataSource.getBoundingBox(getNomCobertura(), getSRS());
	}
	
	/**
	 * Sets the wizard's data source
	 * 
	 * Establece el origen de los datos del wizard
	 * @param dataSource
	 */
	public void setDataSource(WCSWizardDataSource dataSource) {
		this.dataSource = dataSource;
		
	}
	/**
	 * Returns the index of the coverage within the coverages list
	 * 
	 * Devuelve la posicin que ocupa la cobertura en la lista de coberturas
	 * 
	 * @param coverageName
	 * @return The coverage's index if it exists, -1 if it not exists.
	 */
	public int getCoverageIndex(String coverageName) {
		for (int i=0; i<getLstCoverages().getModel().getSize(); i++){
			if (coverageName.equals(getLstCoverages().getModel().getElementAt(i)))
				return i;
		}
		return -1;
	}
	
	/**
	 * Returns the index of the CRS within the CRS list.
	 * 
	 * Devuelve la posicion que ocupa el CRS en la lista de CRS
	 * 
	 * @param crs
	 * @return The CRS's index if it exists, -1 if it not exists.
	 */
	public int getSRSIndex(String crs) {
		for (int i=0; i<getLstCRSs().getModel().getSize(); i++){
			if (crs.equals(getLstCRSs().getModel().getElementAt(i)))
				return i;
		}
		return -1;
	}
	
	/**
	 * Returns the index of the format within the formats list.
	 * 
	 * Devuelve la posicion que ocupa el formato en la lista de formatos
	 * @param format
	 * 
	 * @return The format's index if it exists, -1 if it not exists.
	 */
	public int getFormatIndex(String format) {
		for (int i=0; i<getLstFormats().getModel().getSize(); i++){
			if (format.equals(getLstFormats().getModel().getElementAt(i)))
				return i;
		}
		return -1;
	}
	
	/**
	 * Returns the parameter's name's index within the cmbBox.
	 * 
	 * Devuelve la posicion que ocupa el nombre dle parmetro en el comboBox
	 * de parmetros
	 * 
	 * @param nParam
	 * @return The index if it exists, -1 if it not exists.
	 */
	public int getParamIndex(String nParam) {
		for (int i=0; i<getCmbParam().getModel().getSize(); i++){
			if (nParam.equals(getCmbParam().getItemAt(i)))
				return i;
		}
		return -1;
	}
	
	/**
	 * Returns the position for the parameter's value within the parameter
	 * values list.
	 * 
	 * Devuelve la posicion que ocupa el valor para parametro
	 * en la lista de valores para parmetro
	 * 
	 * @param valor
	 * @return The index if it exists, -1 if it not exists.
	 */
	public int getValueIndex(String valor) {
		for (int i=0; i<getSingleParamValuesList().getModel().getSize(); i++){
			if (valor.equals(getSingleParamValuesList().getModel().getElementAt(i)))
				return i;
		}
		return -1;
	}
	
	/**
	 * Returns the max supported resolution for the coverage.
	 *  
	 * Recupera la resolucin mxima soportada por la cobertura.
	 * @return double
	 */
	public Point2D getMaxRes(){
		return data.getCoverageInfo(getNomCobertura()).getMaxRes();
	}
    /**
     * Returns the next enabled tab's index.
     * 
     * Devuelve el ndicie de la siguiente pestaa habilitada del wizard o -1 si no hay
     * ninguna.
     * 
     * @return The index or -1 if there is no one.
     */
    public int nextPageEnabled() {
        int currentPage = currentPage();
        int nPages = getNumTabs();
        if (currentPage == nPages)
    	    return -1;
        for (int i=currentPage+1; i<nPages; i++){
            if (getJTabbedPane().isEnabledAt(i)){
                return i;
            }
        }
        return -1;
    }
    
    /**
     * Returns the index of the current tab.
     * 
     * Devuelve el ndice de la pgina actual del wizard.
     * 
     * @return
     */
    public int currentPage(){
        //System.out.println("Current page: "+((int)getJTabbedPane().getSelectedIndex()+1));
        return getJTabbedPane().getSelectedIndex();
    }
    
    /**
     * Returns the index of the previous enabled tab.
     * 
     * Devuelve el ndice de la anterior pgina habilitada del wizard o -1 si no hay
     * ninguna.
     * 
     * @return The index, or -1 if there is no one.
     */
    public int previousEnabledPage(){
        int currentPage = currentPage();
        int j=0;
        if (currentPage == 0)
            j= -1;
        for (int i = currentPage-1; i>-1; i--){
            if (getJTabbedPane().isEnabledAt(i)){
                j= i;
            	break;
            }
        }
        //System.out.println("Previous enabled page: "+j );
        return j;
    }
	/**
	 * This method initializes txtInterval	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	public JTextField getTxtInterval() {
		if (txtInterval == null) {
			txtInterval = new JTextField();
			txtInterval.setBounds(61, 171, 149, 23);
			txtInterval.setVisible(false);
		}
		return txtInterval;
	}
	/**
	 * This method initializes txtIntervalBegin	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	public JTextField getTxtIntervalBegin() {
		if (txtIntervalBegin == null) {
			txtIntervalBegin = new JTextField();
			txtIntervalBegin.setBounds(188, 98, 111, 23);
			txtIntervalBegin.setVisible(false);
		}
		return txtIntervalBegin;
	}
	/**
	 * This method initializes txtIntervalEnd
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	public JTextField getTxtIntervalEnd() {
		if (txtIntervalEnd == null) {
			txtIntervalEnd = new JTextField();
			txtIntervalEnd.setBounds(188, 123, 111, 23);
			txtIntervalEnd.setVisible(false);
		}
		return txtIntervalEnd;
	}
	
	/**
	 * This method initializes radioSingleInterval
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	public JRadioButton getRadioSingleInterval() {
		if (radioSingleInterval == null) {
			radioSingleInterval = new JRadioButton();
			radioSingleInterval.setBounds(22, 75, 190, 21);
			radioSingleInterval.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					boolean enabled = radioSingleInterval.isEnabled();
					lblTo.setEnabled(enabled);
					lblFrom.setEnabled(enabled);
					getTxtIntervalBegin().setEnabled(enabled);
					getTxtIntervalEnd().setEnabled(enabled);
					getTxtInterval().setEnabled(!enabled);
					lblSample.setEnabled(!enabled);
				}
			});
			radioSingleInterval.setText(PluginServices.getText(this, "intervalo"));
			radioSingleInterval.setVisible(false);
			group.add(radioSingleInterval);
		}
		return radioSingleInterval;
	}
	/**
	 * This method initializes radioMultipleInterval	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	public JRadioButton getRadioMultipleInterval() {
		if (radioMultipleInterval == null) {
			radioMultipleInterval = new JRadioButton();
			radioMultipleInterval.setBounds(22, 148, 191, 21);
			radioMultipleInterval.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					boolean enabled = radioMultipleInterval.isEnabled();
					lblTo.setEnabled(!enabled);
					lblFrom.setEnabled(!enabled);
					getTxtIntervalBegin().setEnabled(!enabled);
					getTxtIntervalEnd().setEnabled(!enabled);
					getTxtInterval().setEnabled(enabled);
					lblSample.setEnabled(enabled);
				}
			});
			radioMultipleInterval.setText(PluginServices.getText(this, "multiples_intervalos"));
			radioMultipleInterval.setVisible(false);
			group.add(radioMultipleInterval);
		}
		return radioMultipleInterval;
	}
	
	public void setMultipleInterval(String text) {
		getTxtInterval().setText(text);		
	}
	
	public void setIntervalBegin(String text) {
		getTxtIntervalBegin().setText(text);
	}
	
	public void setIntervalEnd(String text) {
		getTxtIntervalEnd().setText(text);	
	}
}