/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.wcs;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Iterator;

import com.iver.cit.gvsig.fmap.layers.WCSParameterModel;

//import com.iver.cit.gvsig.gui.wms.LayerInfo;


/**
 * Class that holds the coverage info.
 * 
 * Most of the content in this file is here for compatibility with LayerInfo and
 * it is not necessary.
 * 
 * Clase que mantiene la informacin de la cobertura
 * 
 * La mayora de su contenido es por compatibilidad con LayerInfo y no se
 * necesita.
 * 
 * @author jaume - jaume.dominguez@iver.es
 *  
 */
public class CoverageInfo {//extends LayerInfo {
    public String text;
    public String name;
    
    public String metadataLink;
    public String label;
    public String[] keywords;
    private ArrayList formats;
    private ArrayList srs = new ArrayList();
    private ArrayList tiempos;
    private ArrayList parametros = new ArrayList();
    //private double maxRes;
    private Point2D maxRes;
    private boolean hayParametros = false;
    
    /**
     * Adds an SRS to the SRSs list
     *
     * @param srs 
     */
    public void addSRS(String srs) {
    	String[] srsArray = srs.split(" ");
    	for (int i = 0; i < srsArray.length; i++){
			this.srs.add(srsArray[i]);
    	}
    }
    
    public void setSRSList(ArrayList srs){
    	this.srs = srs;
    }

    /**
     * Returns an ArrayList containing the strings with the SRSs
     *
     * @return ArrayList
     */
    public ArrayList getSRSs() {
    	return srs;
    }


    /**
     * Compares two CoverageInfo objects
     * Compara dos objetos CoverageInfo
     */
    public boolean equals(Object obj) {
    	try{
	        CoverageInfo objeto = (CoverageInfo) obj;
			return this.name.equals(objeto.name);
		}catch(ClassCastException e){
			e.printStackTrace();
			return false;
		}catch (NullPointerException e) {
			return false;
		}
    }

	/**
	 * Returns an ArrayList containing strings with the coverage supported formats
	 * 
	 * Devuelve la lista de formatos de la cobertura
	 * @return ArrayList
	 */
	public ArrayList getFormats() {
		return formats;
	}
	
	/**
	 * Sets the coverage formats list.
	 * 
	 * Establece la lista de formatos para la cobertura
	 * @param formats
	 */
	public void setFormats(ArrayList formats){
		this.formats = formats;	
	}

	/**
	 * Returns an ArrayList containing strings with the coverage time positions.
	 * 
	 * Devuelve la lista de posiciones para el tiempo de la cobertura
	 * 
	 * @return ArrayList
	 */
	public ArrayList getTimes() {
		return tiempos;
	}
	
	/**
	 * Sets the coverage name
	 * 
	 * Establece el nombre de la cobertura
	 * @param name
	 */
	public void setName(String name){
		this.name = name;
	}
	
	/**
	 * Sets the coverage label
	 * 
	 * Establece la etiqueta de la cobertura
	 * @param label
	 */
	public void setLabel(String label){
		this.label = label;
	}
	/**
	 * Sets the coverage time positions list
	 * 
	 * Establece la lista de posiciones para el tiempo de la cobertura
	 * 
	 * @return 
	 */
	public void setTiempos(ArrayList tiempos){
	    // Since the coverage may have not time positions we ensure
	    // that we won't have nullPointer errors;
		if (tiempos == null) this.tiempos = new ArrayList();
		else this.tiempos = tiempos;
	}

	/**
	 * Sets the coverage's parameter list (and its possible values)
	 * Establece la lista de parametros (y sus posibles valores) de la cobertura
	 * @param parametros
	 */
	public void setParametros(ArrayList parametros){
		// Como la cobertura puede no tener parmetros nos aseguramos de que
		// no tengamos errores de puntero nulo creando un ArrayList vaco
		if (parametros == null) {
			hayParametros = false;
			this.parametros = new ArrayList();
		}
		else{
			hayParametros = true;
			this.parametros = parametros;
		}
	}
	/**
	 * Returns an ArrayList containing a list of Parametro with the parameters list
	 * and its possible values.
	 * 
	 * Devuelve la lista de parmetros (y sus posibles valores) de la cobertura
	 * @return
	 */
	public ArrayList getParametros() {
		return parametros;
	}
	
	/**
	 * Searches and returns the parameter indicated by its name
	 * 
	 * Busca y devuelve el parmetro indicado por nombre.
	 * @param nombreParametro
	 * @return Parametro
	 */
	public WCSParameterModel getParametro(String nombreParametro){
		WCSParameterModel p;
		Iterator it = parametros.iterator();
		while (it.hasNext()){
			p = (WCSParameterModel) it.next();
			if (p.getName().equals(nombreParametro)) 
				return p;
		}
		return null;
		
	}

	/**
	 * @return true if the coverage has parameter, false else.
	 */
	public boolean hasParameter() {
		return hayParametros;
	}

	/**
	 * Gets the coverage label.
	 * 
	 * Obtiene la etiqueta de la cobertura
	 * @return
	 */
	public String getLabel() {
		return this.label;
	}

	/**
	 * Sets the coverage's max resolution.
	 * 
	 * Establece la resolucin mxima de la cobertura
	 * @param maxRes
	 */
	public void setMaxRes(Point2D maxRes) {
		this.maxRes = maxRes;
	}

	/**
	 * Gets the coverage's max resolution.
	 * 
	 * Obtiene la resolucin mxima de la cobertura
	 */
	public Point2D getMaxRes() {
		return maxRes;
	}

	/**
	 * Gets the coverage description (not used in gvSIG)
	 * 
	 * Obtiene la descripcin de la cobertura
	 * (no se usa en gvSIG)
	 * @param coverageDescription
	 */
	public void setText(String coverageDescription) {
		text = coverageDescription;
	}	
}
