/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.wcs;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JOptionPane;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.drivers.wcs.FMapWCSDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWCS;
import com.iver.cit.gvsig.fmap.layers.FMapWCSAdapter;
import com.iver.cit.gvsig.gui.WCSDataSourceAdapter;
import com.iver.cit.gvsig.gui.WizardPanel;
import com.iver.cit.gvsig.gui.panels.WCSParamsPanel;
import com.iver.cit.gvsig.gui.wizards.WizardListener;
import com.iver.cit.gvsig.gui.wizards.WizardListenerSupport;
import com.iver.utiles.NotExistInXMLEntity;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.swing.JComboBox;
 
/**
 * Class implementing the WCSWizard.
 * 
 * Clase que implementa el Wizard para coberturas WCS
 * @author Jaume Domnguez Faus
 */
public class WCSWizard extends WizardPanel {
	private int page = 0;
	private boolean conectado = false;
	private JComboBox cmbHost = null;
	private javax.swing.JButton btnDetalles = null;
	private javax.swing.JPanel jPanel = null;
	private javax.swing.JLabel jLabel1 = null;
	private javax.swing.JLabel lblTitle = null;
	private javax.swing.JScrollPane jScrollPane = null;
	private javax.swing.JTextArea txtAbstract = null;
	private javax.swing.JPanel panelPage1 = null;
	private javax.swing.JPanel panelPage2 = null;
	private javax.swing.JButton btnSiguiente = null;
	private javax.swing.JButton btnAnterior = null;
	private javax.swing.JPanel jPanel1 = null;
	private WizardListenerSupport listenerSupport = new WizardListenerSupport();
	private WCSParamsPanel wcsParamsTabbedPanel = null;
	private WCSWizardDataSource dataSource;
	private WCSWizardData data = null;
	
	/**
	 * This is the default constructor
	 */
	public WCSWizard() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 */
	private void initialize() {
		setTabName("WCS");
		this.setSize(510, 311);
		this.setLayout(null);
		this.setPreferredSize(new java.awt.Dimension(750, 320));
		this.setVisible(true);
		this.add(getPanelPage1(), null);
		this.add(getPanelPage2(), null);
		this.add(getBtnAnterior(), null);
		this.add(getBtnSiguiente(), null);
		activarVisualizarBotones();
	}

	/**
	 * Returns the selected coverage name
	 * 
	 * Devuelve el nombre de la cobertura seleccionada
	 * @return String
	 */
	private String getNomCobertura(){
		return (String) wcsParamsTabbedPanel.getNomCobertura();	
	}
	
	/**
	 * A pure JavaScript alert dialog.
	 * 
	 * Un dilogo de alert al ms puro estilo estilo JavaScript
	 * @param text
	 */
	private void alert(String text) {
		JOptionPane.showMessageDialog(null, text, "Error", JOptionPane.ERROR_MESSAGE);	
	}

	/**
	 * Adds the host addres to a persistent data storage.
	 * 
	 * con addHost guardamos la informacin del combo de servidores en un
	 * almacn persistente
	 * 
	 * @param host
	 */
	private void addHost(String host) {
		host = host.trim();

		DefaultComboBoxModel model = (DefaultComboBoxModel) getTxtHost()
				.getModel();

		if (model.getIndexOf(host) < 0) {
			model.addElement(host);

			PluginServices ps = PluginServices.getPluginServices(this);
			XMLEntity xml = ps.getPersistentXML();

			try {
				String[] servers = xml.getStringArrayProperty("wcs-servers");
				String[] newServers = new String[servers.length + 1];
				System.arraycopy(servers, 0, newServers, 0, servers.length);
				newServers[servers.length] = host;
				xml.putProperty("wcs-servers", newServers);
			} catch (NotExistInXMLEntity e) {
				xml.putProperty("wcs-servers", new String[] { host });
			}

			ps.setPersistentXML(xml);
		}
	}

	/**
	 * Fills up the initial WCSWizard controls.
	 * 
	 * Rellena los primeros controles del WCS Wizard
	 * 
	 * jaume
	 */
	private void rellenarControles() {
		try {
			String host = cmbHost.getModel().getSelectedItem().toString();
			
			data = dataSource.detalles(new URL(host));
			
			addHost(host);
			getLblTitle().setText(data.getTitle());
			getTxtAbstract().setText(data.getDescription());
			wcsParamsTabbedPanel.getLstCoverages().setListData(
					data.getCoverageNames());
			wcsParamsTabbedPanel.setWizardData(data);
			wcsParamsTabbedPanel.setDataSource(dataSource);
			conectado = true;
			activarVisualizarBotones();
		} catch (DriverException soe) {
			alert(PluginServices.getText(this, "servidor_wcs_no_responde"));
		} catch (IOException ioe){
			alert(PluginServices.getText(this, "error_comunicacion_servidor"));
		} catch (Exception e) {
			listenerSupport.callError(e);
		}
	}

	/**
	 * Enables or disables the Next and Previous buttons according with the current
	 * page and the enabled pages.
	 */
	private void activarVisualizarBotones() {
		if (page == 0) {
			getBtnAnterior().setEnabled(false);
			getBtnSiguiente().setVisible(true);
			if (conectado) {
				getBtnSiguiente().setEnabled(true);
			} else {
				getBtnSiguiente().setEnabled(false);
			}
		} else if (page < wcsParamsTabbedPanel.getNumTabs()) {
			getBtnSiguiente().setEnabled(true);
			getBtnSiguiente().setVisible(true);
			getBtnAnterior().setEnabled(true);
			getBtnAnterior().setVisible(true);
			if (wcsParamsTabbedPanel.nextPageEnabled()==-1){
			    getBtnSiguiente().setEnabled(false);
			}
			listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
		} else if (page == wcsParamsTabbedPanel.getNumTabs()) {
			getBtnSiguiente().setVisible(false);
			getBtnSiguiente().setEnabled(false);
			listenerSupport.callStateChanged(wcsParamsTabbedPanel 
					.isCorretlyConfigured());
		}

	}
	


	/**
	 * This method initializes txtHost from the persistent data storage
	 * 
	 * @return javax.swing.JTextField
	 */
	private javax.swing.JComboBox getTxtHost() {
		if (cmbHost == null) {
			cmbHost = new JComboBox();
			cmbHost.setModel(new DefaultComboBoxModel());
			cmbHost.setPreferredSize(new java.awt.Dimension(350, 20));
			cmbHost.setEditable(true);

			XMLEntity xml = PluginServices.getPluginServices(this)
					.getPersistentXML();
			if (xml == null)
				xml = new XMLEntity();

			if (!xml.contains("wcs-servers")) {
				String[] servers = new String[1];
				servers[0] = "http://hypnos.cbs.umn.edu/cgi-bin/mapserv?map=/data/wcs/demo.map";
				//servers[1] = "http://172.19.21.36/cgi-bin/wcs";

				xml.putProperty("wcs-servers", servers);
			}

			try {
				String[] servers = xml.getStringArrayProperty("wcs-servers");

				for (int i = 0; i < servers.length; i++) {
					((DefaultComboBoxModel) cmbHost.getModel())
							.addElement(servers[i]);
				}
			} catch (NotExistInXMLEntity e) {
			}
		}

		return cmbHost;
	}

	
	/**
	 * This method initializes btnDetalles
	 * 
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnDetalles() {
		if (btnDetalles == null) {
			btnDetalles = new javax.swing.JButton();
			btnDetalles.setPreferredSize(new java.awt.Dimension(100, 20));
			btnDetalles.setText(PluginServices.getText(this, "detalles"));
			btnDetalles.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					wcsParamsTabbedPanel.limpiaWizard();
					conectado = false;
					rellenarControles();
				}
			});
		}

		return btnDetalles;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new javax.swing.JPanel();
			jPanel.setLayout(null);
			jPanel.setPreferredSize(new java.awt.Dimension(470, 130));
			jPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
					PluginServices.getText(this, "descripcion"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null,
					null));
			jPanel.setBounds(2, 67, 477, 197);
			jPanel.add(getJLabel1(), null);
			jPanel.add(getLblTitle(), null);
			jPanel.add(getJScrollPane(), null);
		}

		return jPanel;
	}

	/**
	 * This method initializes jLabel1
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel1() {
		if (jLabel1 == null) {
			jLabel1 = new javax.swing.JLabel();
			jLabel1.setText(PluginServices.getText(this, "nombre") + ":");
			jLabel1.setBounds(37, 26, 40, 15);
		}

		return jLabel1;
	}

	/**
	 * This method initializes lblTitle
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getLblTitle() {
		if (lblTitle == null) {
			lblTitle = new javax.swing.JLabel();
			lblTitle.setText("-");
			lblTitle.setPreferredSize(new java.awt.Dimension(350, 16));
			lblTitle.setBounds(82, 26, 350, 16);
		}

		return lblTitle;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private javax.swing.JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new javax.swing.JScrollPane();
			jScrollPane.setViewportView(getTxtAbstract());
			jScrollPane.setPreferredSize(new java.awt.Dimension(450, 60));
			jScrollPane.setBounds(10, 47, 450, 130);
		}

		return jScrollPane;
	}

	/**
	 * This method initializes txtAbstract
	 * 
	 * @return javax.swing.JTextArea
	 */
	private javax.swing.JTextArea getTxtAbstract() {
		if (txtAbstract == null) {
			txtAbstract = new javax.swing.JTextArea();
			txtAbstract.setWrapStyleWord(true);
			txtAbstract.setColumns(30);
			txtAbstract.setLineWrap(true);
		}

		return txtAbstract;
	}

	/**
	 * This method initializes panelPage1
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getPanelPage1() {
		if (panelPage1 == null) {
			panelPage1 = new javax.swing.JPanel();
			panelPage1.setLayout(null);
			panelPage1.setPreferredSize(new java.awt.Dimension(480, 220));
			panelPage1.setVisible(true);
			panelPage1.setBounds(15, 5, 480, 262);
			panelPage1.add(getJPanel1(), null);
			panelPage1.add(getJPanel(), null);
		}

		return panelPage1;
	}

	/**
	 * This method initializes panelPage2
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getPanelPage2() {
		if (panelPage2 == null) {
			panelPage2 = new javax.swing.JPanel();
			panelPage2.setLayout(null);
			panelPage2.setPreferredSize(new java.awt.Dimension(490, 300));
			panelPage2.setVisible(false);
			wcsParamsTabbedPanel = new WCSParamsPanel();
			wcsParamsTabbedPanel.getJTabbedPane().addMouseListener(
					new java.awt.event.MouseAdapter() {
						public void mouseClicked(java.awt.event.MouseEvent e) {
							page = wcsParamsTabbedPanel.getIndiceSolapaActual() + 1;
							activarVisualizarBotones();
						}
					});
			wcsParamsTabbedPanel.getLstCoverages().addListSelectionListener(new javax.swing.event.ListSelectionListener() { 
				public void valueChanged(javax.swing.event.ListSelectionEvent e) {    
					wcsParamsTabbedPanel.actualizaWizard();
				}
			});
			wcsParamsTabbedPanel.getLstCRSs().addListSelectionListener(new javax.swing.event.ListSelectionListener() { 
				public void valueChanged(javax.swing.event.ListSelectionEvent e) {
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
			wcsParamsTabbedPanel.getLstFormats().addListSelectionListener(new javax.swing.event.ListSelectionListener() { 
				public void valueChanged(javax.swing.event.ListSelectionEvent e) {
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
			wcsParamsTabbedPanel.getLstSelectedTimes().addListSelectionListener(new javax.swing.event.ListSelectionListener() { 
				public void valueChanged(javax.swing.event.ListSelectionEvent e) {
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
			wcsParamsTabbedPanel.getSingleParamValuesList().addListSelectionListener(new javax.swing.event.ListSelectionListener() { 
				public void valueChanged(javax.swing.event.ListSelectionEvent e) {
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
			wcsParamsTabbedPanel.getCmbParam().addItemListener(
					new java.awt.event.ItemListener() {
						public void itemStateChanged(java.awt.event.ItemEvent e) {
							String s = (String) wcsParamsTabbedPanel.getLstCoverages().getSelectedValue();
							wcsParamsTabbedPanel.refreshParamValues(s);
						}
					});
			wcsParamsTabbedPanel.getTxtInterval().addKeyListener(new java.awt.event.KeyAdapter() { 
				public void keyReleased(java.awt.event.KeyEvent e) {    
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
			wcsParamsTabbedPanel.getTxtIntervalEnd().addKeyListener(new java.awt.event.KeyAdapter() { 
				public void keyReleased(java.awt.event.KeyEvent e) {    
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
			wcsParamsTabbedPanel.getTxtIntervalBegin().addKeyListener(new java.awt.event.KeyAdapter() { 
				public void keyReleased(java.awt.event.KeyEvent e) {    
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
			wcsParamsTabbedPanel.getRadioSingleInterval().addChangeListener(new javax.swing.event.ChangeListener() { 
				public void stateChanged(javax.swing.event.ChangeEvent e) {    
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
			wcsParamsTabbedPanel.getRadioMultipleInterval().addChangeListener(new javax.swing.event.ChangeListener() { 
				public void stateChanged(javax.swing.event.ChangeEvent e) {    
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
			wcsParamsTabbedPanel.getBtnAddTemps().addActionListener(
					new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							wcsParamsTabbedPanel.addTiempo();
						}
					});
			wcsParamsTabbedPanel.getBtnDelTemps().addActionListener(
					new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							wcsParamsTabbedPanel.delTiempo();
						}
					});
			panelPage2.add(wcsParamsTabbedPanel, null);
			panelPage2.setBounds(10, 5, 496, 276);

		}

		return panelPage2;
	}



	/**
	 * This method initializes btnSiguiente
	 * 
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnSiguiente() {
		if (btnSiguiente == null) {
			btnSiguiente = new javax.swing.JButton();
			btnSiguiente.setPreferredSize(new java.awt.Dimension(100, 30));
			btnSiguiente.setText(PluginServices.getText(this, "siguiente"));
			btnSiguiente.setBounds(205, 282, 100, 21);
			btnSiguiente.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (page > 0) {
						// si page es mayor que cero, vamos enfocando las
						// sucesivas SOLAPAS del WCSParamsPanel
						wcsParamsTabbedPanel.avanzaTab();
						page = wcsParamsTabbedPanel.getIndiceSolapaActual();
					}
					page++;

					getPanelPage1().setVisible(false);
					getPanelPage2().setVisible(true);
					activarVisualizarBotones();
				}
			});
		}
		return btnSiguiente;
	}

	/**
	 * This method initializes btnAnterior
	 * 
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnAnterior() {
		if (btnAnterior == null) {
			btnAnterior = new javax.swing.JButton();
			btnAnterior.setText(PluginServices.getText(this, "anterior"));
			btnAnterior.setPreferredSize(new java.awt.Dimension(100, 30));
			btnAnterior.setBounds(100, 282, 100, 21);
			btnAnterior.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					page --;
					if (page > 0) {
						// Tenemos que retroceder en el wcsParamsPanel
						wcsParamsTabbedPanel.retrocedeTab();
						page = wcsParamsTabbedPanel.getIndiceSolapaActual() + 1;
						activarVisualizarBotones();
					} else if (page==0){
						activarVisualizarBotones();
						page = 1;
						wcsParamsTabbedPanel.limpiaWizard();
						getLblTitle().setText("-");
						getTxtAbstract().setText("");
						wcsParamsTabbedPanel.retrocedeTab();
						getPanelPage1().setVisible(true);
						getPanelPage2().setVisible(false);
						getBtnSiguiente().setEnabled(false);
						conectado = false; 
					}
					listenerSupport.callStateChanged(wcsParamsTabbedPanel.isCorretlyConfigured());
				}
			});
		}
		return btnAnterior;
	}

	/**
	 * This method initializes jPanel1
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new javax.swing.JPanel();
			jPanel1.add(getTxtHost(), null);
			jPanel1.add(getBtnDetalles(), null);
			jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "servidor"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null,
					null));
			jPanel1.setBounds(2, 5, 477, 57);
		}

		return jPanel1;
	}

	/**
	 * Adds the gvSIG's wizard listener 
	 * 
	 * @param listener
	 */
	public void addWizardListener(WizardListener listener) { 
		listenerSupport.addWizardListener(listener);
	}

	/**
	 * Removes the gvSIG's wizard listener
	 * 
	 * @param listener
	 */
	public void removeWizardListener(WizardListener listener) {
		listenerSupport.removeWizardListener(listener);
	}

	/**
	 * Returns the Wizard's data source
	 * 
	 * @return
	 */
	public WCSWizardDataSource getDataSource() {
		return dataSource;
	}

	/**
	 * sets the wizard's data source
	 * 
	 * @param source
	 */
	public void setDataSource(WCSWizardDataSource source) {
		dataSource = source;

	}


	/**
	 * returns the selected SRS
	 * 
	 * @return String
	 */
	public String getSRS() {
		return (String) wcsParamsTabbedPanel.getSRS();
	}

	/**
	 * Returns the selected format
	 * 
	 * @return string
	 */
	public String getFormat() {
		return (String) wcsParamsTabbedPanel.getFormat();
	}

	/**
	 * Returns the selected coverage's bounding box
	 * 
	 * @return Rectangle2D
	 */
	public Rectangle2D getLayersRectangle() {
		return wcsParamsTabbedPanel.getLayersRectangle();
	}

	/**
	 * Returns the rectangle that contains all the others
	 * 
	 * Obtiene el rectngulo que contiene a todos los otros
	 * 
	 * @param rects[]  rectangulos
	 * 
	 * @return Rectangle2D
	 */
	public static Rectangle2D getRectangle(Rectangle2D[] rects) {
		Rectangle2D ret = rects[0];

		for (int i = 1; i < rects.length; i++) {
			ret.add(rects[i]);
		}

		return ret;
	}

	/**
	 * Returns the host typed in the host text field or null if nothing is typed
	 * or it is not a valid URL.
	 * 
	 * Devuelve el host que est escrito en el cuadro de texto del host o null
	 * en caso de que no haya nada escrito o est mal escrito
	 * 
	 * @return URL del host
	 */
	public URL getHost() {
		try {
			return new URL(cmbHost.getModel().getSelectedItem().toString());
		} catch (MalformedURLException e) {
			return null;
		}
	}

	/**
	 * Returns the name of the coverage that appears in the TOC
	 * 
	 * @return String
	 */
	public String getLayerName() {
		return wcsParamsTabbedPanel.getLayerName();
	}
 
	public void initWizard() {
		setDataSource(new WCSDataSourceAdapter());
	}

	public void execute() {
	}
	
	/**
	 * Returns the name and the selected values (if they are) of the parameter in
	 * a query string. Such string may directly be concatenated to the query string.
	 * 
	 * Devuelve el nombre y los valores seleccionados (si hay alguno) del parmetro
	 * en forma de querystring, de manera que puede ser concatenado a la query.
	 * 
	 * Si no se han seleccionado devuelve null.
	 * 
	 * @return String
	 */
	private String getParametro(){
		return wcsParamsTabbedPanel.getParametro();
	}
	
	/**
	 * Returns the time positions in a string that may directly be concatenated
	 * to the query string.
	 * 
	 * Devuelve las posiciones para el tiempo en forma de querystring, 
	 * de manera que puede ser concatenado a la query.
	 * 
	 * @return String
	 */	
	private String getSelectedTimes(){
		return wcsParamsTabbedPanel.getTiempos();
	}
	
	/**
	 * Creates a new layer to be passed to gvSIG. This method is automatically called
	 * by gvSIG and it must ensure that a layer is fully and correctly created or then
	 * return null.
	 */
	public FLayer getLayer() {
		FLyrWCS layer = new FLyrWCS();
		
		FMapWCSAdapter wcsAdap = new FMapWCSAdapter();
		wcsAdap.setDriver((FMapWCSDriver)((WCSDataSourceAdapter)dataSource).getDriver()); 
		layer.setHost(this.getHost().toString());
		layer.setCoverageName(this.getNomCobertura());
		layer.setLabel(this.getEtiquetaCobertura());
		layer.setSRS(this.getSRS());
		layer.setFormat(this.getFormat());
		layer.setFullExtent(this.getLayersRectangle());
		layer.setTime(this.getSelectedTimes());
		layer.setParameter(this.getParametro());
		layer.setName(layer.getLabel());
		layer.setMaxResolution(getMaxResolution());
		return layer; 
	}
	
	/**
	 * Returns the max supported resolution for the coverage.
	 *  
	 * Recupera la resolucin mxima soportada por la cobertura.
	 * @return double
	 */
	//private double getMaxResolution() {
	private Point2D getMaxResolution() {
		return wcsParamsTabbedPanel.getMaxRes();
	}

	/**
	 * Returns the coverage label.
	 * 
	 * @return String
	 */
	private String getEtiquetaCobertura() {
		return wcsParamsTabbedPanel.getEtiquetaCobertura();
	}
}

