/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.wcs;

import java.util.ArrayList;
import java.util.Iterator;

import com.iver.cit.gvsig.gui.wizards.WizardData;



/**
 * This class holds the WCSWizard's info
 * 
 * Contiene la informacin del asistente WCS
 * 
 * @author jaume - jaume.dominguez@iver.es
 *
 */

public class WCSWizardData extends WizardData{
    private String title;
    private String Abstract;
    private ArrayList coverages;
    private String[] coverageNames;

    /**
     * The server description.
     * 
     * La descripcin del servidor
     * @return String
     */
    public String getAbstract() {
    	if (Abstract == null) return "None";
        return Abstract;
    }
    
    /**
     * An ArrayList with the coverage formats.
     * 
     * Los formatos de la cobertura
     *
     * @return
     */ 
    public ArrayList getCoverageFormatos(String nomCobertura) {
        return getCoverageInfo(nomCobertura).getFormats();
    }

    /**
     * Finds the coverage within the coverage list
     * 
     * Busca la cobertura en la lista de coberturas
     *
     * @return CoverageInfo
     */
    public CoverageInfo getCoverageInfo(String name) {
    	Iterator i=coverages.iterator();
    	while (i.hasNext()){
    		CoverageInfo ci = (CoverageInfo) i.next();
    		if (ci.name.equals(name)) return ci;  
    	}
        return null;
    }
    
    /**
     * Returns an ArrayList containing the supported SRS of the coverage specified 
     * by name.
     * 
     * Devuelve una lista de SRSs soportados por la cobertura "name".
     * 
     * @param name
     * @return ArrayList
     */
    public ArrayList getCoverageSRSs(String name){
    	CoverageInfo ci = getCoverageInfo(name);
    	return ci.getSRSs();
    }
    /**
     * Server's title (not used in gvSIG)
     * 
     * Ttulo del servidor (no se usa en gvSIG)
     *
     * @return
     */
    public String getTitle() {
    	if (title == null) return "None";
        return title;
    }

    /**
     * Equivalent to the setDescription method
     * 
     * Equivalente a setDescription
     *
     * @param string
     */
    public void setAbstract(String string) {
        Abstract = string;
    }


    /**
     * Adds a coverage to the coverages list
     * 
     * Aande una cobertura a la lista de coberturas
     *
     * @param CoverageInfo
     */
    public void setCoverage(CoverageInfo info) {
        coverages.add(info);
    }

    /**
     * Sets the server's title (not used in gvSIG)
     * 
     * Establece el ttulo del servidor (no se usa en gvSIG)
     *
     * @param string
     */
    public void setTitle(String string) {
        title = string;
    }
    
    /**
     * Sets the server's description.
     * 
     * Establece la descripcin del servidor.
     * @param String
     */
    public void setDescription(String s) {
    	setAbstract(s);
    }
    
    /**
     * Returns the server's descrition
     * 
     * Recupera la descripcin del servidor
     * @return "None" if the server doesn't specify any description
     */
    public String getDescription(){
    	if (getAbstract() == null) return "None";
    	return getAbstract();
    }

	/**
	 * Sets the coverage names
	 * 
	 * @param String[] coverageNames
	 */
	public void setCoverageNames(String[] coverageNames) {
		this.coverageNames = coverageNames;
	}
	
	/** 
	 * Returns an String[] containing the coverage names.
	 * @return String
	 */
	public String[] getCoverageNames() {
		return this.coverageNames;
	}
	
	/** 
	 * Sets the coverages list from an ArrayList
	 * 
	 * @param coverages
	 */
	public void setCoveragesList(ArrayList coverages){
		this.coverages = coverages;
	}

	/**
	 * Returns an ArrayList containing the positions for the TIME
	 * 
	 * Devuelve las posiciones del tiempo
	 * @param string
	 * @return
	 */
	public ArrayList getCoverageTimes(String name) {
	   	CoverageInfo ci = getCoverageInfo(name);
	   	if (ci.getTimes()== null) return new ArrayList();
    	return ci.getTimes();
 	}
	
	/**
	 * Returns an ArrayList containing a list of Parametro with the parameters list
	 * and its possible values of a coverage specified by its name.
	 * 
	 * Devuelve la lista de parmetros (y sus posibles valores) de la cobertura y sus
	 * posibles valores de una cobertura especificada por su nombre
	 * 
	 * @return ArrayList
	 */
		
	public ArrayList getCoverageParameters(String name){
		CoverageInfo ci = getCoverageInfo(name);
		return ci.getParametros();
	}

	/**
	 * Gets the coverage label.
	 */
	public String getCoverageLabel(String name) {
		CoverageInfo ci = getCoverageInfo(name);
		return ci.getLabel();
	}	
}
