package es.uji.lsi.wcs.XmlWcsParsing;

import java.util.ArrayList;
import java.util.Iterator;


/*
 * Capabilities.java
 *
 * Created on 1 de diciembre de 2004, 23:30
 */

/**
 *
 * @author  jaume
 */



public class Capabilities {
    // Capabilities elements
    private String version;             // Indicates the WCS verison to which the Capabilities XML document conforms. (required)
    private String updateSequence;      // Indicates content or service updates. (A service provider must increase this value when adding new content or changing any aspects orf the service.) (optional)
    
    /* Service section */
    private Service service;
    private Capability capability;
    private ContentMetadata contentMetadata;
    
    /** Creates a new instance of Capabilities */
    public Capabilities(XMLNode node) {
        fillupCapabilities(node);
    }
    
    public String[] getCoverageNames(){
    	ArrayList list = getContentMetadata().getCoverageOfferingBriefList();
    	Iterator i = list.iterator();
        String[] names = new String[list.size()];
        int index = 0;
        while (i.hasNext()){
        	names[index++] = ((CoverageOfferingBrief) i.next()).getName();
        	
        }
        return names;
    }
    private void fillupCapabilities(XMLNode node){
        // Comporovar que s XML de GetCapabilities (a s comprovar que la primera lnia (node) diu que ho s)
        try {
            checkGetCapabilitiesDocument(node);
            for (int i = 0; i<node.getNumSubNodes(); i++){
                XMLNode subnode = node.getSubNode(i);
                if (WCSToolkit.isWCSTab(subnode, "Service")) service = new Service(subnode); 
                if (WCSToolkit.isWCSTab(subnode, "Capability")) capability = new Capability(subnode);
                if (WCSToolkit.isWCSTab(subnode, "ContentMetadata")) contentMetadata = new ContentMetadata(subnode);
            }        
        } catch (NotAnXmlGetCapabilitiesDocumentException e){
            new Error("Unrecognized GetCapabilities format");
        }
        
    }
    
    private boolean checkGetCapabilitiesDocument(XMLNode node) throws NotAnXmlGetCapabilitiesDocumentException{
        if (WCSToolkit.isWCSTab(node, "WCS_Capabilities")||
            WCSToolkit.isWCSTab(node, "WCSCapabilities") ||
            WCSToolkit.isWCSTab(node, "Service") ||
            WCSToolkit.isWCSTab(node, "Capability") ||
            WCSToolkit.isWCSTab(node, "ContentMetadata")) {
                version = node.getAttribute("version");
                updateSequence = node.getAttribute("updateSequence");
                return true;
        } 
        else throw new NotAnXmlGetCapabilitiesDocumentException();
    }
    
    public String getVersion(){
        return version;
    }
     
    public String getUpdateSequence(){
        return updateSequence;
    }
    public String getServiceDescription(){
        return service.getDescription();
    }
    
    public String getServiceLabel(){ 
        return service.getLabel();
    }
    
    public String getServiceName() {
        return service.getName();
    }
    
    public String getServiceMetadatalink(){
        return service.getMetadataLink();
    }
    
    public String getServiceKeywords(){
        return service.getKeywords();
    }
    
    public String getServiceResponsibleParty(){
        return service.getResponsibleParty();
    }
    
    public String getServiceFees(){
        return service.getFees();
    }
    
    public String getServiceAccessConstraints(){
        return service.getAccessConstraints();
    }
    
    public ContentMetadata getContentMetadata(){
        return contentMetadata;
    }
    
    public String toString(){
        return "\nVersion: "+version+
               "\nUpdate sequence: "+updateSequence+
               "\n\n###SERVICE###"+service+
               "\n\n###CAPABILITY###"+capability+
               "\n\n###CONTENT METADATA###"+contentMetadata;
    }
    
    public CoverageOfferingBrief getCoverageOfferingBrief(String name){
        return contentMetadata.getCoverageOfferingBrief(name);
    }
}

class NotAnXmlGetCapabilitiesDocumentException extends Exception{
    public NotAnXmlGetCapabilitiesDocumentException(){
        super();
    }
    
    public NotAnXmlGetCapabilitiesDocumentException(String s){
        super(s);
    }
}