package es.uji.lsi.wcs.XmlWcsParsing;
/*
 * SpatialDomain.java
 *
 * Created on 3 de enero de 2005, 12:36
 */
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 * @author  jaume
 */
public class SpatialDomain {
    private ArrayList gmlEnvelopeWithTimePeriodList = new ArrayList();
    private ArrayList gmlGridList = new ArrayList();
    private GMLRectifiedGrid gmlRectifiedGrid;
    private ArrayList gmlPolygonList = new ArrayList();
    
    /** Creates a new instance of SpatialDomain */
    public SpatialDomain(XMLNode node) {
        for (int i=0; i<node.getNumSubNodes(); i++){
            XMLNode subnode = node.getSubNode(i);
            if (WCSToolkit.isWCSTab(subnode, "gml:Envelope") || subnode.getName().equals("gml:EnvelopeWithTimePeriod")) gmlEnvelopeWithTimePeriodList.add(new GMLEnvelopeWithTimePeriod(subnode));
            if (WCSToolkit.isWCSTab(subnode, "gml:Grid")) gmlGridList.add(new GMLGrid(subnode));
            if (WCSToolkit.isWCSTab(subnode, "gml:RectifiedGrid")) gmlRectifiedGrid = new GMLRectifiedGrid(subnode);
//            if (subnode.getName().equals("gml:Polygon")) gmlGridList.add(new GMLPolygon(subnode));
        }        
    }
    
    public ArrayList getGMLEnvelopeWithTimePeriodList(){
        return gmlEnvelopeWithTimePeriodList;
    }
    
    
    public ArrayList getGMLGridList(){
        return gmlGridList;
    }
    
    public GMLRectifiedGrid getGMLRectifiedGrid(){
        return gmlRectifiedGrid;
    }
    
    public ArrayList getGMLPolygonList(){
        return gmlPolygonList;
    }

    public ArrayList getGMLEnvelopeNames(){
        // Gives the Envelopes' names as string array list;
        ArrayList a = new ArrayList();
        Iterator it = getGMLEnvelopeWithTimePeriodList().iterator();
        while (it.hasNext())
            a.add( ((GMLEnvelopeWithTimePeriod) it.next()).getSRSName() );
        return a;
    }

    public GMLEnvelopeWithTimePeriod getGMLEnvelope(String srsName){
        ArrayList a = new ArrayList();
        Iterator it = getGMLEnvelopeWithTimePeriodList().iterator();
        while (it.hasNext()){
            GMLEnvelopeWithTimePeriod aux = (GMLEnvelopeWithTimePeriod) it.next();
            if ( aux.getSRSName().equals(srsName) ) return aux;
        }
        return null;
    }

    public String toString(){
        Iterator it = gmlEnvelopeWithTimePeriodList.iterator();
        String s = "\nEnvelope with time period list.";
        while (it.hasNext())
            s += "\n"+it.next();
        
        it = gmlGridList.iterator();
        s += "\nGrid List.";
        while (it.hasNext())
            s += it.next();
        
        it = gmlPolygonList.iterator();
        s += "\nPolygon list.";
        while (it.hasNext())
            s += it.next();
        
        
        return s;
    }

	/**
	 * Returns the max native resolution, use this for
	 * if both X and Y resolution are the same.
	 * 
	 * Devuelve la resolucin mxima nativa de la capa,
	 * use este mtodo si las resoluciones de los ejes
	 * X e Y son iguales.
	 * 
	 * @return
	 */
	/*public double getMaxRes() {
		if (getGMLRectifiedGrid()!=null && getGMLRectifiedGrid().getOffsetVector(0) != null)
			return getGMLRectifiedGrid().getOffsetVector(0)[0];
		
		// cuidadin que esto va con servidores MapServer, falta ver si los dems especifican
		// su resolucin igual que MapServer
		return 0D;
	}*/
	
	/**
	 * Returns the max native resolution for the Y axis
	 * 
	 * Devuelve la resolucin nativa mxima de la cobert
	 * @return
	 */
	public double getMaxResX() {
		if (getGMLRectifiedGrid()!=null && getGMLRectifiedGrid().getOffsetVector(0) != null)
			return getGMLRectifiedGrid().getOffsetVector(0)[0];
		
		// cuidadin que esto va con servidores MapServer, falta ver si los dems especifican
		// su resolucin igual que MapServer
		return 0D;		
	}
	
	/**
	 * Returns the max native resolution for the Y axis
	 * 
	 * Devuelve la resolucin nativa mxima de la cobert
	 * @return
	 */
	public double getMaxResY() {
		if (getGMLRectifiedGrid()!=null && getGMLRectifiedGrid().getOffsetVector(0) != null)
			return - getGMLRectifiedGrid().getOffsetVector(1)[1];
		
		// cuidadin que esto va con servidores MapServer, falta ver si los dems especifican
		// su resolucin igual que MapServer
		return 0D;		
	}
	
	public Point2D getMaxRes(){
		if (getGMLRectifiedGrid()!=null && getGMLRectifiedGrid().getOffsetVector(0) != null){
			Point2D p = new Point2D.Double(getGMLRectifiedGrid().getOffsetVector(0)[0], -getGMLRectifiedGrid().getOffsetVector(1)[1]);
			return p;
		}
		return null;
	}
}
