package es.uji.lsi.wcs.client;
/*
 * WCSclient.java
 *
 * Created on 1 de diciembre de 2004, 21:56
 * 
 * This file is supposed to implement the interface beetween server and client.
 * It will implement the GetCapabilities, DescribeCoverage and GetCoverage
 * operations. 
 */
import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownHostException;

import com.devx.io.TempFileManager;

import es.uji.lsi.wcs.XmlWcsParsing.Capabilities;
import es.uji.lsi.wcs.XmlWcsParsing.CoverageOffering;
import es.uji.lsi.wcs.XmlWcsParsing.DescribeCoverageResponse;
import es.uji.lsi.wcs.XmlWcsParsing.WCSExceptionResponse;
import es.uji.lsi.wcs.XmlWcsParsing.XMLNode;


/**
 *
 * @author  jaume
 */
public class WCSClient {
    
	public Capabilities capabilities;
    public DescribeCoverageResponse describeCoverageResponse;
    
    private OperationGetCapabilities operationGetCapabilities;
    private OperationDescribeCoverage operationDescribeCoverage;
    private OperationGetCoverage operationGetCoverage;
    private WCSExceptionResponse wcs_exception;
    // the working address
    private String working_address;
    
    // When working off-line (for development uses only)
    private boolean working_offline = false;
    private File file = null; 
    
    private File fileGetCapabilities, fileDescribeCoverage, fileGetCoverage;
    private String path, datapath, coveragefilename, coveragefileextension;
    private static String characters;
	
    static {
    	characters = "";
		for (int j = 32; j<=127; j++){
			characters += (char) j;
		}
		characters += "ǡ\n\r\f\t";
    }
    
    /** Creates a new instance of WCSclient */
    public WCSClient(String text) {
        this.working_address = text;
        operationGetCapabilities = new OperationGetCapabilities();
        operationDescribeCoverage = new OperationDescribeCoverage();
        operationGetCoverage = new OperationGetCoverage();
    }
    
    public WCSClient(){
        working_address = "http://localhost:8080";
        operationGetCapabilities = new OperationGetCapabilities();
        operationDescribeCoverage = new OperationDescribeCoverage();
        operationGetCoverage = new OperationGetCoverage();
    }
    
    public WCSClient(File file){
        working_offline = true;
        this.file = file;
        operationGetCapabilities = new OperationGetCapabilities();
        operationDescribeCoverage = new OperationDescribeCoverage();
        operationGetCoverage = new OperationGetCoverage();

    }
    
    public Capabilities getTheCapabilities(){
        return capabilities;
    } 
    /**
     * @deprecated
     */
    public void _getCapabilities(){
        // Opens the source of the GetCapabilities XML description
    	// System.out.println("WCSClient>>> GetCapabilities");
        try {
            
            XMLNode node;
            if (working_offline) node = new XMLNode(file);
            else{
                URL url = new URL(construyeURL(working_address, operationGetCapabilities.getParamsString()));
                node = new XMLNode(url.openStream()); 
                // System.out.println("WCSClient>>> GetCapabilities done...");
            } 
            capabilities = new Capabilities(node);
            
        } catch (MalformedURLException e){
            new Error("Bad URL");
        } catch (IOException e){
            new Error("Unable to retrieve server capabilities");
        } catch (Exception e){
            new Error();
        }            

    }
    
    /**
     * Creates a WCS compliant request URL.
     * 
     * @param dir
     * @param param
     * @return string
     */
    private String construyeURL(String dir, String param){
    	if (dir.indexOf('?')==-1) return dir+"?"+param;
    	else return dir+"&"+param;
    }
    
    /**
     * Opens the source of the GetCapabilities XML description and
     * loads it into the capabilities field.
     * 
     * @throws ServerOutOfOrderException
     */
    public void getCapabilities() throws ServerOutOfOrderException{
        try {
            
            XMLNode node;
            if (working_offline) node = new XMLNode(file);
            else{
                URL url = new URL(construyeURL(working_address, operationGetCapabilities.getParamsString()));
                fileGetCapabilities = downloadFile(url, "wcs_GetCapabilities");
                node = new XMLNode(fileGetCapabilities); 
            } 
            capabilities = new Capabilities(node);
            if (capabilities == null){
            	throw new ServerOutOfOrderException();
            }
        } catch (UnknownHostException uhe){ 
        	throw new ServerOutOfOrderException();
        }  catch (ConnectException ce){ 
        	throw new ServerOutOfOrderException();
        } catch (MalformedURLException e){
            new Error("Bad URL");
        } catch (IOException e){
        	throw new ServerOutOfOrderException();
        } catch (Exception e){
            new Error();
        }            

    }

    
    /**
     * Returns a description offered by the DescribeCoverage
     * 
     * Devuelve la descripcin de la cobertura ofrecida en el DescribeCoverage
     * 
     * @param nomCobertura
     * @return es.uji.lsi.XmlWcsParsing.CoverageOffering 
     */
    public CoverageOffering getCoverageDescription(String nomCobertura){
    	return describeCoverageResponse.getCoverageOffering(nomCobertura);
	}
    
    
    /**
     * Unused
     * 
     * Sin uso
     * 
     * @deprecated
     * @param url
     */
    
    public void describeCoverage(URL url){
    	XMLNode node;
    	try {
    		downloadFile(url, this.datapath);
            node = new XMLNode(url.openStream());
    	} catch (Exception e){
    		new Error(e.getMessage());
    	}
    }

    /**
     * Launches a DescribeCoverage request.
     *
     */
    public void describeCoverage(){ 
        try {
            XMLNode node;
            if (working_offline) node = new XMLNode(file);
            else{
                URL url = new URL(construyeURL(working_address, operationDescribeCoverage.getParamsString()));
                fileDescribeCoverage = downloadFile(url, "wcs_DescribeCoverage");
                node = new XMLNode(fileDescribeCoverage);  
            } 
            describeCoverageResponse = new DescribeCoverageResponse(node, capabilities);
        } catch (MalformedURLException e){
            new Error("Bad URL");
        } catch (IOException e){
            new Error("Unable to retrieve coverage description, connection error.");
        } catch (Exception e){
            new Error();
        }            
    }
    /**
     * Unused
     * @deprecated
     *
     */
    public void _describeCoverage(){    
        try {
            XMLNode node;
            if (working_offline) node = new XMLNode(file);
            else{
                URL url = new URL(construyeURL(working_address, operationDescribeCoverage.getParamsString()));
                node = new XMLNode(url.openStream());  
            } 
            describeCoverageResponse = new DescribeCoverageResponse(node, capabilities);
        } catch (MalformedURLException e){
            new Error("Bad URL");
        } catch (IOException e){
            new Error("Unable to retrieve coverage description, connection error.");
        } catch (Exception e){
            new Error();
        }            
    }
    
    /**
     * Launches a coverage specific DescribeCoverage operation (unused in gvSIG)
     * 
     * @param coverage_name
     */
    public void describeCoverage(String coverage_name){
        try {
            operationDescribeCoverage.setParam("coverage", coverage_name);
            describeCoverage();
        } catch (Exception e){
            new Error("Fallo asignando valor al parmetro <coverage>");
        }
    }
    
    /**
     * Unused
     * @deprecated
     * @return
     * @throws ServerErrorResponseException
     */
    public boolean _getCoverage() throws ServerErrorResponseException{
        try {
            String s = construyeURL(working_address, operationGetCoverage.getParamsString());
            URL miurl = new URL(s);
            downloadFile(miurl, this.datapath);
            return (this.coveragefilename!=null);
        } catch (MalformedURLException e){
            new Error("WCS: GetCoverage> URL error.");
        } catch (IOException ioe){
        	new Error("Unable to retrieve coverage.");
        }
        return false;
    }
    /**
     * Launches a GetCoverage exception. It downloads the coverage to a temporary
     * file. If the answer wasn't a coverage but was an error it throws an exception
     * 
     * @return the file downloaded
     * @throws ServerErrorResponseException
     */
    public File getCoverage() throws ServerErrorResponseException, IOException{
        try {
            String s = construyeURL(working_address, operationGetCoverage.getParamsString());
            URL miurl = new URL(s);
            
            File coverageFile = downloadFile(miurl, "wcs_getCoverage");
            boolean correcto = checkExceptionResponse(coverageFile);
            
            if (!correcto){
            	String message = parseException(coverageFile);
            	if (message!=null)
            		throw new ServerErrorResponseException(message);
            	else
            		throw new ServerErrorResponseException();
            }
            return coverageFile;
        } catch (MalformedURLException e){
            new Error("WCS: GetCoverage> URL error.");
        }/* catch (IOException ioe){
        	new Error("Unable to retrieve coverage.");
        }*/
        return null;
    }
    
    /**
     * Gets an string containing the currently set parameters for the GetCoverage
     * request.
     * 
     * @return String
     */
    public String getCovergeParamString(){
    	return operationGetCoverage.getParamsString();
    }
    
    /**
     * Gets an string containing the currently set parameters for the DescribeCoverage
     * request.
     * 
     * @return String
     */
    public String getDescribeCoverageParamString(){
    	return operationDescribeCoverage.getParamsString();
    }
    
    /**
     * Sets the DescribeCoverage Operation parameters.
     * 
     * @param params
     * @return
     */
    public void setDescribeCoverageParams(String paramString){
    	String[] params = paramString.split("&");
    	operationDescribeCoverage.cleanParams();
    	try{
    		for (int i = 0; i < params.length; i++) {
    			String[] par_nombreValor = params[i].split("="); 
    			//System.out.prinln(par_nombreValor[0]+"="+par_nombreValor[1]);)
    			operationDescribeCoverage.setParam(par_nombreValor[0],par_nombreValor[1]);
    		}
    	} catch (Exception e){
    		e.printStackTrace();
    	}
    }

    /**
     * Sets the GetCapabilities Operation parameters.
     * 
     * @param params
     * @return
     */
    public void setGetCapabilitiesParams(String paramString){
    	String[] params = paramString.split("&");
    	operationGetCapabilities.cleanParams();
    	try{
    		for (int i = 0; i < params.length; i++) {
    			String[] par_nombrevalor = params[i].split("="); 
    			operationGetCapabilities.setParam(par_nombrevalor[0],par_nombrevalor[1]);
    		}
    	} catch (Exception e){
    		e.printStackTrace();
    	}
    }
    /**
     * Sets the GetCoverage Operation parameters.
     * 
     * @param params
     * @return
     */
    public void setGetCoverageParams(String paramString){
    	String[] params = paramString.split("&");
    	operationGetCoverage.cleanParams();
    	try{
    		for (int i = 0; i < params.length; i++) {
    			String[] par_nombrevalor = params[i].split("=");
    			if (par_nombrevalor.length==2 && par_nombrevalor[0]!=null && par_nombrevalor[1]!=null)
    			operationGetCoverage.setParam(par_nombrevalor[0],par_nombrevalor[1]);
    			//System.out.println(par_nombrevalor[0]+par_nombrevalor[1]);
    		}
    	} catch (Exception e){
    		e.printStackTrace();
    	}
    }
    
    /**
     * Downloads an URL into a temporary file that is removed the next time the 
     * tempFileManager class is called, which means the next time gvSIG is launched.
     * 
     * @param url
     * @param name
     * @return
     * @throws IOException
     * @throws ServerErrorResponseException
     * @throws ConnectException
     * @throws UnknownHostException
     */
    public File downloadFile(URL url, String name) throws IOException, ServerErrorResponseException, ConnectException, UnknownHostException{
    	File f = null;
	    try{
	    	f = TempFileManager.createTempFile(name, "tmp");
	    	System.out.println("downloading '"+url.toString()+"' to: "+f.getAbsolutePath());
			 
	    	f.deleteOnExit();
		    
	    } catch (IOException io) {
	    	io.printStackTrace();
	    }
	    DataOutputStream dos = new DataOutputStream( new BufferedOutputStream(new FileOutputStream(f)));
	    byte[] buffer = new byte[1024*256];
	    InputStream is = url.openStream();
	    long readed = 0;
	    for (int i = is.read(buffer); i>0; i = is.read(buffer)){
	    	dos.write(buffer, 0, i);
	    	readed += i;
	    }
	    dos.close();
	    /*if (!isNotAnException(f))
	    	// SI que es una excepcin
	    	throw new ServerErrorResponseException();*/
	    return f;
	}

    /**
     * @deprecated
     * @param url
     * @param targetdir
     */
    public void _downloadFile(URL url, String targetdir){
        try{
            String path = this.getDataPath();
            String filename = this.getCoverageFileName()+this.getCoverageFileExtension();
            String where = path+filename;
            DataOutputStream dos = new DataOutputStream( new BufferedOutputStream( new FileOutputStream(where)));
            byte[] buffer = new byte[1024*256]; // 256KB
            InputStream is = url.openStream();
            long readed = 0;
            for (int i = is.read(buffer); i>0; i = is.read(buffer)){
                dos.write(buffer, 0, i);
                readed += i;
            }
            dos.close();
       
            
        } catch (IOException ie){
            new Error("La peticin de la cobertura no tuvo xito.");
        }
        
    }
 
    /**
     * Gets the current working address.
     * 
     * @return
     */
    public String getWorkingAddress(){
        return working_address;
    }
    
    /**
     * Unused in gvSIG
     * @return
     */
    public String getDataPath(){
        return this.datapath;
    }
    
    /**
     * Unused in gvSIG
     */
    public String getCoverageFileName(){
        return this.coveragefilename;
    }
    
    /**
     * Unused in gvSIG
     */
   public String getCoverageFileExtension(){
        return this.coveragefileextension;
    }
    
   /**
    * Typical toString funtion.
    */
    public String toString(){
        return capabilities.toString();
    }
    
    /**
     * Unused in gvSIG
     */
    public void setBaseDirectory(String path){
        this.path = path;
        this.datapath = this.path+"/data/";
        
    }
    
    /**
     * Unused in gvSIG
     */
    public void setCoverageFileName(String name){
        this.coveragefilename = name;
    }

    /**
     * Unused in gvSIG
     */
    public void setCoverageFileExtension(String extension){
        this.coveragefileextension = "."+extension;
    }
    /**
     * Its use is unrecommended, use checkExceptionResponse and parseException
     * 
     * Su uso no est recomendado, usar checkExceptionResponse y parseException
     * @deprecated
     * @param file
     * @return
     */
    public boolean isNotAnException(File file){
    	if (file == null)
    		return false;
    	
        this.wcs_exception = new WCSExceptionResponse(file);
        return (this.wcs_exception.getText() == null);
    }
    
    /**
     * Unused in gvSIG
     */
    public String getWCSException(){
        return this.wcs_exception.getText();
    }
    
    /**
     * Checks the server's answer.
     * TODO figure a good criteria out to catch every kind of error.
     * 
     * Comprueba si la respuesta ha sido un error o era lo que se
     * esperaba. (Versin en pruebas)
     * 
     * @param file
     * @return true si est todo bien, false si hay errores.
     */
    private boolean checkExceptionResponse(File file){
    	// TODO Aix va b si la resposta s un XML de WCS, per tamb s'ha de 
    	// considerar si s alguna resposta d'error d'Apache o algo per l'estil
    	
    	// Un possible criteri s veure el tamany del fitxer, establim un lmit de
    	// fitxer a analitzar (umbral) i comprovem que el que s'analitza s text o
    	// sn dades binaries. Si s text, llavors no s una imatge i es tracta com
    	// a error.
    	wcs_exception = null;
    	int umbral = 1024; 
    	try {
    		FileReader fr = new FileReader(file);
    		for (int i = 0; i < umbral; i++) {
    			int c = fr.read();
    			if (c==-1){
    				// End of file. If we reach this
    				// everything before is printable data.
    				return false;
    			} else {
    				char ch = (char) c;
    				if (characters.indexOf(ch)==-1){
    					// We've found a non-printable character.
    					// Then we'll assume that this file is binary.
    					return true;
    				}
    			}
    		}
    	} catch (FileNotFoundException e) {
    		e.printStackTrace();
    	} catch (IOException e) {
			e.printStackTrace();
		}
    	return false;
 }
    
    /**
     * Parses the OGC Exception XML documents and extracts the error message.
     * 
     * @param f
     * @return String containing the error message.
     */
    private String parseException(File f){
    	wcs_exception = new WCSExceptionResponse(f);
    	return wcs_exception.getText();
    }
    
}

    