/*
 * Created on 22-feb-2005
 */
package org.cresques.io.rasterOld;

import java.awt.Point;
import java.awt.image.DataBuffer;

/**
 * Rectangulo de pixeles. parecido a java.awt.image.Raster
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class RasterBuf {
	public final static int TYPE_BYTE	= DataBuffer.TYPE_BYTE;
	public final static int TYPE_SHORT	= DataBuffer.TYPE_SHORT;
	public final static int TYPE_USHORT = DataBuffer.TYPE_USHORT;
	public final static int TYPE_INT	= DataBuffer.TYPE_INT;
	public final static int TYPE_FLOAT	= DataBuffer.TYPE_FLOAT;
	private byte  [][][] byteBuf;
	private short [][][] shortBuf;
	private int   [][][] intBuf;
	private int width, height, bandNr;
	private int dataType;
	
	public RasterBuf(int dataType, int width, int height, int bandNr, Point orig) {
		this.dataType = dataType;
		this.width = width;
		this.height = height;
		this.bandNr = bandNr;
		if (dataType == TYPE_BYTE)
			byteBuf  = new byte[height][width][bandNr];
		else if (dataType == TYPE_SHORT | dataType == TYPE_USHORT)
			shortBuf = new short[height][width][bandNr];
		else if (dataType == TYPE_INT)
			intBuf   = new int[height][width][bandNr];
	}
	
	public int getWidth() { return width; }
	
	public int getHeight() { return height; }
	
	public int getDataType() { return dataType; }
	
	public int getDataSize() {
		if (dataType == TYPE_BYTE)
			return 1;
		else if (dataType == TYPE_SHORT | dataType == TYPE_USHORT)
			return 2;
		else if (dataType == TYPE_INT)
			return 4;
		return 0;
	}
	
	public int sizeof() {
		return getDataSize()*width*height*bandNr;
	}
	
	public byte [][] getLineByte(int y) {	return byteBuf[y]; }
	
	public short [][] getLineShort(int y) {	return shortBuf[y]; }
	
	public int [][] getLineInt(int y) {	return intBuf[y]; }

	public void setPixelByte(int x, int y, int b, int px) {
		byteBuf[y][x][b] = (byte) (px & 0xff);
	}
	
	public void setPixelShort(int x, int y, int b, int px) {
		shortBuf[y][x][b] = (byte) (px & 0xffff);
	}
	
	public void setPixelInt(int x, int y, int b, int px) {
		intBuf[y][x][b] = px;
	}
	
	public void setPixelShort(int x, int y, int [] px) {
		for (int i=0; i<bandNr; i++) shortBuf[y][x][i] = (short) px[i];
	}
	
	public void getPixelShort(int x, int y, int [] px) {
		for (int i=0; i<bandNr; i++) px[i] = shortBuf[y][x][i];
	}

	public void setPixelInt(int x, int y, int [] px) {
		for (int i=0; i<bandNr; i++) intBuf[y][x][i] = px[i];
	}
	
	public void getPixelInt(int x, int y, int [] px) {
		for (int i=0; i<bandNr; i++) px[i] = intBuf[y][x][i];
	}
	/**
	 * @return Returns the bandNr.
	 */
	public int getBandNr() {
		return bandNr;
	}
}
