/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.wizards;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.geom.Rectangle2D;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;
import java.util.prefs.Preferences;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.AddLayer;
import com.iver.cit.gvsig.fmap.drivers.wms.FMapWMSDriver;
import com.iver.cit.gvsig.fmap.exceptions.CannotReprojectException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWMS;
import com.iver.cit.gvsig.gui.WizardPanel;
import com.iver.cit.gvsig.gui.panels.WMSParamsPanel;
import com.iver.utiles.NotExistInXMLEntity;
import com.iver.utiles.XMLEntity;


/**
 * The WMS properties container panel.
 *
 * @author Jaume Dominguez Faus
 */
public class WMSWizard extends WizardPanel {
	private static Logger logger = Logger.getLogger(WMSWizard.class.getName());
	private int page = 0;
	private boolean connected = false;
	private javax.swing.JComboBox cmbHost = null;
	private javax.swing.JButton btnConnect = null;
	private javax.swing.JPanel jPanel = null;
	private javax.swing.JLabel jLabel1 = null;
	private javax.swing.JLabel lblTitle = null;
	private javax.swing.JScrollPane jScrollPane = null;
	private javax.swing.JTextArea txtAbstract = null;
	private javax.swing.JPanel panelPage1 = null;
	private javax.swing.JButton btnSiguiente = null;
	private javax.swing.JButton btnAnterior = null;
	private javax.swing.JPanel jPanel1 = null;
	private WizardListenerSupport listenerSupport = new WizardListenerSupport();
	private WMSWizardData dataSource;
	private WMSParamsPanel wmsParamsPanel = null;
    private JLabel lblServerType = null;
    private JLabel lblServerTypeValue = null;
	private JCheckBox chkCaching = null;
	private static Preferences fPrefs = Preferences.userRoot().node( "gvsig.wms-wizard" );
	private boolean refreshing = fPrefs.getBoolean("refresh_capabilities", false);
	
	/**
	 * This is the default constructor
	 */
	public WMSWizard() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 */
	private void initialize() {
		setTabName("WMS");
		
		lblServerType = new JLabel();
		lblServerType.setBounds(20, 444, 100, 20);
		lblServerType.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
		lblServerType.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
		lblServerType.setFont(new java.awt.Font("Arial", java.awt.Font.PLAIN, 11));
		lblServerType.setText(PluginServices.getText(this, "server_type")+":");
		lblServerTypeValue = new JLabel();
		lblServerTypeValue.setBounds(128, 444, 148, 20);
		lblServerTypeValue.setFont(new java.awt.Font("Arial", java.awt.Font.BOLD, 11));
		lblServerTypeValue.setText("-");
		this.setSize(510, 468);
		this.setLayout(null);
		this.setPreferredSize(new java.awt.Dimension(750, 420));
		this.setVisible(true);
		this.add(getPanelPage1(), null);
		this.add(getPanelPage2(), null);
		this.add(getBtnAnterior(), null);
		this.add(getBtnSiguiente(), null);
		this.add(lblServerType, null);
		this.add(lblServerTypeValue, null);
		activarVisualizarBotones();
	}


	private JPanel getPanelPage2(){
		if (wmsParamsPanel == null){
			wmsParamsPanel = new WMSParamsPanel();
			wmsParamsPanel.setListenerSupport(this.listenerSupport);
			wmsParamsPanel.setBounds(15, 5, 480, 428);
			wmsParamsPanel.getJTabbedPane().addMouseListener(
                    new java.awt.event.MouseAdapter() {
                        public void mouseClicked(java.awt.event.MouseEvent e) {
                            page = wmsParamsPanel.currentPage() + 1;
                            activarVisualizarBotones();
                        }
                    });
		}
		return wmsParamsPanel;
	}
	/**
	 * DOCUMENT ME!
	 *
	 * @param host DOCUMENT ME!
	 */
	private void addHost(String host) {
		host = host.trim();

		DefaultComboBoxModel model = (DefaultComboBoxModel) getTxtHost()
																.getModel();

		if (model.getIndexOf(host) < 0) {
			model.addElement(host);

			PluginServices ps = PluginServices.getPluginServices(this);
			XMLEntity xml = ps.getPersistentXML();

			try {
				String[] servers = xml.getStringArrayProperty("wms-servers");
				String[] newServers = new String[servers.length + 1];
				System.arraycopy(servers, 0, newServers, 0, servers.length);
				newServers[servers.length] = host;
				xml.putProperty("wms-servers", newServers);
			} catch (NotExistInXMLEntity e) {
				xml.putProperty("wms-servers", new String[] { host });
			}

			ps.setPersistentXML(xml);
		}
	}

	/**
	 * DOCUMENT ME!
	 */
	private void rellenarControles() {
		try {
			String host = cmbHost.getModel().getSelectedItem().toString();
			
            dataSource.setHost(new URL(host), refreshing);
            lblTitle.setText(dataSource.getTitle());
			lblServerTypeValue.setText(dataSource.getServerType());
            txtAbstract.setText(dataSource.getAbstract());
			
			addHost(host);
			wmsParamsPanel.setWizardData(dataSource);
			connected = true;
			activarVisualizarBotones();
		} catch (Exception e) {
			listenerSupport.callError(e);
			e.printStackTrace();
		}
	}

	/**
	 * DOCUMENT ME!
	 */
	private void activarVisualizarBotones() {
		if (page == 0) {
			getBtnAnterior().setEnabled(false);
			getBtnSiguiente().setVisible(true);

			if (connected) {
				getBtnSiguiente().setEnabled(true);
			} else {
				getBtnSiguiente().setEnabled(false);
			}
		} else if (page < wmsParamsPanel.getNumTabs()) {
			getBtnSiguiente().setEnabled(true);
			getBtnSiguiente().setVisible(true);
			getBtnAnterior().setEnabled(true);
			getBtnAnterior().setVisible(true);
			if (wmsParamsPanel.nextEnabledPage()==-1){
			    getBtnSiguiente().setEnabled(false);
			}
			listenerSupport.callStateChanged(wmsParamsPanel.isCorrectlyConfigured());
		} else if (page == wmsParamsPanel.getNumTabs()) {
			getBtnSiguiente().setVisible(false);
			getBtnSiguiente().setEnabled(false);
			listenerSupport.callStateChanged(wmsParamsPanel 
					.isCorrectlyConfigured());
		}
	}

	/**
	 * This method initializes txtHost
	 *
	 * @return javax.swing.JTextField
	 */
	private javax.swing.JComboBox getTxtHost() {
		if (cmbHost == null) {
			cmbHost = new javax.swing.JComboBox();
			cmbHost.setModel(new DefaultComboBoxModel());
			cmbHost.setPreferredSize(new java.awt.Dimension(350, 20));
			cmbHost.setBounds(11, 26, 454, 20);
			cmbHost.setEditable(true);

	

			XMLEntity xml = PluginServices.getPluginServices(this)
										  .getPersistentXML();
			if (xml == null)
				xml = new XMLEntity();
			
			if (!xml.contains("wms-servers")) {
	               String[] servers = new String[11];
	               // Begining of rubbish code
	               //
	               // The following block should not exist and it only will live in the
	               // 0.6 version. It fixes an error that has been happening since the 0.4 version
	               // and recovers the user's server data from the wrong location that has been used to use.
	               // This code will be removed from the 0.7 version
	               boolean bool = false;
	               {
	                   Class gvSIGClass = AddLayer.class; // for example... (we only need to use a class that has been loaded by gvSIG's class loader
	                   XMLEntity xml2 = PluginServices.getPluginServices(PluginServices.getExtension(gvSIGClass)).getPersistentXML();
	                   if (xml2.contains("wms-servers")) {
	                       String oldServers[] = xml2.getStringArrayProperty("wms-servers");
	                       servers = new String[servers.length+oldServers.length];
	                       for (int i = 0; i < oldServers.length; i++) {
	                    	   servers[oldServers.length+i] = oldServers[i];
	                       }
	                       xml2.remove("wms-servers");
	                       bool = true;
	                   }
	                                 }
	               if (!bool) {
	               // end of the rubbish code (don't forget to remove the closing bracket)
	            	   
	               servers[0] = "http://www.idee.es/wms/IDEE-Base/IDEE-Base";
	               servers[1] = "http://wms.jpl.nasa.gov/wms.cgi";
	               servers[2] = "http://www2.dmsolutions.ca/cgi-bin/mswms_gmap?";
	               servers[3] = "http://demo.deegree.org:8080/deegree/wms";
	               servers[4] = "http://orto.cth.gva.es/wmsconnector/com.esri.wms.Esrimap/wms_patricova";
	               servers[5] = "http://orto.cth.gva.es/wmsconnector/com.esri.wms.Esrimap/wms_parque_fondo_prug";
	               servers[6] = "http://orto.cth.gva.es/wmsconnector/com.esri.wms.Esrimap/wms_pgof";
	               servers[7] = "http://orto.cth.gva.es/wmsconnector/com.esri.wms.Esrimap/wms_urbanismo_tematicos";
	               servers[8] = "http://onearth.jpl.nasa.gov/wms.cgi";
	               servers[9] = "http://www.demis.nl/wms/wms.asp?WMS=WorldMap";
	               servers[9] = "http://aes.gsfc.nasa.gov/cgi-bin/wms";
	               servers[10]= "http://mapas.euitto.upm.es/cgi-bin/cnauticas";

	               } // this bracket will be removed too in version <= 0.7
	               xml.putProperty("wms-servers", servers);
	           } 
			try {
				String[] servers = xml.getStringArrayProperty("wms-servers");
				for (int i = 0; i < servers.length; i++) {
					((DefaultComboBoxModel) cmbHost.getModel()).addElement(servers[i]);
				}
			} catch (NotExistInXMLEntity e) {
			}
		}

		return cmbHost;
	}

	/**
	 * This method initializes btnDetalles
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnConnect() {
		if (btnConnect == null) {
			btnConnect = new javax.swing.JButton();
			btnConnect.setPreferredSize(new java.awt.Dimension(100, 20));
			btnConnect.setBounds(366, 50, 100, 20);
			btnConnect.setText(PluginServices.getText(this, "conectar"));
			btnConnect.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						rellenarControles();
					}
				});
		}

		return btnConnect;
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new javax.swing.JPanel();
			jPanel.setLayout(null);
			jPanel.setPreferredSize(new java.awt.Dimension(470, 130));
			jPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "description"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			jPanel.setBounds(2, 96, 477, 324);
			jPanel.add(getJLabel1(), null);
			jPanel.add(getLblTitle(), null);
			jPanel.add(getJScrollPane(), null);
		}

		return jPanel;
	}

	/**
	 * This method initializes jLabel1
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel1() {
		if (jLabel1 == null) {
			jLabel1 = new javax.swing.JLabel();
			jLabel1.setText(PluginServices.getText(this, "name") + ":");
			jLabel1.setBounds(37, 26, 40, 15);
		}

		return jLabel1;
	}

	/**
	 * This method initializes lblTitle
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getLblTitle() {
		if (lblTitle == null) {
			lblTitle = new javax.swing.JLabel();
			lblTitle.setText("-");
			lblTitle.setPreferredSize(new java.awt.Dimension(350, 16));
			lblTitle.setBounds(82, 26, 350, 16);
		}

		return lblTitle;
	}

	/**
	 * This method initializes jScrollPane
	 *
	 * @return javax.swing.JScrollPane
	 */
	private javax.swing.JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new javax.swing.JScrollPane();
			jScrollPane.setViewportView(getTxtAbstract());
			jScrollPane.setPreferredSize(new java.awt.Dimension(450, 60));
			jScrollPane.setBounds(10, 47, 450, 267);
		}

		return jScrollPane;
	}

	/**
	 * This method initializes txtAbstract
	 *
	 * @return javax.swing.JTextArea
	 */
	private javax.swing.JTextArea getTxtAbstract() {
		if (txtAbstract == null) {
			txtAbstract = new javax.swing.JTextArea();
			txtAbstract.setWrapStyleWord(true);
			txtAbstract.setColumns(30);
			txtAbstract.setLineWrap(true);
		}

		return txtAbstract;
	}

	/**
	 * This method initializes panelPage1
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getPanelPage1() {
		if (panelPage1 == null) {
			panelPage1 = new javax.swing.JPanel();
			panelPage1.setLayout(null);
			panelPage1.setPreferredSize(new java.awt.Dimension(480, 220));
			panelPage1.setVisible(true);
			panelPage1.setBounds(15, 5, 480, 427);
			panelPage1.add(getJPanel1(), null);
			panelPage1.add(getJPanel(), null);
		}

		return panelPage1;
	}

	/**
	 * This method initializes btnSiguiente
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnSiguiente() {
		if (btnSiguiente == null) {
			btnSiguiente = new javax.swing.JButton();
			btnSiguiente.setPreferredSize(new java.awt.Dimension(100, 30));
			btnSiguiente.setBounds(395, 444, 100, 20);
			btnSiguiente.setText(PluginServices.getText(this, "siguiente"));
			btnSiguiente.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						if (page > 0) {
							// si page es mayor que cero, vamos enfocando las
							// sucesivas SOLAPAS del WCSParamsPanel
							wmsParamsPanel.avanzaTab();
							page = wmsParamsPanel.currentPage();
						}
						page++;
						getPanelPage1().setVisible(false);
						getPanelPage2().setVisible(true);
						activarVisualizarBotones();
					}
				});
		}
		return btnSiguiente;
	}

	/**
	 * This method initializes btnAnterior
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnAnterior() {
		if (btnAnterior == null) {
			btnAnterior = new javax.swing.JButton();
			btnAnterior.setText(PluginServices.getText(this, "anterior"));
			btnAnterior.setBounds(292, 444, 100, 20);
			btnAnterior.setPreferredSize(new java.awt.Dimension(100, 30));
			btnAnterior.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					page --;
					if (page > 0) {
						// Tenemos que retroceder en el wcsParamsPanel
						wmsParamsPanel.retrocedeTab();
						page = wmsParamsPanel.currentPage() + 1;
						activarVisualizarBotones();
					} else if (page==0){
						activarVisualizarBotones();
						page = 1;
						//wmsParamsPanel.limpiaWizard();
						getLblTitle().setText("-");
						getTxtAbstract().setText("");
						wmsParamsPanel.retrocedeTab();
						getPanelPage1().setVisible(true);
						getPanelPage2().setVisible(false);
						getBtnSiguiente().setEnabled(false);
						connected = false; 
					}
					listenerSupport.callStateChanged(wmsParamsPanel.isCorrectlyConfigured());
				}
			});
		}

		return btnAnterior;
	}

	/**
	 * This method initializes jPanel1
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new javax.swing.JPanel();
			jPanel1.setLayout(null);
			jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "server"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			jPanel1.setBounds(2, 5, 477, 85);
			jPanel1.add(getTxtHost(), null);
			jPanel1.add(getBtnConnect(), null);
			jPanel1.add(getChkCaching(), null);
		}

		return jPanel1;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param listener
	 */
	public void addWizardListener(WizardListener listener) {
		listenerSupport.addWizardListener(listener);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param listener
	 */
	public void removeWizardListener(WizardListener listener) {
		listenerSupport.removeWizardListener(listener);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return
	 */
//	public WMSWizardDataSource getDataSource() {
//		return dataSource;
//	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param source
	 */
//	public void setDataSource(WMSWizardDataSource source) {
//		dataSource = source;
//	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getSRS() {
		return wmsParamsPanel.getSRS();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getFormat() {
		return wmsParamsPanel.getFormat();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public TreePath[] getSelectedLayers() {
		return wmsParamsPanel.getSelectedLayers();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public Rectangle2D getLayersRectangle() throws CannotReprojectException{
		return wmsParamsPanel.getLayersRectangle();
	}



	/**
	 * Devuelve el host que est� escrito en el cuadro de texto del host o null
	 * en caso de que no haya nada escrito o est� mal escrito
	 *
	 * @return URL del host
	 */
	public URL getHost() {
		try {
			return new URL(cmbHost.getModel().getSelectedItem().toString());
		} catch (MalformedURLException e) {
			return null;
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return Nombre de la capa que aparece en el TOC.
	 */
	public String getLayerName() {
		return wmsParamsPanel.getLayerName();
	}


	public void initWizard() {
		dataSource = new WMSWizardData();
	}

	public void execute() {
       
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	private String getLayersQuery() {
		return wmsParamsPanel.getLayersQuery();
	}
	
//    public FLayer _getLayer() {
//        View v = (View) PluginServices.getMDIManager().getActiveView();
//        if (true) {
//            FLayers l = new FLayers(v.getMapControl().getMapContext(), null);
//            l.setName("composite");
//            String[] layerNames = getLayersQuery().split(",");
//            for (int i = 0; i < layerNames.length; i++) {
//                FLyrWMS layer = new FLyrWMS();
//                layer.setHost(getHost());
//                try {
//					layer.setFullExtent(getLayersRectangle());
//				} catch (CanNotReprojectException e) {					
//					e.printStackTrace();
//				}
//                layer.setFormat(getFormat());
//                layer.setLayerQuery(layerNames[i]);
//                layer.setInfoLayerQuery(getQueryableLayerQuery());
//                layer.setSRS(getSRS());
//                layer.setName(layerNames[i]);
//                layer.setWmsTransparency(getTransparency());
//                layer.setStyles(getStyles());
//                l.addLayer(layer);
//            }
//            return l;
//        } else
//            return _getLayer();
//        
//    }

    public FLayer getLayer(){
        FLyrWMS layer = new FLyrWMS();
        layer.setHost(getHost());
        try{
        	layer.setFullExtent(getLayersRectangle());
        }catch(CannotReprojectException ex)
        {
        	ex.printStackTrace();
        	JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(), ex.getMessage());
        	return null;
        }
        layer.setFormat(getFormat());
        layer.setLayerQuery(getLayersQuery());
        layer.setInfoLayerQuery(getQueryableLayerQuery());
        layer.setSRS(getSRS());
        layer.setName(getLayerName());
        layer.setWmsTransparency(getTransparency());
        layer.setStyles(getStyles());
        layer.setDimensions(getDimensions());
        layer.setDriver(getDriver());
        layer.setOnlineResources(getOnlineResources());
        layer.setFixedSize(getFixedSize());
        layer.setQueryable(isQueryable());
        return layer;
    }

    private boolean isQueryable() {
		return dataSource.isQueryable();
	}

	private Dimension getFixedSize() {
    	return wmsParamsPanel.getFixedSize();
    }
	
	private Hashtable getOnlineResources() {
		return wmsParamsPanel.getOnlineResources();
	}

	private FMapWMSDriver getDriver() {
		return wmsParamsPanel.getDriver();
	}

	/**
     * @return
     */
    private Vector getStyles() {
        return wmsParamsPanel.getStyles();
    }
    
    private Vector getDimensions() {
        return wmsParamsPanel.getDimensions();
    }

    /**
     * @return
     */
    private boolean getTransparency() {
        return wmsParamsPanel.getTransparency();
    }

    /**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getQueryableLayerQuery() {
		return wmsParamsPanel.getQueryableLayerQuery();
	}

	/**
	 * This method initializes chkCaching	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getChkCaching() {
		if (chkCaching == null) {
			chkCaching = new JCheckBox();
			chkCaching.setBounds(7, 51, 349, 20);
			chkCaching.setText(PluginServices.getText(this, "refresh_capabilities"));
			chkCaching.setToolTipText(PluginServices.getText(this, "refresh_capabilities_tooltip"));
			chkCaching.setSelected(refreshing);
			chkCaching.addItemListener(new java.awt.event.ItemListener() { 
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					refreshing = chkCaching.isSelected();
				}
			});
			chkCaching.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					fPrefs.putBoolean("refresh_capabilities", chkCaching.isSelected());
				}
			});
			
		}
		return chkCaching;
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"
