package com.iver.andami.persistence.serverData;
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: ServerDataPersistence.java,v 1.1.2.1 2006/03/16 12:40:40 jorpiell Exp $
 * $Log: ServerDataPersistence.java,v $
 * Revision 1.1.2.1  2006/03/16 12:40:40  jorpiell
 * Aadida la clase ServerDataPersistence a andami
 *
 * Revision 1.1  2006/03/07 11:32:13  jorpiell
 * Se ha aadido la clase ServerDataPersistence en el paquete persistence.serverData que se usa para persistir los servidores por orden de ltimo acceso.
 *
 *
 */

import java.util.Date;

import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.View;
import com.iver.utiles.NotExistInXMLEntity;
import com.iver.utiles.XMLEntity;

import com.iver.utiles.DateTime;
import com.iver.utiles.swing.jcomboServer.ServerData;

/**
 * This class is used to save a list of servers (using the 
 * Andami persistence model) to the plugins-persistence.xml file.
 * It has methods to create a set of ServerData objects  from an 
 * xml file. It can also save a set of ServerData objects in an 
 * xml file.
 * 
 * @see es.gva.cit.catalogClient.utils.comboserver.ServerData
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class ServerDataPersistence {
	private XMLEntity xml = null;
	private PluginServices ps = null;
	private String serviceType = null;

	/**
	 * Constructor
	 * @param view
	 * The View Class
	 * @param sevice Type
	 * Service type to load
	 */	
	public ServerDataPersistence(Object pluginClassInstance,String serviceType){
		ps = PluginServices.getPluginServices(pluginClassInstance);
		xml = ps.getPersistentXML();
		this.serviceType = serviceType;		
	}

	/**
	 * This methos is used to save the information in an XML file
	 */
	public void setPersistent(){
		ps.setPersistentXML(xml);
	}

	/**
	 * This method saves an array of ServerData using the Anadami 
	 * persistence model
	 * @param servers
	 * Array of servers
	 */
	public void setArrayOfServerData(ServerData[] servers){
		xml.getXmlTag().removeAllXmlTag();

		for (int i=0 ; i<servers.length ; i++){
			xml.addChild(serverDataToXml(servers[i]));
		}

	}

	/**
	 * This method adds a ServerData using the Anadami 
	 * persistence model. If the server exists just actualizes
	 * the type and subtype fileds and changes the last access 
	 * value to the current time. 
	 * @param server
	 * ServerData
	 */
	public void addServerData(ServerData server){
		ServerData[] servers = getArrayOfServerData();
		ServerData[] newServers = new ServerData[servers.length + 1];

		boolean found = false;

		for (int i = 0; i < servers.length; i++) {
			if (servers[i].getServerAddress().equals(server.getServerAddress())) {
				found = true;
				servers[i].updateLastAccess();
				servers[i].setServiceSubType(server.getServiceSubType());
				servers[i].setServiceType(server.getServiceType());
				servers[i].setDatabase(server.getDatabase());
				setArrayOfServerData(servers);
			}
		}

		if (!found) {
			System.arraycopy(servers, 0, newServers, 0, servers.length);
			newServers[servers.length] = server;
			setArrayOfServerData(newServers);

		}        
	} 
	
	

	/**
	 * This method returns an array of ServerData objects that 
	 * have been saved using the Andami persistence model.
	 * @return
	 * String[]
	 */
	public ServerData[] getArrayOfServerData(){
		ServerData[] servers = new ServerData[xml.getNumChild()];
		for (int i=0 ; i<xml.getNumChild() ; i++){
			servers[i] = xmlToServerData(xml.getChild(i));
		}       
		if (servers.length == 0){
			return retrieveOldServerData();
		}
		return servers;
	} 

	/**
	 * Just retrieve the servers with the old format
	 * ant convert them to the new format
	 * @return
	 */
	private ServerData[] retrieveOldServerData(){
		ServerData[] servers;
		try {
			String[] sServers = xml.getStringArrayProperty("wms-servers");
			servers = new ServerData[sServers.length];
			for (int i = 0; i < servers.length; i++) {
				servers[i] = new ServerData(sServers[i],serviceType);
			}			
		} catch (NotExistInXMLEntity e) {
			servers = new ServerData[0];
		}
		return servers;
	}

	/**
	 * This method creates and returns a new XMLEntity.
	 * @param server
	 * ServerData with all the server information
	 * @return
	 * XMLEntity
	 */
	public XMLEntity serverDataToXml(ServerData server){
		String dFormat="Y-m-d H:i:s.Z";

		XMLEntity xmlEnt = new XMLEntity(); 
		xmlEnt.putProperty("serverURL",server.getServerAddress());
		xmlEnt.putProperty("added",DateTime.dateToString(server.getAdded(),dFormat));
		xmlEnt.putProperty("lastAccess",DateTime.dateToString(server.getLastAccess(),dFormat));
		xmlEnt.putProperty("type",server.getServiceType());
		xmlEnt.putProperty("subType",server.getServiceSubType());
		if (server.getServiceSubType().equals(ServerData.SERVER_SUBTYPE_CATALOG_Z3950)){
			xmlEnt.putProperty("database",server.getDatabase());
		}
		return xmlEnt;
	}

	/**
	 * This method creates a new serverData from a XMLEntity
	 * @param xmlEnt
	 * XMLRntity that contains the server information
	 * @return
	 * ServerData
	 */
	public ServerData xmlToServerData(XMLEntity xmlEnt){
		String serverAddress;
		Date added;
		Date lastAccess;
		String serviceType = "";
		String serviceSubType = "";
		String databaseName = "";

		serverAddress = xmlEnt.getStringProperty("serverURL");
		added = DateTime.stringToDate(xmlEnt.getStringProperty("added"));
		lastAccess = DateTime.stringToDate(xmlEnt.getStringProperty("lastAccess"));
		serviceType = xmlEnt.getStringProperty("type").toUpperCase();
		serviceSubType = xmlEnt.getStringProperty("subType").toUpperCase();

		if (serviceSubType.equals(ServerData.SERVER_SUBTYPE_CATALOG_Z3950)){
			try{
				databaseName = xmlEnt.getStringProperty("database");
			}catch(NotExistInXMLEntity e){

			}
		}
		return new ServerData(serverAddress,added,lastAccess,serviceType,serviceSubType,databaseName);   
	}    
}
