/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.andami.ui.mdiManager;

import java.beans.PropertyVetoException;

import com.iver.andami.ui.mdiFrame.MDIFrame;


/**
 * Las vistas que se aaden al framework derivan de JPanel. Como pasamos de un
 * JPanel al control que vemos (habitualmente un JInternalFrame) Mediante la
 * clase skin que implementa esta interfaz. El framework  invoca los mtodos
 * de la clase pasando como parmetros jPanel y la clase toma el JPanel, lo
 * trata (en la version por defecto lo mete en un JInternalFrame) y lo aade
 * al Frame principal. Para poner un ejemplo se podra implementar ua clase
 * que situara los jpanel en la forma que lo hace eclipse, o instalar un men
 * por cada vista de forma que al seleccionar el men  desapareciera la vista
 * actual y se mostrara la que  se ha seleccionado, ...     Actualmente la
 * clase que implementa esta interfaz es Skin y se usa a pin fijo, pero en
 * un futuro se dejar abierta la posibilidad de sustituir esta clase por una
 * que  implemente el usuario.
 *
 * @author Fernando Gonzlez Corts
 */
public interface MDIManager {
    /**
     * Inicializacin de la clase, se le pasa la consola para mostrar mensajes
     * y el frame principal para actuar sobre l
     *
     * @param f DOCUMENT ME!
     */
    public void init(MDIFrame f);

    /**
     * Aade una nueva vista a la interfaz y lo visualiza. Dicho vista se
     * elimina completamente al cerrarse.
     *
     * @param p Panel con los controles de la vista
     *
     * @return Devuelve la vista aadida o si es un singleton y ya estaba
     *         aadida devuelve la vista ya aadida
     *
     * @throws SingletonDialogAlreadyShownException Si lo que se est aadiendo
     *         es una vista modal singleton y ya est aadida
     */
    public View addView(View p) throws SingletonDialogAlreadyShownException;

    /**
     * Obtiene una referencia al panel de la ultima vista no modal que se
     * activ en la aplicacin.
     *
     * @return referencia al panel activo o null si no hay ninguna vista activa
     */
    public View getActiveView();

    /**
     * Obtiene una referencia a los paneles de todas las vista no modales que
     * hay en la aplicacin en la aplicacin.
     *
     * @return Un vector con todos los paneles que contiene la aplicacin.
     */
    public View[] getAllViews();

    /**
     * Cierra la ventana SingletonView cuya clase se pasa como parmetro y cuyo
     * modelo se pasa como parmetro.
     *
     * @param viewClass Clase de la vista que se quiere cerrar
     * @param model Modelo de la vista que se quiere cerrar
     *
     * @return true si hay abierta una vista con la clase viewClass y con el
     *         modelo que se pasa como parmetro. False si no hay ninguna
     *         ventana de ese tipo.
     */
    public boolean closeSingletonView(Class viewClass, Object model);

    /**
     * Cierra todas las ventanas Singleton que tengan como modelo el que se
     * pasa como parmetro
     *
     * @param model Modelo de las ventanas que se quieren cerrar
     *
     * @return true si se cierra alguna ventana y false si no hay abierta
     *         ninguna ventana singleton con ese modelo
     */
    public boolean closeSingletonView(Object model);

    /**
     * Cierra la vista correspondiente con el JPanel
     *
     * @param p vista que se quiere cerrar
     */
    public void closeView(View p);

    /**
     * Cierra todas las vistas que actualmente estn abiertas
     */
    public void closeAllViews();

    /**
     * Obtiene la referencia al objeto que contiene la informacin de la vista
     * que se pasa como parmetro
     *
     * @param v vista cuya informacin se requiere
     *
     * @return ViewInfo
     */
    public ViewInfo getViewInfo(View v);

    /**
     * Pone como icono de ratn el HourGlass y bloquea todos los eventos de la
     * interfaz que se producen hasta que se recibe una llamada a
     * restoreCursor
     */
    public void setWaitCursor();

    /**
     * Restaura al icono de ratn que haba antes de realizar la operacin
     */
    public void restoreCursor();
    
    /**
     * Maximiza o no la ventana que contiene la vista v
     * @param v
     * @param bMaximum
     * @throws PropertyVetoException
     */
    public void setMaximum(View v, boolean bMaximum) throws PropertyVetoException;
}
