/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.andami.ui.mdiManager;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;


/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class ViewInfo {
    /** DOCUMENT ME! */
    public static final int RESIZABLE = 1;

    /** DOCUMENT ME! */
    public static final int MAXIMIZABLE = 2;

    /** DOCUMENT ME! */
    public static final int ICONIFIABLE = 4;

    /** DOCUMENT ME! */
    public static final int MODALDIALOG = 8;
    public static final int MODELESSDIALOG = 16;
    public static final int PALETTE = 32;
    private PropertyChangeSupport support = new PropertyChangeSupport(this);

    /** DOCUMENT ME! */
    private boolean resizable = false;

    /** DOCUMENT ME! */
    private boolean maximizable = false;

    /** DOCUMENT ME! */
    private boolean iconifiable = false;

    /** DOCUMENT ME! */
    private boolean modal = false;
    private boolean modeless = false;
    
    /**
     * Se usa para poner una ventana de tipo paleta, por
     * encima de las dems. Equivale a poner usar
     * JDesktopPane.PALETTE_LAYER
     */
    private boolean palette = false;

    /** DOCUMENT ME! */
    private int width = -1;

    /** DOCUMENT ME! */
    private int height = -1;
    private int x = 0;
    private int y = 0;
    private boolean visible = true;

    /** DOCUMENT ME! */
    private String title;
    
    private int id;
    
    /**
     * ActionCommand del tool seleccionado. Lo usamos
     * para activar el tool que estaba seleccionado en
     * la vista, cuando volvemos a ella.
     */
    private String selectedTool;

    /**
     * DOCUMENT ME!
     *
     * @return Returns the x.
     */
    public int getX() {
        return x;
    }

    /**
     * DOCUMENT ME!
     *
     * @param x The x to set.
     */
    public void setX(int x) {
    	support.firePropertyChange("x", this.x, x);
    	this.x = x;
    }

    /**
     * DOCUMENT ME!
     *
     * @return Returns the y.
     */
    public int getY() {
        return y;
    }

    /**
     * DOCUMENT ME!
     *
     * @param y The y to set.
     */
    public void setY(int y) {
    	support.firePropertyChange("y", this.y, y);
        this.y = y;
    }

    /**
     * DOCUMENT ME!
     *
     * @param listener
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        support.addPropertyChangeListener(listener);
    }

    /**
     * DOCUMENT ME!
     *
     * @param listener
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        support.removePropertyChangeListener(listener);
    }

    /**
     * Establece las propiedades de la vista
     *
     * @param code Bit-or de las propiedades de la vista
     *
     * @throws IllegalStateException DOCUMENT ME!
     */
    public ViewInfo(int code) {
        resizable = (code % 2) > 0;
        code = code / 2;
        maximizable = (code % 2) > 0;
        code = code / 2;
        iconifiable = (code % 2) > 0;
        code = code / 2;
        modal = (code % 2) > 0;
        code = code / 2;
        modeless = (code % 2) > 0;
        code = code / 2;
        palette = (code % 2) > 0;

        if (modal && modeless) {
            throw new IllegalStateException("modal && modeless");
        }
    }
    
    public ViewInfo(){
    	
    }

    /**
     * Si es iconificable
     *
     * @return
     */
    public boolean isIconifiable() {
        return iconifiable;
    }

    /**
     * Si es maximizable
     *
     * @return
     */
    public boolean isMaximizable() {
        return maximizable;
    }

    /**
     * Si es resizable
     *
     * @return
     */
    public boolean isResizable() {
        return resizable;
    }

    /**
     * Devuelve si el dilogo es modal
     *
     * @return
     */
    public boolean isModal() {
        return modal;
    }

    /**
     * Obtiene la altura de la vista
     *
     * @return
     */
    public int getHeight() {
        return height;
    }

    /**
     * Obtiene la anchura de la vista
     *
     * @return
     */
    public int getWidth() {
        return width;
    }

    /**
     * establece la propiedad altura
     *
     * @param i
     */
    public void setHeight(int i) {
    	support.firePropertyChange("height", this.height, i);
        height = i;
    }

    /**
     * establece la propiedad anchura
     *
     * @param i
     */
    public void setWidth(int i) {
    	support.firePropertyChange("width", this.width, i);
        width = i;
    }

    /**
     * obtiene la propiedad titulo
     *
     * @return
     */
    public String getTitle() {
        return title;
    }

    /**
     * establece la propiedad titulo
     *
     * @param string
     */
    public void setTitle(String string) {
    	support.firePropertyChange("title", this.title, string);
        title = string;
    }

    /**
     * DOCUMENT ME!
     *
     * @return
     */
    public boolean isModeless() {
        return modeless;
    }

    /**
     * DOCUMENT ME!
     *
     * @return
     */
    public boolean isVisible() {
        return visible;
    }
    
    public boolean isPalette()
    {
        return palette;
    }
    
    public void setId(int id){
    	this.id = id;
    }
    
    public int getId(){
    	return id;
    }
    
    public void setViewInfo(ViewInfo vi){
    	this.resizable = vi.resizable;
    	this.maximizable = vi.maximizable;
    	this.iconifiable = vi.iconifiable;
    	this.modal = vi.modal;
    	this.modeless = vi.modeless;
    	this.width = vi.width;
    	this.height = vi.height;
    	this.x = vi.x;
    	this.y = vi.y;
    	this.visible = vi.visible;
    	this.title = vi.title;
    	this.id = vi.id;
    }

    public String getSelectedTool() {
        return selectedTool;
    }

    public void setSelectedTool(String selectedTool) {
        if (selectedTool != null)
            this.selectedTool = selectedTool;
    }
}
