/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.core.mdiManager;

import java.awt.Component;
import java.awt.Image;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;

import com.iver.andami.ui.mdiFrame.MDIFrame;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;


/**
 *
 */
public class FrameViewSupport {
    private Hashtable frameView = new Hashtable();
    private Hashtable viewFrame = new Hashtable();
    private Image icon;
    private ViewInfoSupport vis;
	private JFrame mainFrame;

    /**
     * Creates a new FrameViewSupport object.
     *
     * @param i DOCUMENT ME!
     */
    public FrameViewSupport(MDIFrame mainFrame) {
    	this.mainFrame = mainFrame;
        icon = mainFrame.getIconImage();
    }

    public Iterator getViewIterator(){
    	return viewFrame.keySet().iterator();
    }
    
    public boolean contains(View v){
    	return viewFrame.containsKey(v);
    }

	/**
	 * @param wnd
	 * @return
	 */
	public boolean contains(JInternalFrame wnd) {
		return frameView.contains(wnd);
	}
    
    /**
     * DOCUMENT ME!
     *
     * @param p DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public JDialog getJDialog(View p) {
        JDialog dlg = (JDialog) viewFrame.get(p);

        if (dlg == null) {
            ViewInfo vi = vis.getViewInfo(p);
            JDialog nuevo = new JDialog(mainFrame);

            nuevo.getContentPane().add((JPanel) p);
            nuevo.setSize(getWidth(p), getHeight(p) + 30);
            nuevo.setTitle(vi.getTitle());
            nuevo.setResizable(vi.isResizable());

            viewFrame.put(p, nuevo);
            frameView.put(nuevo, p);

            nuevo.setModal(vi.isModal());
            return nuevo;
        } else {
            return dlg;
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param p DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public JInternalFrame getJInternalFrame(View p) {
        JInternalFrame frame = (JInternalFrame) viewFrame.get(p);

        if (frame == null) {
        	ViewInfo vi = vis.getViewInfo(p);
            JInternalFrame nuevo = createJInternalFrame(p);
            viewFrame.put(p, nuevo);
            frameView.put(nuevo, p);

            return nuevo;
        } else {
            return frame;
        }
    }

    public JInternalFrame createJInternalFrame(View p)
    {
        ViewInfo vi = vis.getViewInfo(p);
        JInternalFrame nuevo = new JInternalFrame();
        if (icon != null){
            nuevo.setFrameIcon(new ImageIcon(icon));
        }
        nuevo.getContentPane().add((JPanel) p);
        nuevo.setClosable(true);
        nuevo.setSize(getWidth(p), getHeight(p));
        nuevo.setTitle(vi.getTitle());
        nuevo.setVisible(vi.isVisible());
        nuevo.setResizable(vi.isResizable());
        nuevo.setIconifiable(vi.isIconifiable());
        nuevo.setMaximizable(vi.isMaximizable());
        nuevo.setLocation(vi.getX(), vi.getY());

        nuevo.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
        return nuevo;
    }
    
    public View getView(Component dlg){
    	return (View) frameView.get(dlg);
    }
    
    public void closeView(View v){
    	Object c = viewFrame.remove(v);
    	frameView.remove(c);
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param x DOCUMENT ME!
     */
    public void setX(View v, int x) {
    	JInternalFrame o = (JInternalFrame) viewFrame.get(v);
        o.setLocation(x, o.getY());
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param y DOCUMENT ME!
     */
    public void setY(View v, int y) {
    	JInternalFrame o = (JInternalFrame) viewFrame.get(v);

        o.setLocation(o.getX(), y);
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    public void setHeight(View v, int height) {
    	JInternalFrame o = (JInternalFrame) viewFrame.get(v);

        o.setSize(o.getWidth(), height);
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param width DOCUMENT ME!
     */
    public void setWidth(View v, int width) {
    	JInternalFrame o = (JInternalFrame) viewFrame.get(v);

    	o.setSize(width, o.getHeight());
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param title DOCUMENT ME!
     */
    public void setTitle(View v, String title) {
    	JInternalFrame o = (JInternalFrame) viewFrame.get(v);
        o.setTitle(title);
    }

    /**
     * DOCUMENT ME!
     *
     * @param vis The vis to set.
     */
    public void setVis(ViewInfoSupport vis) {
        this.vis = vis;
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    private int getWidth(View v) {
        ViewInfo vi = vis.getViewInfo(v);

        if (vi.getWidth() == -1) {
            JPanel p = (JPanel) v;

            return p.getSize().width;
        } else {
            return vi.getWidth();
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    private int getHeight(View v) {
        ViewInfo vi = vis.getViewInfo(v);

        if (vi.getHeight() == -1) {
            JPanel p = (JPanel) v;

            return p.getSize().height;
        } else {
            return vi.getHeight();
        }
    }
}
