/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.core.mdiManager;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Stack;

import javax.swing.DefaultDesktopManager;
import javax.swing.DesktopManager;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JInternalFrame;
import javax.swing.JLayeredPane;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import org.apache.log4j.Logger;

import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiFrame.GlassPane;
import com.iver.andami.ui.mdiFrame.MDIFrame;
import com.iver.andami.ui.mdiManager.MDIManager;
import com.iver.andami.ui.mdiManager.MDIUtilities;
import com.iver.andami.ui.mdiManager.SingletonDialogAlreadyShownException;
import com.iver.andami.ui.mdiManager.SingletonView;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.andami.ui.mdiManager.ViewListener;

/**
 *
 */
public class NewSkin implements MDIManager, Extension {
    /**
     * Variable privada <code>desktopManager</code> para usarlo cuando sale
     * una ventana que no queremos que nos restaure las que tenemos maximizadas.
     * Justo antes de usar el setMaximize(false), le pegamos el cambiazo.
     */
    private static DesktopManager desktopManager = new DefaultDesktopManager();

    /** log */
    private static Logger logger = Logger.getLogger(NewSkin.class.getName());

    /** Panel de la MDIFrame */
    private JDesktopPane panel = new JDesktopPane();

    /** MDIFrame */
    private MDIFrame mainFrame;

    private GlassPane glassPane = new GlassPane();

    private DialogStackSupport dss;

    private FrameViewSupport fvs;

    private ViewInfoSupport vis;

    private ViewStackSupport vss;

    private SingletonViewSupport svs;

    private Cursor lastCursor = null;

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#init(com.iver.andami.ui.mdiFrame.MDIFrame)
     */
    public void init(MDIFrame f) {
        // Inicializa el Frame y la consola
        mainFrame = f;
        mainFrame.setGlassPane(glassPane);
        panel.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);

        mainFrame.getContentPane().add(panel, BorderLayout.CENTER);
        panel.setDesktopManager(desktopManager);

        fvs = new FrameViewSupport(mainFrame);
        dss = new DialogStackSupport(mainFrame);
        svs = new SingletonViewSupport(vis, fvs);
        vis = new ViewInfoSupport(mainFrame, fvs, svs);
        fvs.setVis(vis);
        vss = new ViewStackSupport(vis);
    }

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#addView(com.iver.andami.ui.mdiManager.View)
     */
    public View addView(View p) throws SingletonDialogAlreadyShownException {
        // se obtiene la informacin de la vista
        ViewInfo vi = vis.getViewInfo(p);

        // Se comprueban las incompatibilidades que pudieran haber en la vista
        MDIUtilities.checkViewInfo(vi);
        if ((p instanceof SingletonView) && (vi.isModal())) {
            throw new RuntimeException("A modal view cannot be a SingletonView");
        }

        /*
         * Se obtiene la referencia a la vista anterior por si es una singleton
         * y est siendo mostrada. Se obtiene su informacin si ya fue mostrada
         */
        boolean singletonPreviouslyAdded = false;

        if (p instanceof SingletonView) {
            SingletonView sv = (SingletonView) p;
            if (svs.registerView(sv.getClass(), sv.getViewModel(), vis
                    .getViewInfo(sv))) {
                singletonPreviouslyAdded = true;
            }
        }

        if (singletonPreviouslyAdded) {
            // Si la vista no est actualmente abierta
            if (!svs.contains((SingletonView) p)) {
                JInternalFrame frame = fvs.getJInternalFrame(p);
                svs.openSingletonView((SingletonView) p, frame);
                addJInternalFrame(frame, vi);
                vss.add(p, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        View v = vis.getViewById(Integer.parseInt(e
                                .getActionCommand()));
                        JInternalFrame f = fvs.getJInternalFrame(v);
                        activateJInternalFrame(f);
                    }
                });
                return p;
            } else {
                // La vista est actualmente abierta
                JInternalFrame frame = (JInternalFrame) svs
                        .getFrame((SingletonView) p);
                activateJInternalFrame(frame);
                vss.setActive(p);
                return fvs.getView((JInternalFrame) frame);
            }
        } else {
            if (vi.isModal()) {
                addJDialog(p);
            } else {
                // Se sita la vista en la pila de vistas
                vss.add(p, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        View v = vis.getViewById(Integer.parseInt(e
                                .getActionCommand()));
                        JInternalFrame f = fvs.getJInternalFrame(v);
                        activateJInternalFrame(f);
                    }
                });
                addJInternalFrame(p);
            }

            return p;
        }
    }

    /**
     * DOCUMENT ME!
     * 
     * @param wnd
     *            DOCUMENT ME!
     * @param vi
     *            DOCUMENT ME!
     */
    private void addJInternalFrame(JInternalFrame wnd, ViewInfo vi) {
        wnd.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
        wnd.addInternalFrameListener(new FrameListener());

        if (vi.isModeless() || vi.isPalette()) {
            panel.add(wnd, JDesktopPane.PALETTE_LAYER);
            if (vi.isPalette())
                wnd.setFocusable(false);
        } else {
            panel.add(wnd);
        }

        activateJInternalFrame(wnd);
    }

    /**
     * DOCUMENT ME!
     * 
     * @param p
     */
    private void addJInternalFrame(View p) {
        ViewInfo vi = vis.getViewInfo(p);

        JInternalFrame wnd = fvs.getJInternalFrame(p);

        if (p instanceof SingletonView) {
            SingletonView sv = (SingletonView) p;
            svs.openSingletonView(sv, wnd);
        }

        addJInternalFrame(wnd, vi);
    }

    /**
     * DOCUMENT ME!
     * 
     * @param wnd
     */
    private void activateJInternalFrame(JInternalFrame wnd) {
        try {
            wnd.moveToFront();
            logger.debug("Activando " + wnd.getTitle());
            wnd.setSelected(true);
            wnd.setIcon(false);
        } catch (PropertyVetoException e) {
            logger.error(e);
        }
    }

    /**
     * Situa un dilogo modal en el centro de la pantalla
     * 
     * @param d
     *            Dilogo que se quiere situar
     */
    private void centerDialog(JDialog d) {
        int offSetX = d.getWidth() / 2;
        int offSetY = d.getHeight() / 2;

        d.setLocation((mainFrame.getWidth() / 2) - offSetX, (mainFrame
                .getHeight() / 2)
                - offSetY);
    }

    /**
     * DOCUMENT ME!
     * 
     * @param p
     */
    private void addJDialog(View p) {
        JDialog dlg = fvs.getJDialog(p);

        centerDialog(dlg);

        dlg.addWindowListener(new DialogWindowListener());
        dss.pushDialog(dlg);

        dlg.setVisible(vis.getViewInfo(p).isVisible());
    }

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#getActiveView()
     */
    public View getActiveView() {
        JInternalFrame jif = panel.getSelectedFrame();

        if (jif != null) {
            View theView = fvs.getView(jif);
            if (theView == null)
                return null;
            if (theView.getViewInfo().isPalette())
                return vss.getActiveView();
            else
                return fvs.getView(jif);
        }
        // return vss.getActiveView();

        return null;
    }

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#closeView(com.iver.andami.ui.mdiManager.View)
     */
    public void closeView(View p) {
        // Si es un dilogo modal
        if (p.getViewInfo().isModal()) {
            closeJDialog();
        } else { // Si no es modal se cierra el JInternalFrame
            closeJInternalFrame(fvs.getJInternalFrame(p));
        }
    }

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#closeAllViews()
     */
    public void closeAllViews() {
        ArrayList eliminar = new ArrayList();
        Iterator i = fvs.getViewIterator();

        while (i.hasNext()) {
            eliminar.add((View) i.next());
        }

        for (Iterator iter = eliminar.iterator(); iter.hasNext();) {
            View vista = (View) iter.next();
            closeView(vista);
        }
    }

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#getViewInfo(com.iver.andami.ui.mdiManager.View)
     */
    public ViewInfo getViewInfo(View v) {
        return vis.getViewInfo(v);
    }

    /**
     * DOCUMENT ME!
     * 
     * @param dialog
     * @throws RuntimeException
     *             DOCUMENT ME!
     */
    private void closeJDialog() {
        JDialog dlg = dss.popDialog();

        dlg.setVisible(false);

        View s = (View) fvs.getView(dlg);

        callViewClosed(s);

        fvs.closeView(s);

        // Si es singleton se desasocia el modelo con la vista
        if (s instanceof SingletonView) {
            svs.closeView((SingletonView) s);
        }
    }

    /**
     * DOCUMENT ME!
     * 
     * @param view
     *            DOCUMENT ME!
     */
    private void callViewClosed(View view) {
        if (view instanceof ViewListener) {
            ((ViewListener) view).viewClosed();
        }
    }

    /**
     * DOCUMENT ME!
     * 
     * @param view
     *            DOCUMENT ME!
     */
    private void callViewActivated(View view) {
        logger.debug("View " + view.getViewInfo().getTitle()
                + " activated (callViewActivated)");
        if (view instanceof ViewListener) {
            ((ViewListener) view).viewActivated();
        }
    }

    /**
     * DOCUMENT ME!
     * 
     * @param frame
     */
    private void closeJInternalFrame(JInternalFrame frame) {
        try {
            View s = (View) fvs.getView(frame);

            frame.setClosed(true);
            callViewClosed(s);
        } catch (PropertyVetoException e) {
            logger
                    .error(
                            "Not compatible with property veto's. Use ViewInfo instead.",
                            e);
        }
    }

    /**
     * @see com.iver.andami.plugins.Extension#inicializar()
     */
    public void inicializar() {
    }

    /**
     * @see com.iver.andami.plugins.Extension#execute(java.lang.String)
     */
    public void execute(String actionCommand) {
        if (actionCommand.equals("cascada")) {
        } else if (actionCommand.equals("mosaico")) {
        }
    }

    /**
     * @see com.iver.andami.plugins.Extension#isEnabled()
     */
    public boolean isEnabled() {
        // TODO Auto-generated method stub
        return false;
    }

    /**
     * @see com.iver.andami.plugins.Extension#isVisible()
     */
    public boolean isVisible() {
        // TODO Auto-generated method stub
        return true;
    }

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#setWaitCursor()
     */
    public void setWaitCursor() {
        if (mainFrame != null) {
            glassPane.setVisible(true);
            lastCursor = mainFrame.getCursor();
            dss.setWaitCursor();
            glassPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        }
    }

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#restoreCursor()
     */
    public void restoreCursor() {
        if (mainFrame != null) {
            glassPane.setVisible(false);
            dss.restoreCursor();
            glassPane.setCursor(lastCursor);
        }
    }

    /**
     * Listener para los eventos de cerrado de los dilogos. Tiene su razn de
     * ser en que los dilogos han de devolverse al pool cuando se cierran
     * 
     * @author Fernando Gonzlez Corts
     */
    public class DialogWindowListener extends WindowAdapter {
        /**
         * Captura el evento de cerrado de los dilogos con el fin de realizar
         * tareas de mantenimiento
         * 
         * @param e
         *            evento
         */
        public void windowClosing(WindowEvent e) {
            closeJDialog();
        }
    }

    /**
     * DOCUMENT ME!
     */
    public class FrameListener implements InternalFrameListener {
        /**
         * @see javax.swing.event.InternalFrameListener#internalFrameActivated(javax.swing.event.InternalFrameEvent)
         */
        public void internalFrameActivated(InternalFrameEvent e) {
            // logger.debug("internalFrameActivated " +
            // e.getInternalFrame().getTitle());

            // activatedInternalFramesStack.push(e.getInternalFrame());

            View panel = fvs.getView((JInternalFrame) e.getSource());

            ViewInfo vi = vis.getViewInfo(panel);
            if (vi.isPalette())
                return;

            vss.setActive(panel);

            JInternalFrame frame = fvs.getJInternalFrame(panel);

            if (!frame.isMaximizable() && frame.isMaximum()) {
                try {
                    frame.setMaximum(false);
                } catch (PropertyVetoException e1) {
                }
            }
            mainFrame.enableControls();
            if (vi.getSelectedTool() != null)
                mainFrame.setSelectedTool(vi.getSelectedTool());
            callViewActivated(panel);

        }

        /**
         * @see javax.swing.event.InternalFrameListener#internalFrameClosed(javax.swing.event.InternalFrameEvent)
         */
        public void internalFrameClosed(InternalFrameEvent e) {
        }

        /**
         * @see javax.swing.event.InternalFrameListener#internalFrameClosing(javax.swing.event.InternalFrameEvent)
         */
        public void internalFrameClosing(InternalFrameEvent e) {
            // Se elimina la memoria del JInternalFrame si no es ALWAYS_LIVE
            // logger.debug("internalFrameClosing " +
            // e.getInternalFrame().getTitle());

            JInternalFrame c = (JInternalFrame) e.getSource();
            ViewInfo vi = vis.getViewInfo((View) fvs.getView(c));

            View view = fvs.getView(c);

            boolean alwaysLive;
            if (view instanceof SingletonView) {
                svs.closeView((SingletonView) view);
            }

            fvs.closeView(view);

            panel.remove(c);

            vss.remove(view);

            if (!vi.isPalette())
                mainFrame.enableControls();
            panel.repaint();

            // Para activar el JInternalFrame desde la que hemos
            // abierto la ventana que estamos cerrando
            View lastView = vss.getActiveView();
            // La activamos
            if (lastView != null) {
                System.err.println("Devuelvo el foco a "
                        + lastView.getViewInfo().getTitle());
                JInternalFrame frame = fvs.getJInternalFrame(lastView);
                try {
                    frame.setSelected(true);
                } catch (PropertyVetoException e1) {
                    // TODO Auto-generated catch block
                    // e1.printStackTrace();
                }
                // addView(lastView);
            }

        }

        /**
         * @see javax.swing.event.InternalFrameListener#internalFrameDeactivated(javax.swing.event.InternalFrameEvent)
         */
        public void internalFrameDeactivated(InternalFrameEvent e) {
            // logger.debug("internalDeActivated " +
            // e.getInternalFrame().getTitle());
            JInternalFrame c = (JInternalFrame) e.getSource();
            View andamiView = fvs.getView(c);
            if (andamiView != null) {
                ViewInfo vi = vis.getViewInfo(andamiView);
                if (vi.isPalette())
                    return;
                vi.setSelectedTool(mainFrame.getSelectedTool());
            }

        }

        /**
         * @see javax.swing.event.InternalFrameListener#internalFrameDeiconified(javax.swing.event.InternalFrameEvent)
         */
        public void internalFrameDeiconified(InternalFrameEvent e) {
            mainFrame.enableControls();
        }

        /**
         * @see javax.swing.event.InternalFrameListener#internalFrameIconified(javax.swing.event.InternalFrameEvent)
         */
        public void internalFrameIconified(InternalFrameEvent e) {
            mainFrame.enableControls();
        }

        /**
         * @see javax.swing.event.InternalFrameListener#internalFrameOpened(javax.swing.event.InternalFrameEvent)
         */
        public void internalFrameOpened(InternalFrameEvent e) {
            // logger.debug("internalFrameOpened. Source= " +
            // e.getSource().toString());
        }
    }

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#closeSingletonView(java.lang.Class,
     *      java.lang.Object)
     */
    public boolean closeSingletonView(Class viewClass, Object model) {
        JInternalFrame frame = svs.getFrame(viewClass, model);
        if (frame == null)
            return false;
        closeJInternalFrame(frame);
        return true;
    }

    /**
     * @see com.iver.andami.ui.mdiManager.MDIManager#closeSingletonView(java.lang.Object)
     */
    public boolean closeSingletonView(Object model) {
        JInternalFrame[] frames = svs.getFrames(model);
        if (frames.length == 0)
            return false;
        for (int i = 0; i < frames.length; i++) {
            closeJInternalFrame(frames[i]);
        }
        return true;
    }

    public View[] getAllViews() {
        ArrayList views = new ArrayList();
        Iterator i = fvs.getViewIterator();

        while (i.hasNext()) {
            views.add((View) i.next());
        }
        return (View[]) views.toArray(new View[0]);
    }
    
    public void setMaximum(View v, boolean bMaximum) throws PropertyVetoException
    {
        JInternalFrame f = fvs.getJInternalFrame(v);
        f.setMaximum(bMaximum);
    }
}
