/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.cts;

import org.cresques.cts.gt2.CSDatum;
import org.cresques.cts.gt2.CSGaussPt;
import org.cresques.cts.gt2.CSLambertCC;
import org.cresques.cts.gt2.CSUTM;
import org.cresques.cts.gt2.CoordSys;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;


/**
 * Pool de proyeccions (cs+datum) conocidas.
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class ProjectionPool {
    static TreeMap data = null;

    static {
        CoordSys cs = null;
        data = new TreeMap();

        cs = (new CSUTM(CSDatum.wgs84, 30)).toGeo();
        cs.setAbrev("EPSG:4326"); // WGS84 (World Geodesic Datum)
        data.put(cs.getAbrev(), cs);

        cs = (new CSUTM(CSDatum.ed50, 30)).toGeo();
        cs.setAbrev("EPSG:4230"); // Datum Europeu Internacional ED50
        data.put(cs.getAbrev(), cs);

        cs = (new CSUTM(CSDatum.d73, 30)).toGeo();
        cs.setAbrev("EPSG:4274"); // Datum 73 de Lisboa
        data.put(cs.getAbrev(), cs);

        cs = (new CSUTM(CSDatum.nad27, 30)).toGeo();
        cs.setAbrev("EPSG:4267"); // NAD 27
        data.put(cs.getAbrev(), cs);

        cs = (new CSUTM(CSDatum.nad83, 30)).toGeo();
        cs.setAbrev("EPSG:4269"); // NAD 83
        data.put(cs.getAbrev(), cs);

        cs = (new CSUTM(CSDatum.lomaQuintana, 30)).toGeo();
        cs.setAbrev("EPSG:4288"); // PSAD 56 'Loma Quintana'
        data.put(cs.getAbrev(), cs);

        cs = (new CSUTM(CSDatum.laCanoa, 30)).toGeo();
        cs.setAbrev("EPSG:4247"); // PSAD 56 'Loma Quintana'
        data.put(cs.getAbrev(), cs);

        for (int i = 1; i <= 60; i++) {
            String huso = Integer.toString(i);

            if (i < 10) {
                huso = "0" + huso;
            }

            cs = new CSUTM(CSDatum.wgs84, i);
            cs.setAbrev("EPSG:326" + huso);
            data.put(cs.getAbrev(), cs);

            cs = new CSUTM(CSDatum.ed50, i);
            cs.setAbrev("EPSG:230" + huso);
            data.put(cs.getAbrev(), cs);
        }

        for (int i = 3; i <= 23; i++) {
            String huso = Integer.toString(i);

            if (i < 10) {
                huso = "0" + huso;
            }

            cs = new CSUTM(CSDatum.nad27, i);
            cs.setAbrev("EPSG:267" + huso);
            data.put(cs.getAbrev(), cs);

            cs = new CSUTM(CSDatum.nad83, i);
            cs.setAbrev("EPSG:269" + huso);
            data.put(cs.getAbrev(), cs);
        }

        cs = CSGaussPt.hgd73;
        cs.setAbrev("EPSG:27492"); // Projeco Gauss do Datum 73 de Lisboa (no EPSG found) 
        data.put(cs.getAbrev(), cs);

        cs = new CSUTM(CSDatum.d73, 29);
        cs.setAbrev("EPSG:27429"); // Projeco Gauss do Datum 73 de Lisboa (no EPSG found) 
        data.put(cs.getAbrev(), cs);

        for (int i = 18; i <= 21; i++) {
            String huso = Integer.toString(i);

            if (i < 10) {
                huso = "0" + huso;
            }

            cs = new CSUTM(CSDatum.lomaQuintana, i);
            cs.setAbrev("EPSG:288" + huso);
            data.put(cs.getAbrev(), cs);

            cs = new CSUTM(CSDatum.laCanoa, i);
            cs.setAbrev("EPSG:247" + huso);
            data.put(cs.getAbrev(), cs);
        }

        //		cs = new CSLambertCC(CSDatum.nad27, -105D, 49D, 49D, 77D, 0, 0);
        //		cs.setAbrev("LCCCan");
        //		data.put(cs.getAbrev(), cs);
        /* Para el server WMS de canad:
         * EPSG:42101
         * EPSG:42304
         * EPSG:4269
         */
        /*
         * 42101,PROJCS["WGS 84 / LCC Canada",
         * GEOGCS["WGS 84",DATUM["WGS_1984",
         * SPHEROID["WGS_1984",6378137,298.257223563]],
         * PRIMEM["Greenwich",0],UNIT["Decimal_Degree",0.0174532925199433]],
         *
         * PROJECTION["Lambert_Conformal_Conic_2SP"],
         * PARAMETER["central_meridian",-95.0],
         * PARAMETER["latitude_of_origin",0],
         * PARAMETER["standard_parallel_1",49.0],
         * PARAMETER["standard_parallel_2",77.0],
         * PARAMETER["false_easting",0.0],
         * PARAMETER["false_northing",-8000000.0],
         * UNIT["Meter",1],AUTHORITY["EPSG","42101"]]
         */
        cs = new CSLambertCC(CSDatum.wgs84, -95, 0, 49, 77, 0, -8000000.0);
        cs.setAbrev("EPSG:42101");
        data.put(cs.getAbrev(), cs);

        /* 42304,PROJCS["NAD83 / NRCan LCC Canada",
         * GEOGCS["NAD83",DATUM["North_American_Datum_1983",
         * SPHEROID["GRS_1980",6378137,298.257222101]],
         * PRIMEM["Greenwich",0],
         * UNIT["Decimal_Degree",0.0174532925199433]],
         *
         * PROJECTION["Lambert_Conformal_Conic_2SP"],
         * PARAMETER["central_meridian",-95.0],
         * PARAMETER["latitude_of_origin",49.0],
         * PARAMETER["standard_parallel_1",49.0],
         * PARAMETER["standard_parallel_2",77.0],
         * PARAMETER["false_easting",0.0],
         * PARAMETER["false_northing",0.0],
         * UNIT["Meter",1],AUTHORITY["EPSG","42304"]]
         */
        cs = new CSLambertCC(CSDatum.nad83, -95, 49, 49, 77, 0, 0);
        cs.setAbrev("EPSG:42304");
        data.put(cs.getAbrev(), cs);

        /*
         * EPSG:26915 - NAD83 / UTM zone 15N
         * EPSG:31466 - Gau-Krger band 2
         * EPSG:31467 - Gau-Krger band 3
         * EPSG:4314  - DHDN
         */
    }

    /**
     * Mete una nueva proyeccion en la Pool.
     * @param name abreviatura de la proyecccion (i.e. EPSG:23030)
     * @param proj Proyeccion
     */
    public static void add(String name, IProjection proj) {
        data.put(name, proj);
    }

    /**
     * Devuelve una proyeccion a partir de una cadena.
     * @param name abreviatura de la proyecccion (i.e. EPSG:23030)
     * @return Proyeccion si existe
     */
    public static IProjection get(String name) {
        IProjection proj = null;

        if (ProjectionPool.data.containsKey(name)) {
            proj = (IProjection) ProjectionPool.data.get(name);
        } else {
            System.err.println("ProjectionPool: Key '" + name + "' not set.");
        }

        return proj;
    }

    public static Iterator iterator() {
        ArrayList projs = new ArrayList();

        Iterator iter = data.entrySet().iterator();

        while (iter.hasNext()) {
            projs.add(((Map.Entry) iter.next()).getValue());
        }

        return projs.iterator();
    }

    /**
     * Devuelve una proyeccion a partir de un codig epsg.
     * @param name codigo de la proyecccion (i.e. 23030)
     * @return Proyeccion si existe
     */
    private static IProjection getByEpsg(String code) {
        IProjection proj = get("EPSG:" + code);

        return proj;
    }
}
