/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 * 
 * Copyright (C) 2004-5. 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 * 
 * cresques@gmail.com
 */
package org.cresques.io.raster;



/**
 *  
 * @author Nacho Brodin (brodin_ign@gva.es)
 */

public class ColorSpaceConversion {
	public static int H = 0, R = 0;
	public static int S = 1, G = 1;
	public static int L = 2, B = 2;

	
	/**
	 * Convierte HSL a RGB
	 * @param h hue
	 * @param s saturation
	 * @param l ligthness
	 * @return
	 */
	public static int[] HSLtoRGB(int h, int s, int i){
		double   red;                   /* the red band output                       */
		double   red255;                /* the red band output                       */
		double   green;                 /* the green band output                     */
		double   green255;              /* the green band output                     */
		double   blue;                  /* the blue band output                      */
		double   blue255;               /* the blue band output                      */
		double   m1;                    /* value used for determining RGB            */
		double   m2;                    /* value used for determining RGB            */
		double   scalei;                /* intensity value                           */
		double   scales;                /* saturation value                          */
		double   hue;                   /* hue                                       */
		double   savehue;               /* save the hue for future processing        */
		int[]   res = new int[3];
		 
		if(h == s && h == 0){
			res[0] = res[1] = res[2] = i;
			return res;
		}
				
		red = green = blue = 0.0;
		scalei = (double) (i / 255.0);
		scales = (double) (s / 255.0);
		m2 = 0.0;
		
		if (scalei <= 0.50)
			m2 = scalei*(1.0 + scales);
		else if (scalei > 0.50)
		    m2 = scalei+scales-(scalei*scales);
		m1 = 2.0 * scalei - m2;
		 
		hue = (double) 360.0 * h / 255.0;
		 
		if (scales == 0.0){
		  if (hue == -1.0){
		    red = scalei;
		    green = scalei;
		    blue = scalei;
		  }
		}else{
		   savehue = hue + 120.0;
		   if (savehue > 360.0)
		      savehue -= 360.0;
		   if (savehue < 0.0)
		      savehue += 360.0;
		   if (savehue < 60.0)
		      red = m1 + (m2-m1) * savehue/60.0;
		   else if (savehue < 180.0)
		          red = m2;
		        else if (savehue < 240.0)
		          red = m1 + (m2-m1) * (240.0-savehue)/60.0;
		        else
		          red = m1;
		 
		   savehue = hue;
		   if (savehue > 360.0)
		     savehue -= 360.0;
		   if (savehue < 0.0)
		      savehue += 360.0;
		   if (savehue < 60.0)
		      green = m1 + (m2-m1) * savehue/60.0;
		   else if (savehue < 180.0)
		      green = m2;
		   else if (savehue < 240.0)
		      green = m1 + (m2-m1) * (240.0-savehue)/60.0;
		   else
		      green = m1;
		 
		   
		   savehue = hue - 120.0;
		   if (savehue > 360.0)
		      savehue -= 360.0;
		   if (savehue < 0.0)
		      savehue += 360.0;
		   if (savehue < 60.0)
		      blue = m1 + (m2-m1) * savehue/60.0;
		   else if (savehue < 180.0)
		      blue = m2;
		   else if (savehue < 240.0)
		      blue = m1 + (m2-m1) * (240.0-savehue)/60.0;
		   else
		      blue = m1;
		}
		 
		red255 = red*255.0;
		green255 = green*255.0;
		blue255 = blue*255.0;
		if (red255 > 255.0)
		   red = 255.0;
		else
		   red = red255;
		if (green255 > 255.0)
		   green = 255.0;
		else
		   green = green255;
		if (blue255 > 255.0)
		   blue = 255.0;
		else
		   blue = blue255;
		 
		if (red   > 254.5) red   = 254.5;
		if (red   <   0.0) red   =   0.0;
		if (green > 254.5) green = 254.5;
		if (green <   0.0) green =   0.0;
		if (blue  > 254.5) blue  = 254.5;
		if (blue  <   0.0) blue  =   0.0;
		 
		res[0] = (int) (red + 0.5);
		res[1] = (int) (green +0.5);
		res[2] = (int) (blue + 0.5);
		return res;
	}
	
	/**
	 * Convierte RGB a HSL
	 * @param r red
	 * @param g green
	 * @param b blue
	 * @return
	 */
	public static double[] RGBtoHSL(int rojo, int verde, int azul){
		double scaler, scaleg, scaleb;
		double sat;
		double red, green, blue;
		double low, high, intens;
		double   hue=0.0;
		double[] res1 = new double[3];
		
		scaler = (double) (rojo / 255.0);
	    scaleg = (double) (verde / 255.0);
	    scaleb = (double) (azul / 255.0);
	  	 
	    high = scaler;
	    
	    if (scaleg > high)
	    	high = scaleg;
	    if (scaleb > high)
	    	high = scaleb;
	    low = scaler;
	    if (scaleg < low)
	    	low = scaleg;
	    if (scaleb < low)
	    	low = scaleb;
	    
	    intens = ((high + low) / 2.0);
	    if (high == low){
		  sat = 0.0;
		  hue = 0.0;
		  res1[0] = (double) hue;
		  res1[2] = (double) intens;
		  res1[1] = (double) sat;
	    }else if (high != low){
		  if (intens <= 0.5)
		    sat = (high-low)/(high+low);
		  else
		    sat = (high-low)/(2 - high - low);
		   
			red = (high-scaler)/(high-low);
			green = (high-scaleg)/(high-low);
			blue =(high-scaleb)/(high-low);
						
			if (scaler == high)
			    hue = blue - green;
			else
			  if (scaleg == high)
			   hue = 2 + red - blue;
			  else
			    if (scaleb == high)
			    	hue = 4 + green - red;
			  
			hue *= 60.0;
			
			if (hue < 0.0)
			 hue += 360.0;
			 
			//res[0] = (byte)(255.0 * hue / 360.0 + 0.5);
			//res[1] = (byte) (intens*255. + 0.5);
			//res[2] = (byte) (sat*255. + 0.5);
			res1[0] = (double)(hue);
			res1[2] = (double) (intens);
			res1[1] = (double) (sat);
	    }
		return res1;
	}
}
