/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;

import java.io.File;


/**
 * Clase base para los filtros de realzado lineal. Lee el mnimo y mxmo de la clase
 * RasterStats que sern calculados por PercentTailTrimFilter o ComputeMinMaxFilter dependiendo
 * de si est activado el recorte de colas o no. En RasterStats tambin estn los segundos
 * valores despus del mnimo y mximo que son los que se utilizan con la opcin eliminar
 * extremos activada. Estos se usaran en vez del mnimo y mximo cuando la variable
 * removeExtrema est a true.
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public abstract class LinearEnhancementFilter extends RasterFilter {
    protected double[] scale = new double[3];
    protected double[] offset = new double[3];
    protected int[] minBandValue = null;
    protected int[] maxBandValue = null;
    protected boolean removeExtrema = false;
    protected String filename = null;

    /**
     * Constructor
     *
     */
    public LinearEnhancementFilter() {
        super();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
    	 if (removeExtrema) { //Si est activado eliminar extremos gastamos el 2 mximo/mnimo

            if ((filename != null) && !filename.equals("")) {
                for (int i = 0; i < stats.history.size(); i++) {
                    RasterStats.History history = (RasterStats.History) stats.history.get(i);
                    if (history.file.equals(filename.substring((filename.lastIndexOf(File.separator) +
                                                                   1),
                                                                   filename.length()))) {
                        this.minBandValue = history.secMin;
                        this.maxBandValue = history.secMax;
                    }
                }
            }

            if ((this.minBandValue == null) || (this.maxBandValue == null)) {
                this.minBandValue = stats.secondMinBandValue;
                this.maxBandValue = stats.secondMaxBandValue;
            }
        } else { //Si no est activado eliminar extremos

            if ((filename != null) && !filename.equals("")) {
                for (int i = 0; i < stats.history.size(); i++) {
                    RasterStats.History history = (RasterStats.History) stats.history.get(i);

                    if (history.file.equals(filename.substring((filename.lastIndexOf(File.separator) +
                                                                   1),
                                                                   filename.length()))) {
                        this.minBandValue = history.min;
                        this.maxBandValue = history.max;
                    }
                }
            }

            if ((this.minBandValue == null) || (this.maxBandValue == null)) {
                this.minBandValue = stats.minBandValue;
                this.maxBandValue = stats.maxBandValue;
            }
        }

        for (int i = 0; i < 3; i++) {
            scale[i] = 255D / (maxBandValue[i] - minBandValue[i]);
            offset[i] = (255D * minBandValue[i]) / (minBandValue[i] -
                        maxBandValue[i]);
        }
    }

    /**
     * Obtiene true si est activado el flag de eliminar extremos y false si no lo est
     */
    public Boolean getRemoveExtrema() {
        return new Boolean(removeExtrema);
    }
}
