/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;

import java.awt.Image;
import java.awt.image.BufferedImage;


/**
 * Proceso del filtro de recorte de colas aplicado a objetos Image
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class PercentTailTrimImageFilter extends PercentTailTrimFilter {
    public PercentTailTrimImageFilter() {
    }

    /**
     * Obtiene parmetros para el filtro y obtiene el ancho y alto de
     * la imagen sobre la que se aplica el filtro
     */
    public void pre() {
        //Obtencin de parmetros
        this.image = (Image) params.get("raster");
        height = image.getHeight(null);
        width = image.getWidth(null);
        super.pre();
    }

    /**
     * Carga sobre el vector de muestras los valores de estas cogidos
     * de la imagen
     */
    public void process(int x, int y) {
        int pt = ((BufferedImage) image).getRGB(x, y);
        sample[0][count] = ((pt & 0xff0000) >> 16) & 0xff;
        sample[1][count] = ((pt & 0xff00) >> 8) & 0xff;
        sample[2][count] = (pt & 0xff);
        count++;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
        super.post();
    }
}
