/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;

import java.util.ArrayList;


/**
 * Clase para estadisticas
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class RasterStats {
    /**
     * Mnimo valor de pixel por banda
     */
    public int[] minBandValue = {
                                    Integer.MAX_VALUE, Integer.MAX_VALUE,
                                    Integer.MAX_VALUE
                                };

    /**
     * Mximo valor de pixel por banda
     */
    public int[] maxBandValue = {
                                    Integer.MIN_VALUE, Integer.MIN_VALUE,
                                    Integer.MIN_VALUE
                                };

    /**
     * Segundo valor ms pequeo de pixel por banda
     */
    public int[] secondMinBandValue = {
                                          Integer.MAX_VALUE, Integer.MAX_VALUE,
                                          Integer.MAX_VALUE
                                      };

    /**
     * Segundo valor ms grande de pixel por banda
     */
    public int[] secondMaxBandValue = {
                                          Integer.MIN_VALUE, Integer.MIN_VALUE,
                                          Integer.MIN_VALUE
                                      };

    /**
     * Historial de valores para una imagen. Esto se usa cuando se solicita varias
     * veces para una misma imagen solo tenga que calcular los valores una vez
     */
    public ArrayList history = new ArrayList();

    /**
     * Porcentaje de recorte de colas
     */
    public double tailPercent = 0D;

    /**
     * Constructor
     */
    public RasterStats() {
        super();
    }

    public void pinta() {
        for (int i = 0; i < 3; i++)
            System.out.println("  Band [" + i + "]: " + minBandValue[i] +
                               "..." + maxBandValue[i]);

        System.out.println("Porcentaje de recorte: " + tailPercent);
    }

    /**
     * Clase que contiene los valores calculados para una imagen.
     * Esto se usa cuando se solicita varias veces para una misma imagen solo
     * tenga que calcular los valores una vez.
     * @author Nacho Brodin (brodin_ign@gva.es)
     */
    public class History {
        public String file = null;
        public int[] min = null;
        public int[] max = null;
        public int[] secMin = null;
        public int[] secMax = null;

        public History(String file, int[] min, int[] max, int[] secMin,
                       int[] secMax) {
            this.file = file;
            this.min = (int[]) min.clone();
            this.max = (int[]) max.clone();
            this.secMin = (int[]) secMin.clone();
            this.secMax = (int[]) secMax.clone();
        }
    }
}
