/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;


/**
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public abstract class RemoveBandsFilter extends RasterFilter {
    protected String bands = "RGB";
    protected int mascara = 0xffffffff;
    protected int m0 = 0xff;
    protected int m1 = 0xff;
    protected int m2 = 0xff;
    protected int m3 = 0xff;

    /**
     * Constructor
     *
     */
    public RemoveBandsFilter() {
        super();
    }

    /**
     * Operaciones que se realizan antes de la ejecucin del filtro
     */
    public void pre() {
        //Obtencin de parmetros comunes a todos
        this.stats = (RasterStats) params.get("stats");
        this.bands = (String) params.get("bands");

        if (bands.compareTo("RGB") == 0) { //No modifica ninguna banda -> no se ejecuta el filtro
            this.exec = false;

            return;
        }

        if (bands.compareTo("R") == 0) {
            mascara = 0xff00ffff;
            m1 = 0x00;
        }

        if (bands.compareTo("G") == 0) {
            mascara = 0xffff00ff;
            m2 = 0x00;
        }

        if (bands.compareTo("B") == 0) {
            mascara = 0xffffff00;
            m3 = 0x00;
        }

        if (bands.compareTo("RG") == 0) {
            mascara = 0xff0000ff;
            m1 = m2 = 0x00;
        }

        if (bands.compareTo("RB") == 0) {
            mascara = 0xff00ff00;
            m1 = m3 = 0x00;
        }

        if (bands.compareTo("GB") == 0) {
            mascara = 0xffff0000;
            m2 = m3 = 0x00;
        }
    }

    /**
     * Operaciones que se realizan despus de la ejecucin del filtro
     */
    public void post() {
    }
}
