/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;

import java.awt.Image;

import org.cresques.io.GeoRasterFile;


/**
 * Clase base para los filtros de sharpening en sus diferentes tipos
 * de datos.
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public abstract class SharpeningFilter extends RasterFilter {
    
	/**
	 * Posicin en la lista de GeoRasterFiles de la imagen de refinado.
	 */
	public int				 	posPancromatica = 0;
	
	/**
	 * Lista de GeoRasterFiles que componen los datos de entrada
	 */
	public GeoRasterFile[]	files = null;
	
	/**
	 * Alto de la imagen pancromtica
	 */
	protected int 				heightPancr = 0;
	
	/**
	 * Ancho de la imagen pancromtica
	 */
	protected int 				widthPancr = 0;
	
	/**
	 * Alto de las bandas multiespectrales
	 */
	protected int 				heightMultiespec = 0;
	
	/**
	 * Ancho de las bandas multiespectrales
	 */
	protected int 				widthMultiespec = 0;
	
	/**
	 * Image que corresponde a la imagen de refinado
	 */
	protected Image 			imagePancr = null;
	
	/**
	 * Image que corresponde a la imagen de entrada
	 */
	protected Image 			imageMultiespec = null;
	
	/**
	 * Nmero de bandas de la operacin incluido la de refinado
	 */
	protected int				nBands = 0;
	
	/**
	 * Relacin entre la pancromatica y la multiespectral en X
	 */
	protected int				relX = 0;
	
	/**
	 * Relacin entre la pancromatica y la multiespectral en Y
	 */
	protected int				relY = 0;
	
	/**
	 * Vector de 3 elementos que corresponden a R, G y B respectivamente. El valor contenido
	 * en cada uno de ellos corresponde al nmero de banda de fichero que se visualiza en R, G y B.
	 */
	public int[]				bandOrder = null;
	
	/**
	 * Valor de alpha aplicado a toda la imagen
	 */
	public int					alpha = 0;
	
	/**
	 * Mtodo de calculo del refinado
	 */
	public String				method = "ihs";
	
	/**
	 * Coeficiente
	 */
	public double				coef = 0.20;
	
	/**
	 * Coeficiente Brovey
	 */
	public int					coefBrovey = 0;
	
	/**
	 * Nombre de la banda pancromtica
	 */
	protected String			pancrName = null;
	
    /**
     * Constructor
     *
     */
    public SharpeningFilter() {
        super();
    }

    /**
     * 
     */
    public void pre() {
      
    }

}
