/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;

import java.awt.Image;
import java.awt.image.BufferedImage;


/**
 * Filtro de transparencia aplicada a un rango de valores de un pixel sobre un objeto image
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class TransparencyImageFilter extends TransparencyFilter {
    /**
     * Constructor
     *
     */
    public TransparencyImageFilter() {
        super();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
        //Obtencin de parmetros
        this.image = (Image) params.get("raster");
        height = image.getHeight(null);
        width = image.getWidth(null);
        super.pre();
    }

    /**
     * Dado un pixel y un rango, asigna ese pixel al color de transparencia por
     * defecto si el pixel en una banda determinada est en el rango definido.
     * @param range        rango
     * @param banda banda
     * @param px        pixel
     */
    private void processRange(int[][] range, int banda, int[] px) {
        for (int i = 0; i < range.length; i++) {
            if (((px[banda] >= range[i][0]) && (px[banda] <= range[i][1]))) {
                px[0] = this.alpha;
                px[1] = this.transparencyColorRed;
                px[2] = this.transparencyColorGreen;
                px[3] = this.transparencyColorBlue;
            }
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#process(int, int)
     */
    public void process(int x, int y) {
        int pt = ((BufferedImage) image).getRGB(x, y);
        int[] px4 = {
                        (pt & 0xff000000) >> 24, (pt & 0xff0000) >> 16,
                        (pt & 0x00ff00) >> 8, (pt & 0x0000ff)
                    };

        if (rangesR != null) {
            processRange(rangesR, 1, px4);
        }

        if (rangesG != null) {
            processRange(rangesG, 2, px4);
        }

        if (rangesB != null) {
            processRange(rangesB, 3, px4);
        }

        ((BufferedImage) image).setRGB(x, y,
                                       (((px4[0] << 24) & 0xff000000) |
                                       ((px4[1] << 16) & 0x00ff0000) |
                                       ((px4[2] << 8) & 0x0000ff00) |
                                       (px4[3] & 0x0000ff)));
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        if (name.equals("raster")) {
            return (Object) this.image;
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
    }
}
