/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.px;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;

import org.cresques.geo.Projected;
import org.cresques.geo.ViewPortData;

import org.cresques.io.EcwFile;
import org.cresques.io.GdalFile;
import org.cresques.io.GeoRasterFile;
import org.cresques.io.raster.RasterBuf;
import org.cresques.io.raster.RasterFilterStack;
import org.cresques.io.raster.RasterFilterStackManager;
import org.cresques.io.raster.RasterStats;
import org.cresques.io.raster.RasterToImageFilter;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.awt.image.ImageObserver;

import java.util.ArrayList;
import java.util.Date;
import java.util.Vector;


/**
 *
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class PxRaster extends PxObj implements Projected {
    protected GeoRasterFile[] geoFile = null;
    protected ImageObserver component = null;
    Vector pts = null;

    // Soporte para n bandas, visibles de 3 en 3, en ficheros separados
    //protected GeoRasterFile [] colorBand = null;
    protected int rBand = 1;

    // Soporte para n bandas, visibles de 3 en 3, en ficheros separados
    //protected GeoRasterFile [] colorBand = null;
    protected int gBand = 2;

    // Soporte para n bandas, visibles de 3 en 3, en ficheros separados
    //protected GeoRasterFile [] colorBand = null;
    protected int bBand = 3;
    public RasterStats stats = new RasterStats();
    int transparente = 0x10ffff80;
    String vName = null;
    protected boolean pintaMarco = false; //true;
    IProjection proj = null;
    protected Extent extentOrig = null;
    ICoordTrans rp = null;
    public RasterFilterStack filterStack = new RasterFilterStack(stats);
    private BandSwitch bandSwitch = new BandSwitch();
    private boolean firstPxRaster = true;
    private final double percentFilterInit = 0.02;
    private RasterFilterStackManager stackManager = null;
    private Image geoImage = null;
    private ViewPortData lastViewPort = null;

    /*public boolean transparencyFilterEnabled = false;
    public boolean enhancedFilterEnabled = false;
    public boolean tailFilterEnabled = false;*/

    /**
     * Constructor.
     * @param component
     */
    public PxRaster(ImageObserver component) {
        this.component = component;
    }

    /**
     * Contructor para un solo fichero
     * @param proj        Proyecccin
     * @param fname Nombre del fichero
     * @param component
     */
    public PxRaster(IProjection proj, String fname, ImageObserver component) {
        geoFile = new GeoRasterFile[1];
        geoFile[0] = GeoRasterFile.openFile(proj, fname); //loadECW(fname);
        geoFile[0].setUpdatable((Component) component);
        this.proj = proj;
        this.component = component;
        setExtent(geoFile[0].getExtent());
        geoFile[0].setView(geoFile[0].getExtent());
        extentOrig = extent;
        bandSwitch.addFile(geoFile[0]);

        if (geoFile[0].getBandCount() >= 3) {
            setBand(GeoRasterFile.RED_BAND, 0);
            setBand(GeoRasterFile.GREEN_BAND, 1);
            setBand(GeoRasterFile.BLUE_BAND, 2);
        } else if (geoFile[0].getBandCount() == 2) {
            setBand(GeoRasterFile.RED_BAND, 0);
            setBand(GeoRasterFile.GREEN_BAND, 1);
            setBand(GeoRasterFile.BLUE_BAND, 1);
        } else if (geoFile[0].getBandCount() == 1) {
            //setBand(GeoRasterFile.RED_BAND|GeoRasterFile.GREEN_BAND|GeoRasterFile.BLUE_BAND, 0);
            setBand(GeoRasterFile.RED_BAND, 0);
            setBand(GeoRasterFile.GREEN_BAND, 0);
            setBand(GeoRasterFile.BLUE_BAND, 0);
        }
    }

    /**
     * Constructor para multiples ficheros
     */
    public PxRaster(IProjection proj, String[] fnames, ImageObserver component) {
        this.proj = proj;
        this.component = component;
        geoFile = new GeoRasterFile[fnames.length];

        for (int i = 0; i < geoFile.length; i++) {
            geoFile[i] = GeoRasterFile.openFile(proj, fnames[i]); //loadECW(fname);
            geoFile[i].setUpdatable((Component) component);
            setExtent(geoFile[i].getExtent());
            geoFile[i].setView(geoFile[i].getExtent());
            bandSwitch.addFile(geoFile[i]);
        }

        //geoFile = geoFile[0];
        extentOrig = extent;

        if ((fnames.length >= 3) || (geoFile[0].getBandCount() > 2)) {
            setBand(GeoRasterFile.RED_BAND, 0);
            setBand(GeoRasterFile.GREEN_BAND, 1);
            setBand(GeoRasterFile.BLUE_BAND, 2);
        } else {
            setBand(GeoRasterFile.RED_BAND | GeoRasterFile.GREEN_BAND |
                    GeoRasterFile.BLUE_BAND, 0);
        }
    }

    public PxRaster(GeoRasterFile eFile, ImageObserver component, Extent view) {
        geoFile = new GeoRasterFile[1];
        geoFile[0] = eFile; //loadECW(fname);
        geoFile[0].setUpdatable((Component) component);
        setProjection(geoFile[0].getProjection());
        this.component = component;

        setExtent(geoFile[0].getExtent());
        if(view != null){
	        geoFile[0].setView(view); //geoFile.getExtent());
	        extentOrig = extent;
	        bandSwitch.addFile(eFile);
	
	        if (geoFile[0].getBandCount() >= 3) {
	            setBand(GeoRasterFile.RED_BAND, 0);
	            setBand(GeoRasterFile.GREEN_BAND, 1);
	            setBand(GeoRasterFile.BLUE_BAND, 2);
	        } else if (geoFile[0].getBandCount() == 2) {
	            setBand(GeoRasterFile.RED_BAND, 0);
	            setBand(GeoRasterFile.GREEN_BAND, 1);
	            setBand(GeoRasterFile.BLUE_BAND, 1);
	        } else if (geoFile[0].getBandCount() == 1) {
	            //setBand(GeoRasterFile.RED_BAND|GeoRasterFile.GREEN_BAND|GeoRasterFile.BLUE_BAND, 0);
	            setBand(GeoRasterFile.RED_BAND, 0);
	            setBand(GeoRasterFile.GREEN_BAND, 0);
	            setBand(GeoRasterFile.BLUE_BAND, 0);
	        }
        }
    }

    /**
     * Aade un GeoRasterFile al PxRaster
     * @param fileName Nombre del fichero
     */
    public void addFile(String fileName) {
        if (geoFile != null) {
            GeoRasterFile[] listFiles = new GeoRasterFile[geoFile.length + 1];

            for (int i = 0; i < geoFile.length; i++)
                listFiles[i] = geoFile[i];

            listFiles[geoFile.length] = GeoRasterFile.openFile(proj, fileName);
            listFiles[geoFile.length].setUpdatable((Component) component);
            setExtent(listFiles[geoFile.length].getExtent());
            listFiles[geoFile.length].setView(listFiles[geoFile.length].getExtent());
            bandSwitch.addFile(listFiles[geoFile.length]);
            geoFile = listFiles;
        } else {
            System.err.println("PxRaster.addFile(): Imagen no cargada.");

            return;
        }
    }

    /**
     * Devuelve el Extent de un fichero son necesidad de aadirlo
     * al PxRaster
     * @param fileName
     * @return
     */
    public Extent testExtentFile(String fileName) {
        GeoRasterFile grf = GeoRasterFile.openFile(proj, fileName);

        return grf.getExtent();
    }

    /**
     * Obtiene el valor del pixel del Image en la posicin wcx,wcy
     * @param wcx Posicin x
     * @param wcx Posicin y
     * @return valor de pixel
     */
    public int[] getPixel(double wcx, double wcy) {
        if (geoImage != null) {
            int ptox = 0;
            int ptoy = 0;

            try {
                //Extent de la imagen completa
                Extent extOrtofoto = geoFile[0].getExtent();

                //Variables q definen el extent del objeto image
                double minx = 0;

                //Variables q definen el extent del objeto image
                double miny = 0;

                //Variables q definen el extent del objeto image
                double maxx = 0;

                //Variables q definen el extent del objeto image
                double maxy = 0;

                if (lastViewPort.getExtent().getMin().getX() < extOrtofoto.minX()) {
                    minx = extOrtofoto.minX();
                } else {
                    minx = lastViewPort.getExtent().getMin().getX();
                }

                if (lastViewPort.getExtent().getMax().getX() > extOrtofoto.maxX()) {
                    maxx = extOrtofoto.maxX();
                } else {
                    maxx = lastViewPort.getExtent().getMax().getX();
                }

                if (lastViewPort.getExtent().getMin().getY() < extOrtofoto.minY()) {
                    miny = extOrtofoto.minY();
                } else {
                    miny = lastViewPort.getExtent().getMin().getY();
                }

                if (lastViewPort.getExtent().getMax().getY() > extOrtofoto.maxY()) {
                    maxy = extOrtofoto.maxY();
                } else {
                    maxy = lastViewPort.getExtent().getMax().getY();
                }

                //Comprobamos que estemos dentro del extent del Image
                if ((wcx < minx) || (wcx > maxx) || (wcy < miny) ||
                        (wcy > maxy)) {
                    int[] res = { -1, -1, -1, -1 };

                    return res;
                }

                //Pasamos a coordenadas del Image las coordenadas del mundo real
                int w = ((BufferedImage) geoImage).getWidth();
                int h = ((BufferedImage) geoImage).getHeight();
                double wcw = maxx - minx;
                double wch = maxy - miny;
                ptox = (int) (((wcx - minx) * w) / wcw);
                ptoy = (int) (((wcy - miny) * h) / wch);

                //Obtenemos el pto seleccionado del Image y extraemos el RGB
                int px = ((BufferedImage) geoImage).getRGB(ptox, h - ptoy);
                int[] values = new int[4];
                values[0] = ((px & 0xff000000) >> 24);
                values[1] = ((px & 0x00ff0000) >> 16);
                values[2] = ((px & 0x0000ff00) >> 8);
                values[3] = (px & 0x000000ff);

                return values;
            } catch (ArrayIndexOutOfBoundsException e) {
                e.printStackTrace();
            }
        }

        return null;
    }

    /**
     * Elimina un GeoRasterFile al PxRaster
     * @param finaName Nombre del fichero
     */
    public void delFile(String fileName) {
        if (geoFile != null) {
            Vector grfTemp = new Vector();

            for (int i = 0; i < geoFile.length; i++) {
                if (!fileName.endsWith(geoFile[i].getName())) {
                    grfTemp.add(geoFile[i]);
                }
            }

            GeoRasterFile[] listFiles = new GeoRasterFile[grfTemp.size()];

            for (int i = 0; i < listFiles.length; i++)
                listFiles[i] = (GeoRasterFile) grfTemp.get(i);

            //Lo eliminamos del bandSwitch
            for (int i = 0; i < geoFile.length; i++) {
                if (fileName.endsWith(geoFile[i].getName())) {
                    bandSwitch.removeFile(geoFile[i]);
                }
            }

            geoFile = listFiles;

            /*if(geoFile!=null)
                    setExtent(geoFile[0].getExtent());*/
        } else {
            System.err.println("PxRaster.addFile(): Imagen no cargada.");

            return;
        }
    }

    /**
     * Obtiene el tamao de bloque
     * @return
     */
    public int getBlockSize() {
        return geoFile[0].getBlockSize();
    }

    /**
     *
     * @return
     */
    public GeoRasterFile[] getGeoFiles() {
        return geoFile;
    }

    public GeoRasterFile getGeoFile() {
        return geoFile[0];
    }

    /**
     * Obtiene el nmero de bandas del PxRaster. Si ls longitud de geoFile es 1 quiere
     * decir que hay un solo fichero que contiene todas las bandas. Si hay ms de un geoFile
     * el nmero de bandas ser la suma de todas las bandas de las imgenes que forman
     * el geoFile
     * @return Nmero de bandas
     */
    public int getBandCount() {
        return bandSwitch.getBandCount();
    }

    /**
     * Obtiene el tipo de dato del primer GeoRasterFile
     * @return
     */
    public int getDataType() {
        if (geoFile != null) {
            return geoFile[0].getDataType();
        } else {
            System.err.println("PxRaster.getDataType(): Imagen no cargada.");
        }

        return 0;
    }

    /**
     * Obtiene el vector de GeoRasterFile que componen el PxRaster
     * @return vector GeoRasterFile
     */
    public GeoRasterFile[] getFiles() {
        return geoFile;
    }

    /**
     * Asocia un colorBand al rojo, verde o azul.
     * @param flag cual (o cuales) de las bandas.
     * @param nBand        que colorBand
     */
    public void setBand(int flag, int nBand) {
        bandSwitch.setBand(flag, nBand);
    }

    /**
     * Obtiene la posicin del fichero asignado a la banda
     * que se le pasa por parmetro
     * @return
     */
    public int getPosFile(int flag) {
        if (flag == GeoRasterFile.RED_BAND) {
            return bandSwitch.getBandR().getPos();
        } else if (flag == GeoRasterFile.GREEN_BAND) {
            return bandSwitch.getBandG().getPos();
        } else if (flag == GeoRasterFile.BLUE_BAND) {
            return bandSwitch.getBandB().getPos();
        } else {
            return -1;
        }
    }

    /**
     * Devuelve el colorBand activo en la banda especificada.
     * @param flag banda.
     */
    public int getBand(int flag) {
        if (flag == GeoRasterFile.RED_BAND) {
            return bandSwitch.getBandR().getBand();
        } else if (flag == GeoRasterFile.GREEN_BAND) {
            return bandSwitch.getBandG().getBand();
        } else if (flag == GeoRasterFile.BLUE_BAND) {
            return bandSwitch.getBandB().getBand();
        } else {
            return -1;
        }
    }

    /**
     * @param pm
     */
    public void setDrawBorder(boolean pm) {
        pintaMarco = pm;
    }

    /**
     * Obtiene el nombre del fichero si solo hay uno
     * @return Nombre del fichero
     */
    public String getFName() {
        return geoFile[0].getName();
    }

    /**
     * Obtiene el nombre del fichero GeoRasterFile seleccionado
     * @param i        posicin del GeoRasterFile
     * @return Nombre del fichero
     */
    public String getFName(int i) {
        if (geoFile != null) {
            if (i < geoFile.length) {
                return geoFile[i].getName();
            } else {
                return null;
            }
        } else {
            System.err.println("PxRaster.getFName(): Imagen no cargada.");

            return null;
        }
    }

    /**
     * Obtiene una lista de Strings con los nombres de todos los ficheros
     * que tiene el PxRaster
     * @return Lista de nombres
     */
    public String[] getLisName() {
        if (geoFile != null) {
            String[] list = new String[geoFile.length];

            for (int i = 0; i < geoFile.length; i++)
                list[i] = geoFile[i].getName();

            return list;
        } else {
            System.err.println("PxRaster.getListName(): Imagen no cargada.");

            return null;
        }
    }

    /**
     * Devuelve la anchura total del fichero, en pixeles.
     * @return ancho en pixeles
     */
    public int getFWidth() {
        if (geoFile != null) {
            return geoFile[0].getWidth();
        } else {
            System.err.println("PxRaster.getFWidth(): Imagen no cargada.");

            return 0;
        }
    }

    /**
     * Devuelve la anchura total del fichero, en pixeles.
     * @return ancho en pixeles
     */
    public int getFWidth(int i) {
        if (i < geoFile.length) {
            return geoFile[i].getWidth();
        } else {
            System.err.println("PxRaster.getFWidth(): Imagen no cargada.");

            return 0;
        }
    }

    /**
     * Devuelve la altura total del fichero, en pixeles.
     * @return alto en pixeles
     */
    public int getFHeight() {
        if (geoFile != null) {
            return geoFile[0].getHeight();
        } else {
            System.err.println("PxRaster.getFHeight(): Imagen no cargada.");

            return 0;
        }
    }

    /**
     * Devuelve la altura total del fichero, en pixeles.
     * @return alto en pixeles
     */
    public int getFHeight(int i) {
        if (i < geoFile.length) {
            return geoFile[i].getHeight();
        } else {
            System.err.println("PxRaster.getFHeight(): Imagen no cargada.");

            return 0;
        }
    }

    /**
     * Devuelve el nmero de ficheros que componen el PxRaster
     * @return Nmero de ficheros
     */
    public int nFiles() {
        if (geoFile != null) {
            return geoFile.length;
        } else {
            return 0;
        }
    }

    /**
     * Activa o desactiva la transparencia de los ficheros que forman el PxRaster
     * @param t true o false para activar o desactivar transparencia
     */
    public void setTransparency(boolean t) {
        if (geoFile != null) {
            for (int i = 0; i < geoFile.length; i++) {
                geoFile[i].setTransparency(t);
            }
        } else {
            System.err.println("PxRaster.setTransparency(): Imagen no cargada.");

            return;
        }
    }

    /**
     * Pone la transparencia de los ficheros de la imagen a un valor
     * @param t Valor para la transparencia
     */
    public void setTransparency(int t) {
        if (geoFile != null) {
            for (int i = 0; i < geoFile.length; i++) {
                geoFile[i].setTransparency(t);
            }
        } else {
            System.err.println("PxRaster.setTransparency(): Imagen no cargada.");

            return;
        }
    }

    /**
     * Obtiene el alpha del primer fichero. Han de ser iguales en todos
     * los ficheros de la imagen.
     * @return alpha
     */
    public int getAlpha() {
        if (geoFile != null) {
            return geoFile[0].getAlpha();
        } else {
            System.err.println("PxRaster.getAlpha(): Imagen no cargada.");

            return 0;
        }
    }

    /**
     * Asigna el extent
     * @param e extent
     */
    public void setExtent(Extent e) {
        super.extent = e;
        if(e != null){
	        pts = new Vector();
	        pts.add(proj.createPoint(e.minX(), e.minY()));
	        pts.add(proj.createPoint(e.maxX(), e.minY()));
	        pts.add(proj.createPoint(e.maxX(), e.maxY()));
	        pts.add(proj.createPoint(e.minX(), e.maxY()));
        }
    }

    /**
     * Cambia la vista (viewport) sobre el raster.
     *
     * @param v extent
     * @param vName nombre
     */
    public void setView(Extent v, String vName) {
        if (geoFile != null) {
            for (int i = 0; i < geoFile.length; i++) {
                geoFile[i].setView(v);
            }
        } else {
            System.err.println("PxRaster.setView(): Imagen no cargada.");

            return;
        }

        this.vName = vName;
    }

    /**
     * Obtiene la escala.
     *
     * @param width
     * @param height
     * @return
     */
    public double[] getScale(int width, int height) {
        double[] scale = new double[2];

        if (geoFile != null) {
            scale[0] = ((double) width) / geoFile[0].getView().width();
            scale[1] = ((double) height) / geoFile[0].getView().height();

            return scale;
        } else {
            System.err.println("PxRaster.getScale(): Imagen no cargada.");

            return null;
        }
    }

    /**
     * 'Normaliza' la vista en funcin del extent del raster.
     *
     * @param mat
     * @param sz
     */
    protected void calculateNewView(Extent sz) {
        double vx = sz.minX();
        double vy = sz.minY();
        double vx2 = sz.maxX();
        double vy2 = sz.maxY();

        if (sz.minX() < extent.minX()) {
            vx = extent.minX();
        }

        if (sz.minY() < extent.minY()) {
            vy = extent.minY();
        }

        if (sz.maxX() > extent.maxX()) {
            vx2 = extent.maxX();
        }

        if (sz.maxY() > extent.maxY()) {
            vy2 = extent.maxY();
        }

        if (geoFile != null) {
            for (int i = 0; i < geoFile.length; i++)
                geoFile[i].setView(new Extent(vx, vy, vx2, vy2));
        } else {
            System.err.println("PxRaster.calculateNewView(): Imagen no cargada.");
        }
    }

    /**
     * Dibuja el raster
     */
    public synchronized void draw(Graphics2D g, ViewPortData vp) {
        geoImage = null;

        long t2;
        long t1 = new Date().getTime();
        lastViewPort = vp;

        //System.out.println("PxRaster.draw(): vp.extent = "+ vp.getExtent());
        //System.out.println("PxRaster.draw():    extent = "+ getExtent());
        if ((vp.getExtent().minX() > extent.maxX()) ||
                (vp.getExtent().minY() > extent.maxY()) ||
                (vp.getExtent().maxX() < extent.minX()) ||
                (vp.getExtent().maxY() < extent.minY())) {
            return;
        }

        calculateNewView(vp.getExtent());

        Extent v = geoFile[0].getView();
        double x = v.minX();
        double y = v.minY();
        double w = v.width();
        double h = v.height();

        //System.out.println("Pinto PxRaster:" + v);
        double scalex = vp.mat.getScaleX() /* g.getTransform().getScaleX()*/;

        //System.out.println("Pinto PxRaster:" + v);
        double scaley = vp.mat.getScaleY() /* g.getTransform().getScaleY() */;
        int wImg = (int) Math.ceil(Math.abs(w * scalex) + 1);
        int hImg = (int) Math.ceil(Math.abs(h * scaley) + 1);

        //System.out.println("Sz=("+wImg+","+hImg+"); scale=("+scalex+","+scaley+")");
        if ((wImg <= 0) || (hImg <= 0)) {
            return;
        }

        Point2D.Double pt = new Point2D.Double(x, y + h);

        try {
            //System.out.println("Dibujando PxRaster: pt0 = "+pt);
            vp.mat.transform(pt, pt);

            //System.out.println("Dibujando PxRaster: pt1 = "+pt);
            if ((geoFile != null) && geoFile[0] instanceof GdalFile &&
                    (geoFile[0].getDataType() != DataBuffer.TYPE_BYTE)) {
                //System.out.println("PxRaster: Has dado con un Raster de 16 bits");
                RasterBuf raster = ((GdalFile) geoFile[0]).getRaster(wImg,
                                                                     hImg, rp);
                t2 = new Date().getTime();

                t2 = new Date().getTime();
                System.out.println("Dibujando PxRaster: " +
                                   ((t2 - t1) / 1000D) +
                                   ", secs. Filtrando/Renderizando");
                t1 = t2;

                filterStack.setInitRasterBuf(raster);

                //Si la imagen es de 16 bits lleva un filtro de realce por defecto
                if (stackManager == null) {
                    stackManager = new RasterFilterStackManager(filterStack);
                    stackManager.addEnhancedFilter(false, geoFile[0].getName());
                    stackManager.removeFilter(stackManager.getTypeFilter("computeminmax"));
                    stackManager.addTailFilter(this.percentFilterInit, 0D, false);
                }

                geoImage = renderizeRaster(raster, vp, v);
                g.drawImage(geoImage, (int) Math.round(pt.getX()),
                            (int) Math.round(pt.getY()), component);

                t2 = new Date().getTime();
                System.out.println("Dibujando PxRaster: " +
                                   ((t2 - t1) / 1000D) + ", secs. Dibujando");
            } else if((geoFile != null) && (geoFile.length > 1) && 
            		(!bandSwitch.getBandR().getGeoRasterFile().getName().equals(bandSwitch.getBandG().getGeoRasterFile().getName()) || 
            		!bandSwitch.getBandR().getGeoRasterFile().getName().equals(bandSwitch.getBandB().getGeoRasterFile().getName()) ||
            		!bandSwitch.getBandG().getGeoRasterFile().getName().equals(bandSwitch.getBandB().getGeoRasterFile().getName()))
            		) { // multiFiles
                System.out.println("Dibujando PxRaster (Multifile) ... Bands " +
                                   geoFile.length);

                if (bandSwitch.getBandR().getGeoRasterFile() instanceof EcwFile) {
                    ((EcwFile) bandSwitch.getBandR().getGeoRasterFile()).setMultifile(true);
                }

                if (bandSwitch.getBandG().getGeoRasterFile() instanceof EcwFile) {
                    ((EcwFile) bandSwitch.getBandG().getGeoRasterFile()).setMultifile(true);
                }

                if (bandSwitch.getBandB().getGeoRasterFile() instanceof EcwFile) {
                    ((EcwFile) bandSwitch.getBandB().getGeoRasterFile()).setMultifile(true);
                }
                
                //TODO:Solucin para que no se pinte si hay sharpening. Esto hay que pensarlo mejor
                if (stackManager == null) 
                    stackManager = new RasterFilterStackManager(filterStack);
                if(!filterStack.isActive(stackManager.getTypeFilter("sharpening"))){
	                geoImage = bandSwitch.getBandR().getGeoRasterFile().updateImage(wImg, hImg, rp, null, 0, 0);
	                geoImage = bandSwitch.getBandG().getGeoRasterFile().updateImage(wImg, hImg, rp, geoImage, 
	                						bandSwitch.getBandG().getBand(), GeoRasterFile.GREEN_BAND);
	                geoImage = bandSwitch.getBandB().getGeoRasterFile().updateImage(wImg, hImg, rp, geoImage, 
	                						bandSwitch.getBandB().getBand(), GeoRasterFile.BLUE_BAND);
                }else{
                	geoImage = new BufferedImage(wImg, hImg, BufferedImage.TYPE_INT_RGB);
                }
                
                filterStack.setInitRasterBuf(geoImage);

                geoImage = renderizeRaster(geoImage, vp, v);
                g.drawImage(geoImage, (int) Math.round(pt.getX()),
                            (int) Math.round(pt.getY()), component);
                t2 = new Date().getTime();
                System.out.println("Dibujando PxRaster: " +
                                   ((t2 - t1) / 1000D) + ", secs.");
            } else if ((geoFile != null) ) { // Una solo fichero
                geoImage = bandSwitch.getBandR().getGeoRasterFile().updateImage(wImg, hImg, rp, null, 0, 0);

                filterStack.setInitRasterBuf(geoImage);

                geoImage = renderizeRaster(geoImage, vp, v);
                g.drawImage(geoImage, (int) Math.round(pt.getX()),
                            (int) Math.round(pt.getY()), component);
                t2 = new Date().getTime();
                System.out.println("Dibujando PxRaster: " +
                                   ((t2 - t1) / 1000D) + ", secs.");
            } else { // no cargada
                System.err.println("Dibujando PxRaster: Foto no cargada.");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        if (pintaMarco) {
            drawMarco(g, vp);
        }
    }

    /**
     * Aplica la pila de fitros sobre el RasterBuf pasado como parmetro
     * y lo devuelve en Image
     * @param raster        RasterBuf con la imagen sobre la que se aplicaran filtros
     * @return        Image con la imagen con los filtros puestos
     */
    public Image renderizeRaster(RasterBuf raster, ViewPortData vp, Extent e) {
        if (filterStack != null) {
        	filterStack.setViewPortData(vp);
        	filterStack.setExtent(e);
            filterStack.execute(raster);
        }
        System.out.println("Image renderizeRaster(RasterBuf raster, ViewPortData vp) ");
        //Aplicamos el filtro para convertir a Image
        RasterToImageFilter rti = new RasterToImageFilter();
        rti.addParam("raster", raster);
        rti.addParam("alpha", new Integer(this.getAlpha()));
        rti.execute();
        raster = null;

        return (Image) rti.getResult("raster");
    }

    /**
     * Aplica la pila de filtros sobre el Image pasado como parmetro y lo devuelve.
     * Si la salida del ltimo filtro es un RasterBuf lo convertir a Image
     * @param image Image con la imagen sobre la que se aplicaran filtros
     * @return        Image con la imagen con los filtros puestos
     */
    public Image renderizeRaster(Image image, ViewPortData vp, Extent e) {
        if (filterStack != null) {
        	filterStack.setViewPortData(vp);
        	filterStack.setExtent(e);
            filterStack.execute(image);
        }
      
        if (filterStack.getOutDataType() != RasterBuf.TYPE_IMAGE) {
            //Aplicamos el filtro para convertir a Image
            RasterToImageFilter rti = new RasterToImageFilter();
            rti.addParam("raster", (RasterBuf) filterStack.getResult());
            rti.addParam("alpha", new Integer(this.getAlpha()));
            rti.execute();
         
            return (Image) rti.getResult("raster");
        }
        return (Image) filterStack.getResult();
       // return image;
    }

    /**
     *
     * @param g
     * @param vp
     */
    public void drawMarco(Graphics2D g, ViewPortData vp) {
        //		Color color = new Color(255,222,165,128), fillColor = new Color(255,214,132,128);
        Color color = new Color(128, 128, 128);

        //		Color color = new Color(255,222,165,128), fillColor = new Color(255,214,132,128);
        Color fillColor = new Color(255, 220, 220, 0x20);
        GeneralPath gp = newGP(vp);
        g.setColor(fillColor);
        g.fill(gp);
        g.setColor(color);
        g.draw(gp);
    }

    private GeneralPath newGP(ViewPortData vp) {
        //if (gp != null) return;
        GeneralPath gp = new GeneralPath();
        Point2D.Double pt0 = new Point2D.Double(0.0, 0.0);
        Point2D.Double pt1 = new Point2D.Double(0.0, 0.0);
        Point2D.Double pt2 = new Point2D.Double(0.0, 0.0);
        Point2D.Double pt3 = new Point2D.Double(0.0, 0.0);
        vp.mat.transform((Point2D) pts.get(0), pt0);
        vp.mat.transform((Point2D) pts.get(1), pt1);
        vp.mat.transform((Point2D) pts.get(2), pt2);
        vp.mat.transform((Point2D) pts.get(3), pt3);

        // Aspa desde el extent
        gp.moveTo((float) pt0.getX(), (float) pt0.getY());
        gp.lineTo((float) pt2.getX(), (float) pt2.getY());
        gp.moveTo((float) pt1.getX(), (float) pt1.getY());
        gp.lineTo((float) pt3.getX(), (float) pt3.getY());

        // Extent
        gp.moveTo((float) pt0.getX(), (float) pt0.getY());
        gp.lineTo((float) pt1.getX(), (float) pt1.getY());
        gp.lineTo((float) pt2.getX(), (float) pt2.getY());
        gp.lineTo((float) pt3.getX(), (float) pt3.getY());

        if (extentOrig != extent) {
            gp.lineTo((float) pt0.getX(), (float) pt0.getY());

            Vector pts = new Vector();
            pts.add(proj.createPoint(extentOrig.minX(), extentOrig.minY()));
            pts.add(proj.createPoint(extentOrig.maxX(), extentOrig.minY()));
            pts.add(proj.createPoint(extentOrig.maxX(), extentOrig.maxY()));
            pts.add(proj.createPoint(extentOrig.minX(), extentOrig.maxY()));

            vp.mat.transform((Point2D) pts.get(0), pt0);
            vp.mat.transform((Point2D) pts.get(1), pt1);
            vp.mat.transform((Point2D) pts.get(2), pt2);
            vp.mat.transform((Point2D) pts.get(3), pt3);
            gp.moveTo((float) pt0.getX(), (float) pt0.getY());
            gp.lineTo((float) pt1.getX(), (float) pt1.getY());
            gp.lineTo((float) pt2.getX(), (float) pt2.getY());
            gp.lineTo((float) pt3.getX(), (float) pt3.getY());
        }

        gp.closePath();

        return gp;
    }

    public IProjection getProjection() {
        return proj;
    }

    public void setProjection(IProjection p) {
        proj = p;
    }

    public void reProject(ICoordTrans rp) {
        this.rp = rp.getInverted();
        System.out.println("PxRaster: reProject()");

        //geoFile.reProject(rp);
        Vector savePts = pts;

        pts = new Vector();
        extent = new Extent();

        Point2D ptDest = null;

        for (int i = 0; i < savePts.size(); i++) {
            ptDest = rp.getPDest().createPoint(0.0, 0.0);
            ptDest = rp.convert((Point2D) savePts.get(i), ptDest);
            pts.add(ptDest);
            extent.add(ptDest);
        }

        setProjection(rp.getPDest());
    }

    /**
     * Obtiene el Stack Manager
     * @return
     */
    public RasterFilterStackManager getStackManager() {
        return this.stackManager;
    }

    /**
     * Asigna el Stack Manager
     * @return
     */
    public void setStackManager(RasterFilterStackManager sm) {
        this.stackManager = sm;
    }

    /**
     * Estructura que representa la relacin entre un fichero y la banda que se
     * utiliza de este.
     * @author Nacho Brodin <brodin_ign@gva.es>
     */
    class FileBands {
        GeoRasterFile grf; //Fichero
        int band; //Banda asinada
        int filePos; //posicin del fichero

        /**
         * @return Returns the band.
         */
        public int getBand() {
            return band;
        }

        /**
         * @param band The band to set.
         */
        public void setBand(int band) {
            this.band = band;
        }

        /**
         * @return Returns the band.
         */
        public int getPos() {
            return filePos;
        }

        /**
         * @param band The band to set.
         */
        public void setPos(int pos) {
            this.filePos = pos;
        }

        /**
         * @return Returns the grf.
         */
        public GeoRasterFile getGeoRasterFile() {
            return grf;
        }

        /**
         * @param grf The grf to set.
         */
        public void setGeoRasterFile(GeoRasterFile grf) {
            this.grf = grf;
        }
    }

    /**
     * Clase que lleva la gestin entre la relacin de ficheros representados
     * por un GeoRasterFile y el nmero de bandas que contienen. Esto es necesario
     * para la actualizacin de la vista ya que para un Image dado podemos tener
     * multiples bandas que pueden ser leidas desde diferentes ficheros.
     *
     * @author Nacho Brodin <brodin_ign@gva.es>
     */
    class BandSwitch {
        private ArrayList geoFiles = new ArrayList(); //Lista de GeoRasterFile con los ficheros raster cargados
        private FileBands[] listBands = new FileBands[3];
        private boolean debug = false;

        BandSwitch() {
            for (int i = 0; i < listBands.length; i++)
                listBands[i] = new FileBands();
        }

        /**
         * Aade un fichero
         * @param grf
         */
        public void addFile(GeoRasterFile grf) {
            geoFiles.add(grf);

            if (debug) {
                this.show("addFile");
            }
        }

        /**
         * Elimina un fichero
         * @param grf
         */
        public void removeFile(GeoRasterFile grf) {
            for (int iFile = 0; iFile < geoFiles.size(); iFile++) {
                if (grf.equals(geoFiles.get(iFile))) {
                    geoFiles.remove(iFile);
                    iFile--;
                }
            }

            if (debug) {
                this.show("removeFile");
            }
        }

        /**
         *
         * @param flag
         * @param grf
         * @param nBand
         */
        public void setBand(int flag, GeoRasterFile grf, int nBand) {
            if ((flag & GeoRasterFile.RED_BAND) == GeoRasterFile.RED_BAND) {
                listBands[0].setBand(nBand);
                listBands[0].setGeoRasterFile(grf);
            } else if ((flag & GeoRasterFile.GREEN_BAND) == GeoRasterFile.GREEN_BAND) {
                listBands[1].setBand(nBand);
                listBands[1].setGeoRasterFile(grf);
            } else if ((flag & GeoRasterFile.BLUE_BAND) == GeoRasterFile.BLUE_BAND) {
                listBands[2].setBand(nBand);
                listBands[2].setGeoRasterFile(grf);
            } else {
                return;
            }

            grf.setBand(flag, nBand);
        }

        /**
         * Asigna las bandas
         * @param flag
         * @param nBand
         */
        public void setBand(int flag, int nBand) {
            int cont = 0;

            for (int iGrf = 0; iGrf < geoFiles.size(); iGrf++) {
                for (int iBand = 0;
                         iBand < ((GeoRasterFile) geoFiles.get(iGrf)).getBandCount();
                         iBand++) {
                    if (((flag & GeoRasterFile.RED_BAND) == GeoRasterFile.RED_BAND) &&
                            (cont == nBand)) {
                        listBands[0].setGeoRasterFile(((GeoRasterFile) geoFiles.get(iGrf)));
                        listBands[0].setBand(iBand);
                        listBands[0].setPos(iGrf);
                        ((GeoRasterFile) geoFiles.get(iGrf)).setBand(flag, iBand);

                        //System.out.println("==>Asignando banda R FILE="+iGrf+" BANDA="+iBand);
                    } else if (((flag & GeoRasterFile.GREEN_BAND) == GeoRasterFile.GREEN_BAND) &&
                                   (cont == nBand)) {
                        listBands[1].setGeoRasterFile(((GeoRasterFile) geoFiles.get(iGrf)));
                        listBands[1].setBand(iBand);
                        listBands[1].setPos(iGrf);
                        ((GeoRasterFile) geoFiles.get(iGrf)).setBand(flag, iBand);

                        //System.out.println("==>Asignando banda G FILE="+iGrf+" BANDA="+iBand);
                    } else if (((flag & GeoRasterFile.BLUE_BAND) == GeoRasterFile.BLUE_BAND) &&
                                   (cont == nBand)) {
                        listBands[2].setGeoRasterFile(((GeoRasterFile) geoFiles.get(iGrf)));
                        listBands[2].setBand(iBand);
                        listBands[2].setPos(iGrf);
                        ((GeoRasterFile) geoFiles.get(iGrf)).setBand(flag, iBand);

                        //System.out.println("==>Asignando banda B FILE="+iGrf+" BANDA="+iBand);
                    }

                    cont++;
                }
            }

            if (debug) {
                this.show("setBand");
            }
        }

        /**
         * Obtiene el nmero de bandas
         * @return
         */
        public int getBandCount() {
            int nbandas = 0;

            for (int iGrf = 0; iGrf < geoFiles.size(); iGrf++)
                nbandas += ((GeoRasterFile) geoFiles.get(iGrf)).getBandCount();

            return nbandas;
        }

        /**
         * Obtiene el GeoRasterFile que hay que leer para cargar la banda del rojo
         * y la banda de este fichero que se utiliza para esta asignacin.
         * @return        Estructura FileBand con la relacin fichero-banda
         */
        public FileBands getBandR() {
            return listBands[0];
        }

        /**
         * Obtiene el GeoRasterFile que hay que leer para cargar la banda del verde
         * y la banda de este fichero que se utiliza para esta asignacin.
         * @return        Estructura FileBand con la relacin fichero-banda
         */
        public FileBands getBandG() {
            return listBands[1];
        }

        /**
         * Obtiene el GeoRasterFile que hay que leer para cargar la banda del azul
         * y la banda de este fichero que se utiliza para esta asignacin.
         * @return        Estructura FileBand con la relacin fichero-banda
         */
        public FileBands getBandB() {
            return listBands[2];
        }

        public void show(String op) {
            String banda = null;
            System.out.println("** " + op + " **");

            for (int i = 0; i < 3; i++) {
                if (i == 0) {
                    banda = new String("Rojo");
                } else if (i == 1) {
                    banda = new String("Verde");
                } else if (i == 2) {
                    banda = new String("Azul");
                }

                System.out.println("** BANDA IMAGE=" + banda + " FILEPOS=" +
                                   listBands[i].getBand() +
                                   " BANDA DEL FICHERO=" +
                                   listBands[i].getBand());
            }
        }
    }
	/**
	 * @return Returns the vName.
	 */
	public String getVName() {
		return vName;
	}
	/**
	 * @param name The vName to set.
	 */
	public void setVName(String name) {
		vName = name;
	}
}
