/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.cmd;

import org.cresques.cts.IProjection;

import org.cresques.px.PxLayerList;

import org.cresques.ui.CQCursor;
import org.cresques.ui.CQMapCanvas;
import org.cresques.ui.CQSaveRaster;
import org.cresques.ui.raster.DataInputSaveRaster;
import org.cresques.ui.raster.SaveRasterDialogPanel;
import org.cresques.ui.raster.SaveSetupPanel;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;


/**
 * Comando save raster.
 * Permite seleccionar una zona del canvas para que pueda ser guardado
 * a formato raster georeferenciado.
 * @see org.cresques.geo.cover.Coverage
 * @author "Nacho Brodin" (brodin_ign@gva.es)
 */
public class CmdSaveRaster extends Cmd {
    private Point2D ptIni = null;
    private CQSaveRaster saveRaster = null;
    private IProjection prj = null;
    private PxLayerList layerList = null;

    /**
     * Construye un nuevo CmdInfo para el Canvas
     * @param canvas
     */
    public CmdSaveRaster(CQMapCanvas canvas) {
        super(canvas);
        eventsWanted = (LEFT | RIGHT | PRESS | RELEASE | DRAG);
        cursor = CQCursor.getCursor(CQCursor.INFO_CURSOR);
    }

    /**
     *
     * @param saveRaster
     */
    public void setCQSaveRaster(CQSaveRaster saveRaster) {
        System.out.println("setCQSaveRaster");
        this.saveRaster = saveRaster;
    }

    /**
     * Asigna la proyeccin
     * @param prj Proyeccin
     */
    public void setProjection(IProjection prj) {
        //System.out.println("setProjection");
        this.prj = prj;
    }

    /**
     * Asigna la lista de capas
     * @param layerList Lista de capas
     */
    public void setLayerList(PxLayerList layerList) {
        //System.out.println("setLayerList");
        this.layerList = layerList;
    }

    /**
     * Recibe los eventos del ratn.
     */
    public void cmd(Point2D pt, int bt, int mouseEvent) {
        //System.out.println("Save Raster: Evento = "+ mouseEvent);
        if ((mouseEvent == PRESS) && (bt == MouseEvent.BUTTON1)) {
            ptIni = pt;
        } else if ((mouseEvent == RELEASE) && (bt == MouseEvent.BUTTON1)) {
            if ((pt.getX() == ptIni.getX()) || (pt.getY() == ptIni.getY())) {
                System.err.println("Ptos coincidentes");
            } else {
                double iniX;
                double iniY;
                double finX;
                double finY;

                if (pt.getX() > ptIni.getX()) {
                    iniX = ptIni.getX();
                    finX = pt.getX();
                } else {
                    finX = ptIni.getX();
                    iniX = pt.getX();
                }

                if (pt.getY() > ptIni.getY()) {
                    iniY = ptIni.getY();
                    finY = pt.getY();
                } else {
                    finY = ptIni.getY();
                    iniY = pt.getY();
                }

                saveRaster.setProjection(this.prj);
                saveRaster.setLayerList(this.layerList);

                DataInputSaveRaster dialog = ((SaveSetupPanel) (((SaveRasterDialogPanel) saveRaster.getContentPane()).getContentPanel())).getSaveParameters();
                int indexPoint = String.valueOf(finX).indexOf('.');
                dialog.getTinf_derX().setText(String.valueOf(finX).substring(0,
                                                                             indexPoint +
                                                                             2));
                indexPoint = String.valueOf(finY).indexOf('.');
                dialog.getTinf_derY().setText(String.valueOf(finY).substring(0,
                                                                             indexPoint +
                                                                             2));
                indexPoint = String.valueOf(iniX).indexOf('.');
                dialog.getTsup_izqX().setText(String.valueOf(iniX).substring(0,
                                                                             indexPoint +
                                                                             2));
                indexPoint = String.valueOf(iniY).indexOf('.');
                dialog.getTsup_izqY().setText(String.valueOf(iniY).substring(0,
                                                                             indexPoint +
                                                                             2));
                saveRaster.show();
            }
        }
    }
}
