/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JTextField;


/**
 * Panel que contiene los elementos para el dialogo del filtro de realce
 * @author Nacho Brodin <brodin_ign@gva.es>
 */
public class EnhancedPanel extends JPanel {
    final private static long serialVersionUID = -3370601314380922368L;
    /**
     * Nombre del panel
     */
    private String nom = "Realce";
    FilterRasterDialogPanel parent = null;
    private JPanel jPanel = null;
    private JPanel jPanel1 = null;
    private JPanel jPanel2 = null;
    private JPanel jPanel3 = null;
    private JRadioButton jRadioButton = null;
    private JRadioButton jRadioButton1 = null;
    private JCheckBox jCheckBox = null;
    
    /**
     * Variable accesible para la traduccin
     */
    public JLabel lLineal = null;
    
    /**
     * Variable accesible para la traduccin
     */
    public JLabel lWithoutEnhanced = null;
    
    /**
     * Variable accesible para la traduccin
     */
    public JPanel pGeneralEnhanced = null;
    private ButtonGroup grupo = new ButtonGroup();
    private Image iconoLineal = null;
    private Image iconoSinRealce = null;
    private JPanel jPanel5 = null;
    private JSlider jSlider = null;
    private JTextField jTextField = null;
    private JPanel jPanel6 = null;
    private JCheckBox jCheckBox1 = null;
    
    /**
     * Variable accesible para la traduccin
     */
    public JLabel lCut = null;
    
    /**
     * Variable accesible para la traduccin
     */
    public JPanel pTail = null;
    
    /**
     * Variable accesible para la traduccin
     */
    public JLabel lQueue = null;
    
    /**
     * Variable accesible para su traduccin
     */
    public JPanel pSlider = null;
    
    /**
     * Variable accesible para la traduccin
     */
    public JPanel pRemove = null;
    
    /**
     * Variable accesible para la traduccin
     */
    public JLabel lRemove = null;

    /**
     * Constructor.
     * @param parent
     */
    public EnhancedPanel(FilterRasterDialogPanel parent) {
        this.parent = parent;
        initialize();
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    private void initialize() {
        this.setLayout(new BorderLayout());
        this.setSize(445, 239);
        this.setPreferredSize(new java.awt.Dimension(445, 239));
        this.add(getJPanel(), java.awt.BorderLayout.CENTER);

        initControls();
    }

    /**
     * Obtiene el nombre del panel
     * @return Cadena con el nombre del panel
     */
    public String getName(){
    	return this.nom;
    }
    
    /**
     * Inicializa controles a sus valores por defecto
     */
    public void initControls() {
        jRadioButton1.setSelected(true);
        this.getTailText().setText("0.0");
        this.getTailSlider().setValue(0);
        this.setActiveTailControl(false);
        this.getTailCheck().setEnabled(false);
        this.getRemoveCheck().setEnabled(false);
    }

    /**
     * This method initializes jPanel
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel() {
        if (jPanel == null) {
            FlowLayout flowLayout2 = new FlowLayout();
            GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
            jPanel = new JPanel();
            jPanel.setLayout(flowLayout2);
            gridBagConstraints4.gridx = 0;
            gridBagConstraints4.gridy = 0;
            jPanel.setPreferredSize(new java.awt.Dimension(445, 239));
            flowLayout2.setAlignment(java.awt.FlowLayout.CENTER);
            jPanel.add(getJPanel4(), null);
            jPanel.add(getJPanel8(), null);
        }

        return jPanel;
    }

    /**
     * This method initializes jPanel1
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel1() {
        if (jPanel1 == null) {
            lLineal = new JLabel("Lineal directo",
                                new ImageIcon("images/lineal.gif"),
                                JLabel.CENTER);
            jPanel1 = new JPanel();
            jPanel1.setLayout(new BorderLayout());
            jPanel1.add(getLinealDirectoRadioButton(),
                        java.awt.BorderLayout.WEST);
            jPanel1.add(lLineal, java.awt.BorderLayout.EAST);
        }

        return jPanel1;
    }

    /**
     * This method initializes jPanel2
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel2() {
        if (jPanel2 == null) {
            jPanel2 = new JPanel();
            jPanel2.setLayout(new BorderLayout());
            jPanel2.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,
                                                                         1));
            jPanel2.setPreferredSize(new java.awt.Dimension(220, 95));
            jPanel2.add(getJPanel5(), java.awt.BorderLayout.CENTER);
            jPanel2.add(getJPanel6(), java.awt.BorderLayout.SOUTH);
            jPanel2.add(getJPanel7(), java.awt.BorderLayout.NORTH);
        }

        return jPanel2;
    }

    /**
     * This method initializes jPanel3
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel3() {
        if (jPanel3 == null) {
            lWithoutEnhanced = new JLabel("Sin Realce",
                                 new ImageIcon("images/sinrealce.gif"),
                                 JLabel.CENTER);
            jPanel3 = new JPanel();
            jPanel3.setLayout(new BorderLayout());
            jPanel3.add(getSinRealceRadioButton(), java.awt.BorderLayout.WEST);
            jPanel3.add(lWithoutEnhanced, java.awt.BorderLayout.EAST);
        }

        return jPanel3;
    }

    /**
     * This method initializes jRadioButton
     *
     * @return javax.swing.JRadioButton
     */
    public JRadioButton getLinealDirectoRadioButton() {
        if (jRadioButton == null) {
            jRadioButton = new JRadioButton();
            grupo.add(jRadioButton);
        }

        return jRadioButton;
    }

    /**
     * This method initializes jRadioButton1
     *
     * @return javax.swing.JRadioButton
     */
    public JRadioButton getSinRealceRadioButton() {
        if (jRadioButton1 == null) {
            jRadioButton1 = new JRadioButton();
            jRadioButton1.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
            grupo.add(jRadioButton1);
        }

        return jRadioButton1;
    }

    /**
     * This method initializes jCheckBox
     *
     * @return javax.swing.JCheckBox
     */
    public JCheckBox getTailCheck() {
        if (jCheckBox == null) {
            jCheckBox = new JCheckBox();
        }

        return jCheckBox;
    }

    /**
     * This method initializes jPanel4
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel4() {
        if (pGeneralEnhanced == null) {
            GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
            GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
            pGeneralEnhanced = new JPanel();
            pGeneralEnhanced.setLayout(new GridBagLayout());
            gridBagConstraints9.gridx = 0;
            gridBagConstraints9.gridy = 0;
            gridBagConstraints9.insets = new java.awt.Insets(5, 0, 5, 2);
            gridBagConstraints9.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints10.gridx = 0;
            gridBagConstraints10.gridy = 1;
            gridBagConstraints10.insets = new java.awt.Insets(5, 0, 5, 5);
            pGeneralEnhanced.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0,
                                                                          0, 0));
            pGeneralEnhanced.setPreferredSize(new java.awt.Dimension(175, 200));
            pGeneralEnhanced.add(getJPanel3(), gridBagConstraints9);
            pGeneralEnhanced.add(getJPanel1(), gridBagConstraints10);
        }

        return pGeneralEnhanced;
    }

    /**
     * This method initializes jPanel5
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel5() {
        if (jPanel5 == null) {
            jPanel5 = new JPanel();
            jPanel5.add(getTailSlider(), null);
        }

        return jPanel5;
    }

    /**
     * This method initializes jSlider
     *
     * @return javax.swing.JSlider
     */
    public JSlider getTailSlider() {
        if (jSlider == null) {
            jSlider = new JSlider();
            jSlider.setPreferredSize(new java.awt.Dimension(240, 16));
        }

        return jSlider;
    }

    /**
     * This method initializes jTextField
     *
     * @return javax.swing.JTextField
     */
    public JTextField getTailText() {
        if (jTextField == null) {
            jTextField = new JTextField();
            jTextField.setPreferredSize(new java.awt.Dimension(40, 19));
            jTextField.setText("0.0");
        }

        return jTextField;
    }

    /**
     * Activa/Desactiva los controles de Recorte de colas
     * @param active
     */
    public void setActiveTailControl(boolean active) {
        this.getTailSlider().setEnabled(active);
        this.getTailText().setEnabled(active);
        this.lQueue.setEnabled(active);
        this.lCut.setEnabled(active);
    }

    /**
     *
     * @param active
     */
    public void setActiveRemoveCheck(boolean active) {
        this.getRemoveCheck().setEnabled(active);
        this.lRemove.setEnabled(active);
    }

    /**
     * Asigna el procentaje de recorte de colas aplicado
     * @param tail
     */
    public void setPercentTail(int tail) {
        jTextField.setText(String.valueOf(tail));
        jSlider.setValue(tail);
    }

    /**
     * Asigna el filtro seleccionado cuando se abre el dialogo
     * @param filter
     */
    public void setSelectedFilter(int filter) {
        switch (filter) {
        case 0:
            jRadioButton1.setSelected(true); //Sin realce
            jCheckBox.setEnabled(false);
            setActiveTailControl(false);
            this.setActiveRemoveCheck(false);

            break;

        case 1:
            this.getRemoveCheck().setSelected(true);

        case 2:
            jRadioButton.setSelected(true); //Realce lineal
            this.getTailCheck().setEnabled(true);
            this.setActiveRemoveCheck(true);

            if (!this.getTailText().getText().equals("") &&
                    !this.getTailText().getText().equals("0.0")) {
                this.getTailCheck().setSelected(true);
                this.setActiveTailControl(true);
            }

            break;
        }
    }

    /**
     * This method initializes jPanel6
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel6() {
        if (jPanel6 == null) {
            FlowLayout flowLayout3 = new FlowLayout();
            lCut = new JLabel();
            jPanel6 = new JPanel();
            jPanel6.setLayout(flowLayout3);
            lCut.setText("% recorte");
            flowLayout3.setAlignment(java.awt.FlowLayout.LEFT);
            jPanel6.add(getTailText(), null);
            jPanel6.add(lCut, null);
        }

        return jPanel6;
    }

    /**
     * This method initializes jCheckBox1
     *
     * @return javax.swing.JCheckBox
     */
    public JCheckBox getRemoveCheck() {
        if (jCheckBox1 == null) {
            jCheckBox1 = new JCheckBox();
        }

        return jCheckBox1;
    }

    /**
     * This method initializes jPanel7
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel7() {
        if (pTail == null) {
        	lQueue = new JLabel();

            FlowLayout flowLayout1 = new FlowLayout();
            pTail = new JPanel();
            pTail.setLayout(flowLayout1);
            lQueue.setText("Recorte de colas");
            flowLayout1.setAlignment(java.awt.FlowLayout.LEFT);
            pTail.add(getTailCheck(), null);
            pTail.add(lQueue, null);
        }

        return pTail;
    }

    /**
     * This method initializes jPanel8
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel8() {
        if (pSlider == null) {
        	pSlider = new JPanel();
        	pSlider.setLayout(new BorderLayout());
        	pSlider.setPreferredSize(new java.awt.Dimension(250, 130));
        	pSlider.add(getJPanel2(), java.awt.BorderLayout.SOUTH);
        	pSlider.add(getJPanel9(), java.awt.BorderLayout.NORTH);
        }

        return pSlider;
    }

    /**
     * This method initializes jPanel9
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel9() {
        if (pRemove == null) {
            lRemove = new JLabel();

            FlowLayout flowLayout11 = new FlowLayout();
            pRemove = new JPanel();
            pRemove.setLayout(flowLayout11);
            lRemove.setText("Eliminar Extremos");
            flowLayout11.setAlignment(java.awt.FlowLayout.LEFT);
            pRemove.add(getRemoveCheck(), null);
            pRemove.add(lRemove, null);
        }

        return pRemove;
    }
} //  @jve:decl-index=0:visual-constraint="36,15"
