/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.util.TreeMap;

import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.cresques.io.GeoRasterFile;
import org.cresques.ui.DefaultTabPanel;


/**
 * Panel que contiene todos los paneles de los tabs del cuadro de dialogo
 * de bandas y transparencias
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class FilterRasterDialogPanel extends DefaultTabPanel
    implements ActionListener, ChangeListener, MouseListener, FocusListener,
               KeyListener {
    final private static long serialVersionUID = -3370601314380922368L;

    /**
     * Tamao en X del panel interior.El tab de dentro es proporcional
     */
    protected int sizePanelX = 450;
    
    /**
     * Tamao en Y del panel interior. El tab de dentro es proporcional
     */
    protected int sizePanelY = 260;
    private RasterTransparencyPanel pTrans = null;
    
    /**
     * Variable que contiene el panel de control de bandas. 
     */
    protected BandSetupPanel pSetup = null;
    
    /**
     * Variable que contiene el panel de control de realce. 
     */
    protected EnhancedPanel pEnhan = null;
    
    /**
     * Variable que contiene el panel de Sharpening. 
     */
    protected SharpeningPanel pSharp = null;
    
    /**
     * Variable que contiene el panel de informacin de raster. 
     */
    protected InfoPanel pInfo = null;
    private int[][] rangeRed = null;
    private int[][] rangeGreen = null;
    private int[][] rangeBlue = null;

    /**
     * Propiedades guardadas en una matriz Nx2 con la forma propiedad/valor.
     */
    public Object[][] props = null;
    
    /**
     * Nmero de bandas.
     */
    protected int nbands = 0;

    /**
     * Lista de georrasterfiles correspondiente a los ficheros de bandas
     */
    protected GeoRasterFile[] grf = null;
    private TreeMap shadePanel = new TreeMap();
	
    /**
     * Constructor. Inicializa los paneles y propiedades
     * @param props        Propiedades
     */
    public FilterRasterDialogPanel(Object[][] props) {
        super();
        init(props);
    }

    /**
     * Constructor
     */
    public FilterRasterDialogPanel() {
        super();
    }

    /**
     * Inicializa las propiedades
     * @param props        Propiedades. Guardadas en una matriz Nx2 con la forma proiedad/valor
     */
    public void init(Object[][] props) {
        contentPane.setPreferredSize(new java.awt.Dimension(sizePanelX,
                                                            sizePanelY));
        this.tabbedPane.setPreferredSize(new java.awt.Dimension(sizePanelX,
                                                                sizePanelY - 5));
        this.props = props;
        initPanels();
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    protected void initPanels() {
        this.setBounds(0, 0, 355, 230);
        pTrans = new RasterTransparencyPanel(this);
        pSetup = new BandSetupPanel();
        pEnhan = new EnhancedPanel(this);
        pInfo = new InfoPanel(this);
        pSharp = new SharpeningPanel();
        this.addTab(pInfo.getName(), pInfo);
        this.addTab(pSetup.getName(), pSetup);
        this.addTab(pTrans.getName(), pTrans);
        this.addTab(pEnhan.getName(), pEnhan);
        this.addTab(pSharp.getName(), pSharp);

        pTrans.getOpacityCheck().addActionListener(this);
        pTrans.getTransparencyCheck().addActionListener(this);
        pTrans.getTRojo().addActionListener(this);
        pTrans.getTVerde().addActionListener(this);
        pTrans.getTAzul().addActionListener(this);
        pTrans.getOpacitySlider().addChangeListener(this);
        pTrans.getOpacityText().addActionListener(this);
        pTrans.getOpacityText().addFocusListener(this);
        pTrans.getTRojo().addFocusListener(this);
        pTrans.getTVerde().addFocusListener(this);
        pTrans.getTAzul().addFocusListener(this);
        pTrans.getTRojo().addMouseListener(this);
        pTrans.getTVerde().addMouseListener(this);
        pTrans.getTAzul().addMouseListener(this);
        pTrans.addFocusListener(this);
        this.getTab().addFocusListener(this);

        pEnhan.getTailCheck().addActionListener(this);
        pEnhan.getTailSlider().addChangeListener(this);
        pEnhan.getSinRealceRadioButton().addMouseListener(this);
        pEnhan.getLinealDirectoRadioButton().addMouseListener(this);
        pEnhan.getTailText().addActionListener(this);
        pEnhan.getTailText().addFocusListener(this);

        //Ocultamos el botn de aplicar
        //this.getApplyButton().setVisible(false);
    }

    /**
     * Asigna la visibilidad de un tab a verdadero o falso. La
     * seleccin del tab se hace por el identificador.
     * @param tab	Identificador del tab. Variable nom del mismo
     * @param active	True o false para visible o invisible.
     */
    public void setTabVisible(String tab, boolean active){
    	if(active == false){
	    	for(int i=0; i<this.getTab().getTabCount(); i++){
	    		if(tab.equals( ((JPanel)this.getTab().getComponentAt(i)).getName() )){
	    			shadePanel.put(tab, this.getTab().getComponentAt(i));
	    			this.getTab().removeTabAt(i);
	    		}
	    	}
    	}else
    		this.addTab(tab, ((JPanel)shadePanel.get(tab)) );
    }
    
    /**
     * Selecciona el panel indicado por index
     * @param index        panel seleccionado
     */
    public void setSelectedIndex(int index) {
        tabbedPane.setSelectedIndex(index);
    }

    /**
     * Obtiene el panel que corresponde a la seleccin de bandas
     * @return Panel de seleccin de bandas
     */
    public BandSetupPanel getBandSetup() {
        return this.pSetup;
    }

    /**
     * Obtiene el panel que corresponde a la seleccin de transparencia y opacidad
     * @return Panel de transparencia y opacidad
     */
    public RasterTransparencyPanel getTransparencyPanel() {
        return this.pTrans;
    }

    /**
     * Obtiene el panel que corresponde a la seleccin de sharpening
     * @return Panel de sharpening
     */
    public SharpeningPanel getSharpeningPanel() {
        return this.pSharp;
    }
    
    /**
     * Obtiene el panel que corresponde al filtro de realce
     * @return Panel del filtro de realce
     */
    public EnhancedPanel getEnhancedPanel() {
        return this.pEnhan;
    }

    /**
     * Obtiene el panel Info
     * @return InfoPanel
     */
    public InfoPanel getInfoPanel() {
        return this.pInfo;
    }

    /**
     * Eventos sobre TextField y CheckBox. Controla eventos de checkbox de opacidad, transparencia,
     * recorte de colas y los textfield de opacidad, valores de transparencia por banda y
     * porcentaje de recorte.
     */
    public void actionPerformed(ActionEvent e) {
        //Evento sobre el checkbox de opacidad
        if (e.getSource().equals(pTrans.getOpacityCheck())) {
            //Check de opacidad activado -> Activar controles de opacidad
            if (pTrans.getOpacityCheck().isSelected()) {
                pTrans.getOpacitySlider().setEnabled(true);
                pTrans.getOpacityText().setEnabled(true);
            } else {
                pTrans.getOpacitySlider().setEnabled(false);
                pTrans.getOpacityText().setEnabled(false);
            }
        }

        //Evento sobre el checkbox de transparencia
        if (e.getSource().equals(pTrans.getTransparencyCheck())) {
            //Check de opacidad activado -> Activar controles de opacidad
            if (pTrans.getTransparencyCheck().isSelected()) {
                if (nbands == 1) {
                    pTrans.getTRojo().setEnabled(true);
                    pTrans.getTVerde().setEnabled(false);
                    pTrans.getTAzul().setEnabled(false);
                }

                if (nbands == 2) {
                    pTrans.getTRojo().setEnabled(true);
                    pTrans.getTVerde().setEnabled(true);
                    pTrans.getTAzul().setEnabled(false);
                }

                if (nbands >= 3) {
                    pTrans.getTRojo().setEnabled(true);
                    pTrans.getTVerde().setEnabled(true);
                    pTrans.getTAzul().setEnabled(true);
                }
            } else {
                pTrans.getTRojo().setEnabled(false);
                pTrans.getTVerde().setEnabled(false);
                pTrans.getTAzul().setEnabled(false);
            }
        }

        //Evento sobre el textfield de opacidad
        if (e.getSource().equals(pTrans.getOpacityText())) {
            checkOpacityText();
        }

        //Evento sobre el textfield de valor para el Rojo
        if (e.getSource().equals(pTrans.getTRojo())) {
            try {
                rangeRed = getTransparencyValues(pTrans.getTRojo().getText());
            } catch (IOException exc) {
                pTrans.getTRojo().setText("");
            }
        }

        //Evento sobre el textfield de valor para el Verde
        if (e.getSource().equals(pTrans.getTVerde())) {
            try {
                rangeGreen = getTransparencyValues(pTrans.getTVerde().getText());
            } catch (IOException exc) {
                pTrans.getTVerde().setText("");
            }
        }

        //Evento sobre el textfield de valor para el Azul
        if (e.getSource().equals(pTrans.getTAzul())) {
            try {
                rangeBlue = getTransparencyValues(pTrans.getTAzul().getText());
            } catch (IOException exc) {
                pTrans.getTAzul().setText("");
            }
        }

        //Evento sobre el checkbox de Recorte de colas
        if (e.getSource().equals(pEnhan.getTailCheck())) {
            if (pEnhan.getTailCheck().isSelected()) {
                pEnhan.setActiveTailControl(true);
            } else {
                pEnhan.setActiveTailControl(false);
            }
        }

        //Evento sobre el textfield de porcentaje de recorte
        if (e.getSource().equals(pEnhan.getTailText())) {
            checkTailText();
        }
    }

    /**
     * Obtiene el rango de valores a partir de la cadena de
     * texto introducida por el usuario.
     * @param values
     */
    private int[][] getTransparencyValues(String values)
                                   throws IOException {
        int[][] rangeTransparency = null;

        for (int i = 0; i < values.length(); i++) {
            char c = values.charAt(i);

            //Control de caracter valido
            if ((c != ':') && (c != ',')) {
                try {
                    Integer.parseInt(String.valueOf(c));
                } catch (Exception exp) {
                    System.err.println("Caracteres incorrectos en la cadena:" +
                                       c);
                    throw new IOException("Caracteres incorrectos en la cadena.");
                }
            }

            //Control de comienzo por un simbolo
            if (values.startsWith(",") || values.startsWith(":") ||
                    values.endsWith(",") || values.endsWith(":")) {
                System.err.println("La cadena empieza o acaba con simbolos incorrectos");
                throw new IOException();
            }

            //Control de signos consecutivos
            if (i < (values.length() - 1)) {
                char cmas = values.charAt(i + 1);

                if (((c == ',') || (c == ':')) &&
                        ((cmas == ',') || (cmas == ':'))) {
                    System.err.println("Signos consecutivos");
                    throw new IOException();
                }
            }

            //Control de dos : seguidos con un nmero en el medio
            if ((i < (values.length() - 3)) && (c == ':')) {
                int n = i + 1;

                while ((n < (values.length() - 1)) &&
                           (values.charAt(n) != ',') &&
                           (values.charAt(n) != ':'))
                    n++;

                char signoSgte = values.charAt(n);

                if (signoSgte == ':') {
                    System.err.println("Dos separadores de rango consecutivos");
                    throw new IOException();
                }
            }
        }

        //Obtenemos los valores de los intervalos
        if (!values.equals("")) {
            String[] t1 = null;
            t1 = values.split("\\,");
            rangeTransparency = new int[t1.length][2];

            for (int i = 0; i < t1.length; i++) {
                if (t1[i].indexOf(":") == -1) {
                    rangeTransparency[i][0] = rangeTransparency[i][1] = Integer.parseInt(t1[i]);
                } else {
                    String[] t2 = null;
                    t2 = t1[i].split("\\:");

                    if (Integer.parseInt(t2[1]) > Integer.parseInt(t2[0])) {
                        rangeTransparency[i][0] = Integer.parseInt(t2[0]);
                        rangeTransparency[i][1] = Integer.parseInt(t2[1]);
                    } else {
                        rangeTransparency[i][0] = Integer.parseInt(t2[1]);
                        rangeTransparency[i][1] = Integer.parseInt(t2[0]);
                    }
                }

                if ((rangeTransparency[i][0] < 0) ||
                        (rangeTransparency[i][0] > 255) ||
                        (rangeTransparency[i][1] < 0) ||
                        (rangeTransparency[i][1] > 255)) {
                    System.err.println("Valores fuera de rango (0-255)");
                    throw new IOException();
                }
            }

            return rangeTransparency;

            //for(int i=0;i<rangeTransparency.length;i++)
            //	System.out.println("("+rangeTransparency[i][0]+":"+rangeTransparency[i][1]+")");
        }

        return null;
    }

    /**
     * Devuelve el rango de valores para poner transparencia en el Rojo
     * @return
     */
    public int[][] getRangeRed() {
        return rangeRed;
    }

    /**
     * Devuelve el rango de valores para poner transparencia en el Verde
     * @return
     */
    public int[][] getRangeGreen() {
        return rangeGreen;
    }

    /**
     * Devuelve el rango de valores para poner transparencia en el Azul
     * @return
     */
    public int[][] getRangeBlue() {
        return rangeBlue;
    }

    /**
     * Asigna valore para los rangos
     * @param red
     * @param green
     * @param blue
     */
    public void setRanges(int[][] red, int[][] green, int[][] blue) {
        rangeRed = red;
        rangeGreen = green;
        rangeBlue = blue;

        if (pTrans != null) {
            StringBuffer rango = new StringBuffer();

            if (red != null) {
                for (int i = 0; i < red.length; i++) {
                    if (red[i][0] == red[i][1]) {
                        rango.append(String.valueOf(red[i][0]) + ",");
                    } else {
                        rango.append(String.valueOf(red[i][0]) + ":" +
                                     String.valueOf(red[i][1]) + ",");
                    }
                }

                String t = rango.toString();

                if (t.endsWith(",")) {
                    t = t.substring(0, t.length() - 1);
                }

                pTrans.getTRojo().setText(t);
                pTrans.setActiveTransparencyControl(true);
            }

            if (green != null) {
                rango = new StringBuffer();

                for (int i = 0; i < green.length; i++) {
                    if (green[i][0] == green[i][1]) {
                        rango.append(String.valueOf(green[i][0]) + ",");
                    } else {
                        rango.append(String.valueOf(green[i][0]) + ":" +
                                     String.valueOf(green[i][1]) + ",");
                    }
                }

                String t = rango.toString();

                if (t.endsWith(",")) {
                    t = t.substring(0, t.length() - 1);
                }

                pTrans.getTVerde().setText(t);
                pTrans.setActiveTransparencyControl(true);
            }

            if (blue != null) {
                rango = new StringBuffer();

                for (int i = 0; i < blue.length; i++) {
                    if (blue[i][0] == blue[i][1]) {
                        rango.append(String.valueOf(blue[i][0]) + ",");
                    } else {
                        rango.append(String.valueOf(blue[i][0]) + ":" +
                                     String.valueOf(blue[i][1]) + ",");
                    }
                }

                String t = rango.toString();

                if (t.endsWith(",")) {
                    t = t.substring(0, t.length() - 1);
                }

                pTrans.setActiveTransparencyControl(true);
                pTrans.getTAzul().setText(t);
            }
        }
    }

    /**
     * Aade las bandas de los georrasterfile a los paneles 
     * que lo necesitan
     * @param files
     */
    public void addFiles(GeoRasterFile[] files){
    	if(pSharp != null)
    		pSharp.addFiles(files);
        if(pSetup != null)
        	pSetup.addFiles(files);
    }
    
    /**
     * Elimina las bandas de los georrasterfile a los paneles
     * @param file Banda a eliminar
     */
    public void removeFile(String file) {
    	if(pSharp != null)
    		pSharp.removeFile(file);
        if(pSetup != null)
        	pSetup.removeFile(file);
    }
    
    /**
     * Controla que si el formato introducido en el textfield
     * de opacidad es numerico se actualiza la posicin del
     * slider.
     */
    private void checkOpacityText() {
        String op = pTrans.getOpacityText().getText();
        int value = 0;

        try {
            if (!op.equals("")) {
                value = Integer.parseInt(op);
            }

            pTrans.getOpacitySlider().setValue(value);
        } catch (NumberFormatException exc) {
            System.err.println("Formato no numerico");
            pTrans.getOpacityText().setText("100");
            pTrans.getOpacitySlider().setValue(100);
        }
    }

    /**
     * Controla que si el formato introducido en el textfield
     * de Recorte de colas es numerico se actualiza la posicin del
     * slider
     */
    private void checkTailText() {
        String op = pEnhan.getTailText().getText();
        double value = 0;

        try {
            if (!op.equals("")) {
                value = (Double.parseDouble(op) * 100) / 50;
            }

            pEnhan.getTailSlider().setValue((int) Math.round(value));
            pEnhan.getTailText().setText(op);
        } catch (NumberFormatException exc) {
            System.err.println("Formato no numerico");
            pEnhan.getTailText().setText("0.0");
            pEnhan.getTailSlider().setValue(0);
        }
    }

    /**
     * Llama a las funciones que obtienen los valores de los pixels a los que se
     * quiere aplicar transparencia desde las cajas de texto.
     */
    public void checkTransparencyValues() {
        if (!pTrans.getTRojo().getText().equals("")) {
            try {
                rangeRed = getTransparencyValues(pTrans.getTRojo().getText());
            } catch (IOException exc) {
                pTrans.getTRojo().setText("");
            }
        }

        if (!pTrans.getTVerde().getText().equals("")) {
            try {
                rangeGreen = getTransparencyValues(pTrans.getTVerde().getText());
            } catch (IOException exc) {
                pTrans.getTVerde().setText("");
            }
        }

        if (!pTrans.getTAzul().getText().equals("")) {
            try {
                rangeBlue = getTransparencyValues(pTrans.getTAzul().getText());
            } catch (IOException exc) {
                pTrans.getTAzul().setText("");
            }
        }
    }

    /*
     *
     */
    public void stateChanged(ChangeEvent e) {
        //Ponemos el valor del texto de la opacidad de pendiendo de la posicin del Slider
        if (e.getSource().equals(pTrans.getOpacitySlider())) {
            pTrans.getOpacityText().setText(String.valueOf(pTrans.getOpacitySlider()
                                                                 .getValue()));
        }

        if (e.getSource().equals(pEnhan.getTailSlider())) {
            pEnhan.getTailText().setText(String.valueOf((pEnhan.getTailSlider()
                                                               .getValue() >> 1)) +
                                         ".0");
        }
    }

    public void focusGained(FocusEvent e) {
        //pTrans.updateTextBox();
    }

    public void focusLost(FocusEvent e) {
        checkOpacityText();
        checkTailText();
        checkTransparencyValues();
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        checkTransparencyValues();

        //Evento sobre los radio button de Realce 
        if (e.getSource().equals(pEnhan.getSinRealceRadioButton())) {
            if (pEnhan.getSinRealceRadioButton().isEnabled()) {
                pEnhan.getTailCheck().setEnabled(false);
                pEnhan.setActiveTailControl(false);
                pEnhan.getSinRealceRadioButton().setSelected(true);
                pEnhan.setActiveRemoveCheck(false);
            }
        }

        if (e.getSource().equals(pEnhan.getLinealDirectoRadioButton())) {
            pEnhan.getTailCheck().setEnabled(true);
            pEnhan.setActiveRemoveCheck(true);

            if (pEnhan.getTailCheck().isSelected()) {
                pEnhan.setActiveTailControl(true);
            }
        }
    }

    public void keyTyped(KeyEvent e) {
    }

    public void keyPressed(KeyEvent e) {
    }

    public void keyReleased(KeyEvent e) {
    }
}
