/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableColumn;


/**
 * Panel que contiene la informacin sobre un raster
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class InfoPanel extends JPanel {
    final private static long serialVersionUID = -3370601314380922368L;
    private JPanel jPanel = null;
    private JTable jTable = null;
    private JScrollPane jScrollPane = null;

    /**
     * Nombre del panel
     */
    private String nom = "Info";

    /**
     * Dialogo padre que contiene a este
     */
    public FilterRasterDialogPanel parent = null;

    /**
     * Propiedades guardadas en una matriz Nx2 con la forma proiedad/valor
     */
    public Object[][] props = null;

    /**
     * Cabeceras de las columnas del dialogo accesibles desde fuera para su traduccin
     */
    public Object[] columnNames = { "Propiedad", "Valor" };

    /**
     * Constructor que asigna el objeto del panel padre y las propiedades (props) contenidas este.
     * Ejecuta el mtodo initialize.
     * @param parent Panel que contiene a este
     */
    public InfoPanel(FilterRasterDialogPanel parent) {
        super();
        this.parent = parent;
        this.props = parent.props;
        initialize();
    }

    /**
     * Mtodo de inicializacin
     *
     * @return void
     */
    public void initialize() {
        this.setLayout(new BorderLayout());
        this.setSize(350, 210);
        this.add(getJPanel(), java.awt.BorderLayout.CENTER);
        initControls();
    }

    /**
     * Obtiene el nombre del panel
     * @return Cadena con el nombre del panel
     */
    public String getName(){
    	return this.nom;
    }
    
    /**
     * Resetea los controles por defecto
     */
    public void resetTable() {
        this.jTable = null;
        this.jPanel = null;
        this.jScrollPane = null;
    }

    /**
     * Inicializa controles a sus valores por defecto
     */
    public void initControls() {
        TableColumn column = null;
        column = jTable.getColumnModel().getColumn(0);
        column.setPreferredWidth(10);
    }

    /**
     * This method initializes jPanel
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel() {
        if (jPanel == null) {
            jPanel = new JPanel();
            jPanel.setLayout(new BorderLayout());
            jPanel.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
        }

        return jPanel;
    }

    /**
     * This method initializes jTable
     *
     * @return javax.swing.JTable
     */
    private JTable getJTable() {
        if (jTable == null) {
            jTable = new JTable(props, columnNames);
            jTable.setShowGrid(true);
            jTable.setShowHorizontalLines(false);

            //jTable.setRowHeight(30);
            jTable.setCellSelectionEnabled(true);
        }

        return jTable;
    }

    /**
     * This method initializes jScrollPane
     *
     * @return javax.swing.JScrollPane
     */
    private JScrollPane getJScrollPane() {
        if (jScrollPane == null) {
            jScrollPane = new JScrollPane();
            jScrollPane.setViewportView(getJTable());
        }

        return jScrollPane;
    }
} //  @jve:decl-index=0:visual-constraint="40,32"
