/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;


/**
 * Dialogo para asignar la transparencia por pixel y global al raster.
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class RasterTransparencyPanel extends JPanel {
    final private static long serialVersionUID = -3370601314380922368L;

    /**
     * Nombre del panel
     */
    private String nom = "Transparencia";
    FilterRasterDialogPanel parent = null;

    /**
     * Nmero de bandas del raster
     */
    public int nBands = 3;
    private String[] nameBands = { "Rojo", "Verde", "Azul" };
    private JPanel jPanel = null;
    private JPanel jPanel1 = null;
    private JPanel jPanel2 = null;
    private JPanel jPanel3 = null;
    private JCheckBox cbTransparencia = null;

    /**
     * Variable pblica para la traduccin
     */
    public JLabel lRedValue = null;
    private JTextFieldExt tRojo = null;

    /**
     * Variable pblica para la traduccin
     */
    public JLabel lGreenValue = null;

    /**
     * Variable pblica para la traduccin
     */
    public JLabel lBlueValue = null;

    /**
     * Variable pblica para la traduccin
     */
    public JPanel pGeneralTrans = null;

    /**
     * Variable pblica para la traduccin
     */
    public JLabel lRange = null;

    /**
     * Variable pblica para la traduccin
     */
    public JLabel lPixelValue = null;
    private JTextFieldExt tVerde = null;
    private JTextFieldExt tAzul = null;
    private JCheckBox cbOpacidad = null;
    private JSlider slOpacidad = null;
    private JTextField tOpacidad = null;
    private JPanel jPanel5 = null;
    private JPanel jPanel6 = null;

    /**
     * Constructor. Asigna la variable panel que contiene este
     * @param parent FilterRasterDialogPanel
     */
    public RasterTransparencyPanel(FilterRasterDialogPanel parent) {
        this.parent = parent;
        initialize();
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    private void initialize() {
        this.setLayout(new BorderLayout());
        this.setBounds(0, 0, 445, 239);
        this.setPreferredSize(new java.awt.Dimension(445, 239));
        this.add(getJPanel(), java.awt.BorderLayout.CENTER);
        initControls();
    }

    /**
     * Obtiene el nombre del panel
     * @return Cadena con el nombre del panel
     */
    public String getName(){
    	return this.nom;
    }
    
    /**
     * Asigna el nmero de bandas de la imagen
     * @param nBands
     */
    public void setBands(int nBands) {
        this.nBands = nBands;
    }

    /**
     * Asigna el nmero de bandas de la imagen
     * @param nBands
     */
    public void setNameBands(String[] nameBands) {
        this.nameBands = nameBands;
    }

    /**
     * Inicializa controles a sus valores por defecto
     */
    public void initControls() {
        this.getOpacityText().setText("100");
        this.setActiveOpacityControl(false);
        this.setActiveTransparencyControl(false);
    }

    /**
     * Esta funcin no est en uso.
     * Oculta los controles de transparencia por bandas en caso de que haya menos de 3 bandas.
     *
     */
    public void updateTextBox() {
        if (parent.nbands == 1) {
            this.getTRojo().setVisible(true);
            this.getTVerde().setVisible(false);
            this.getTAzul().setVisible(false);
            lRedValue.setVisible(true);
            lGreenValue.setVisible(false);
            lBlueValue.setVisible(false);
        }

        if (parent.nbands == 2) {
            this.getTRojo().setVisible(true);
            this.getTVerde().setVisible(true);
            this.getTAzul().setVisible(false);
            lRedValue.setVisible(true);
            lGreenValue.setVisible(true);
            lBlueValue.setVisible(false);
        }

        if (parent.nbands >= 3) {
            this.getTRojo().setVisible(true);
            this.getTVerde().setVisible(true);
            this.getTAzul().setVisible(true);
            lRedValue.setVisible(true);
            lGreenValue.setVisible(true);
            lBlueValue.setVisible(true);
        }
    }

    /**
     * This method initializes jPanel
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel() {
        if (jPanel == null) {
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
            GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
            GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
            GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
            GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
            jPanel = new JPanel();
            jPanel.setLayout(new GridBagLayout());
            gridBagConstraints5.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints5.ipady = 0;
            gridBagConstraints5.gridheight = 1;
            gridBagConstraints5.gridwidth = 0;
            gridBagConstraints5.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints5.insets = new java.awt.Insets(0, 0, 0, 0);
            gridBagConstraints6.gridwidth = 0;
            gridBagConstraints6.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints7.gridwidth = 1;
            gridBagConstraints7.insets = new java.awt.Insets(20, 0, 0, 0);
            gridBagConstraints8.gridwidth = 0;
            gridBagConstraints8.anchor = java.awt.GridBagConstraints.WEST;
            jPanel.setPreferredSize(new java.awt.Dimension(333, 113));
            gridBagConstraints9.gridx = 0;
            gridBagConstraints9.gridy = 1;
            gridBagConstraints9.insets = new java.awt.Insets(0, 180, 0, 0);
            gridBagConstraints1.gridx = 0;
            gridBagConstraints1.gridy = 2;
            gridBagConstraints1.insets = new java.awt.Insets(0, 180, 0, 0);
            jPanel.add(getJPanel1(), gridBagConstraints5);
            jPanel.add(getJPanel2(), gridBagConstraints6);
            jPanel.add(getJPanel3(), gridBagConstraints8);
            jPanel.add(getJPanel4(), gridBagConstraints7);
            jPanel.add(getJPanel5(), gridBagConstraints9);
            jPanel.add(getJPanel6(), gridBagConstraints1);
        }

        return jPanel;
    }

    /**
     * This method initializes jPanel1
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel1() {
        if (jPanel1 == null) {
            lRedValue = new JLabel();
            jPanel1 = new JPanel();
            jPanel1.setLayout(new BorderLayout());
            lRedValue.setText("Valor Rojo: ");
            jPanel1.setPreferredSize(new java.awt.Dimension(170, 42));
            jPanel1.add(getTransparencyCheck(), java.awt.BorderLayout.NORTH);
            jPanel1.add(lRedValue, java.awt.BorderLayout.WEST);
            jPanel1.add(getTRojo(), java.awt.BorderLayout.EAST);
        }

        return jPanel1;
    }

    /**
     * This method initializes jPanel2
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel2() {
        if (jPanel2 == null) {
            lGreenValue = new JLabel();
            jPanel2 = new JPanel();
            jPanel2.setLayout(new BorderLayout());
            lGreenValue.setText("Valor Verde: ");
            jPanel2.setPreferredSize(new java.awt.Dimension(170, 19));
            jPanel2.add(lGreenValue, java.awt.BorderLayout.WEST);
            jPanel2.add(getTVerde(), java.awt.BorderLayout.EAST);
        }

        return jPanel2;
    }

    /**
     * This method initializes jPanel3
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel3() {
        if (jPanel3 == null) {
            lBlueValue = new JLabel();
            jPanel3 = new JPanel();
            jPanel3.setLayout(new BorderLayout());
            lBlueValue.setText("Valor Azul: ");
            jPanel3.setPreferredSize(new java.awt.Dimension(170, 19));
            jPanel3.add(lBlueValue, java.awt.BorderLayout.WEST);
            jPanel3.add(getTAzul(), java.awt.BorderLayout.EAST);
        }

        return jPanel3;
    }

    /**
     * This method initializes jCheckBox
     *
     * @return javax.swing.JCheckBox
     */
    public JCheckBox getTransparencyCheck() {
        if (cbTransparencia == null) {
            cbTransparencia = new JCheckBox();
            cbTransparencia.setText("Transparencia");
        }

        return cbTransparencia;
    }

    /**
     * This method initializes jTextField
     *
     * @return javax.swing.JTextField
     */
    public JTextFieldExt getTRojo() {
        if (tRojo == null) {
            tRojo = new JTextFieldExt(this.getBackground());
            tRojo.setPreferredSize(new java.awt.Dimension(80, 19));
        }

        return tRojo;
    }

    /**
     * This method initializes jTextField1
     *
     * @return javax.swing.JTextField
     */
    public JTextFieldExt getTVerde() {
        if (tVerde == null) {
            tVerde = new JTextFieldExt(this.getBackground());
            tVerde.setPreferredSize(new java.awt.Dimension(80, 19));
        }

        return tVerde;
    }

    /**
     * This method initializes jTextField2
     *
     * @return javax.swing.JTextField
     */
    public JTextFieldExt getTAzul() {
        if (tAzul == null) {
            tAzul = new JTextFieldExt(this.getBackground());
            tAzul.setPreferredSize(new java.awt.Dimension(80, 19));
        }

        return tAzul;
    }

    /**
     * This method initializes jPanel4
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel4() {
        if (pGeneralTrans == null) {
        	pGeneralTrans = new JPanel();
        	pGeneralTrans.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,
                                                                         1));
        	pGeneralTrans.setPreferredSize(new java.awt.Dimension(415, 35));
        	pGeneralTrans.add(getOpacityCheck(), null);
        	pGeneralTrans.add(getOpacitySlider(), null);
        	pGeneralTrans.add(getOpacityText(), null);
        }

        return pGeneralTrans;
    }

    /**
     * This method initializes jCheckBox1
     *
     * @return javax.swing.JCheckBox
     */
    public JCheckBox getOpacityCheck() {
        if (cbOpacidad == null) {
            cbOpacidad = new JCheckBox();
            cbOpacidad.setText("Opacidad");
        }

        return cbOpacidad;
    }

    /**
     * This method initializes jSlider
     *
     * @return javax.swing.JSlider
     */
    public JSlider getOpacitySlider() {
        if (slOpacidad == null) {
            slOpacidad = new JSlider();
            slOpacidad.setPreferredSize(new java.awt.Dimension(280, 16));
        }

        return slOpacidad;
    }

    /**
     * This method initializes jTextField3
     *
     * @return javax.swing.JTextField
     */
    public JTextField getOpacityText() {
        if (tOpacidad == null) {
            tOpacidad = new JTextField();
            tOpacidad.setPreferredSize(new java.awt.Dimension(30, 19));
        }

        return tOpacidad;
    }

    /**
     * This method initializes jPanel5
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel5() {
        if (jPanel5 == null) {
            lRange = new JLabel();
            jPanel5 = new JPanel();
            lRange.setText("Usa un rango: 1,3,5:8");
            jPanel5.setPreferredSize(new java.awt.Dimension(161, 21));
            jPanel5.add(lRange, null);
        }

        return jPanel5;
    }

    /**
     * Activa/Desactiva los controles de opacidad
     * @param active
     */
    public void setActiveOpacityControl(boolean active) {
        this.getOpacityCheck().setSelected(active);
        this.getOpacitySlider().setEnabled(active);
        this.getOpacityText().setEnabled(active);
    }

    /**
     * Activa/Desactiva los controles de transparencia
     * @param active
     */
    public void setActiveTransparencyControl(boolean active) {
        this.getTransparencyCheck().setSelected(active);
        this.getTRojo().setEnabled(active);

        if ((active && (parent.nbands >= 2)) || !active) {
            this.getTVerde().setEnabled(active);
        } else {
            this.getTVerde().setEnabled(false);
        }

        if ((active && (parent.nbands >= 3)) || !active) {
            this.getTAzul().setEnabled(active);
        } else {
            this.getTAzul().setEnabled(false);
        }
    }

    /**
     * Asigna el valor de opacidad a los controles de la ventana
     * para que cuando esta se abra tenga los valores seleccionados en la
     * imagen.
     * @param alpha
     */
    public void setOpacity(int alpha) {
        int opacityPercent = (int) ((alpha * 100) / 255);
        this.getOpacityText().setText(String.valueOf(opacityPercent));
        this.getOpacitySlider().setValue(opacityPercent);
        this.setActiveOpacityControl(true);

        if (opacityPercent == 100) {
            this.setActiveOpacityControl(false);
        }
    }

    /**
     * This method initializes jPanel6
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel6() {
        if (jPanel6 == null) {
            lPixelValue = new JLabel();
            jPanel6 = new JPanel();
            jPanel6.setPreferredSize(new java.awt.Dimension(161, 20));
            lPixelValue.setText("Valor de pixel: 0 a 255");
            jPanel6.add(lPixelValue, null);
        }

        return jPanel6;
    }

    public class JTextFieldExt extends JTextField {
        final private static long serialVersionUID = -3370601314380922368L;
        private Color color = null;

        public JTextFieldExt(Color color) {
            this.color = color;
        }

        public JTextFieldExt() {
            super();
        }

        public void setEnabled(boolean enabled) {
            super.setEnabled(enabled);

            if (!enabled) {
                if (color == null) {
                    this.setBackground(new Color(210, 210, 210));
                } else {
                    this.setBackground(color);
                }
            } else {
                this.setBackground(new Color(255, 255, 255));
            }
        }
    }
} //  @jve:decl-index=0:visual-constraint="36,15"
