/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;


/**
 * Panel encargado de manejar los eventos del los controles de Salvar a Raste
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class SaveSetupPanel extends JPanel implements ActionListener,
                                                      MouseListener,
                                                      FocusListener,
                                                      KeyListener {
    final private static long serialVersionUID = -3370601314380922368L;
    private DataInputSaveRaster saveRaster = null;
    private SaveRasterDialogPanel dialogPanel = null;
    private String fName = null;

    /**
     * This method initializes
     *
     */
    public SaveSetupPanel() {
        super();
        initialize();
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    void initialize() {
        this.setPreferredSize(new Dimension(350, 210)); //AnchoxAlto  panel interior
        this.setBounds(0, 0, 300, 250);
        this.setLayout(null);
        this.add(getSaveParameters(), null);

        //Aadimos gestin de eventos        
        saveRaster.getJComboBox().addActionListener(this);
        saveRaster.getTescala().addActionListener(this);
        saveRaster.getTescala().addMouseListener(this);
        saveRaster.getTescala().addFocusListener(this);
        saveRaster.getTinf_derX().addActionListener(this);
        saveRaster.getTinf_derY().addActionListener(this);
        saveRaster.getTsup_izqX().addActionListener(this);
        saveRaster.getTsup_izqY().addActionListener(this);
        saveRaster.getTinf_derX().addMouseListener(this);
        saveRaster.getTinf_derY().addMouseListener(this);
        saveRaster.getTsup_izqX().addMouseListener(this);
        saveRaster.getTsup_izqY().addMouseListener(this);
        saveRaster.getTinf_derX().addFocusListener(this);
        saveRaster.getTinf_derY().addFocusListener(this);
        saveRaster.getTsup_izqX().addFocusListener(this);
        saveRaster.getTsup_izqY().addFocusListener(this);
        saveRaster.getBseleccion().addActionListener(this);
        saveRaster.getBPropiedades().addActionListener(this);
        saveRaster.getTescala().addKeyListener(this);
        saveRaster.getJComboBox().addKeyListener(this);
    }

    /**
     * Asigna un valor al panel padre
     * @param dialogPanel
     */
    public void setDialogPanel(SaveRasterDialogPanel dialogPanel) {
        this.dialogPanel = dialogPanel;
    }

    /**
     * Obtiene el Panel de Controles interior
     * @return
     */
    public DataInputSaveRaster getSaveParameters() {
        if (saveRaster == null) {
            saveRaster = new DataInputSaveRaster();

            //saveRaster.setBounds(9, 9, 328, 75);
        }

        return saveRaster;
    }

    /**
     * Calcula el tamao en pixeles de la imagen resultante. 
     */
    private void calcSizeImage(){
    	if(!saveRaster.getProjection().isProjected())
    		calcSizeImageGeodesicas();
    	else
    		calcSizeImageUTM();
    }
    
    /**
     * Calcula el tamao en pixeles cuando la imagen est en coordenadas 
     * geogrficas. 
     *
     */
    public void calcSizeImageGeodesicas(){
    	double lrX = Double.parseDouble(saveRaster.getTinf_derX().getText());
    	double ulX = Double.parseDouble(saveRaster.getTsup_izqX().getText());
    	
    	//Nos aseguramos de que la escala sea un entero. Si no lo es ponemos un 0
        try {
            Integer.parseInt(saveRaster.getTescala().getText());
        } catch (NumberFormatException e) {
            saveRaster.getTescala().setText("0");
        }
        
    	double scale = saveRaster.getProjection().getScale(ulX, lrX, 
    			saveRaster.getWidthInPixelsGeodesicas(), 
    			Integer.parseInt((String)saveRaster.getJComboBox().getSelectedItem()));
    	double scaleIntro = Double.parseDouble(saveRaster.getTescala().getText());
    	int widthPixels = (int)((scaleIntro * saveRaster.getWidth()) / scale);
    	int heightPixels = (int)((scaleIntro * saveRaster.getHeight()) / scale);
    	
    	saveRaster.getTancho().setText(String.valueOf(widthPixels));
        saveRaster.getTalto().setText(String.valueOf(heightPixels));
    }
    
    /**
     * Calculo del tamao en pixels de la imagen a partir de las coordenadas del
     * mundo real, la escala y los puntos por pulgada
     */
    private void calcSizeImageUTM() {
        double lrX = Double.parseDouble(saveRaster.getTinf_derX().getText());
        double lrY = Double.parseDouble(saveRaster.getTinf_derY().getText());
        double ulX = Double.parseDouble(saveRaster.getTsup_izqX().getText());
        double ulY = Double.parseDouble(saveRaster.getTsup_izqY().getText());
        double widthMts = 0;

        if (ulX > lrX) {
            widthMts = ulX - lrX;
        } else {
        	widthMts = lrX - ulX;
        }

        double heightMts = 0;

        if (ulY > lrY) {
        	heightMts = ulY - lrY;
        } else {
        	heightMts = lrY - ulY;
        }

        //Nos aseguramos de que la escala sea un entero. Si no lo es ponemos un 0
        try {
            Integer.parseInt(saveRaster.getTescala().getText());
        } catch (NumberFormatException e) {
            saveRaster.getTescala().setText("0");
        }

        if ((widthMts <= 0) || (heightMts <= 0) ||
                (Integer.parseInt(saveRaster.getTescala().getText()) == 0)) {
            saveRaster.getTancho().setText("0");
            saveRaster.getTalto().setText("0");

            return;
        }

        //Calculo del tamao de la imagen definitiva en pulgadas
        double anchopulgadas = (widthMts / Double.parseDouble(saveRaster.getTescala().getText())) * 39.73007874D;
        double altopulgadas = (heightMts / Double.parseDouble(saveRaster.getTescala().getText())) * 39.73007874D;

        //Ancho en pixeles = ppp * sizepulgadas
        int anchopixels = (int) (Integer.parseInt(saveRaster.getJComboBox().getSelectedItem().toString()) * 
        						anchopulgadas);
        int altopixels = (int) (Integer.parseInt(saveRaster.getJComboBox().getSelectedItem().toString()) * 
        						altopulgadas);

        //int anchopixels =(int) (Toolkit.getDefaultToolkit().getScreenResolution() * anchopulgadas);
        //int altopixels = (int) (Toolkit.getDefaultToolkit().getScreenResolution() * altopulgadas);
        saveRaster.getTancho().setText(String.valueOf(anchopixels));
        saveRaster.getTalto().setText(String.valueOf(altopixels));
    }
    
    /**
     * Controla cuando se cumplen todos los requisitos en el formulario para
     * poder activar el botn de aceptar.
     */
    private void enableButtons() {
        if ((saveRaster.getTescala() == null) ||
                saveRaster.getTescala().getText().equals("") ||
                saveRaster.getTescala().getText().equals("0") ||
                (this.fName == null) || this.fName.equals("")) {
            if (dialogPanel != null) {
                dialogPanel.getAcceptButton().setEnabled(false);
            }

            return;
        }

        if (saveRaster.getTancho().getText().equals("0") ||
                saveRaster.getTalto().getText().equals("0")) {
            if (dialogPanel != null) {
                dialogPanel.getAcceptButton().setEnabled(false);
            }

            return;
        }

        if (dialogPanel != null) {
            dialogPanel.getAcceptButton().setEnabled(true);
        }
    }

    /**
     * Gestiona los eventos del cambio de escalla de la imagen y lla apertura del
     * dialogo de seleccin del nombre del fichero.
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(saveRaster.getJComboBox()) ||
                e.getSource().equals(saveRaster.getTescala()) ||
                e.getSource().equals(saveRaster.getTinf_derX()) ||
                e.getSource().equals(saveRaster.getTinf_derY()) ||
                e.getSource().equals(saveRaster.getTsup_izqX()) ||
                e.getSource().equals(saveRaster.getTsup_izqY())) {
        	this.calcSizeImage();
            this.enableButtons();
        }

        //Selector de Fichero sobre el cual se va a salvar a raster
        if (e.getSource().equals(saveRaster.getBseleccion())) {
            JFileChooser chooser = new JFileChooser("./");
            chooser.setDialogTitle("Seleccionar el fichero");

            String os = System.getProperties().getProperty("os.version");

            if (os.startsWith("2.4")) {
                chooser.addChoosableFileFilter(new WriterFilter(chooser, "ecw"));
            }

            chooser.addChoosableFileFilter(new WriterFilter(chooser, "jp2"));
            chooser.addChoosableFileFilter(new WriterFilter(chooser, "tif"));
            chooser.addChoosableFileFilter(new WriterFilter(chooser, "jpg"));
            
            int returnVal = chooser.showOpenDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                this.fName = chooser.getSelectedFile().toString();

                FileFilter filter = chooser.getFileFilter();

                if ((fName != null) && !fName.equals("")) {
                    if (this.fName.endsWith(".ecw") && os.startsWith("2.4")) {
                        saveRaster.getBPropiedades().setText("Propiedades Ecw");
                    }if (this.fName.endsWith(".jp2")) {
                        saveRaster.getBPropiedades().setText("Propiedades Jp2");
                    }else if (this.fName.endsWith(".tif")) {
                        saveRaster.getBPropiedades().setText("Propiedades GeoTiff");
                    }else if (this.fName.endsWith(".jpg") || this.fName.endsWith(".jpeg")) {
                        saveRaster.getBPropiedades().setText("Propiedades Jpeg");
                    } else if (filter.getDescription().equals(".ecw") && os.startsWith("2.4")) {
                        saveRaster.getBPropiedades().setText("Propiedades Ecw");
                    } else if (filter.getDescription().equals(".tif")) {
                        saveRaster.getBPropiedades().setText("Propiedades GeoTiff");
                    }else if (filter.getDescription().equals(".jpg")) {
                        saveRaster.getBPropiedades().setText("Propiedades Jpeg");
                    }else if (filter.getDescription().equals(".jp2")) {
                        saveRaster.getBPropiedades().setText("Propiedades Jp2");
                    }
                }

                if (!fName.endsWith(".ecw") && 
                	!fName.endsWith(".tif") &&
					!fName.endsWith(".jpg") && 
                	!fName.endsWith(".jpeg") && 
                	!fName.endsWith(".jp2") ) {
                    fName = new String(chooser.getSelectedFile().toString() +
                                       filter.getDescription());
                }

                enableButtons();
            }
        }
    }

    /**
     * Devuelve el nombre del fichero seleccionado
     * @return Nombre del fichero seleccionado
     */
    public String getFileName() {
        return fName;
    }

    /**
     * Unused
     */
    public void mouseExited(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mouseEntered(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    	this.calcSizeImage();
        enableButtons();
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mousePressed(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void focusGained(FocusEvent e) {
    }

    /**
     * Recalcula el tamao de la imagen con los datos existentes y
     * activa o desactiva los botones.
     */
    public void focusLost(FocusEvent e) {
   		this.calcSizeImage();
        enableButtons();
    }

    /**
     * Activa o Desactiva los botones a travs de la funcin que valida los campos
     */
    public void keyTyped(KeyEvent e) {
        enableButtons();
    }

    /**
     * Activa o Desactiva los botones a travs de la funcin que valida los campos
     */
    public void keyPressed(KeyEvent e) {
        enableButtons();
    }

    /**
     * Activa o Desactiva los botones a travs de la funcin que valida los campos
     */
    public void keyReleased(KeyEvent e) {
        enableButtons();
    }
}

/**
 * @author Nacho Brodin <brodin_ign@gva.es>
 * Filtro Gdal para el selector de formatos de escritura
 */
class WriterFilter extends javax.swing.filechooser.FileFilter {
    private JFileChooser chooser = null;
    private String filter;

    public WriterFilter(JFileChooser ch, String fil) {
        this.chooser = ch;
        this.filter = fil;
    }

    public boolean accept(File f) {
        return f.isDirectory() || f.getName().toLowerCase().endsWith("."+filter);
    }

    public String getDescription() {
        return "."+filter;
    }
}

