/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap;

import com.iver.cit.gvsig.fmap.layers.CancelationException;
import com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent;
import com.iver.cit.gvsig.fmap.layers.LayerCollectionListener;
import com.iver.cit.gvsig.fmap.layers.LayerEvent;
import com.iver.cit.gvsig.fmap.layers.LayerListener;
import com.iver.cit.gvsig.fmap.layers.LayerPositionEvent;
import com.iver.cit.gvsig.fmap.layers.LegendListener;
import com.iver.cit.gvsig.fmap.layers.SelectionEvent;
import com.iver.cit.gvsig.fmap.layers.SelectionListener;
import com.iver.cit.gvsig.fmap.rendering.LegendChangedEvent;

import java.util.ArrayList;
import java.util.Iterator;


/**
 * Evento sobre el buffer.
 *
 * @author Fernando Gonzlez Corts
 */
public class EventBuffer implements LegendListener, LayerCollectionListener,
	SelectionListener, ViewPortListener, LayerListener {
	private ArrayList events = new ArrayList();
	private ArrayList listeners = new ArrayList();
	private boolean dispatching = true;

	/**
	 * Pone el buffer en modo acumulacin de eventos. Los eventos que se
	 * reciban a partir de una llamada a este mtodo sern acumulados y no
	 * sern notificados a los listeners hasta que no se reciba una llamada a
	 * endAtomicEvent
	 */
	public void beginAtomicEvent() {
		dispatching = false;
	}

	/**
	 * Termina la acumulacin de eventos y notifica a los  lsteners.
	 */
	public void endAtomicEvent() {
		fireAtomicEventListener();
		events.clear();
		dispatching = true;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LegendListener#legendChanged(com.iver.cit.gvsig.fmap.rendering.LegendChangedEvent)
	 */
	public void legendChanged(LegendChangedEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerAdded(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
	 */
	public void layerAdded(LayerCollectionEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerMoved(com.iver.cit.gvsig.fmap.layers.LayerPositionEvent)
	 */
	public void layerMoved(LayerPositionEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerRemoved(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
	 */
	public void layerRemoved(LayerCollectionEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerAdding(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
	 */
	public void layerAdding(LayerCollectionEvent e) throws CancelationException {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerMoving(com.iver.cit.gvsig.fmap.layers.LayerPositionEvent)
	 */
	public void layerMoving(LayerPositionEvent e) throws CancelationException {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerRemoving(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
	 */
	public void layerRemoving(LayerCollectionEvent e)
		throws CancelationException {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#activationChanged(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
	 */
	public void activationChanged(LayerCollectionEvent e)
		throws CancelationException {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#visibilityChanged(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
	 */
	public void visibilityChanged(LayerCollectionEvent e)
		throws CancelationException {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.SelectionListener#selectionChanged(com.iver.cit.gvsig.fmap.layers.SelectionEvent)
	 */
	public void selectionChanged(SelectionEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.ExtentListener#extentChanged(com.iver.cit.gvsig.fmap.ExtentEvent)
	 */
	public void extentChanged(ExtentEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * Aade un AtomicEventListener a la lista de listener.
	 *
	 * @param listener AtomicEventListener.
	 *
	 * @return True si ha sido aadido correctamente.
	 */
	public boolean addAtomicEventListener(AtomicEventListener listener) {
		return listeners.add(listener);
	}

	/**
	 * Borra el AtomicEventListener de la lista de listeners.
	 *
	 * @param listener AtomicEventListener.
	 *
	 * @return True si ha sido borrado correctamente.
	 */
	public boolean removeAtomicEventListener(AtomicEventListener listener) {
		return listeners.remove(listener);
	}

	/**
	 * Ejecuta el mtodo atomicEvent de todos los listeners.
	 */
	private void fireAtomicEventListener() {
		for (Iterator i = listeners.iterator(); i.hasNext();) {
			AtomicEventListener listener = (AtomicEventListener) i.next();
			AtomicEvent e = new AtomicEvent(events);
			listener.atomicEvent(e);
		}

		events.clear();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerListener#visibilityChanged(com.iver.cit.gvsig.fmap.layers.LayerEvent)
	 */
	public void visibilityChanged(LayerEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerListener#activationChanged(com.iver.cit.gvsig.fmap.layers.LayerEvent)
	 */
	public void activationChanged(LayerEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerListener#nameChanged(com.iver.cit.gvsig.fmap.layers.LayerEvent)
	 */
	public void nameChanged(LayerEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.ViewPortListener#backColorChanged(com.iver.cit.gvsig.fmap.ColorEvent)
	 */
	public void backColorChanged(ColorEvent e) {
		events.add(e);

		if (dispatching) {
			fireAtomicEventListener();
		}
	}
}
