/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.core;

import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.v02.FLabel;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.rendering.styling.FStyle2D;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.geom.GeometryFactory;

import org.cresques.cts.ICoordTrans;
import org.geotools.data.postgis.attributeio.WKBEncoder;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;


/**
 * Coleccin de Geometras.
 *
 * @author Vicente Caballero Navarro
 */
public class FGeometryCollection implements IGeometry {
	private IGeometry[] geometries;

	/**
	 * Crea un nuevo FGeometryCollection.
	 *
	 * @param geoms vector de geometras.
	 */
	public FGeometryCollection(IGeometry[] geoms) {
		geometries = geoms;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#draw(java.awt.Graphics2D,
	 * 		com.iver.cit.gvsig.fmap.ViewPort, FStyle2D)
	 */
	public void draw(Graphics2D g, ViewPort vp, FStyle2D symbol) {
		for (int i = 0; i < geometries.length; i++)
			geometries[i].draw(g, vp, symbol);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#createLabels(int, boolean)
	 */
	public FLabel[] createLabels(int position, boolean duplicates) {
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#intersects(java.awt.geom.Rectangle2D)
	 */
	public boolean intersects(Rectangle2D r) {
	    boolean resul = false;
		for (int i = 0; i < geometries.length; i++)
		{
			resul = geometries[i].intersects(r);
			if (resul) break;
		}
		
		return resul;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#getBounds2D()
	 */
	public Rectangle2D getBounds2D() {
		Rectangle2D rAux = null;

		for (int i = 0; i < geometries.length; i++)
			rAux.createUnion(geometries[i].getBounds2D());

		return rAux;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#getGeometryType()
	 */
	public int getGeometryType() {
		int ret = 0;

		for (int i = 0; i < geometries.length; i++) {
			ret = ret & geometries[i].getGeometryType();
		}

		return ret;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#draw(java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.core.v02.FSymbol)
	 */
	public void draw(Graphics2D g, ViewPort vp, FSymbol symbol) {
		for (int i = 0; i < geometries.length; i++)
			geometries[i].draw(g, vp, symbol);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#cloneGeometry()
	 */
	public IGeometry cloneGeometry() {
		IGeometry[] newGeometries = new IGeometry[geometries.length];

		for (int i = 0; i < geometries.length; i++)
			newGeometries[i] = geometries[i].cloneGeometry();

		return new FGeometryCollection(newGeometries);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#toJTSGeometry()
	 */
	public Geometry toJTSGeometry() {
        Geometry[] theGeoms = new Geometry[geometries.length];
        for (int i = 0; i < geometries.length; i++)
        {
            theGeoms[i] = geometries[i].toJTSGeometry();
        }
        GeometryCollection geomCol = new GeometryFactory().createGeometryCollection(theGeoms);
		return geomCol;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#reProject(org.cresques.cts.ICoordTrans)
	 */
	public void reProject(ICoordTrans ct) {
		for (int i = 0; i < geometries.length; i++)
			geometries[i].reProject(ct);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#getGeneralPathXIterator()
	 */
	public GeneralPathXIterator getGeneralPathXIterator() {
		//TODO no esta implementado.
		return null;
	}

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.core.IGeometry#fastIntersects(double, double, double, double)
     */
    public boolean fastIntersects(double x, double y, double w, double h) {
	    boolean resul = false;
		for (int i = 0; i < geometries.length; i++)
		{
			resul = geometries[i].fastIntersects(x,y,w,h);
			if (resul) break;
		}		
		return resul;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.core.IGeometry#toWKB()
     */
    public byte[] toWKB() throws IOException {
        return WKBEncoder.encodeGeometry(toJTSGeometry());
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.core.IGeometry#drawInts(java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.core.v02.FSymbol)
     */
    public void drawInts(Graphics2D g, ViewPort vp, FSymbol symbol) {
        for (int i = 0; i < geometries.length; i++)
            geometries[i].drawInts(g, vp, symbol);        
        
    }
}
