/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.core;

import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.v02.FGraphicUtilities;
import com.iver.cit.gvsig.fmap.core.v02.FLabel;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.rendering.styling.FStyle2D;

import com.vividsolutions.jts.geom.Geometry;

import org.cresques.cts.ICoordTrans;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;


/**
 * Multipunto 2D.
 *
 * @author Vicente Caballero Navarro
 */
public class FMultiPoint2D implements IGeometry {
	double[] x = null;
	double[] y = null;

	/**
	 * Crea un nuevo MultiPoint2D.
	 *
	 * @param x DOCUMENT ME!
	 * @param y DOCUMENT ME!
	 */
	public FMultiPoint2D(double[] x, double[] y) {
		this.x = x;
		this.y = y;
	}
	public FMultiPoint2D(FPoint2D[] points) {
		double[] auxX=new double[points.length];
		double[] auxY=new double[points.length];
		for (int i=0;i<points.length;i++){
			auxX[i]=points[i].getX();
			auxY[i]=points[i].getY();
		}
	}
	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#draw(java.awt.Graphics2D,
	 * 		ViewPort, FStyle2D)
	 */
	public void draw(Graphics2D g, ViewPort vp, FStyle2D symbol) {
		int size = 2;
		int hw = 4;

		for (int i = 0; i < x.length; i++) {
			java.awt.geom.Point2D.Double p = new java.awt.geom.Point2D.Double(x[i],
					y[i]);
			vp.getAffineTransform().transform(p, p);
			g.setColor(Color.red);
			g.fillOval((int) p.x - size, (int) p.y - size, (int) hw, (int) hw);
			g.setColor(Color.black);
			g.drawOval((int) p.x - size, (int) p.y - size, (int) hw, (int) hw);
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#toJTSGeometry()
	 */
	public Geometry toJTSGeometry() {
        // TODO.
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#createLabels(int, boolean)
	 */
	public FLabel[] createLabels(int position, boolean duplicates) {
        FLabel[] aux = new FLabel[getNumPoints()];
        for (int i=0; i < getNumPoints(); i++)
        {
            aux[i] = FLabel.createFLabel(new FPoint2D(x[i],y[i]));
        }
        
		return aux;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#intersects(java.awt.geom.Rectangle2D)
	 */
	public boolean intersects(Rectangle2D r) {
		for (int i=0;i<getNumPoints();i++){
			if (r.contains(x[i],y[i]))return true;
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#getBounds2D()
	 */
	public Rectangle2D getBounds2D() {
		Rectangle2D r=null;
		if (getNumPoints()>0){
			r=new Rectangle2D.Double(x[0],y[0],0.001,0.001);
		}
		for (int i=1;i<getNumPoints();i++){
			r.add(x[i],y[i]);
		}
		return r;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#getGeometryType()
	 */
	public int getGeometryType() {
		return FShape.MULTIPOINT;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#draw(java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.core.v02.FSymbol)
	 */
	public void draw(Graphics2D g, ViewPort vp, FSymbol symbol) {
		int size = 2;
		int hw = 4;

		for (int i = 0; i < x.length; i++) {
			java.awt.geom.Point2D.Double p = new java.awt.geom.Point2D.Double(x[i],
					y[i]);
			vp.getAffineTransform().transform(p, p);
			FGraphicUtilities.DrawShape(g, vp.getAffineTransform(), new FPoint2D(p.getX(),p.getY()), symbol);
			
		/*	java.awt.geom.Point2D.Double p = new java.awt.geom.Point2D.Double(x[i],
					y[i]);
			vp.getAffineTransform().transform(p, p);
			g.setColor(Color.red);
			g.fillOval((int) p.x - size, (int) p.y - size, (int) hw, (int) hw);
			g.setColor(Color.black);
			g.drawOval((int) p.x - size, (int) p.y - size, (int) hw, (int) hw);
		*/
		}
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#cloneGeometry()
	 */
	public IGeometry cloneGeometry() {
		return new FMultiPoint2D((double[])x.clone(),(double[])y.clone());
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#reProject(org.cresques.cts.ICoordTrans)
	 */
	public void reProject(ICoordTrans ct) {
		// TODO Auto-generated method stub
	}
	public int getNumPoints(){
		return x.length;
	}
	public FPoint2D getPoint(int i){
		return new FPoint2D(x[i],y[i]);
	}
	/**
	 * @see com.iver.cit.gvsig.fmap.core.IGeometry#getGeneralPathXIterator()
	 */
	public GeneralPathXIterator getGeneralPathXIterator() {
		GeneralPathX gpx=new GeneralPathX();
		if (getNumPoints()>0){
			gpx.moveTo(x[0],y[0]);
		}
		for (int i=1;i<getNumPoints();i++){
			gpx.lineTo(x[i],y[i]);
		}
		return (GeneralPathXIterator)gpx.getPathIterator(null);
	}
    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.core.IGeometry#fastIntersects(double, double, double, double)
     */
    public boolean fastIntersects(double x, double y, double w, double h) {
        // TODO Auto-generated method stub
        return false;
    }
    /**
     * @see com.iver.cit.gvsig.fmap.core.IGeometry#toWKB()
     */
    public byte[] toWKB() throws IOException {
        // TODO Auto-generated method stub
        return null;
    }
    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.core.IGeometry#drawInts(java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.core.v02.FSymbol)
     */
    public void drawInts(Graphics2D g, ViewPort vp, FSymbol symbol) {
        draw(g,vp, symbol);
        
    }
}
