/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.core;

import org.cresques.cts.ICoordTrans;

import com.iver.cit.gvsig.fmap.core.v02.FConverter;

import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;


/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class FPolyline2D implements FShape {
	protected GeneralPathX gp;
	/**
	 * Crea un nuevo FPolyline2D.
	 *
	 * @param gpx GeneralPathX.
	 */
	public FPolyline2D(GeneralPathX gpx) {
		gp = gpx;
	}

	/* (non-Javadoc)
	 * @see java.awt.Shape#contains(double, double)
	 */
	public boolean contains(double x, double y) {
		return gp.contains(x, y);
	}

	/* (non-Javadoc)
	 * @see java.awt.Shape#contains(double, double, double, double)
	 */
	public boolean contains(double x, double y, double w, double h) {
		return gp.contains(x, y, w, h);
	}

	/* (non-Javadoc)
	 * @see java.awt.Shape#intersects(double, double, double, double)
	 */
	public boolean intersects(double x, double y, double w, double h) {
	    // Ms rpido
		return gp.intersects(x, y, w, h);
	}

	/* (non-Javadoc)
	 * @see java.awt.Shape#getBounds()
	 */
	public Rectangle getBounds() {
		return gp.getBounds();
	}

	/* (non-Javadoc)
	 * @see java.awt.Shape#contains(java.awt.geom.Point2D)
	 */
	public boolean contains(Point2D p) {
		return gp.contains(p);
	}

	/* (non-Javadoc)
	 * @see java.awt.Shape#getBounds2D()
	 */
	public Rectangle2D getBounds2D() {
		return gp.getBounds2D();
	}

	/* (non-Javadoc)
	 * @see java.awt.Shape#contains(java.awt.geom.Rectangle2D)
	 */
	public boolean contains(Rectangle2D r) {
		return gp.contains(r);
	}

	/**
	 * El mtodo intersects de java.awt.Shape que define la interseccin entre
	 * una polilnea y un Rectangle2D considera la polilnea como un Shape
	 * genrico y se producen errores en la seleccin de polilneas. Por este
	 * motivo se ha modificado este mtodo intersect() de FPolyline2D para que
	 * realize la interseccin estricta entre el Rectangle2D y la polilnea en
	 * cuestin. El precio es un incremento de tiempo mximo del 50%.
	 *
	 * @param r Rectngulo.
	 *
	 * @return True si intersecta con el rectangulo que se pasa como parmetro.
	 */
	public boolean intersects(Rectangle2D r) {
		//return gp.intersects(r);
	    // Ms exacto
		boolean bool = false;
		   if (gp.intersects(r)) {
		           ArrayList arrayCoords;
		           int theType;
		           //Use this array to store segment coordinate data
		           double[] theData = new double[6];
		           PathIterator theIterator;
		
		       Point2D p1 = new Point2D.Double(r.getMinX(),r.getMinY());
		       Point2D p2 = new Point2D.Double(r.getMinX(),r.getMaxY());
		       Point2D p3 = new Point2D.Double(r.getMaxX(),r.getMaxY());
		       Point2D p4 = new Point2D.Double(r.getMaxX(),r.getMinY());
		       Line2D l1 = new Line2D.Double(p1,p2);
		       Line2D l2 = new Line2D.Double(p2,p3);
		       Line2D l3 = new Line2D.Double(p3,p4);
		       Line2D l4 = new Line2D.Double(p4,p1);
		
		           theIterator = this.getPathIterator(null);
		           arrayCoords = new ArrayList();
		            while(!theIterator.isDone()) {
		                    theType = theIterator.currentSegment(theData);
		           if (theType==PathIterator.SEG_MOVETO) {
		                    arrayCoords.add(new Point2D.Double(theData[0], theData[1]));
		           } else if (theType==PathIterator.SEG_LINETO) {
		                   arrayCoords.add(new Point2D.Double(theData[0], theData[1]));
		                   Point2D pAnt = (Point2D)arrayCoords.get(arrayCoords.size()-2);
		                   Line2D l = new Line2D.Double(pAnt.getX(),pAnt.getY(),theData[0],theData[1]);
		                   if (l.intersectsLine(l1.getX1(),l1.getY1(),l1.getX2(),l1.getY2())
		                                   || l.intersectsLine(l2.getX1(),l2.getY1(),l2.getX2(),l2.getY2())
		                                   || l.intersectsLine(l3.getX1(),l3.getY1(),l3.getX2(),l3.getY2())
		                                   || l.intersectsLine(l4.getX1(),l4.getY1(),l4.getX2(),l4.getY2())
		                                   || r.intersectsLine(l)) {
		                           bool = true;
		                   }
		           } else {
		                    System.out.println("Not supported here");
		           }
		                theIterator.next();
		            }
		   }
		   return bool;
	}

	/* (non-Javadoc)
	 * @see java.awt.Shape#getPathIterator(java.awt.geom.AffineTransform)
	 */
	public PathIterator getPathIterator(AffineTransform at) {
		return gp.getPathIterator(at);
	}

	/* (non-Javadoc)
	 * @see java.awt.Shape#getPathIterator(java.awt.geom.AffineTransform, double)
	 */
	public PathIterator getPathIterator(AffineTransform at, double flatness) {
		return gp.getPathIterator(at, flatness);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param at DOCUMENT ME!
	 */
	public void transform(AffineTransform at) {
        
        // TODO: PRUEBA. BORRAR ESTA LINEA
        // gp = FConverter.transformToInts(gp, at);
        
		gp.transform(at);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.core.FShape#getShapeType()
	 */
	public int getShapeType() {
		return FShape.LINE;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.FShape#cloneFShape()
	 */
	public FShape cloneFShape() {
		return new FPolyline2D((GeneralPathX) gp.clone());
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.core.FShape#reProject(org.cresques.cts.ICoordTrans)
	 */
	public void reProject(ICoordTrans ct) {
		gp.reProject(ct);
	}
		
}
