/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.core;

import com.iver.cit.gvsig.fmap.core.gt2.FLiteShape;


/**
 * Clase que crea las geometras, contendra un mtodo create por cada tipo de
 * geometria que soporte gvSIG
 */
public class ShapeFactory {
	/**
	 * Crea una geometra que contiene como shape un punto 2D.
	 *
	 * @param x Coordenada x.
	 * @param y Coordenada y.
	 *
	 * @return Geometra.
	 */
	public static IGeometry createPoint2D(double x, double y) {
		return new FGeometry(new FPoint2D(x, y));
	}
	public static IGeometry createPoint2D(FPoint2D p) {
		return new FGeometry(p);
	}

	
	/**
	 * Crea una geometra que contiene como shape un Multipunto 2D.
	 *
	 * @param x Coordenada x.
	 * @param y Coordenada y.
	 *
	 * @return Geometra.
	 */
	public static IGeometry createMultipoint2D(double[] x, double[] y) {
		return new FMultiPoint2D(x, y);
	}

	/**
	 * Crea una geometra que contiene como shape un punto 3D.
	 *
	 * @param x Coordenada x.
	 * @param y Coordenada y.
	 * @param z Coordenada z.
	 *
	 * @return Geometra.
	 */
	public static IGeometry createPoint3D(double x, double y, double z) {
		return new FGeometry(new FPoint3D(x, y, z));
	}

	/**
	 * Crea una geometra que contiene como shape un Multipunto 3D.
	 *
	 * @param x Coordenada x.
	 * @param y Coordenada y.
	 * @param z Coordenada z.
	 *
	 * @return Geometra.
	 */
	public static IGeometry createMultipoint3D(double[] x, double[] y,
		double[] z) {
		return new FMultipoint3D(x, y, z);
	}

	/**
	 * Crea una geometra que contiene como shape un Polilnea 2D.
	 *
	 * @param shape GeneralPathX.
	 *
	 * @return Geometra.
	 */
	public static IGeometry createPolyline2D(GeneralPathX shape) {
		return new FGeometry(new FPolyline2D(shape));
	}

	/**
	 * Crea una geometra que contiene como shape un Polilnea 3D.
	 *
	 * @param shape GeneralPathX.
	 * @param pZ Vector de Z.
	 *
	 * @return Geometra.
	 */
	public static IGeometry createPolyline3D(GeneralPathX shape, double[] pZ) {
		return new FGeometry(new FPolyline3D(shape, pZ));
	}
	/**
	 * Crea una geometra que contiene como shape un Polgono 3D.
	 *
	 * @param shape GeneralPathX.
	 * @param pZ Vector de Z.
	 *
	 * @return Geometra.
	 */
	public static IGeometry createPolygon3D(GeneralPathX shape, double[] pZ) {
		return new FGeometry(new FPolygon3D(shape, pZ));
	}

	/**
	 * Crea una geometra que contiene como shape un Polgono 2D.
	 *
	 * @param shape GeneralPathX.
	 *
	 * @return Geometra.
	 */
	public static IGeometry createPolygon2D(GeneralPathX shape) {
		return new FGeometry(new FPolygon2D(shape));
	}

	/**
	 * Crea a partir de un FShape una geometra.
	 *
	 * @param shp FShape.
	 *
	 * @return Geometra.
	 */
	public static FGeometry createGeometry(FShape shp) {
		return new FGeometry(shp);
	}
    
    public static IGeometry createGeometry(FLiteShape gt2geometry) {
        return new Gt2Geometry(gt2geometry);
    }

    
}
