/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers.dgn;

import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.driver.FileDriver;
import com.hardcode.gdbms.engine.data.file.FileDataWare;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;


/**
 * Driver DGN.
 *
 * @author Vicente Caballero Navarro
 */
public class DGNDriver implements VectorialFileDriver, FileDriver {
	DGNReader m_DgnReader;
	private final int ID_FIELD_ID = 0;
	private final int ID_FIELD_ENTITY = 1;
	private final int ID_FIELD_LAYER = 2;
	private final int ID_FIELD_COLOR = 3;
	private final int ID_FIELD_HEIGHTTEXT = 4;
	private final int ID_FIELD_ROTATIONTEXT = 5;
	private final int ID_FIELD_TEXT = 6;
	private String path;
	private File m_Fich;

	// Campos del registro en cache:
	Object[] auxRow = new Object[7];
	Object[] cellRow = new Object[7];
	Object[] complexRow = new Object[7];

	/**
	 * Traduce las posiciones de comienzo de las entidades del DGN a nuestras
	 * entidades FShape. (Una FShape puede corresponder a varias entidades de
	 * un DGN).
	 */
	private int[] m_posShapes;

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#open(java.io.File)
	 */
	public void open(File f) throws IOException {
		m_Fich = f;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#close()
	 */
	public void close() throws IOException {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getShape(int)
	 */
	public IGeometry getShape(int index) throws IOException {
		boolean bElementoCompuesto = false;
		boolean bEsPoligono = false;
		boolean bInsideCell = false;
		boolean bFirstHoleEntity = false;
		boolean bConnect = false; // Se usa para que los polgonos cierren bien cuando son formas compuestas

		// int contadorSubElementos =0, numSubElementos = 0;
		int complex_index_fill_color = -1;
		int nClass; // Para filtrar los elementos de construccin, etc.

		ArrayList arrayFields = new ArrayList();
		arrayFields.add("ID");
		arrayFields.add("Entity");
		arrayFields.add("Layer");
		arrayFields.add("Color");
		arrayFields.add("HeightText");
		arrayFields.add("RotationText");
		arrayFields.add("Text");

		IGeometry aux;
		GeneralPathX elementoCompuesto = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD);

		m_DgnReader.DGNGotoElement(index);

		DGNElemCore elemento = m_DgnReader.DGNReadElement();
		nClass = 0;

		if (elemento.properties != 0) {
			nClass = elemento.properties & DGNFileHeader.DGNPF_CLASS;
		}

		if ((elemento != null) && (elemento.deleted == 0) && (nClass == 0)) //Leer un elemento
		 {
			aux = null;

			// if ((elemento.element_id > 3800) && (elemento.element_id < 3850))
			// 	m_DgnReader.DGNDumpElement(m_DgnReader.getInfo(),elemento,""); 
			if ((elemento.stype == DGNFileHeader.DGNST_MULTIPOINT) ||
					(elemento.stype == DGNFileHeader.DGNST_ARC) ||
					(elemento.stype == DGNFileHeader.DGNST_CELL_HEADER) ||
					(elemento.stype == DGNFileHeader.DGNST_SHARED_CELL_DEFN) ||
					(elemento.stype == DGNFileHeader.DGNST_COMPLEX_HEADER)) {
				if (elemento.complex != 0) {
					bElementoCompuesto = true;
				} else {
					if (bElementoCompuesto) {
						if (bInsideCell) {
							auxRow[ID_FIELD_ENTITY] = cellRow[ID_FIELD_ENTITY];
						} else {
							auxRow = complexRow;
						}

						// System.err.println("Entidad compuesta. bInsideCell = " + bInsideCell + " auxRow = " + auxRow[ID_FIELD_ENTITY]);
						// TODO: CREACION DE LINEA lyrLines.addShape(new FShape(FConstant.SHAPE_TYPE_POLYLINE, elementoCompuesto), auxRow);
						if (bEsPoligono) {
							if (complex_index_fill_color != -1) {
								auxRow[ID_FIELD_COLOR] = new Integer(complex_index_fill_color);
							}

							// TODO: CREAR POLIGONO lyrPolygons.addShape(new FShape(FConstant.SHAPE_TYPE_POLYGON, elementoCompuesto), auxRow);
						}

						elementoCompuesto = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD);
					}

					// System.err.println("Entidad simple");
					bElementoCompuesto = false;
					bEsPoligono = false;
					bConnect = false;

					// elementoCompuesto = new GeneralPathX();
					bInsideCell = false;
				}
			}

			switch (elemento.stype) {
				case DGNFileHeader.DGNST_SHARED_CELL_DEFN:
					bInsideCell = true;
					cellRow[ID_FIELD_ID] = new Integer(elemento.element_id);
					cellRow[ID_FIELD_LAYER] = new Integer(elemento.level);
					cellRow[ID_FIELD_COLOR] = new Integer(elemento.color);
					cellRow[ID_FIELD_ENTITY] = new String("Shared Cell");

					break;

				case DGNFileHeader.DGNST_CELL_HEADER:
					bInsideCell = true;

					DGNElemCellHeader psCellHeader = (DGNElemCellHeader) elemento;
					cellRow[ID_FIELD_ID] = new Integer(elemento.element_id);
					cellRow[ID_FIELD_LAYER] = new Integer(elemento.level);
					cellRow[ID_FIELD_COLOR] = new Integer(elemento.color);
					cellRow[ID_FIELD_ENTITY] = new String("Cell");
					complex_index_fill_color = m_DgnReader.DGNGetShapeFillInfo(elemento);

					// System.err.println("Cell Header " + complex_index_fill_color);
					break;

				case DGNFileHeader.DGNST_COMPLEX_HEADER:

					// bElementoCompuesto = true;
					// System.err.println("Complex Header");
					// contadorSubElementos = 0;
					DGNElemComplexHeader psComplexHeader = (DGNElemComplexHeader) elemento;

					// numSubElementos = psComplexHeader.numelems;
					complexRow[ID_FIELD_ID] = new Integer(elemento.element_id);
					complexRow[ID_FIELD_LAYER] = new Integer(elemento.level);
					complexRow[ID_FIELD_COLOR] = new Integer(elemento.color);
					complexRow[ID_FIELD_ENTITY] = new String("Complex");

					if (psComplexHeader.type == DGNFileHeader.DGNT_COMPLEX_SHAPE_HEADER) {
						bEsPoligono = true;

						// Si es un agujero, no conectamos con el anterior
						if ((psComplexHeader.properties & 0x8000) != 0) {
							bFirstHoleEntity = true;
						} else {
							// Miramos si tiene color de relleno
							// complex_index_fill_color = -1;
							// if (elemento.attr_bytes > 0) {
							complex_index_fill_color = m_DgnReader.DGNGetShapeFillInfo(elemento);

							// System.err.println("complex shape fill color = " + elemento.color);
							// }
						}

						bConnect = true;
					} else {
						bEsPoligono = false;
						bConnect = false;
					}

					break;

				case DGNFileHeader.DGNST_MULTIPOINT:

					// OJO: Si lo que viene en este multipoint es un elemento con type=11 (curve), se trata de una "parametric
					// spline curve". La vamos a tratar como si no fuera curva, pero segn la documentacin, los 2 primeros puntos
					// y los 2 ltimos puntos definen "endpoint derivatives" y NO se muestran.
					// TODAVA HAY UN PEQUEO FALLO CON EL FICHERO dgn-sample.dgn, pero lo dejo por ahora.
					// Es posible que tenga que ver con lo de los arcos (arco distorsionado), que
					// todava no est metido.
					DGNElemMultiPoint psLine = (DGNElemMultiPoint) elemento;
					auxRow[ID_FIELD_LAYER] = new Integer(elemento.level);
					auxRow[ID_FIELD_COLOR] = new Integer(elemento.color);

					if ((psLine.num_vertices == 2) &&
							(psLine.vertices[0].x == psLine.vertices[1].x) &&
							(psLine.vertices[0].y == psLine.vertices[1].y)) {
						auxRow[ID_FIELD_ENTITY] = new String("Point");

						// TODO: CREAR PUNTO lyrPoints.addShape(new FShape(new FPoint(psLine.vertices[0].x,
						// 		psLine.vertices[0].y,psLine.vertices[0].z),FConstant.SHAPE_TYPE_POINT), auxRow);
						return ShapeFactory.createPoint3D(psLine.vertices[0].x,
							psLine.vertices[0].y, psLine.vertices[0].z);
					} else {
						GeneralPathX elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD);

						if (psLine.type == DGNFileHeader.DGNT_CURVE) {
							psLine.num_vertices = psLine.num_vertices - 4;

							for (int aux_n = 0; aux_n < psLine.num_vertices;
									aux_n++) {
								psLine.vertices[aux_n] = psLine.vertices[aux_n +
									2];
							}
						}

						if ((psLine.type == DGNFileHeader.DGNT_SHAPE) &&
								((psLine.properties & 0x8000) != 0)) {
							// Invertimos el orden porque es un agujero
							elShape.moveTo(psLine.vertices[psLine.num_vertices -
								1].x, psLine.vertices[psLine.num_vertices - 1].y);

							for (int i = psLine.num_vertices - 2; i >= 0;
									i--)
								elShape.lineTo(psLine.vertices[i].x,
									psLine.vertices[i].y);
						} else {
							elShape.moveTo(psLine.vertices[0].x,
								psLine.vertices[0].y);

							for (int i = 1; i < psLine.num_vertices; i++)
								elShape.lineTo(psLine.vertices[i].x,
									psLine.vertices[i].y);
						}

						auxRow[ID_FIELD_ID] = new Integer(elemento.element_id);
						auxRow[ID_FIELD_ENTITY] = new String("Multipoint");

						if ((psLine.vertices[0].x == psLine.vertices[psLine.num_vertices -
								1].x) &&
								(psLine.vertices[0].y == psLine.vertices[psLine.num_vertices -
								1].y)) {
							// Lo aadimos tambin como polgono
							bEsPoligono = true;

							// Miramos si tiene color de relleno
							if (elemento.attr_bytes > 0) {
								elemento.color = m_DgnReader.DGNGetShapeFillInfo(elemento);

								// System.err.println("fill color = " + elemento.color);
								if (elemento.color != -1) {
									auxRow[ID_FIELD_COLOR] = new Integer(elemento.color);
								}
							}

							if (elemento.complex == 0) {
								// TODO: Aadir POLIGONO lyrPolygons.addShape(new FShape(FConstant.SHAPE_TYPE_POLYGON, elShape), auxRow);
								return ShapeFactory.createPolygon2D(elShape);
							}
						}

						if (elemento.complex != 0) {
							// Si es un agujero o 
							// es la primera entidad del agujero, lo aadimos sin unir al anterior
							if (bFirstHoleEntity ||
									((psLine.type == DGNFileHeader.DGNT_SHAPE) &&
									((psLine.properties & 0x8000) != 0))) {
								elementoCompuesto.append(elShape, false);
								bFirstHoleEntity = false;
							} else {
								elementoCompuesto.append(elShape, bConnect);
							}
						} else {
							// TODO: Aadir LINEA lyrLines.addShape(new FShape(FConstant.SHAPE_TYPE_POLYLINE, elShape), auxRow);
							return ShapeFactory.createPolyline2D(elShape);
						}
					}

					break;

				case DGNFileHeader.DGNST_ARC:

					// m_DgnReader.DGNDumpElement(m_DgnReader.getInfo(), elemento,"");
					DGNElemArc psArc = (DGNElemArc) elemento;

					// La definicin de arco de MicroStation es distinta a la de Java.
					// En el dgn el origin se entiende que es el centro del arco,
					// y a la hora de crear un Arc2D las 2 primeras coordenadas son
					// la esquina inferior izquierda del rectngulo que rodea al arco.
					// 1.- Creamos la elipse sin rotacin.
					// 2.- Creamos el arco
					// 3.- Rotamos el resultado

					/* System.out.println("Arco con primari axis: " + psArc.primary_axis +
					   " start angle: " + psArc.startang + " sweepang = " + psArc.sweepang);
					   System.out.println("secondaria axis: " + psArc.secondary_axis +
					                                    " rotation = " + psArc.rotation); */
					AffineTransform mT = AffineTransform.getRotateInstance(Math.toRadians(
								psArc.rotation), psArc.origin.x, psArc.origin.y);

					// mT.preConcatenate(AffineTransform.getScaleInstance(100.0,100.0));
					Arc2D.Double elArco = new Arc2D.Double(psArc.origin.x -
							psArc.primary_axis,
							psArc.origin.y - psArc.secondary_axis,
							2.0 * psArc.primary_axis,
							2.0 * psArc.secondary_axis, -psArc.startang,
							-psArc.sweepang, Arc2D.OPEN);

					// Ellipse2D.Double elArco = new Ellipse2D.Double(psArc.origin.x - psArc.primary_axis,
					// 		psArc.origin.y - psArc.secondary_axis,2.0 * psArc.primary_axis, 2.0 * psArc.secondary_axis);
					GeneralPathX elShapeArc = new GeneralPathX(elArco);

					// Transformamos el GeneralPahtX porque si transformamos elArco nos lo convierte
					// a GeneralPath y nos guarda las coordenadas en float, con la correspondiente prdida de precisin
					elShapeArc.transform(mT);

					if (m_DgnReader.getInfo().dimension == 3) {
						//Aqu podramos hacer cosas con la coordenada Z
					}

					auxRow[ID_FIELD_ID] = new Integer(elemento.element_id);
					auxRow[ID_FIELD_ENTITY] = new String("Arc");
					auxRow[ID_FIELD_LAYER] = new Integer(elemento.level);
					auxRow[ID_FIELD_COLOR] = new Integer(elemento.color);

					/* Line2D.Double ejeMayor = new Line2D.Double(psArc.origin.x - psArc.primary_axis, psArc.origin.y,
					   psArc.origin.x + psArc.primary_axis, psArc.origin.y);
					
					   lyrLines.addShape(new FShape(FConstant.SHAPE_TYPE_POLYLINE, new GeneralPathX(ejeMayor)), auxRow); */

					// lyrLines.addShape(new FShape(FConstant.SHAPE_TYPE_POLYLINE, elShapeArc), auxRow);
					if (elemento.complex != 0) {
						// Esto es una posible fuente de fallos si detrs de una
						// elipse vienen ms cosas pegadas. Deberamos volver
						// a conectar una vez pasada la elipse.
						if (elemento.type == DGNFileHeader.DGNT_ELLIPSE) {
							bConnect = false;
						}

						// SI LA ELIPSE ES UN AGUJERO, SE AADE SIN PEGAR
						// Y EL ELEMENTO ES UN POLIGONO		               	   
						if (bFirstHoleEntity ||
								((elemento.type == DGNFileHeader.DGNT_SHAPE) &&
								((elemento.properties & 0x8000) != 0))) {
							elementoCompuesto.append(elShapeArc, false);
							bFirstHoleEntity = false;
						} else {
							elementoCompuesto.append(elShapeArc, bConnect);
						}
					} else {
						// TODO: AADIR LINEA lyrLines.addShape(new FShape(FConstant.SHAPE_TYPE_POLYLINE, elShapeArc), auxRow);
						if (psArc.type == DGNFileHeader.DGNT_ELLIPSE) {
							// TODO: Aadir POLIGONO lyrPolygons.addShape(new FShape(FConstant.SHAPE_TYPE_POLYGON, elShapeArc), auxRow);
						}
					}

					// System.err.println("Entra un Arco");
					break;

				case DGNFileHeader.DGNST_TEXT:

					DGNElemText psText = (DGNElemText) elemento;

					// TODO: DEVOLVER UN TEXTO??

					/* FPoint2D elShapeTxt = new FPoint2D(
					   psText.origin.x, psText.origin.y,psText.origin.z);
					
					   auxRow[ID_FIELD_ID] = new Integer(elemento.element_id);
					   auxRowTxt[ID_FIELD_ENTITY] = new String("Text");
					   auxRowTxt[ID_FIELD_LAYER] = new Integer(elemento.level);
					   auxRowTxt[ID_FIELD_COLOR] = new Integer(elemento.color);
					   heightText = (float) psText.height_mult;
					   auxRowTxt[ID_FIELD_HEIGHTTEXT] = new Double(heightText);
					   auxRowTxt[ID_FIELD_ROTATIONTEXT] = new Double(psText.rotation);
					   auxRowTxt[ID_FIELD_TEXT] = psText.string; // .trim();
					   lyrTexts.addShape(new FShape(elShapeTxt,FConstant.SHAPE_TYPE_POINT),
					                   auxRowTxt);
					 */
					break;

				/* default:
				   m_DgnReader.DGNDumpElement(m_DgnReader.getInfo(), elemento, "");
				 */
			} // switch
		} // if

		// } // for
		if (bElementoCompuesto) {
			if (bInsideCell) {
				auxRow = cellRow;
			} else {
				auxRow = complexRow;
			}

			// System.err.println("Entidad compuesta. bInsideCell = " + bInsideCell + " auxRow = " + auxRow[ID_FIELD_ENTITY]);			
			// TODO: LINEA lyrLines.addShape(new FShape(FConstant.SHAPE_TYPE_POLYLINE, elementoCompuesto), auxRow);
			if (bEsPoligono) {
				if (complex_index_fill_color != -1) {
					auxRow[ID_FIELD_COLOR] = new Integer(complex_index_fill_color);
				}

				// TODO: POLIGONO lyrPolygons.addShape(new FShape(FConstant.SHAPE_TYPE_POLYGON, elementoCompuesto), auxRow);
			}
		}

		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getShapeCount()
	 */
	public int getShapeCount() throws IOException {
		return m_DgnReader.getNumEntities();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws IOException {
		return m_DgnReader.getBoundingBox();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#initialize()
	 */
	public void initialize() throws IOException {
		m_DgnReader = new DGNReader(m_Fich.getAbsolutePath());
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getShapeType()
	 */
	public int getShapeType() {
		return 15;
	}

	/**
	 * @see com.hardcode.driverManager.Driver#getType()
	 */
	public String getName() {
		return "gvSIG DGN Driver";
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#accept(java.io.File)
	 */
	public boolean accept(File f) {
		return f.getName().toUpperCase().endsWith("DGN");
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getDataDriverName()
	 */
	public String getDataDriverName() {
		//TODO implementar bien
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getDataFile(java.io.File)
	 */
	public File getDataFile(File f) {
		//TODO implementar bien
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.FileDriver#fileAccepted(java.io.File)
	 */
	public boolean fileAccepted(File f) {
		return f.getName().toUpperCase().endsWith("DGN");
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldValue(long, int)
	 */
	public Value getFieldValue(long rowIndex, int fieldId)
		throws DriverException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldCount()
	 */
	public int getFieldCount() throws DriverException {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldName(int)
	 */
	public String getFieldName(int fieldId) throws DriverException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getRowCount()
	 */
	public long getRowCount() throws DriverException {
		try {
			return getShapeCount();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return -1;
	}

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getDriverAttributes()
     */
    public DriverAttributes getDriverAttributes() {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldType(int)
     */
    public int getFieldType(int arg0) throws DriverException {
        throw new RuntimeException("por implementar");
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.FileDriver#writeFile(com.hardcode.gdbms.engine.data.file.FileDataWare, java.io.File)
     */
    public void writeFile(FileDataWare arg0) throws DriverException {
        // TODO Auto-generated method stub
        
    }

    public void createSource(String arg0, String[] arg1, int[] arg2) throws IOException {
        // TODO Auto-generated method stub
        
    }

    public void setDataSourceFactory(DataSourceFactory dsf) {
    }
}
