/*
 * @(#)DXFDriver    13-dic-2004
 * 
 * @author jmorell (jose.morell@gmail.com)
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers.dxf;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;

import org.cresques.cts.IProjection;
import org.cresques.cts.ProjectionPool;
import org.cresques.io.DxfFile;
import org.cresques.px.Extent;
import org.cresques.px.IObjList;
import org.cresques.px.dxf.DxfFeatureMaker;
import org.cresques.px.gml.Feature;
import org.cresques.px.gml.Geometry;
import org.cresques.px.gml.InsPoint;
import org.cresques.px.gml.LineString;
import org.cresques.px.gml.Point;
import org.cresques.px.gml.Polygon;

import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.driver.FileDriver;
import com.hardcode.gdbms.engine.data.file.FileDataWare;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.drivers.BoundedShapes;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;

/**
 * @author jmorell (jose.morell@gmail.com)
 * @version 13-dic-2004
 */
public class DXFDriver implements VectorialFileDriver, BoundedShapes, FileDriver {

	private String path;
	
	private final int ID_FIELD_ID=0;
	private final int ID_FIELD_ENTITY=1;
	private final int ID_FIELD_LAYER=2;
	private final int ID_FIELD_COLOR=3;
	private final int ID_FIELD_ELEVATION=4;
	private final int ID_FIELD_THICKNESS=5;
	private final int ID_FIELD_TEXT=6;
	private final int ID_FIELD_HEIGHTTEXT=7;
	private final int ID_FIELD_ROTATIONTEXT=8;
	
	private DxfFile.EntityFactory featureMaker;
	private DxfFile dxfFeatureFile;
	
	private IObjList.vector features;
	private File dxfFile = null;
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#open(java.io.File)
	 */
	public void open(File f) throws IOException {
		dxfFile = f;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#close()
	 */
	public void close() throws IOException {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getShape(int)
	 */
	public IGeometry getShape(int i) throws IOException {
		Feature fea = (Feature)features.get(i);
		if (fea.getGeometry() instanceof Point) {
			Point point = (Point)fea.getGeometry();
			Point2D pto = new Point2D.Double();
			pto = (Point2D.Double)point.get(0);
	        //DGNElemMultiPoint psLine = new DGNElemMultiPoint();
        	//return ShapeFactory.createPoint3D(psLine.vertices[0].x, psLine.vertices[0].y,psLine.vertices[0].z);
        	return ShapeFactory.createPoint3D(pto.getX(),pto.getY(),0);
		} else if (fea.getGeometry() instanceof InsPoint) {
			InsPoint insPoint = (InsPoint)fea.getGeometry();
			Point2D pto = new Point2D.Double();
			pto = (Point2D.Double)insPoint.get(0);
			//GeneralPathX elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD);
			//return ShapeFactory.createPolygon2D(elShape);
			return ShapeFactory.createPoint3D(pto.getX(),pto.getY(),0);
		} else if (fea.getGeometry() instanceof LineString) {
			GeneralPathX genPathX = new GeneralPathX();
			Point2D[] pts = new Point2D[fea.getGeometry().pointNr()];
			for (int j=0; j<fea.getGeometry().pointNr(); j++) {
				pts[j] = (Point2D)fea.getGeometry().get(j);
			}
			genPathX.moveTo(pts[0].getX(), pts[0].getY());
			for (int j=1; j < pts.length; j++) {
				genPathX.lineTo(pts[j].getX(), pts[j].getY());
			}
			double[] zs = new double[1];
			zs[0] = 0.0;
			return ShapeFactory.createPolyline3D(genPathX,zs);
		} else if (fea.getGeometry() instanceof Polygon) {
			GeneralPathX genPathX = new GeneralPathX();
			Point2D[] pts = new Point2D[fea.getGeometry().pointNr()];
			for (int j=0; j<fea.getGeometry().pointNr(); j++) {
				pts[j] = (Point2D)fea.getGeometry().get(j);
			}
			genPathX.moveTo(pts[0].getX(), pts[0].getY());
			for (int j=1; j < pts.length; j++) {
				genPathX.lineTo(pts[j].getX(), pts[j].getY());
			}
			return ShapeFactory.createPolygon2D(genPathX);
		} else {
			System.out.println("getShape(): ERROR:Geometra desconocida.");
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getShapeCount()
	 */
	public int getShapeCount() throws IOException {
		return features.size();
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws IOException {
		return (Rectangle2D)featureMaker.getExtent().toRectangle2D();
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#initialize()
	 */
	public void initialize() throws IOException {
		IProjection proj = ProjectionPool.get("EPSG:23030");
		featureMaker = new DxfFeatureMaker(proj);
		dxfFeatureFile = new DxfFile(proj, dxfFile.getAbsolutePath(), featureMaker);
		dxfFeatureFile.load();
		features = (IObjList.vector)((DxfFeatureMaker)featureMaker).getObjects();
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#accept(java.io.File)
	 */
	public boolean accept(File f) {
		return f.getName().toUpperCase().endsWith("DXF");
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getDataDriverName()
	 */
	public String getDataDriverName() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getDataFile(java.io.File)
	 */
	public File getDataFile(File f) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getShapeType()
	 */
	public int getShapeType() {
		return 1;
	}

	/* (non-Javadoc)
	 * @see com.hardcode.driverManager.Driver#getName()
	 */
	public String getName() {
		return "gvSIG DXF Driver";
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.BoundedShapes#getShapeBounds(int)
	 */
	public Rectangle2D getShapeBounds(int index) throws IOException {
		Feature fea = (Feature)features.get(index);
		Geometry geo = (Geometry)fea.getGeometry();
		return ((Extent)(geo.getExtent())).toRectangle2D();
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.BoundedShapes#getShapeType(int)
	 */
	public int getShapeType(int index) {
        int auxType = 0;
        Feature fea = (Feature)features.get(index);
		if (fea.getGeometry() instanceof Point) {
            auxType = auxType | FShape.POINT;
		} else if (fea.getGeometry() instanceof InsPoint) {
            auxType = auxType | FShape.POINT;
		} else if (fea.getGeometry() instanceof LineString) {
            auxType = auxType | FShape.LINE;
		} else if (fea.getGeometry() instanceof Polygon) {
            auxType = auxType | FShape.POLYGON;
		} else {
			System.out.println("getShape(): ERROR:Geometra desconocida.");
		}
        return auxType;
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.FileDriver#fileAccepted(java.io.File)
	 */
	public boolean fileAccepted(File f) {
		return f.getName().toUpperCase().endsWith("DXF");
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldValue(long, int)
	 */
	public Value getFieldValue(long arg0, int arg1) throws DriverException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldCount()
	 */
	public int getFieldCount() throws DriverException {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldName(int)
	 */
	public String getFieldName(int arg0) throws DriverException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getRowCount()
	 */
	public long getRowCount() throws DriverException {
		try {
			return getShapeCount();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return -1;
		//return 0;
	}

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getDriverAttributes()
     */
    public DriverAttributes getDriverAttributes() {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldType(int)
     */
    public int getFieldType(int arg0) throws DriverException {
        throw new RuntimeException("por implementar");
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.FileDriver#writeFile(com.hardcode.gdbms.engine.data.file.FileDataWare, java.io.File)
     */
    public void writeFile(FileDataWare arg0) throws DriverException {
        // TODO Auto-generated method stub
        
    }

    public void createSource(String arg0, String[] arg1, int[] arg2) throws IOException {
        // TODO Auto-generated method stub
        
    }

    public void setDataSourceFactory(DataSourceFactory dsf) {
    }

}
