/*
 * Created on 17-dic-2004
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers.raster;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.cresques.geo.ViewPortData;
import org.cresques.io.EcwWriter;
import org.cresques.io.GdalFile;
import org.cresques.io.GdalWriter;
import org.cresques.io.GeoRasterFile;
import org.cresques.io.raster.RasterFilterStack;
import org.cresques.px.Extent;
import org.cresques.px.PxRaster;

import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.GeorreferencedRasterDriver;

/**
 * Driver de raster (tal y como los abre CMS.
 * Primera aproximacin al raster. Borrador para revisar.
 * 
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class CmsRasterDriver implements GeorreferencedRasterDriver {
	private File file = null;
	private IProjection proj = null;
	private GeoRasterFile rasterFile = null;
	private PxRaster raster = null;
	private Extent tempExtent = null;
	
	int trans = 255;

	static {
		// Fuerza la carga del soporte de img (gdal):
		 System.err.println("CmsRasterDriver: inicializando");
		 Class c[] = {GdalFile.class, GdalWriter.class, EcwWriter.class};
	}
	/* (non-Javadoc)
	 * @see com.hardcode.driverManager.Driver#getName()
	 */
	public String getName() {
		return "gvSIG Image Driver";
	}
	
	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#open(java.io.File)
	 */
	public void open(File f) throws IOException {
		file = f;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#close()
	 */
	public void close() throws IOException {		
		rasterFile.close();
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#initialize()
	 */
	public void initialize() throws IOException {
		if (proj != null){
			String fName = file.getAbsolutePath();
			int test = fName.indexOf("ecwp:");

			if (test != -1) {
	    		String urlECW = fName.substring(test + 6);
	    		//urlECW.replaceAll("\\", "/");
	    		fName = "ecwp://" + urlECW;
				System.err.println(test+" "+fName);
	    	}

			rasterFile = GeoRasterFile.openFile(proj, fName);
			
			rasterFile.load();
			
			createPxRaster();
		} else
			throw new IOException("Proyeccin no asignada");
		System.out.println(getName()+": Inicializado (con PxRaster)");
	}
	
	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.GeorreferencedRasterDriver#initialize(org.cresques.cts.IProjection)
	 */
	public void initialize(IProjection proj) throws IOException {
		this.proj = proj;
		rasterFile = GeoRasterFile.openFile(proj, file.getAbsolutePath());
		
		rasterFile.load();
		
		createPxRaster();
	}
	
	private void createPxRaster() {
		raster = new PxRaster(rasterFile, null, rasterFile.getExtent());
		raster.setTransparency(false);
	}

	/**
	 * Aade un fichero al PxRaste
	 * @param fileName Nombre del fichero a aadir
	 */
	public void addFile(String fileName){
		if(raster!=null)
			raster.addFile(fileName);
	}
	
	/**
	 * Elimina un fichero al PxRaste
	 * @param fileName Nombre del fichero a aadir
	 */
	public void delFile(String fileName){
		if(raster!=null)
			raster.delFile(fileName);
	}
	
	/*
	 * @see com.iver.cit.gvsig.fmap.drivers.GeorreferencedRasterDriver#getFullExtent()
	 */
	public Rectangle2D getFullExtent() {
		return rasterFile.getExtent().toRectangle2D();
	}
	
	public int getTransparency() {
		return 255-(raster.getAlpha());
	}
	
	public void setTransparency(int trans) {
		this.trans = trans;
		if (raster != null) {
			raster.setTransparency(trans);
		}
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#draw(java.awt.image.BufferedImage, java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort vp) throws DriverIOException {
		Extent e = new Extent(vp.getAdjustedExtent());
		// System.err.println("CmsRasterDriver: Extent = "+e);
		Dimension imgSz = vp.getImageSize();
		ViewPortData vp2 = new ViewPortData(vp.getProjection(), e, imgSz );
		vp2.setMat(vp.getAffineTransform());
		raster.draw(g, vp2);
	}

	/*
	 * @see com.iver.cit.gvsig.fmap.drivers.GeorreferencedRasterDriver#getProjection()
	 */
	public IProjection getProjection() {
		return proj;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.GeorreferencedRasterDriver#setProjection(org.cresques.cts.IProjection)
	 */
	public void setProjection(IProjection proj) {
		this.proj = proj;
	}

	/*
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getNumBands()
	 */
	public int getNumBands() {
		return rasterFile.getBandCount();
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getPixel(int, int, byte[])
	 */
	public byte[] getPixel(int x, int y, byte[] dArray) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getPixel(int, int, int[])
	 */
	public int[] getPixel(int x, int y, int[] dArray) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getRasterDataType()
	 */
	public int getRasterDataType() {
		return raster.getDataType();
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getData(int, int, int)
	 */
	public Object getData(int x, int y, int band) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getDataAsByte(int, int, int)
	 */
	public byte getDataAsByte(int x, int y, int band) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getDataAsFloat(int, int, int)
	 */
	public float getDataAsFloat(int x, int y, int band) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getDataAsDouble(int, int, int)
	 */
	public double getDataAsDouble(int x, int y, int band) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getDataAsInt(int, int, int)
	 */
	public int getDataAsInt(int x, int y, int band) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#fileAccepted(java.io.File)
	 */
	public boolean fileAccepted(File file) {
		return GeoRasterFile.fileIsSupported(file.getName());
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getImage(java.awt.Dimension, java.awt.geom.Rectangle2D, org.cresques.cts.ICoordTrans)
	 */
	public Image getImage(Dimension size, Rectangle2D userSize, ICoordTrans rp) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getAttributes()
	 */
	public ArrayList getAttributes() {
		ArrayList attr = new ArrayList();
		String dataType = "Byte";
		if (rasterFile.getDataType() == DataBuffer.TYPE_BYTE) dataType = "Byte";
		else if (rasterFile.getDataType() == DataBuffer.TYPE_SHORT)
			dataType = "Short";
		else if (rasterFile.getDataType() == DataBuffer.TYPE_USHORT)
			dataType = "Unsigned Short";
		else if (rasterFile.getDataType() == DataBuffer.TYPE_INT)
			dataType = "Integer";
		else if (rasterFile.getDataType() == DataBuffer.TYPE_FLOAT)
			dataType = "Float";
		else if (rasterFile.getDataType() == DataBuffer.TYPE_DOUBLE)
			dataType = "Double";
		else
			dataType = "Unknown";
		Object [][] a = {
			{"Filename",rasterFile.getName()},
			{"Filesize",new Long(rasterFile.getFileSize())},
			{"Width",new Integer(rasterFile.getWidth())},
			{"Height", new Integer(rasterFile.getHeight())},
			{"Bands", new Integer(rasterFile.getBandCount())}
//			{"BandDataType", dataType}
		};
		for (int i=0; i<a.length; i++)
			attr.add(a[i]);
		return attr;
	}
	
	/**
	 * Devuelve el colorBand activo en la banda especificada.
	 * @param flag banda.
	 * @return color de banda activo
	 */
	public int getBand(int flag){
		return raster.getBand(flag);
	}
	
	/**
	 * Devuelve la posicin del fichero para la banda especificada.
	 * @param flag banda.
	 * @return posicin del fichero
	 */
	public int getPosFile(int flag){
		return raster.getPosFile(flag);
	}
		
	/**
	 * Activa o desactiva la transparencia
	 * @param t	true activa la transparencia y false la desactiva
	 */
	public void setTransparency(boolean t){
		raster.setTransparency(t);
	}
	
	/**
	 * Asocia un colorBand al rojo, verde o azul.
	 * @param flag cual (o cuales) de las bandas.
	 * @param nBand	que colorBand
	 */
	public void setBand(int flag, int nBand){
		raster.setBand(flag, nBand);
	}
	
	/**
	 * Obtiene la pila de filtros 
	 * @return pila de filtros
	 */
	public RasterFilterStack getFilterStack(){
		return raster.filterStack;
	}
	
	/**
	 * Asigna la pila de filtros 
	 * @return pila de filtros
	 */
	public void setFilterStack(RasterFilterStack stack){
		raster.filterStack = stack;
	}
	
	/**
	 * Obtiene el valor del pixel del Image en la posicin x,y
	 * @param x Posicin x
	 * @param y Posicin y
	 * @return valor de pixel
	 */
	public int[] getPixel(double wcx, double wcy){
		return raster.getPixel(wcx, wcy);
	}
	
	/**
	 * 
	 */
	public GeoRasterFile [] getFiles(){
		return raster.getFiles();
	}
		
}
