package com.iver.cit.gvsig.fmap.edition;

import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;

import java.io.IOException;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 */
public interface EditableFeatureSource {
	/**
	 * Mtodo invocado cuando se comienza la edicin, para poner en marcha las
	 * estructuras de datos necesarias para la misma, notificar al servidor en
	 * protocolos en los que sea necesario, ...
	 *
	 * @throws EditionException Si no se logra poner la fuente de datos en
	 * 		   edicin
	 */
	void startEdition() throws EditionException;

	/**
	 * Invocado cuando termina la edicin. En funcin de la clase concreta que
	 * implemente este mtodo se generar el fichero con los resultados de la
	 * edicin, se realizar una transaccin a la base de datos, etc.
	 *
	 * @throws EditionException Si no se consiguen llevar a cabo las
	 * 		   modificaciones
	 */
	void stopEdition() throws EditionException;

	/**
	 * Si el ndice se corresponde a una geometria de las originales de la capa
	 * en edicin y no ha sido modificada ni eliminada devuelve la geometria
	 * original. Si ha sido modificada debera de buscar en el fichero de
	 * expansin y si ha sido eliminada debera devolver null
	 *
	 * @param index ndice de la geometra.
	 *
	 * @return Geometra.
	 *
	 * @throws IOException Si se produce un error con el fichero de expansin
	 * @throws DriverIOException Si se produce un error accediendo a las
	 * 		   geometras originales
	 */
	IGeometry getGeometry(int index) throws IOException, DriverIOException;

	/**
	 * Devuelve el nmero de geometrias que hay actualmente en edicin.
	 *
	 * @return Nmero de geometras.
	 */
	int getGeometryCount();

	/**
	 * Aade una geometria al fichero de expansin y guarda la correspondencia
	 * en una tabla asociada.
	 *
	 * @param g geometra a guardar.
	 *
	 * @throws DriverIOException Si se produce un error accediendo a las
	 * 		   geometras originales
	 * @throws IOException Si se produce un error con el fichero de expansin
	 */
	void addGeometry(IGeometry g) throws DriverIOException, IOException;

	/**
	 * Elimina una geometria. Si es una geometra original de la capa en
	 * edicin se marca como eliminada (haya sido modificada o no). Si es una
	 * geometra aadida posteriormente se invalida en el fichero de
	 * expansin, para que una futura compactacin termine con ella.
	 *
	 * @param index ndice de la geometra que se quiere eliminar
	 */
	void removeGeometry(int index);

	/**
	 * Si se intenta modificar una geometra original de la capa en edicin se
	 * aade al fichero de expansin y se registra la posicin en la que se
	 * aadi. Si se intenta modificar una geometria que se encuentra en el
	 * fichero de expansin (por ser nueva o original pero modificada) se
	 * invoca el mtodo modifyGeometry y se actualiza el ndice de la
	 * geometria en el fichero.
	 *
	 * @param index ndice de la geometra que se quiere eliminar
	 * @param g Geometra nueva
	 *
	 * @throws IOException Si se produce un error accediendo al fichero de
	 * 		   expansion
	 */
	void modifyGeometry(int index, IGeometry g) throws IOException;

	/**
	 * Compacta el almacenamiento de las geometras que estn en edicin. Tras
	 * esta operacin, el orden de las geometras seguramente cambiar y toda
	 * llamada a getGeometry devolver una geometra distinta de null, ya que
	 * las eliminadas son borradas definitivamente. Hay que tener especial
	 * cuidado al invocar este mtodo ya que cualquier tipo de asociacin
	 * entre geometras y otro tipo de objetos (comandos de edicin, snapping,
	 * ...) que use el ndice de la geometra se ver afectado por ste mtodo
	 */
	void compact();
}
