package com.iver.cit.gvsig.fmap.edition;

import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;

import java.io.IOException;


/**
 * Clase proxy que implementa los mtodos de EditableFeatureSource apilando los
 * comandos correspondientes en el fichero dietario. Estos comandos al
 * apilarse debern de ejecutarse contra el EditableFeatureSource concreto que
 * se est editando
 */
public class EditableFeatureSourceProxy implements EditableFeatureSource {
	private DefaultEditableFeatureSource defs;
	private CommandRecord cr;

	/**
	 * Crea un nuevo EditableFeatureSourceProxy.
	 *
	 * @param defs DOCUMENT ME!
	 * @param cr DOCUMENT ME!
	 */
	public EditableFeatureSourceProxy(DefaultEditableFeatureSource defs,
		CommandRecord cr) {
		this.defs = defs;
		this.cr = cr;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @throws EditionException
	 *
	 * @see com.iver.cit.gvsig.fmap.edition.EditableFeatureSource#startEdition()
	 */
	public void startEdition() throws EditionException {
		defs.getOriginalFeatureAdapter().startEdition();
		cr.start();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @throws EditionException
	 *
	 * @see com.iver.cit.gvsig.fmap.edition.EditableFeatureSource#stopEdition()
	 */
	public void stopEdition() throws EditionException {
		defs.getOriginalFeatureAdapter().stopEdition();
		cr.stop();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param index DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws IOException
	 * @throws DriverIOException
	 *
	 * @see com.iver.cit.gvsig.fmap.edition.EditableFeatureSource#getGeometry(int)
	 */
	public IGeometry getGeometry(int index)
		throws IOException, DriverIOException {
		return defs.getGeometry(index);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.EditableFeatureSource#getGeometryCount()
	 */
	public int getGeometryCount() {
		return defs.getGeometryCount();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param g DOCUMENT ME!
	 *
	 * @throws DriverIOException
	 * @throws IOException
	 *
	 * @see com.iver.cit.gvsig.fmap.edition.EditableFeatureSource#addGeometry(com.iver.cit.gvsig.fmap.core.FGeometry)
	 */
	public void addGeometry(IGeometry g) throws DriverIOException, IOException {
		defs.addGeometry(g);

		//cr.addCommand(new AddGeometryCommand());
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.EditableFeatureSource#removeGeometry(int)
	 */
	public void removeGeometry(int index) {
		defs.removeGeometry(index);

		//cr.addCommand(new RemoveGeometryCommand());
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param index DOCUMENT ME!
	 * @param g DOCUMENT ME!
	 *
	 * @throws IOException
	 *
	 * @see com.iver.cit.gvsig.fmap.edition.EditableFeatureSource#modifyGeometry(int,
	 * 		com.iver.cit.gvsig.fmap.core.FGeometry)
	 */
	public void modifyGeometry(int index, IGeometry g)
		throws IOException {
		defs.modifyGeometry(index, g);

		//cr.addCommand(new ModifyGeometryCommand());
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.EditableFeatureSource#compact()
	 */
	public void compact() {
	}
}
