package com.iver.cit.gvsig.fmap.edition;

import com.iver.cit.gvsig.fmap.core.IGeometry;

import java.io.IOException;

import java.util.HashMap;


/**
 * Maneja el fichero de extensin en el que se almacenan las modificacionesy adicines
 * durante la edicin. Los ndices que devuelve esta clase en sus mtodos
 * addGeometry y modifyGeometry son invariables, es decir, que si se invoca un
 * mtodo addGeometry que retorna un 8, independientemente de las operaciones
 * que se realicen posteriormente, una llamada a getGeometry(8) retornar
 * dicha geometra. Si esta geometra es eliminada posteriormente, se retornar un
 * null. Esto ltimo se cumple mientras no se invoque el mtodo compact, mediante
 * el que se reorganizan las geometras dejando en el fichero slo las que tienen
 * validez en el momento de realizar la invocacin.
 */
public interface ExpansionFile {
	/**
	 * Aade una geometria al final del fichero y retorna el ndice que ocupa
	 * esta geometria en el mismo
	 *
	 * @param g DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws IOException
	 */
	int addGeometry(IGeometry g) throws IOException;

	/**
	 * Modifica la index-sima geometra del fichero devolviendo la posicin en
	 * la que se pone la geometria modificada.
	 *
	 * @param index DOCUMENT ME!
	 * @param g DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws IOException
	 */
	int modifyGeometry(int index, IGeometry g) throws IOException;

	/**
	 * Obtiene la geometria que hay en el ndice 'index' o null si la geometra
	 * ha sido invalidada.
	 *
	 * @param index DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws IOException
	 */
	IGeometry getGeometry(int index) throws IOException;

	/**
	 * Invalida una geometra, de cara a una futura compactacin del fichero
	 *
	 * @param index DOCUMENT ME!
	 */
	void invalidateGeometry(int index);

	/**
	 * Realiza una compactacin del fichero que maneja esta clase
	 *
	 * @param relations DOCUMENT ME!
	 */
	void compact(HashMap relations);

	/**
	 * Devuelve el nmero de geometras del fichero.
	 *
	 * @return nmero de geometras.
	 */
	int getGeometryCount();
}
