/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.edition;

import com.iver.cit.gvsig.fmap.core.IGeometry;

import java.io.IOException;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.HashMap;
import java.util.Iterator;


/**
 * Implementacin de prueba en memoria de ExpansionFile.
 *
 * @author Vicente Caballero Navarro
 */
public class MemoryExpansionFile implements ExpansionFile {
	ArrayList geometries = new ArrayList();
	BitSet invalidGeometries = new BitSet();
	/**
	 * @see com.iver.cit.gvsig.fmap.edition.ExpansionFile#addGeometry(com.iver.cit.gvsig.fmap.core.IGeometry)
	 */
	public int addGeometry(IGeometry g) throws IOException {
		geometries.add(g);

		return geometries.size() - 1;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.ExpansionFile#modifyGeometry(int,
	 * 		com.iver.cit.gvsig.fmap.core.IGeometry)
	 */
	public int modifyGeometry(int index, IGeometry g) throws IOException {
		if (invalidGeometries.get(index)) {
			throw new RuntimeException(
				"Se ha intentado modificar una geometra que ha sido borrada anteriormente");
		}

		invalidateGeometry(index);
		geometries.add(g);

		return geometries.size() - 1;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.ExpansionFile#getGeometry(int)
	 */
	public IGeometry getGeometry(int index) throws IOException {
		if (invalidGeometries.get(index)) {
			return null;
		}

		return (IGeometry) geometries.get(index);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.ExpansionFile#invalidateGeometry(int)
	 */
	public void invalidateGeometry(int index) {
		invalidGeometries.set(index, true);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.ExpansionFile#compact()
	 */
	public void compact(HashMap relations) {
		ArrayList geoAux = new ArrayList();
		Iterator iter = relations.keySet().iterator();
		HashMap aux = new HashMap();
		int n = 0;
		
		while (iter.hasNext()) {
			Integer virtualIndex = (Integer) iter.next();
			Integer expansionIndex = (Integer) relations.get(virtualIndex);
			
			if (!invalidGeometries.get(expansionIndex.intValue())){
				geoAux.add(geometries.get(expansionIndex.intValue()));
				aux.put(new Integer(n), new Integer(geoAux.size()-1));
				n++;
			}
		}

		invalidGeometries.clear();
		geometries = geoAux;
		relations.clear();
		relations.putAll(aux);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.ExpansionFile#getGeometryCount()
	 */
	public int getGeometryCount() {
		return geometries.size() - invalidGeometries.cardinality();
	}
}
