/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.ArrayList;
import java.util.BitSet;

import org.apache.log4j.Logger;
import org.cresques.cts.ICoordTrans;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.NoSuchTableException;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.hardcode.gdbms.engine.values.DoubleValue;
import com.hardcode.gdbms.engine.values.FloatValue;
import com.hardcode.gdbms.engine.values.NullValue;
import com.hardcode.gdbms.engine.values.NumericValue;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FLabel;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.BoundedShapes;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Labelable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.RandomVectorialData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.operations.Cancellable;
import com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.Strategy;
import com.iver.cit.gvsig.fmap.operations.strategies.StrategyManager;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.fmap.rendering.Legend;
import com.iver.cit.gvsig.fmap.rendering.LegendChangedEvent;
import com.iver.cit.gvsig.fmap.rendering.LegendFactory;
import com.iver.cit.gvsig.fmap.rendering.SingleSymbolLegend;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.utiles.IPersistance;
import com.iver.utiles.PostProcessSupport;
import com.iver.utiles.XMLEntity;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.index.quadtree.Quadtree;


/**
 * Capa bsica Vectorial.
 *
 * @author Fernando Gonzlez Corts
 */

//TODO Cuando no sea para pruebas debe no ser public
public class FLyrVect extends FLyrDefault implements Labelable, Selectable,
	AlphanumericData, ClassifiableVectorial, SingleLayer, VectorialData,
	RandomVectorialData {
	private static Logger logger = Logger.getLogger(FLyrVect.class.getName());

	/** Leyenda de la capa vectorial */
	private VectorialLegend legend;
	private int typeShape = -1;
	private SelectionSupport selectionSupport = new SelectionSupport();
	private LayerChangeSupport layerChangeSupport = new LayerChangeSupport();
	private VectorialAdapter source;
	private SelectableDataSource sds;
    
    private Quadtree spatialIndex = null;
    
    private boolean bHasJoin = false;

	/**
	 * Aade un SelectionListener a la lista de listeners.
	 *
	 * @param listener SelectionListener.
	 */
	public void addSelectionListener(SelectionListener listener) {
		selectionSupport.addSelectionListener(listener);
	}

	/**
	 * Borra un selectionListener de la lista de listeners.
	 *
	 * @param listener SelectionListener
	 */
	public void removeSelectionListener(SelectionListener listener) {
		selectionSupport.removeSelectionListener(listener);
	}

	/**
	 * Cuando ocurre un evento de cambio en la seleccin, ste puede ser uno de
	 * una gran cantidad de eventos. Con el fin de no propagar todos estos
	 * eventos, se realiza la propagacin de manera manual al final de la
	 * "rfaga" de eventos
	 */
	public void fireSelectionEvents() {
		selectionSupport.fireSelectionEvents();
	}

	/**
	 * Devuelve el VectorialAdapater de la capa.
	 *
	 * @return VectorialAdapter.
	 */
	public VectorialAdapter getSource() {
		return source;
	}

	/**
	 * Inserta el VectorialAdapter a la capa.
	 *
	 * @param va VectorialAdapter.
	 */
	public void setSource(VectorialAdapter va) {
		source = va;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerOperations#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws DriverException {
		try {
			Rectangle2D rAux;
			logger.debug("source.start()");
			source.start();
			rAux = source.getFullExtent();
			logger.debug("source.stop()");
			source.stop();

			// Si existe reproyeccin, reproyectar el extent
			ICoordTrans ct = getCoordTrans();

			if (ct != null) {
				Point2D pt1 = new Point2D.Double(rAux.getMinX(), rAux.getMinY());
				Point2D pt2 = new Point2D.Double(rAux.getMaxX(), rAux.getMaxY());
				pt1 = ct.convert(pt1, null);
				pt2 = ct.convert(pt2, null);
				rAux = new Rectangle2D.Double();
				rAux.setFrameFromDiagonal(pt1, pt2);
			}

			return rAux;
		} catch (DriverIOException e) {
			throw new DriverException(e);
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerOperations#draw(java.awt.image.BufferedImage,
	 * 		java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
		Cancellable cancel,double scale) throws DriverException {
	    
		if (isWithinScale(scale)){	
            // Las que solo tienen etiquetado sin pintar el shape,
            // no pasamos por ellas
            boolean bDrawShapes = true;
            if (legend instanceof SingleSymbolLegend)
            {
                if (legend.getDefaultSymbol().isShapeVisible() == false)
                    bDrawShapes = false;
            }
            if (bDrawShapes)
            {
                Strategy strategy = StrategyManager.getStrategy(this);    
                try{
                strategy.draw(image, g, viewPort, cancel);
                }catch (DriverException e) {
					this.setVisible(false);
					this.setActive(false);
					throw e;
				}
            }
    
    		if (getVirtualLayers() != null) {
    			getVirtualLayers().draw(image, g, viewPort, cancel,scale);
    		}
    
    		if (getLayerText() != null) {
    			getLayerText().draw(image, g, viewPort, cancel,scale);
    		}
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#print(java.awt.Graphics2D,
	 * 		com.iver.cit.gvsig.fmap.ViewPort,
	 * 		com.iver.cit.gvsig.fmap.operations.Cancellable)
	 */
	public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel,double scale)
		throws DriverException {
		if (isVisible() && isWithinScale(scale)){	
		Strategy strategy = StrategyManager.getStrategy(this);

		strategy.print(g, viewPort, cancel);

		if (getLayerText() != null) {
			getLayerText().draw(null, g, viewPort, cancel,scale);
		}
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.VectorialOperations#createLabelLayer(int)
	 */
	// public FLayer createLabelLayer(int fieldId) {
    public FLayer createLabelLayer(SelectableDataSource ds) {
		FLyrText layerText = null;
        try {
            layerText = new FLyrText();
			layerText.setLegend((VectorialLegend) getLegend());
            layerText.createLabels(this);
		} catch (FieldNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (DriverException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
        }


		setLayerText(layerText);
		layerText.setCoordTrans(getCoordTrans());
		return layerText;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.VectorialOperations#removeLabels()
	 */
	public void removeLabels() {
		setLayerText(null);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.RandomVectorialData#createIndex()
	 */
	public void createSpatialIndex() {
        // FJP: ESTO HABR QUE CAMBIARLO. PARA LAS CAPAS SECUENCIALES, TENDREMOS
        // QUE ACCEDER CON UN WHILE NEXT. (O mejorar lo de los FeatureVisitor
        // para que acepten recorrer sin geometria, solo con rectangulos.
        spatialIndex = new Quadtree();
        VectorialAdapter va = getSource();
        ICoordTrans ct = getCoordTrans();
        BoundedShapes shapeBounds = (BoundedShapes) va.getDriver();
        try {
            va.start();

            for (int i=0; i < va.getShapeCount(); i++)
            {               
                Rectangle2D r = shapeBounds.getShapeBounds(i);
                // TODO: MIRAR COMO SE TRAGARA ESTO LO DE LAS REPROYECCIONES
                if (ct != null) {
                    r = ct.convert(r);
                }
                Coordinate c1 = new Coordinate(r.getMinX(), r.getMinY());
                Coordinate c2 = new Coordinate(r.getMaxX(), r.getMaxY());
                Envelope env = new Envelope(c1, c2);
                spatialIndex.insert(env, new Integer(i));
            } // for
            va.stop();
        } catch (DriverIOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.VectorialOperations#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor,
	 * 		FBitSet)
	 */
	public void process(FeatureVisitor visitor, FBitSet subset)
		throws DriverException, VisitException {
		Strategy s = StrategyManager.getStrategy(this);
		s.process(visitor, subset);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor)
	 */
	public void process(FeatureVisitor visitor)
		throws DriverException, VisitException {
		Strategy s = StrategyManager.getStrategy(this);
		s.process(visitor);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#setSelection(com.iver.cit.gvsig.fmap.operations.selection.VectorialSubSet)
	 */
	public void setSelection(FBitSet selection) {
		selectionSupport.setSelection(selection);
		fireSelectionEvents();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#isSelected(int)
	 */
	public boolean isSelected(int index) {
		return selectionSupport.isSelected(index);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#getSelection()
	 */
	public FBitSet getSelection() {
		return selectionSupport.getSelection();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#clearSelection()
	 */
	public void clearSelection() {
		selectionSupport.clearSelection();
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.RandomVectorialData#queryByRect(java.awt.geom.Rectangle2D)
	 */
	public FBitSet queryByRect(Rectangle2D rect) throws DriverException {
		Strategy s = StrategyManager.getStrategy(this);

		return s.queryByRect(rect);
	}

    public FBitSet queryByPoint(Point2D p, double tolerance) throws DriverException
    {
        Strategy s = StrategyManager.getStrategy(this);
        return s.queryByPoint(p, tolerance);       
    }
    
    public FBitSet queryByShape(IGeometry g, int relationship) throws DriverException, VisitException
    {
        Strategy s = StrategyManager.getStrategy(this);
        return s.queryByShape(g, relationship);        
    }
	/**
	 * @throws DriverException
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#getRecordset()
	 */
	public SelectableDataSource getRecordset() throws DriverException {
		if (sds == null){
			try {
				DataSource ds = source.getRecordset();

				if (ds == null) {
					return null;
				}

				sds = new SelectableDataSource(ds);
				sds.setSelectionSupport(selectionSupport);
				
				return sds;
			} catch (DriverLoadException e) {
				throw new DriverException(e);
			} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
				throw new DriverException(e);
            }
		}
		return sds;
	}
    
    /**
     * Para cuando haces una unin, sustituyes el recorset por el nuevo.
     * De esta forma, podrs poner leyendas basadas en el nuevo
     * recordset
     * @param newSds
     * @throws DriverException 
     * @throws FieldNotFoundException 
     * @throws FieldNotFoundException 
     */
    public void setRecordset(SelectableDataSource newSds) throws DriverException, FieldNotFoundException
    {
        sds = newSds;
        sds.setSelectionSupport(selectionSupport);
        legend.setDataSource(sds);
        logger.debug("Recordset cambiado a " + sds.getName());
    }

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.CommonOperations#setLegend(int,
	 * 		com.iver.cit.gvsig.fmap.rendering.Legend)
	 */
	public void setLegend(VectorialLegend r)
		throws DriverException, FieldNotFoundException {
		VectorialLegend oldLegend = legend;
		legend = r;

		try {
			legend.setDataSource(getRecordset());

			if (legend.getLabelField() != null) {
			    // sds.start();
				// int idLabelField = getRecordset().getFieldIndexByName(legend.getLabelField());
				createLabelLayer(sds);
				// sds.stop();
			}
			else
			    removeLabels();
		} catch (DriverException e) {
			throw new DriverException(e);
		} catch (FieldNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
        }

		LegendChangedEvent e = LegendChangedEvent.createLegendChangedEvent(oldLegend, legend);
		callLegendChanged(e);
	}

	/**
	 * Devuelve la Leyenda de la capa.
	 *
	 * @return Leyenda.
	 */
	public Legend getLegend() {
		return legend;
	}

	/**
	 * Devuelve el tipo de shape que contiene la capa.
	 *
	 * @return tipo de shape.
	 *
	 * @throws DriverException
	 */
	public int getShapeType() throws DriverException {
		if (typeShape == -1) {
			try {
				logger.debug("source.start()");
				source.start();
				typeShape = source.getShapeType();
				logger.debug("source.stop()");
				source.stop();
			} catch (DriverIOException e) {
				throw new DriverException(e);
			}
		}

		return typeShape;
	}

	/**
	 * @throws XMLException
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getProperties()
	 */
	public XMLEntity getXMLEntity() throws XMLException {
		XMLEntity xml = super.getXMLEntity();
        xml.addChild(legend.getXMLEntity());
        xml.addChild(selectionSupport.getXMLEntity());

		if (source instanceof VectorialFileAdapter) {
			xml.putProperty("type", "vectorial");
			xml.putProperty("file", ((VectorialFileAdapter) source).getFile());
			try {
				xml.putProperty("recordset-name", getRecordset().getName());
			} catch (DriverException e) {
				throw new XMLException(e);
			}
		} else if (source instanceof VectorialDBAdapter) {
            xml.putProperty("type", "vectorial");
            
            VectorialDatabaseDriver dbDriver = (VectorialDatabaseDriver) source.getDriver();
            
            // Guardamos el nombre del driver para poder recuperarlo
            // con el DriverManager de Fernando.
            xml.putProperty("db", dbDriver.getName());            
            try {
                xml.putProperty("recordset-name", getRecordset().getName());
            } catch (DriverException e) {
                throw new XMLException(e);
            }
            xml.addChild(dbDriver.getXMLEntity()); // Tercer child. Antes hemos metido la leyenda y el selection support
        } else if (source instanceof VectorialAdapter) { 
            // Se supone que hemos hecho algo genrico.
            xml.putProperty("type", "vectorial");
            
            VectorialDriver driver = (VectorialDriver) source.getDriver();
            
            // Guardamos el nombre del driver para poder recuperarlo
            // con el DriverManager de Fernando.
            xml.putProperty("other", driver.getName());            
            try {
                xml.putProperty("recordset-name", getRecordset().getName());
            } catch (DriverException e) {
                throw new XMLException(e);
            }
            if (driver instanceof IPersistance)
            {
                // xml.putProperty("className", driver.getClass().getName());
                IPersistance persist = (IPersistance) driver;
                xml.addChild(persist.getXMLEntity()); // Tercer child. Antes hemos metido la leyenda y el selection support
            }
		} 
		xml.putProperty("driverName", getSource().getDriver().getName());
        if (bHasJoin)
            xml.putProperty("hasJoin", "true");
        
        
		return xml;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity03(XMLEntity xml)
		throws XMLException {
		
		super.setXMLEntity(xml);
		legend = LegendFactory.createFromXML03(xml.getChild(0));

		try {
			// legend.setDataSource(getRecordset());
			setLegend(legend);
		} catch (FieldNotFoundException e) {
			throw new XMLException(e);
		} catch (DriverException e) {
			throw new XMLException(e);
		}	

		selectionSupport.setXMLEntity03(xml.getChild(1));
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity(XMLEntity xml)
		throws XMLException {
        super.setXMLEntity(xml);
		
		legend = LegendFactory.createFromXML(xml.getChild(0));
		selectionSupport.setXMLEntity(xml.getChild(1));

		String recordsetName = xml.getStringProperty("recordset-name");
		try {
			LayerFactory.getDataSourceFactory().changeDataSourceName(
                    getRecordset().getName(), recordsetName
					);
            // Hacemos que el recordset que hemos creado antes (los
            // recordset se crean antes que todo) sea el recordset
            // de esta capa. Y del que se crea automaticamente
            // en el setLegend del createLayer, nos olvidamos
            setRecordset(new SelectableDataSource(LayerFactory.getDataSourceFactory()
                        .createRandomDataSource(recordsetName, DataSourceFactory.AUTOMATIC_OPENING)));
		} catch (NoSuchTableException e1) {
			throw new XMLException(e1);
		} catch (DriverException e1) {
			throw new XMLException(e1);		
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e1) {
            throw new XMLException(e1);
        } catch (DriverLoadException e1) {
            throw new XMLException(e1);
        } catch (FieldNotFoundException e) {
            e.printStackTrace();
            throw new XMLException(e);
        }
        // Si tiene una unin, lo marcamos para que no se cree la leyenda hasta el final
        // de la lectura del proyecto
        if (xml.contains("hasJoin"))
        {
            setIsJoined(true);
            PostProcessSupport.addToPostProcess(this,"setLegend", legend, 1);
        }
        else
        {
            try {
                // legend.setDataSource(getRecordset());
                setLegend(legend);
            } catch (FieldNotFoundException e) {
                throw new XMLException(e);
            } catch (DriverException e) {
                throw new XMLException(e);
            }   
        }       
        
	}

	/**
	 * Aade un LegendListener a la lista de Listeners.
	 *
	 * @param listener LegendListener.
	 */
	public void addLegendListener(LegendListener listener) {
		layerChangeSupport.addLayerListener(listener);
	}

	/**
	 * Llamada al mtodo callLegendChanged de los listener.
	 *
	 * @param e Evento.
	 */
	private void callLegendChanged(LegendChangedEvent e) {
		layerChangeSupport.callLegendChanged(e);
	}

	/**
	 * Borra un LegendListener de la lista de Listeners
	 *
	 * @param listener LegendListener.
	 */
	public void removeLegendListener(LegendListener listener) {
		layerChangeSupport.removeLayerListener(listener);
	}

	/**
	 * Sobreimplementacin del mtodo toString para que las bases de datos
	 * identifiquen la capa.
	 *
	 * @return DOCUMENT ME!
	 */
	public String toString() {
		/*
		 * Se usa internamente para que la parte de datos
		 * identifique de forma unvoca las tablas
		 */
		String ret = super.toString();

		return "layer" + ret.substring(ret.indexOf('@') + 1);
	}

    public boolean isJoined() {
        return bHasJoin;
    }

    public void setIsJoined(boolean hasJoin) {
        bHasJoin = hasJoin;
    }

    /**
     * @return Returns the spatialIndex.
     */
    public Quadtree getSpatialIndex() {
        return spatialIndex;
    }

	public SelectionSupport getSelectionSupport() {
		return selectionSupport;
	}

	public void setSelectionSupport(SelectionSupport selectionSupport) {
		this.selectionSupport = selectionSupport;
	}
}
