/*
 * Created on 19-sep-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 * 
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 * 
 *    or
 * 
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 * 
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.operations.Cancellable;
import com.iver.cit.gvsig.fmap.rendering.FGraphic;

public class GraphicLayer extends FLyrDefault {
    private ArrayList graphics = new ArrayList();
    private ArrayList symbols = new ArrayList();
    private Rectangle2D fullExtent;

    public GraphicLayer() {
        super();
        System.err.println("GraphicLayer: Constructor");
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#getFullExtent()
     */
    public Rectangle2D getFullExtent() throws DriverException {
        return fullExtent;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#draw(java.awt.image.BufferedImage, java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.operations.Cancellable, double)
     */
    public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort, Cancellable cancel, double scale) throws DriverException {
        if (isVisible() && isWithinScale(scale)){   
            drawGraphics(image, g, viewPort, cancel);
            }        
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#print(java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.operations.Cancellable, double)
     */
    public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel, double scale) throws DriverException {
        if (isVisible() && isWithinScale(scale)){   
            drawGraphics(null, g, viewPort, cancel);
            }        
    }
    
    private void drawGraphics(BufferedImage image, Graphics2D g,
            ViewPort viewPort, Cancellable cancel) 
    {
        int numReg;
        Rectangle2D elExtent = viewPort.getAdjustedExtent();
        if (elExtent == null) return;
    
        //int anchoMapa;
        //int altoMapa;
        //double anchoReal;
        //double altoReal;
        //double escala;
        FSymbol theSymbol = null;
        System.out.println("Dibujando grficos...(" + graphics.size() + ")");
    
        for (numReg = 0; numReg < graphics.size(); numReg++) {
            if (cancel.isCanceled()) {
                break;
            }
    
            FGraphic theGraphic = (FGraphic) graphics.get(numReg);
            IGeometry geom = theGraphic.getGeom();
            if (geom.fastIntersects(elExtent.getMinX(), 
                        elExtent.getMinY(), elExtent.getWidth(), elExtent.getHeight()))
             {
                theSymbol = (FSymbol) symbols.get(theGraphic.getIdSymbol());
                theGraphic.draw(g, viewPort, theSymbol);
            }
        }
    }    
    
    public int addSymbol(FSymbol newSymbol)
    {        
        if (symbols.add(newSymbol))
            return symbols.size()-1;
        return -1;
            
    }
    /**
     * Add a graphic to the graphic layer.
     * @param g
     * @return index of graphic created.
     */
    public int addGraphic(FGraphic g)
    {
        if (graphics.add(g))
        {
            if (fullExtent == null) {
                fullExtent = g.getGeom().getBounds2D();
            } else {
                fullExtent.add(g.getGeom().getBounds2D());
            }

            return graphics.size()-1;
        }
        return -1;

    }

    /**
     * 
     */
    public void clearAllGraphics()
    {
        System.err.println("Clear all graphics. Antes haba " + graphics.size());
        graphics.clear();
    }
    /**
     * 
     */
    public void clearSymbolsGraphics()
    {
        symbols.clear();
    }
    
}
