/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.NoSuchTableException;
import com.hardcode.gdbms.engine.data.driver.ObjectDriver;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DBLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;
import com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver;



/**
 * Adapta un driver de base de datos vectorial a la interfaz vectorial,
 * manteniendo adems el estado necesario por una capa vectorial de base de
 * datos (parmetros de la conexin)
 */
public class VectorialDBAdapter extends VectorialAdapter implements VectorialSecuencial {
    private int numReg=-1;
    private DataSource ds = null;
	/**
	 * incrementa el contador de las veces que se ha abierto el fichero.
	 * Solamente cuando el contador est a cero pide al driver que conecte con
	 * la base de datos
	 */
	public void start() {
            try {
                ((VectorialDatabaseDriver)driver).open();
            } catch (DriverException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
	}

	/**
	 * decrementa el contador de nmero de aperturas y cuando llega a cero pide
	 * al driver que cierre la conexion con el servidor de base de datos
	 */
	public void stop() {
	    ((VectorialDatabaseDriver)driver).close();
	}

	/**
	 * @return devuelve la Conexin a la base de datos, para que 
	 * el usuario pueda hacer la consulta que quiera, si lo desea.
	 * Por ejemplo, esto puede ser til para abrir un cuadro de dialogo
	 * avanazado y lanzar peticiones del tipo "Devuelveme un buffer
	 * a las autopistas", y con el resultset que te venga, escribir
	 * un shape, o cosas as.
	 */
	/* public Connection getConnection()
	{
	    return ((VectorialDatabaseDriver)driver).getConnection();
	}*/
	/* public IFeatureIterator getFeatureIterator(String sql) throws DriverException
	{
	    return ((VectorialDatabaseDriver)driver).getFeatureIterator(sql);
	}*/
	public IFeatureIterator getFeatureIterator(Rectangle2D r, String strEPSG) throws DriverException
	{
	    return ((VectorialDatabaseDriver)driver).getFeatureIterator(r, strEPSG);
	}
    
    public IFeatureIterator getFeatureIterator(Rectangle2D r, String strEPSG, String[] alphaNumericFieldsNeeded) throws DriverException
    {
        return ((VectorialDatabaseDriver)driver).getFeatureIterator(r, strEPSG, alphaNumericFieldsNeeded);
    }
	
	public String[] getFields()
	{
	    return ((VectorialDatabaseDriver)driver).getFields();	    
	}
	public String getWhereClause()
	{
	    return ((VectorialDatabaseDriver)driver).getWhereClause();
	}
	public String getTableName()
	{
	    return ((VectorialDatabaseDriver)driver).getTableName();
	}
	   
	
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShape(int)
	 */
	public IGeometry getShape(int index) throws DriverIOException {
	    IGeometry geom = null;
	    try {
            geom = ((VectorialDatabaseDriver)driver).getShape(index);
        } catch (IOException e) {
            throw new DriverIOException(e);
        }
	    return geom;
	}

	/**
	 * @throws DriverException
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShapeCount()
	 */
	public int getShapeCount() throws DriverIOException {
        try {
            return ((VectorialDatabaseDriver)driver).getShapeCount();
        } catch (IOException e) {
            e.printStackTrace();
            throw new DriverIOException(e.getMessage());
        }	    	
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getFullExtent()
	 */
	public Rectangle2D getFullExtent() {
	    return ((VectorialDatabaseDriver)driver).getFullExtent();
	}


	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShapeType()
	 */
	public int getShapeType() throws DriverIOException {
		return ((VectorialDatabaseDriver)driver).getShapeType();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.VectorialAdapter#getRecordset()
	 */
	public DataSource getRecordset() throws DriverLoadException {
	    if (driver instanceof ObjectDriver)
	    {
            if (ds == null)
            {
    			String name = LayerFactory.getDataSourceFactory().addDataSource((ObjectDriver)driver);
    			try {
                    ds = LayerFactory.getDataSourceFactory().createRandomDataSource(name, DataSourceFactory.AUTOMATIC_OPENING);
                } catch (NoSuchTableException e) {
                    throw new RuntimeException(e);
    			} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
    				throw new RuntimeException(e);
    			}
            }            
	    }
		return ds;
	}

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.VectorialAdapter#getFeature(int)
     */
    public IFeature getFeature(int numReg) throws DriverException {
        IGeometry geom;
        IFeature feat = null;
        try {
            geom = getShape(numReg);
            DataSource rs = getRecordset();
            int idFieldID = getLyrDef().getIdFieldID();
            Value[] regAtt = new Value[rs.getFieldCount()];
            String theID = null;
            for (int fieldId=0; fieldId < rs.getFieldCount(); fieldId++ )
            {       
                regAtt[fieldId] =  rs.getFieldValue(numReg, fieldId);
                if (fieldId == idFieldID)
                    theID = regAtt[fieldId].toString();
            }
            
            feat = new DefaultFeature(geom, regAtt, theID);
        } catch (DriverIOException e) {
            throw new DriverException(e);
        } catch (DriverLoadException e) {
            throw new DriverException(e);
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            throw new DriverException(e);
        }
        return feat;
    }

    public DBLayerDefinition getLyrDef()
    {
        return ((VectorialDatabaseDriver)driver).getLyrDef();
    }
}
