/*
 * Created on 08-mar-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 * 
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 * 
 *    or
 * 
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 * 
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.sql.SQLException;
import java.util.BitSet;

import org.cresques.cts.ICoordTrans;

import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;
import com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.VectorialDBAdapter;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.fmap.operations.Cancellable;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;

/**
 * @author FJP
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class DBStrategy extends DefaultStrategy {

	public DBStrategy(FLayer capa) {
		super(capa);
	}


    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#draw(java.awt.image.BufferedImage, java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.operations.Cancellable)
     */
    public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort, Cancellable cancel) throws DriverException {
        // Nos aprovechamos del SQL para lanzar la consulta
        // teniendo en cuenta el boundingbox que toca.
        FLyrVect lyr = (FLyrVect) getCapa();
        VectorialDBAdapter dbAdapter = (VectorialDBAdapter) ((SingleLayer) capa).getSource();
        VectorialDatabaseDriver dbDriver = (VectorialDatabaseDriver) dbAdapter.getDriver();
        dbAdapter.start();
		Selectable selection = (Selectable) lyr;
		ICoordTrans ct = lyr.getCoordTrans();
		FBitSet bitSet = selection.getSelection();
        
        String strEPSG = viewPort.getProjection().getAbrev().substring(5);
        // TODO: EXPLORAR LAS LEYENDAS PARA SABER LOS CAMPOS QUE VAMOS
        // A NECESITAR RECUPERAR, PARA INCLUIR EN LA CONSULTA SOLO
        // AQUELLOS QUE VAMOS A NECESITAR. CON ESO GANAREMOS VELOCIDAD.
        // Ejemplo:
        // En ArcSDE:
        // con Vias cogiendo un campo solo: 5 segundos
        // con todos los campos de Vias: 11 segundos.
        // => MODIFICAR EL getFeatureIterator para que admita los nombres
        // de los campos adems del rectngulo que pides.
        String[] usedFields = null;
        if (lyr.getLegend() instanceof VectorialLegend)
        {
            VectorialLegend legend = (VectorialLegend) lyr.getLegend();
            // TODO: Meter en el interaz Legend un mtodo String[] getUsedFields()
        }
        
        Rectangle2D rectAux = viewPort.getAdjustedExtent();
        if (ct != null) {
            ICoordTrans invertedCT = ct.getInverted();
            rectAux = invertedCT.convert(rectAux);            
        }

        VectorialLegend l = (VectorialLegend) ((ClassifiableVectorial) capa).getLegend();
        usedFields = l.getUsedFields();

        
        IFeatureIterator geomIt = dbAdapter.getFeatureIterator(rectAux, strEPSG, usedFields);
        if (geomIt == null)
        {
            // dbAdapter.stop();
            return;
        }
        
        
        DriverAttributes attr = dbAdapter.getDriverAttributes();
        boolean bMustClone = false;
        if (attr != null)
        {
            if (attr.isLoadedInMemory())
            {
                bMustClone = attr.isLoadedInMemory();               
            }
        }

        
        int i;
        FSymbol symbol;
        try {
            while (geomIt.hasNext())
            {
            	if (cancel.isCanceled()) {
            	    geomIt.closeIterator();
            		break;
            	}
            	IFeature feat = geomIt.next();
                IGeometry geom = feat.getGeometry();
                
                if (ct != null) {
                    if (bMustClone)
                        geom = geom.cloneGeometry();
                    geom.reProject(ct);
                }

                i = dbDriver.getRowIndexByFID(feat);
                // System.out.println("Antes de pintar " + i);
                // Value val = feat.getAttribute(0);                
                symbol = l.getSymbolByFeature(feat);
                // symbol = l.getSymbol(i);
                if (symbol == null) continue;
                if (bitSet.get(i)) {
                    symbol = FSymbol.getSymbolForSelection(symbol);
                } 
                
                // symbol = l.getDefaultSymbol();
                geom.draw(g,viewPort, symbol);
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new DriverException(e);
        }
        dbAdapter.stop();
        
    }
    

}
