/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.BitSet;

import org.apache.log4j.Logger;
import org.cresques.cts.ICoordTrans;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.fmap.operations.Cancellable;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.cit.gvsig.fmap.rendering.styling.FStyle2D;


/**
 * Implementa la Strategy por defecto. Los mtodos que tendrn en comn la
 * mayor parte de las estrategias
 */
public class DefaultStrategy implements Strategy {
    public static final int EQUALS = 0;
    public static final int DISJOINT = 1;
    public static final int INTERSECTS = 2;
    public static final int TOUCHES = 3;
    public static final int CROSSES = 4;
    public static final int WITHIN = 5;
    public static final int CONTAINS = 6;
    public static final int OVERLAPS = 7;
    
	private static Logger logger = Logger.getLogger(DefaultStrategy.class.getName());
	FLayer capa = null;

	/**
	 * Crea un nuevo DefaultStrategy.
	 *
	 * @param capa DOCUMENT ME!
	 */
	public DefaultStrategy(FLayer capa) {
		this.capa = capa;

		SingleLayer foo = (SingleLayer) capa;
		ClassifiableVectorial vectorial = (ClassifiableVectorial) capa;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#queryByRect(java.awt.geom.Rectangle2D)
	 */
	public FBitSet queryByRect(Rectangle2D rect) throws DriverException {
		QueryByRectVisitor visitor = new QueryByRectVisitor();

		visitor.setRect(rect);

		try {
			process(visitor);
		} catch (VisitException e) {
			throw new RuntimeException(
				"QueryByRectVisitor lanza una VisitException?");
		}

		return visitor.getBitSet();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#queryByShape(com.iver.cit.gvsig.fmap.fshape.IGeometry,
	 * 		int)
	 */
	public FBitSet queryByShape(IGeometry g, int relationship)
		throws DriverException, VisitException {
		QueryByShapeVisitor visitor = new QueryByShapeVisitor();
		visitor.setRelationShip(relationship);
		visitor.setShape(g);
		process(visitor);

		return visitor.getBitSet();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#getSelectionBounds()
	 */
	public Rectangle2D getSelectionBounds() {
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#createIndex()
	 */
	public void createIndex() {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.LayerOperations#draw(java.awt.image.BufferedImage,
	 * 		java.awt.Graphics2D, FStyle2D)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
		Cancellable cancel) throws DriverException {
		try {
			VectorialAdapter adapter = ((SingleLayer) capa).getSource();
			ICoordTrans ct = getCapa().getCoordTrans();
			logger.debug("adapter.start()");
			adapter.start();

			logger.debug("getCapa().getRecordset().start()");
			SelectableDataSource rsSel = ((AlphanumericData) getCapa()).getRecordset(); 
			if (rsSel != null)
			    rsSel.start();

			int sc;
			Rectangle2D extent = viewPort.getAdjustedExtent();
			AffineTransform at = viewPort.getAffineTransform();

			sc = adapter.getShapeCount();
			// TODO: A revisar si es o no conveniente este sistema
			// de comunicacin con los drivers.
			DriverAttributes attr = adapter.getDriverAttributes();
			boolean bMustClone = false;
			if (attr != null)
			{
			    if (attr.isLoadedInMemory())
			    {
			        bMustClone = attr.isLoadedInMemory();			    
			    }
			}
			VectorialLegend l = (VectorialLegend) ((ClassifiableVectorial) capa).getLegend();
            FBitSet bitSet = null;
            if (getCapa() instanceof Selectable){
                Selectable selection = (Selectable) getCapa();
                bitSet = selection.getSelection();
            }            
			for (int i = 0; i < sc; i++) {
				if (cancel.isCanceled()) {
					break;
				}

				
				IGeometry geom = adapter.getShape(i);

				if (geom == null) {
					continue;
				}

				if (ct != null) {
				    if (bMustClone)
				        geom = geom.cloneGeometry();				    
					geom.reProject(ct);
				}
				
				// if (geom.intersects(extent)) {
				if (geom.fastIntersects(extent.getMinX(), extent.getMinY(), 
				         extent.getWidth(), extent.getHeight())) {
					FSymbol symbol = l.getSymbol(i);
					
                    if (symbol ==null)
                        continue;
                    if (bitSet != null)
                        if (bitSet.get(i)) {
    						symbol = FSymbol.getSymbolForSelection(symbol);
    					}
					geom.draw(g, viewPort, symbol);
				}
				/* else
				{
				    System.out.println("no pinto id=" + i);
				} */
			}

			logger.debug("getCapa().getRecordset().stop()");
			if (rsSel != null)
			    rsSel.stop();


			logger.debug("adapter.stop()");
			adapter.stop();
			// TODO: A revisar si es o no conveniente este sistema
			// de comunicacin con los drivers.
			// DriverAttributes attr = adapter.getDriverAttributes();
			/* if (attr != null)
			{
			    if (attr.isLoadedInMemory())
			    {
			        // Quitamos lo de la reproyeccin al vuelo para que 
			        // solo se haga una vez.
			        getCapa().setCoordTrans(null);
			    }
			} */

		} catch (DriverIOException e) {
			throw new DriverException(e);
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
		    throw new DriverException(e);
		} catch (DriverException e) {
		    throw new DriverException(e);
        }
	}

	/**
	 * Devuelve la capa.
	 *
	 * @return Returns the capa.
	 */
	public FLayer getCapa() {
		return capa;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor,
	 * 		java.util.BitSet)
	 */
	public void process(FeatureVisitor visitor, BitSet subset)
		throws DriverException, VisitException {
		try {
			logger.debug("visitor.start()");

			if (visitor.start(capa)) {
				VectorialAdapter va = ((SingleLayer) capa).getSource();

				va.start();
				for (int i = 0; i < va.getShapeCount(); i++) {
					if (subset.get(i)) {
						visitor.visit(va.getShape(i), i);
					}
				}
				va.stop();

				logger.debug("visitor.stop()");
				visitor.stop(capa);
			}
		} catch (DriverIOException e) {
			throw new DriverException(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param visitor DOCUMENT ME!
	 *
	 * @throws DriverException DOCUMENT ME!
	 * @throws VisitException
	 *
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor)
	 */
	public void process(FeatureVisitor visitor)
		throws DriverException, VisitException {
		try {
			logger.debug("visitor.start()");

			if (visitor.start(capa)) {
				VectorialAdapter va = ((SingleLayer) capa).getSource();
				ICoordTrans ct = getCapa().getCoordTrans();
				va.start();
				for (int i = 0; i < va.getShapeCount(); i++) {
				    IGeometry geom = va.getShape(i);
				    if (ct != null) {
						geom.reProject(ct);
					}

					visitor.visit(geom, i);
				}
				va.stop();
				logger.debug("visitor.stop()");
				visitor.stop(capa);
			}
		} catch (DriverIOException e) {
			throw new DriverException(e);
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#queryByPoint(Point2D,
	 * 		double)
	 */
	public FBitSet queryByPoint(Point2D p, double tolerance)
		throws DriverException {
        // TODO: OJO!!!!. Est implementado como un rectangulo.
        // Lo correcto debera ser calculando las distancias reales
        // es decir, con un crculo.
        Rectangle2D recPoint = new Rectangle2D.Double(p.getX() - (tolerance / 2),
                p.getY() - (tolerance / 2), tolerance, tolerance);
		return queryByRect(recPoint);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#print(java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort)
	 */
	public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel)
		throws DriverException {
		draw(null, g, viewPort, cancel); // Quiero ejecutar el draw del padre, que es el que va sin acelaracin!!
	}
}
