/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import com.hardcode.gdbms.engine.data.DataSource;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.fmap.operations.Cancellable;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.cit.gvsig.fmap.rendering.styling.FStyle2D;

import org.apache.log4j.Logger;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;


/**
 * Extiende DefaultStrategy de forma optimizada.
 *
 * @author Vicente Caballero Navarro
 */
public class OptimiceStrategy extends DefaultStrategy {
	private static Logger logger = Logger.getLogger(OptimiceStrategy.class.getName());

	/**
	 * Crea una nueva OptimiceStrategy.
	 *
	 * @param capa
	 */
	public OptimiceStrategy(FLayer capa) {
		super(capa);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.LayerOperations#draw(java.awt.image.BufferedImage,
	 * 		java.awt.Graphics2D, FStyle2D)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
		Cancellable cancel) throws DriverException {
		try {
			VectorialAdapter adapter = ((SingleLayer) getCapa()).getSource();
			DataSource ds = ((AlphanumericData) getCapa()).getRecordset();
			// ShapeInfo shapeinfo = adapter.getShapeInfo();
			logger.debug("adapter.start()");
			adapter.start();
			logger.debug("ds.start()");
			ds.start();

			VectorialFileDriver driver = (VectorialFileDriver) adapter.getDriver();
			int sc;
			long t1 = System.currentTimeMillis();
			Rectangle2D extent = viewPort.getAdjustedExtent();
			AffineTransform at = viewPort.getAffineTransform();

			sc = adapter.getShapeCount();

			for (int i = 0; i < sc; i++) {
				if (cancel.isCanceled()) {
					break;
				}

				IGeometry geom = adapter.getShape(i);
				VectorialLegend l = (VectorialLegend) ((ClassifiableVectorial) getCapa()).getLegend();

				// TODO Falta rematar esta parte.
				// if (shapeinfo.getBoundingBox(sc).intersects(extent)) {
					FSymbol symbol = l.getSymbol(i);
					geom.draw(g, viewPort, symbol);
				// }
			}

			long t2 = System.currentTimeMillis();
			logger.debug("adapter.stop()");
			adapter.stop();
			logger.debug("ds.stop()");
			ds.stop();

			System.out.println(t2 - t1);
		} catch (DriverIOException e) {
			throw new DriverException(e);
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			throw new DriverException(e);
		}
	}
}
