/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;

import org.cresques.cts.ICoordTrans;

import java.awt.geom.Rectangle2D;


/**
 * Query by Rectangle Visitor.
 *
 * @author Vicente Caballero Navarro
 */
public class QueryByRectVisitor implements FeatureVisitor {
	private Rectangle2D rect = null;
	private FBitSet bitset = null;
	private ICoordTrans ct = null;

	/**
	 * Inserta el Rectngulo para realizar la consulta.
	 *
	 * @param r Rectngulo para la consulta.
	 */
	public void setRect(Rectangle2D r) {
		rect = r;
	}

	/**
	 * Devuelve un FBitSet como respuesta a la consulta.
	 *
	 * @return FBitSet con los ndices de los registros de la consulta.
	 */
	public FBitSet getBitSet() {
		return bitset;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor#visit(com.iver.cit.gvsig.fmap.core.IGeometry,
	 * 		int)
	 */
	public void visit(IGeometry g, int index) {
		if (g==null)return;
		// Comentamos esto porque la geometry ya viene reproyectada
		/* if (ct != null) {
			g.reProject(ct);
		} */

		if (g.intersects(rect)) {
			bitset.set(index, true);
		} else {
			bitset.set(index, false);
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor#stop(com.iver.cit.gvsig.fmap.layers.FLayer)
	 */
	public void stop(FLayer layer) {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor#start(com.iver.cit.gvsig.fmap.layers.FLayer)
	 */
	public boolean start(FLayer layer) {
		bitset = new FBitSet();
		// ct = layer.getCoordTrans();

		return true;
	}
}
