/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import java.util.BitSet;

import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.vividsolutions.jts.geom.Geometry;


/**
 * Query by Shape.
 *
 * @author Vicente Caballero Navarro
 */
public class QueryByShapeVisitor implements FeatureVisitor {
	private FBitSet bitset = null;
	private IGeometry shape = null;
	private Geometry jtsShape = null;
	private int relationship;

	/**
	 * Inserta la relacin.
	 *
	 * @param relationship Relacin.
	 */
	public void setRelationShip(int relationship) {
		this.relationship = relationship;
	}

	/**
	 * Inserta la geometra.
	 *
	 * @param shape Geometra.
	 */
	public void setShape(IGeometry shape) {
		this.shape = shape;
		this.jtsShape = shape.toJTSGeometry();
	}

	/**
	 * Devuelve un BitSet con los ndices de los registros, como respuesta a la
	 * consulta.
	 *
	 * @return BitSet con los ndices de los registros de la consulta.
	 */
	public FBitSet getBitSet() {
		return bitset;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor#visit(com.iver.cit.gvsig.fmap.core.IGeometry,
	 * 		int)
	 */
	public void visit(IGeometry g, int index) {
		if (g==null)return;
		Geometry jtsGeom = g.toJTSGeometry();
		if (jtsShape.intersects(jtsGeom)) {
			bitset.set(index, true);
		} else {
			bitset.set(index, false);
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor#stop()
	 */
	public void stop(FLayer lyr) {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor#start()
	 */
	public boolean start(FLayer lyr) {
		bitset = new FBitSet();

		return true;
	}
}
