/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.BitSet;
import java.util.List;

import org.apache.log4j.Logger;
import org.cresques.cts.ICoordTrans;
import org.geotools.resources.geometry.XRectangle2D;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.BoundedShapes;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.fmap.operations.Cancellable;
import com.iver.cit.gvsig.fmap.rendering.ClassifiedLegendInfo;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.cit.gvsig.fmap.rendering.styling.FStyle2D;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.IntersectionMatrix;


/**
 * Esta clase definir las operaciones de la interfaz FLyrVect de la manera ms
 * ptima para los ficheros shp.
 */
public class ShpStrategy extends DefaultStrategy {
	private static Logger logger = Logger.getLogger(ShpStrategy.class.getName());

	/**
	 * Crea una ShpStrategy.
	 *
	 * @param capa
	 */
	public ShpStrategy(FLayer capa) {
		super(capa);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.operations.LayerOperations#draw(java.awt.image.BufferedImage,
	 * 		java.awt.Graphics2D, FStyle2D)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
		Cancellable cancel) throws DriverException {
		try {
			VectorialAdapter adapter = ((SingleLayer) getCapa()).getSource();
			if (adapter.getShapeCount() <= 0)
			{
			    logger.debug("Layer:" + getCapa().getName() + " sin registros");
			    return;
			}
			Selectable selection = (Selectable) getCapa();
			ICoordTrans ct = getCapa().getCoordTrans();
			BitSet bitSet = selection.getSelection();
			BoundedShapes shapeBounds = (BoundedShapes) adapter.getDriver();
			VectorialFileDriver driver = (VectorialFileDriver) adapter.getDriver();
			// logger.debug("adapter.start() -> Layer:" + getCapa().getName());
			adapter.start();
			IGeometry geom;
			if (adapter.getShapeCount()>0){
			geom = adapter.getShape(0);
			}
			VectorialLegend l = (VectorialLegend) ((ClassifiableVectorial) getCapa()).getLegend();

			if (l instanceof ClassifiedLegendInfo) {
				ClassifiedLegendInfo clsfLegend = (ClassifiedLegendInfo) l;
				FSymbol[] symbs = clsfLegend.getSymbols();
				//double rSym = 0;
				//double maxRSym = -1;

				for (int i = 0; i < symbs.length; i++) {
					// TODO: REVISAR LOS SIMBOLOS Y SUS TAMAOS

					/* Style2D pointSymbol = symbs[i].getPointStyle2D();
					   if (pointSymbol instanceof MarkStyle2D)
					   {
					           MarkStyle2D mrk2D = (MarkStyle2D) pointSymbol;
					           rSym = viewPort.toMapDistance(mrk2D.getSize());
					           if (maxRSym < rSym)
					                   maxRSym = rSym;
					   }*/
				}
			}

			Rectangle2D extent = viewPort.getAdjustedExtent();
			//AffineTransform at = viewPort.getAffineTransform();

			int sc;

			Rectangle2D bounds;

			sc = adapter.getShapeCount();

			long t1 = System.currentTimeMillis();
			// logger.debug("getCapa().getRecordset().start()");
			((AlphanumericData) getCapa()).getRecordset().start();
			
			// TODO: A revisar si es o no conveniente este sistema
			// de comunicacin con los drivers.
			DriverAttributes attr = adapter.getDriverAttributes();
			boolean bMustClone = false;
			if (attr != null)
			{
			    if (attr.isLoadedInMemory())
			    {
			        bMustClone = attr.isLoadedInMemory();			    
			    }
			}
			

			for (int i = 0; i < sc; i++) {
				// Salimos si alguien cancela
				if (cancel.isCanceled()) {
					break;
				}

				bounds = shapeBounds.getShapeBounds(i);

				if (ct != null) {
					bounds = ct.convert(bounds);
				}

				if (XRectangle2D.intersectInclusive(extent, bounds)) {
					FSymbol symbol = l.getSymbol(i);

					if (bitSet.get(i)) {
						symbol = FSymbol.getSymbolForSelection(symbol);
					} 

					boolean bPoint = (shapeBounds.getShapeType(i) == FShape.POINT);

					if (bPoint ||
							((bounds.getHeight() > viewPort.getDist1pixel()) ||
							(bounds.getWidth() > viewPort.getDist1pixel()))) {
						geom = driver.getShape(i);
                        
                        // PRUEBA DE VELOCIDAD
                        // geom = ShapeFactory.createPolygon2D(new GeneralPathX(bounds));

						if (ct != null) {
						    if (bMustClone)
						        geom = geom.cloneGeometry();
							geom.reProject(ct);
						}
                        // FJP: CAMBIO: Sabemos que vamos a dibujar sobre una 
                        // imagen, con coordenadas enteras, as
                        // que lo tenemos en cuenta.
						// ANTES: geom.draw(g, viewPort, symbol);
                        // AHORA:
                        geom.drawInts(g, viewPort, symbol);
                        // geom.draw(g, viewPort, symbol);
                        

					} else {
						Point2D.Double pOrig = new Point2D.Double(bounds.getMinX(),
								bounds.getMinY());
						Point2D pDest, pDest2;

						pDest = viewPort.getAffineTransform().transform(pOrig, null);
                        pDest2 = g.getTransform().transform(pDest, null);

						int pixX = (int) pDest2.getX();
						int pixY = (int) pDest2.getY();
						if (symbol ==null)
							continue;
						if ((pixX > 0) && (pixX < image.getWidth())) {
							if ((pixY > 0) && (pixY < image.getHeight())) {
								image.setRGB(pixX, pixY, symbol.getRgb());
							}
						}
					}
				}
			}

			// logger.debug("getCapa().getRecordset().stop()");
			((AlphanumericData) getCapa()).getRecordset().stop();

			long t2 = System.currentTimeMillis();
			// logger.debug("adapter.stop()");
			adapter.stop();

			// System.out.println(t2 - t1);
		} catch (DriverIOException e) {
			throw new DriverException(e);
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			throw new DriverException(e);
		} catch (DriverException e) {
			throw new DriverException(e);
		} catch (IOException e) {
			throw new DriverException(e);
		}
	}

	/**
	 * Mtodo utilizado para dibujar sobre el graphics que se pasa como
	 * parmetro, pensado para utilizarse para imprimir.
	 *
	 * @param g Graphics2D
	 * @param viewPort ViewPort.
	 * @param cancel
	 *
	 * @throws DriverException
	 */
	public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel)
		throws DriverException {
		// super.draw(null, g, viewPort, cancel); // Quiero ejecutar el draw del padre, que es el que va sin acelaracin!!
        try {
            VectorialAdapter adapter = ((SingleLayer) getCapa()).getSource();
            if (adapter.getShapeCount() <= 0)
            {
                logger.debug("Layer:" + getCapa().getName() + " sin registros");
                return;
            }
            Selectable selection = (Selectable) getCapa();
            ICoordTrans ct = getCapa().getCoordTrans();
            BitSet bitSet = selection.getSelection();
            BoundedShapes shapeBounds = (BoundedShapes) adapter.getDriver();
            VectorialFileDriver driver = (VectorialFileDriver) adapter.getDriver();
            // logger.debug("adapter.start() -> Layer:" + getCapa().getName());
            adapter.start();
            IGeometry geom;
            if (adapter.getShapeCount()>0){
            geom = adapter.getShape(0);
            }
            VectorialLegend l = (VectorialLegend) ((ClassifiableVectorial) getCapa()).getLegend();

            Rectangle2D extent = viewPort.getAdjustedExtent();
            //AffineTransform at = viewPort.getAffineTransform();

            int sc;

            Rectangle2D bounds;

            sc = adapter.getShapeCount();

            long t1 = System.currentTimeMillis();
            // logger.debug("getCapa().getRecordset().start()");
            ((AlphanumericData) getCapa()).getRecordset().start();
            
            // TODO: A revisar si es o no conveniente este sistema
            // de comunicacin con los drivers.
            DriverAttributes attr = adapter.getDriverAttributes();
            boolean bMustClone = false;
            if (attr != null)
            {
                if (attr.isLoadedInMemory())
                {
                    bMustClone = attr.isLoadedInMemory();               
                }
            }
            

            for (int i = 0; i < sc; i++) {

                bounds = shapeBounds.getShapeBounds(i);

                if (ct != null) {
                    bounds = ct.convert(bounds);
                }

                if (XRectangle2D.intersectInclusive(extent, bounds)) {
                    FSymbol symbol = l.getSymbol(i);

                    if (bitSet.get(i)) {
                        symbol = FSymbol.getSymbolForSelection(symbol);
                    } 

                    boolean bPoint = (shapeBounds.getShapeType(i) == FShape.POINT);

                    geom = driver.getShape(i);
                    
                    // PRUEBA DE VELOCIDAD
                    // geom = ShapeFactory.createPolygon2D(new GeneralPathX(bounds));

                    if (ct != null) {
                        if (bMustClone)
                            geom = geom.cloneGeometry();
                        geom.reProject(ct);
                    }
                    geom.draw(g, viewPort, symbol);
                }
            }

            // logger.debug("getCapa().getRecordset().stop()");
            ((AlphanumericData) getCapa()).getRecordset().stop();

            long t2 = System.currentTimeMillis();
            // logger.debug("adapter.stop()");
            adapter.stop();

            // System.out.println(t2 - t1);
        } catch (DriverIOException e) {
            throw new DriverException(e);
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            throw new DriverException(e);
        } catch (DriverException e) {
            throw new DriverException(e);
        } catch (IOException e) {
            throw new DriverException(e);
        }
        
	}
    
    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#queryByShape(com.iver.cit.gvsig.fmap.core.IGeometry, int)
     */
    public FBitSet queryByShape(IGeometry g, int relationship)
    throws DriverException, VisitException {
        // Si hay un ndice espacial, lo usamos para hacer el query.
        FLyrVect lyr = (FLyrVect) capa;
        if (lyr.getSpatialIndex() == null)
            return super.queryByShape(g, relationship);
        
        long t1 = System.currentTimeMillis();
        VectorialAdapter va = lyr.getSource();
        ICoordTrans ct = lyr.getCoordTrans();
        Rectangle2D bounds = g.getBounds2D();
        Coordinate c1 = new Coordinate(bounds.getMinX(), bounds.getMinY());
        Coordinate c2 = new Coordinate(bounds.getMaxX(), bounds.getMaxY());
        Envelope env = new Envelope(c1, c2);
        
        List lstRecs = lyr.getSpatialIndex().query(env);
        Integer idRec;
        FBitSet bitset = new FBitSet();
        Geometry jtsShape = g.toJTSGeometry();
        IntersectionMatrix m;
        int index;
        try {
            va.start();

            for (int i=0; i < lstRecs.size(); i++)
            {
                idRec = (Integer) lstRecs.get(i);
                index = idRec.intValue();
                IGeometry geom = va.getShape(index);
                if (ct != null) {
                    geom.reProject(ct);
                }                
                Geometry jtsGeom = geom.toJTSGeometry();
                switch (relationship) {
                case CONTAINS:
                    m = jtsShape.relate(jtsGeom);
                    if (m.isContains()) {
                        bitset.set(index, true);
                    }
                    break;

                case CROSSES:
                    m = jtsShape.relate(jtsGeom);
                    if (m.isCrosses(jtsGeom.getDimension(), jtsShape.getDimension())) {
                        bitset.set(index, true);
                    }
                    break;

                case DISJOINT:
                    // TODO: CREO QUE EL DISJOINT NO SE PUEDE METER AQUI
                    m = jtsShape.relate(jtsGeom);            
                    if (m.isDisjoint()) {
                        bitset.set(index, true);
                    }
                    break;

                case EQUALS:
                    m = jtsShape.relate(jtsGeom); 
                    if (m.isEquals(jtsGeom.getDimension(), jtsShape.getDimension())) {
                        bitset.set(index, true);
                    }
                    break;

                case INTERSECTS:
                    m = jtsShape.relate(jtsGeom);
                    if (m.isIntersects()) {
                        bitset.set(index, true);
                    }
                    break;

                case OVERLAPS:
                    m = jtsShape.relate(jtsGeom);
                    if (m.isOverlaps(jtsGeom.getDimension(), jtsShape.getDimension()))
                    {
                        bitset.set(index, true);
                    }

                    break;

                case TOUCHES:
                    m = jtsShape.relate(jtsGeom);
                    if (m.isTouches(jtsGeom.getDimension(), jtsShape.getDimension()))
                    {
                        bitset.set(index, true);
                    }

                    break;

                case WITHIN:
                    m = jtsShape.relate(jtsGeom);
                    if (m.isWithin()) {
                        bitset.set(index, true);
                    }

                    break;
                }
            }
            va.stop();
        } catch (DriverIOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        long t2 = System.currentTimeMillis();
        logger.debug("queryByShape optimizado sobre la capa " + lyr.getName() + ". " + (t2-t1) + " mseg.");
        return bitset;
    }
    public FBitSet queryByRect(Rectangle2D rect) throws DriverException {
        // Si hay un ndice espacial, lo usamos para hacer el query.
        FLyrVect lyr = (FLyrVect) capa;
        if (lyr.getSpatialIndex() == null)
            return super.queryByRect(rect);

        VectorialAdapter va = lyr.getSource();
        ICoordTrans ct = lyr.getCoordTrans();
        Rectangle2D bounds = rect;
        Coordinate c1 = new Coordinate(bounds.getMinX(), bounds.getMinY());
        Coordinate c2 = new Coordinate(bounds.getMaxX(), bounds.getMaxY());
        Envelope env = new Envelope(c1, c2);
        
        List lstRecs = lyr.getSpatialIndex().query(env);
        Integer idRec;
        FBitSet bitset = new FBitSet();
        int index;
        try {
            va.start();

            for (int i=0; i < lstRecs.size(); i++)
            {
                idRec = (Integer) lstRecs.get(i);
                index = idRec.intValue();
                IGeometry geom = va.getShape(index);
                if (ct != null) {
                    geom.reProject(ct);
                }
                if (geom.intersects(rect))
                    bitset.set(index, true);
            }
            va.stop();
        } catch (DriverIOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return bitset;

    }
    
    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#queryByPoint(java.awt.geom.Point2D, double)
     */
    public FBitSet queryByPoint(Point2D p, double tolerance)
    throws DriverException {
        // TODO: OJO!!!!. Est implementado como un rectangulo.
        // Lo correcto debera ser calculando las distancias reales
        // es decir, con un crculo.
        Rectangle2D recPoint = new Rectangle2D.Double(p.getX() - (tolerance / 2),
                p.getY() - (tolerance / 2), tolerance, tolerance);
        return queryByRect(recPoint);
    }
}
