/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import com.iver.cit.gvsig.fmap.drivers.BoundedShapes;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;
import com.iver.cit.gvsig.fmap.layers.VectorialDBAdapter;
//import com.iver.cit.gvsig.fmap.layers.WFSAdapter;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;


/**
 * Clase con mtodos estticos para crear la estategia.
 *
 * @author Vicente Caballero Navarro
 */
public class StrategyManager {
	/**
	 * Se encarga de, dada una clase que implementa el interfaz vectorial,
	 * seleccionar la estrategia ptima de acceso a las capas, devolviendo el
	 * objeto Strategy con la capa vectorial asociada.
	 *
	 * @param v SingleLayer.
	 *
	 * @return Estrategia.
	 */
	public static Strategy getStrategy(SingleLayer v) {
        if (v.getStrategy() != null)
        {
            return v.getStrategy();
        }
		if (v.getSource().getDriver() instanceof BoundedShapes) {
			return new ShpStrategy((FLayer) v);
		} else {
		    if (v.getSource() instanceof VectorialDBAdapter)
		        return new DBStrategy((FLayer) v);
		    /* else if (v.getSource() instanceof WFSAdapter)
		        return new WFSStrategy((FLayer) v); */
			else  
		        return new DefaultStrategy((FLayer) v);
		}
	}

	/**
	 * Crea un ShapeInfo en memoria o en disco en funcin de la memoria
	 * disponible
	 *
	 * @param adapter VectorialAdapter.
	 *
	 * @return ShapeInfo.
	 */
	public static ShapeInfo createShapeInfo(VectorialAdapter adapter) {
		//TODO falta que implementar el DiskShapeInfo.
		return new MemoryShapeInfo();
	}
}
