/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering.indexes;

import com.iver.cit.gvsig.fmap.drivers.DriverIOException;

import java.awt.geom.Rectangle2D;


/**
 * Interfaz que define el comportamiento de una estructura de datos de ndice
 */
public interface Index {
	/**
	 * Dado un rectngulo en coordenadas del mapa se devuelve un array de
	 * indices de los registros cuyas features estn contenidas o intersecten
	 * en dicho rectngulo. En caso de que ninguna est contenida se devuelve
	 * un array vaco. Si se invoca esta funcin pero no hay ningn ndice
	 * definido se lanzar una IndexNotExistsException
	 *
	 * @param rect Rectngulo.
	 *
	 * @return ndices.
	 *
	 * @throws DriverIOException
	 * @throws IndexNotExistsException
	 */
	int[] getRecordIndexes(Rectangle2D rect)
		throws DriverIOException, IndexNotExistsException;

	/**
	 * Abre el fichero de ndices
	 *
	 * @param file Fichero.
	 */
	void openIndexFile(String file);

	/**
	 * Cierra el fichero de ndices
	 */
	void closeIndexFile();
}
