/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools;

import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.geom.Point2D;

import javax.swing.ImageIcon;

import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.tools.Events.MeasureEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.PolylineListener;


/**
 * Implementacin de la interfaz MeasureListener como herramienta para medir el
 * rea.
 *
 * @author Vicente Caballero Navarro
 */
public class AreaListenerImpl implements PolylineListener {
	private final Image iarea = new ImageIcon(MapControl.class.getResource(
				"images/AreaCursor.gif")).getImage();
	private Cursor cur = Toolkit.getDefaultToolkit().createCustomCursor(iarea,
			new Point(16, 16), "");
	protected MapControl mapCtrl;
	protected MeasureEvent event;

	/**
	 * Crea un nuevo AreaListenerImpl.
	 *
	 * @param mc MapControl.
	 */
	public AreaListenerImpl(MapControl mc) {
		this.mapCtrl = mc;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.PolylineListener#points(com.iver.cit.gvsig.fmap.tools.Events.MeasureEvent)
	 */
	public void points(MeasureEvent event) {
		this.event = event;

		double dist = 0;
		double distAll = 0;

		ViewPort vp = mapCtrl.getMapContext().getViewPort();

		for (int i = 0; i < (event.getXs().length - 1); i++) {
			dist = 0;

			Point p = new Point(event.getXs()[i].intValue(),
					event.getXs()[i].intValue());
			Point p2 = new Point(event.getXs()[i + 1].intValue(),
					event.getXs()[i + 1].intValue());

			///dist = vp.toMapDistance((int) p.distance(p2));
			dist = vp.distanceWorld(p, p2);
			distAll += dist;
		}

		System.out.println("Per�metro = " + distAll + " �rea = " +
			(returnArea(vp.toMapPoint(
					new Point2D.Double(
						event.getXs()[event.getXs().length - 2].doubleValue(),
						event.getYs()[event.getYs().length - 2].doubleValue())))));
	}

	/**
	 * <p>
	 * Computes the area using the trapezoid method. This is, we decompose a polygon in a set of
	 * trapezoids defined each two points of the polygon starting and ending in the first user's 
	 * entered point. For each trapezoid by these each two points and the points over the y-axis (when y equals 0)
	 * that have the same x value than the points defined by the user and then we obtain
	 * the area of this trapezoid with this formula.
	 * </p>
	 * <p>
	 * The four points that define the trapezoid we are considering are
	 * p1 = (x1, y1) -> the first user point
	 * p2 = (x2, y2) -> the second user point
	 * p3 = (x3, y3) -> the image over the y-axis of p1
	 * p4 = (x4, y4) -> the image over the y-axis
	 * </p>
	 * <pre>
	 * 		  (y1 - y2) * (x2 - x1)
	 *  A = | ----------------------------| + |((x2 - x1) * (y2 - y1)| (of course, simplify this)
	 *                 2 
	 * <pre>
	 * 
	 * Calculated the area of every trapezoid generated at the descomposition, only rests
	 * to acumulate it to the total amout when (x2 - x1)>0 and substract it when (x2 - x1)< 0
	 * 
	 * @param aux, the last point.
	 *
	 * @return area.
	 */
	protected double returnArea(Point2D aux) {

		double elArea;
		int pos;
		Point2D p = new Point2D.Double();
		Point2D.Double pAnt = new Point2D.Double();
		Double[] x = event.getXs();
		Double[] y = event.getYs();

		System.out.println("[equis]");
		for (int j = 0; j < x.length; j++) {
			System.out.print(x[j].doubleValue()+", ");
		}
		// double baseMayor, baseMenor, altura; // De cada trapezoide
		// El area de cada pareja de puntos sera (x2-x1) * baseMenor + (baseMayor-baseMenor)*(x2-x1)
		// Simplificando la ecuacin queda: (x2-x1)*((baseMayor+baseMenor)/2). Es decir, que da igual
		// quien es la base mayor y quien la manor.
		// si la coordenada X del punto i+1 es mayor que la del punto i => Area negativa
		// si no => Area positiva
		if (x.length < 2) {
			return 0.0;
		}

		elArea = 0.0;

		for (pos = 0; pos < x.length; pos++) {
			p = new Point2D.Double(x[pos].doubleValue(), y[pos].doubleValue());
			
			if (pos == 0) {
				pAnt.x = aux.getX();
				pAnt.y = aux.getY();
			}

			elArea = elArea + ((pAnt.x - p.getX()) * (pAnt.y + p.getY()));
			pAnt.setLocation(p);
		}

		elArea = elArea + ((pAnt.x - aux.getX()) * (pAnt.y + aux.getY()));
		elArea = Math.abs(elArea / 2.0);

		return elArea;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#getCursor()
	 */
	public Cursor getCursor() {
		return cur;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.PolylineListener#pointFixed(com.iver.cit.gvsig.fmap.tools.Events.MeasureEvent)
	 */
	public void pointFixed(MeasureEvent event) {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.PolylineListener#polylineFinished(com.iver.cit.gvsig.fmap.tools.Events.MeasureEvent)
	 */
	public void polylineFinished(MeasureEvent event) {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#cancelDrawing()
	 */
	public boolean cancelDrawing() {
		return false;
	}
}
