/*
 * Created on 31-ene-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.utiles.console;

import java.awt.BorderLayout;
import java.awt.event.KeyEvent;

import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;


/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class JConsole extends JPanel {
	private JTextArea txt = null;
	private int startingCaretPosition = 0;
	private ArrayList entries = new ArrayList();
	private int currentEntry = -1;
	private JScrollPane jScrollPane = null;
	private ResponseListenerSupport listenerSupport = new ResponseListenerSupport();

	/**
	 * This is the default constructor
	 */
	public JConsole() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 */
	private void initialize() {
		this.setLayout(new BorderLayout());
		this.setSize(300, 200);
		this.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
	}

	/**
	 * Obtiene una referencia al JTextArea de la consola
	 *
	 * @return javax.swing.JTextArea
	 */
	public JTextArea getTxt() {
		if (txt == null) {
			txt = new JTextArea();
			txt.addCaretListener(new CaretListener() {
					public void caretUpdate(CaretEvent e) {
						if (txt.getCaretPosition() < startingCaretPosition) {
							if (startingCaretPosition <= txt.getText().length()) {
								txt.setCaretPosition(startingCaretPosition);
							}
						}
					}
				});
			txt.addKeyListener(new java.awt.event.KeyAdapter() {
					public void keyPressed(KeyEvent e) {
						if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
							if (startingCaretPosition >= txt.getCaretPosition()) {
								int caretPos = txt.getCaretPosition();
								String text = txt.getText();
								text = text.substring(0, caretPos) + " " +
									text.substring(caretPos);
								txt.setText(text);
								txt.setCaretPosition(caretPos);
							}
						}
					}

					public void keyReleased(KeyEvent e) {
						if (e.getKeyCode() == KeyEvent.VK_ENTER) {
							String texto = txt.getText();
							String response = texto.substring(startingCaretPosition);

							listenerSupport.callAcceptResponse(response);

							if (response.trim().length() > 0) {
								entries.add(response.trim());
							}

							currentEntry = -1;
						} else if (e.getKeyCode() == KeyEvent.VK_UP) {
							if (entries.size() == 0) {
								return;
							}

							if (currentEntry == -1) {
								currentEntry = entries.size() - 1;
							} else {
								currentEntry--;

								if (currentEntry < 0) {
									currentEntry = 0;
								}
							}

							putEntry();
						} else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
							if (entries.size() == 0) {
								return;
							}

							if (currentEntry != -1) {
								currentEntry++;

								if (currentEntry >= entries.size()) {
									currentEntry = entries.size() - 1;
								}
							}

							putEntry();
						} else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
							txt.setText(txt.getText());
							listenerSupport.callAcceptResponse(null);
						}
					}

					private void putEntry() {
						String anterior = txt.getText();
						anterior = anterior.substring(0, startingCaretPosition);
						txt.setText(anterior + entries.get(currentEntry));
					}
				});
		}

		return txt;
	}

	/**
	 * Aade un texto a la consola
	 *
	 * @param text Texto que se aade a la consola
	 */
	public void addText(String text) {
		txt.setText(txt.getText() + text);
		txt.setCaretPosition(txt.getText().length());
		startingCaretPosition = txt.getText().length();
	}

	/**
	 * Aade un texto a la consola que es tomado como si lo
	 * hubiese escrito el usuario. Formar parte de la respuesta
	 *
	 * @param text Texto que se aade a la consola
	 */
	public void addResponseText(String text) {
		txt.setText(txt.getText() + text);
		txt.setCaretPosition(txt.getText().length());
	}

	/**
	 * This method initializes jScrollPane
	 *
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getTxt());
		}

		return jScrollPane;
	}
	public void addResponseListener(ResponseListener listener) {
		listenerSupport.addResponseListener(listener);
	}
	public void removeResponseListener(ResponseListener listener) {
		listenerSupport.removeResponseListener(listener);
	}
}
